@@ -60,18 +60,7 @@ _cloudsdk_root_dir() {
 }
 CLOUDSDK_ROOT_DIR=$(_cloudsdk_root_dir "$0")
 
-# Cloud SDK requires python 2 (2.6 or 2.7)
-case $CLOUDSDK_PYTHON in
-*python2*)
-  ;;
-*python[0-9]*)
-  CLOUDSDK_PYTHON=
-  ;;
-esac
-# if CLOUDSDK_PYTHON is empty
-if [ -z "$CLOUDSDK_PYTHON" ]; then
-  CLOUDSDK_PYTHON=/usr/local/bin/python2.7
-fi
+CLOUDSDK_PYTHON=/usr/local/bin/python2.7
 
 # if CLOUDSDK_PYTHON_SITEPACKAGES and VIRTUAL_ENV are empty
 case :$CLOUDSDK_PYTHON_SITEPACKAGES:$VIRTUAL_ENV: in
