@@ -21,27 +21,25 @@ import urllib2
 from googlecloudsdk.api_lib.app import logs_requestor
 from googlecloudsdk.api_lib.app import util
 from googlecloudsdk.api_lib.app import yaml_parsing
-from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.credentials import devshell as c_devshell
+from googlecloudsdk.core.credentials import http
 from googlecloudsdk.core.credentials import service_account as c_service_account
 from googlecloudsdk.core.credentials import store as c_store
 from googlecloudsdk.third_party.appengine.datastore import datastore_index
 from googlecloudsdk.third_party.appengine.tools import appengine_rpc_httplib2
-from oauth2client import gce as oauth2client_gce
+from oauth2client.contrib import gce as oauth2client_gce
 import yaml
 
 
-APPCFG_SCOPES = ('https://www.googleapis.com/auth/appengine.admin',
-                 'https://www.googleapis.com/auth/cloud-platform')
+APPCFG_SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
 
 # Parameters for reading from the GCE metadata service.
 METADATA_BASE = 'http://metadata.google.internal'
-SERVICE_ACCOUNT_BASE = (
-    'computeMetadata/v1beta1/instance/service-accounts/default')
+SERVICE_ACCOUNT_BASE = ('computeMetadata/v1/instance/service-accounts/default')
 
 RpcServerClass = appengine_rpc_httplib2.HttpRpcServerOAuth2  # pylint: disable=invalid-name
 
@@ -79,13 +77,18 @@ class AppengineClient(object):
     self.oauth2_refresh_token = None
     self.oauth_scopes = APPCFG_SCOPES
     self.authenticate_service_account = False
+    self.client_id = None
+    self.client_secret = None
 
     account = properties.VALUES.core.account.Get()
     # This statement will raise a c_store.Error if there is a problem
     # fetching credentials.
     credentials = c_store.Load(account=account)
-    if (isinstance(credentials, c_devshell.DevshellCredentials) or
-        isinstance(credentials, c_service_account.ServiceAccountCredentials)):
+    if isinstance(credentials, c_service_account.ServiceAccountCredentials):
+      self.oauth2_access_token = credentials.access_token
+      self.client_id = credentials.client_id
+      self.client_secret = credentials.client_secret
+    elif isinstance(credentials, c_devshell.DevshellCredentials):
       # TODO(user): This passes the access token to use for API calls to
       # appcfg which means that commands that are longer than the lifetime
       # of the access token may fail - e.g. some long deployments.  The proper
@@ -93,12 +96,18 @@ class AppengineClient(object):
       # this code will go away then and the standard credentials flow will be
       # used.
       self.oauth2_access_token = credentials.access_token
+      self.client_id = None
+      self.client_secret = None
     elif isinstance(credentials, oauth2client_gce.AppAssertionCredentials):
       # If we are on GCE, use the service account
       self.authenticate_service_account = True
+      self.client_id = None
+      self.client_secret = None
     else:
       # Otherwise use a stored refresh token
       self.oauth2_refresh_token = credentials.refresh_token
+      self.client_id = credentials.client_id
+      self.client_secret = credentials.client_secret
 
   def CleanupIndexes(self, index_yaml):
     """Removes unused datastore indexes.
@@ -142,12 +151,12 @@ class AppengineClient(object):
           warning_message += index.ToYAML()
         log.warning(warning_message)
 
-  def GetLogs(self, module, version, severity, vhost, include_vhost,
+  def GetLogs(self, service, version, severity, vhost, include_vhost,
               include_all, num_days, end_date, output_file):
-    """Get application logs for the given version of the module.
+    """Get application logs for the given version of the service.
 
     Args:
-      module: str, The module of the app to fetch logs from.
+      service: str, The service of the app to fetch logs from.
       version: str, The version of the app to fetch logs for.
       severity: int, App log severity to request (0-4); None for request logs
         only.
@@ -164,16 +173,16 @@ class AppengineClient(object):
     """
     rpcserver = self._GetRpcServer()
     requestor = logs_requestor.LogsRequester(
-        rpcserver, self.project, module, version, severity, vhost,
+        rpcserver, self.project, service, version, severity, vhost,
         include_vhost, include_all)
     requestor.DownloadLogs(num_days, end_date, output_file)
 
-  def GetLogsAppend(self, module, version, severity, vhost, include_vhost,
+  def GetLogsAppend(self, service, version, severity, vhost, include_vhost,
                     include_all, end_date, output_file):
     """Get application logs and append them to an existing file.
 
     Args:
-      module: str, The module of the app to fetch logs from.
+      service: str, The service of the app to fetch logs from.
       version: str, The version of the app to fetch logs for.
       severity: int, App log severity to request (0-4); None for request logs
         only.
@@ -188,70 +197,63 @@ class AppengineClient(object):
     """
     rpcserver = self._GetRpcServer()
     requestor = logs_requestor.LogsRequester(
-        rpcserver, self.project, module, version, severity, vhost,
+        rpcserver, self.project, service, version, severity, vhost,
         include_vhost, include_all)
     requestor.DownloadLogsAppend(end_date, output_file)
 
   def PrepareVmRuntime(self):
     """Prepare the application for vm runtimes and return state."""
-    rpcserver = self._GetRpcServer()
+    rpcserver = self._GetRpcServer(timeout_max_errors=5)
     rpcserver.Send('/api/vms/prepare', app_id=self.project)
 
-  def SetManagedByGoogle(self, module, version, instance=None, wait=True):
-    """Sets a module version (and optionally an instance) to Google managed.
+  # TODO(b/29059251): vm_name and instance id are different, this API client
+  # needs the VM name. The Zeus API will use instance id instead.
+  def SetManagedByGoogle(self, service, version, vm_name=None, wait=True):
+    """Sets a service version (and optionally an instance) to Google managed.
 
     This will reboot the machine and restore the instance with a fresh runtime.
 
     Args:
-      module: str, The module to update.
-      version: str, The version of the module to update.
-      instance: str, The instance id of a single instance to update.
+      service: str, The service to update.
+      version: str, The version of the service to update.
+      vm_name: str, The vm name of the instance to update.
       wait: bool, True to wait until it takes effect.
-
-    Returns:
-      None, if not waiting.  If waiting, returns (bool, message) for the last
-      attempt at checking state.
     """
-    return self._SetManagedBy(module, version, instance, '/api/vms/lock', wait)
+    self._SetManagedBy(service, version, vm_name, '/api/vms/lock', wait)
 
-  def SetManagedBySelf(self, module, version, instance=None, wait=True):
-    """Sets a module version (and optionally a single instance) as self managed.
+  def SetManagedBySelf(self, service, version, vm_name=None, wait=True):
+    """Sets a service version (optionally a single instance) as self managed.
 
     This is the 'break the glass' mode that lets you ssh into the machine and
     debug.
 
     Args:
-      module: str, The module to update.
-      version: str, The version of the module to update.
-      instance: str, The instance id of a single instance to update.
+      service: str, The service to update.
+      version: str, The version of the service to update.
+      vm_name: str, The vm name of the instance to update.
       wait: bool, True to wait until it takes effect.
-
-    Returns:
-      None, if not waiting.  If waiting, returns (bool, message) for the last
-      attempt at checking state.
     """
-    return self._SetManagedBy(module, version, instance, '/api/vms/debug', wait)
+    self._SetManagedBy(service, version, vm_name, '/api/vms/debug', wait)
 
-  def _SetManagedBy(self, module, version, instance, url, wait):
-    """Switches a module version between management modes.
+  def _SetManagedBy(self, service, version, vm_name, url, wait):
+    """Switches a service version between management modes.
 
     Args:
-      module: str, The module to update.
-      version: str, The version of the module to update.
-      instance: str, The instance id of a single instance to update.
+      service: str, The service to update.
+      version: str, The version of the service to update.
+      vm_name: str, The vm name of the instance to update.
       url: str, The URL of the API to call to make the update.
       wait: bool, True to wait until it takes effect.
 
-    Returns:
-      None, if not waiting.  If waiting, returns (bool, message) for the last
-      attempt at checking state.
+    Raises:
+      Error: if changing the instance debug state failed.
     """
     rpcserver = self._GetRpcServer()
     kwargs = {'app_id': self.project,
               'version_match': version,
-              'module': module}
-    if instance:
-      kwargs['instance'] = instance
+              'module': service}
+    if vm_name:
+      kwargs['instance'] = vm_name
 
     rpcserver.Send(url, **kwargs)
 
@@ -259,41 +261,44 @@ class AppengineClient(object):
       def GetState():
         yaml_data = rpcserver.Send(
             '/api/vms/debugstate', app_id=self.project, version_match=version,
-            module=module)
+            module=service)
         state = yaml.safe_load(yaml_data)
         done = state['state'] != 'PENDING'
-        return (done, state['message'])
+        return (done, (state['state'], state['message']))
 
-      def PrintRetryMessage(msg, delay):
+      def PrintRetryMessage((unused_state, msg), delay):
         log.status.Print('{0}.  Will try again in {1} seconds.'
                          .format(msg, delay))
 
-      return util.RetryWithBackoff(GetState, PrintRetryMessage, initial_delay=1,
-                                   backoff_factor=2, max_delay=5, max_tries=20)
+      _, (state, message) = util.RetryWithBackoff(
+          GetState, PrintRetryMessage, initial_delay=1, backoff_factor=2,
+          max_delay=5, max_tries=20)
+      if state == 'ERROR':
+        raise Error(message)
 
-  def StartModule(self, module, version):
-    """Starts serving a the given version of the module.
+  def StartService(self, service, version):
+    """Starts serving a the given version of the service.
 
     This only works if scaling is set to manual.
 
     Args:
-      module: str, The module to start.
-      version: str, The version of the module to start.
+      service: str, The service to start.
+      version: str, The version of the service to start.
     """
     self._GetRpcServer().Send('/api/modules/start', app_id=self.project,
-                              module=module, version=version)
+                              module=service, version=version)
 
-  def StopModule(self, module, version):
-    """Stop serving a the given version of the module.
+  def StopService(self, service, version):
+    """Stop serving a the given version of the service.
 
     This only works if scaling is set to manual.
 
     Args:
-      module: str, The module to stop.
-      version: str, The version of the module to stop.
+      service: str, The service to stop.
+      version: str, The version of the service to stop.
     """
     self._GetRpcServer().Send('/api/modules/stop', app_id=self.project,
-                              module=module, version=version)
+                              module=service, version=version)
 
   def UpdateConfig(self, config_name, parsed_yaml):
     """Updates any of the supported config file types.
@@ -367,25 +372,34 @@ class AppengineClient(object):
     self._GetRpcServer().Send('/api/queue/update',
                               app_id=self.project, payload=queue_yaml.ToYAML())
 
-  def _GetRpcServer(self):
+  def _GetRpcServer(self, timeout_max_errors=2):
     """Returns an instance of an AbstractRpcServer.
 
+    Args:
+      timeout_max_errors: How many timeout errors to retry.
     Returns:
       A new AbstractRpcServer, on which RPC calls can be made.
     """
     log.debug('Host: {0}'.format(self.server))
 
+    if self._IsGceEnvironment():
+      credentials = oauth2client_gce.AppAssertionCredentials()
+    else:
+      credentials = None
+
     # In this case, the get_user_credentials parameters to the RPC server
     # constructor is actually an OAuth2Parameters.
+
     get_user_credentials = (
         appengine_rpc_httplib2.HttpRpcServerOAuth2.OAuth2Parameters(
             access_token=self.oauth2_access_token,
-            client_id=config.CLOUDSDK_CLIENT_ID,
-            client_secret=config.CLOUDSDK_CLIENT_NOTSOSECRET,
+            client_id=self.client_id,
+            client_secret=self.client_secret,
             scope=APPCFG_SCOPES,
             refresh_token=self.oauth2_refresh_token,
             credential_file=None,
-            token_uri=self._GetTokenUri()))
+            token_uri=None,
+            credentials=credentials))
     # Also set gflags flag... this is a bit of a hack.
     if hasattr(appengine_rpc_httplib2.tools, 'FLAGS'):
       appengine_rpc_httplib2.tools.FLAGS.auth_local_webserver = True
@@ -398,20 +412,22 @@ class AppengineClient(object):
         host_override=None,
         save_cookies=True,
         auth_tries=3,
+        timeout_max_errors=timeout_max_errors,
         account_type='HOSTED_OR_GOOGLE',
         secure=True,
-        ignore_certs=self.ignore_bad_certs)
+        ignore_certs=self.ignore_bad_certs,
+        http_object=http.Http())
     # TODO(user) Hack to avoid failure due to missing cacerts.txt resource.
     server.certpath = None
     # Don't use a cert file if the user passed ignore-bad-certs.
     server.cert_file_available = not self.ignore_bad_certs
     return util.RPCServer(server)
 
-  def _GetTokenUri(self):
-    """Returns the OAuth2 token_uri, or None to use the default URI.
+  def _IsGceEnvironment(self):
+    """Determine if we are running in a GCE environment.
 
     Returns:
-      A string that is the token_uri, or None.
+      True if we are running in a GCE environment.
 
     Raises:
       Error: The user has requested authentication for a service account but the
@@ -423,7 +439,7 @@ class AppengineClient(object):
       # with access to the appengine.admin scope.
       url = '%s/%s/scopes' % (METADATA_BASE, SERVICE_ACCOUNT_BASE)
       try:
-        req = urllib2.Request(url)
+        req = urllib2.Request(url, headers={'Metadata-Flavor': 'Google'})
         vm_scopes_string = urllib2.urlopen(req).read()
       except urllib2.URLError, e:
         raise Error(
@@ -434,9 +450,13 @@ class AppengineClient(object):
       missing = list(set(self.oauth_scopes).difference(vm_scopes))
       if missing:
         raise Error(
-            'Required scopes %s missing from %s. '
-            'This VM instance probably needs to be recreated '
-            'with the missing scopes.' % (missing, vm_scopes))
-      return '%s/%s/token' % (METADATA_BASE, SERVICE_ACCOUNT_BASE)
+            'You are currently logged into gcloud using a service account '
+            'which does not have the appropriate access to [{0}]. The account '
+            'has the following scopes: [{1}].  It needs [{2}] in order to '
+            'succeed.\nPlease recreate this VM instance with the missing '
+            'scopes. You may also log into a standard account that has the '
+            'appropriate access by using `gcloud auth login`.'
+            .format(self.project, ', '.join(vm_scopes), ', '.join(missing)))
+      return True
     else:
-      return None
+      return False
