@@ -21,8 +21,10 @@ DOCKER_PY_VERSION = 'auto'
 # Timeout of HTTP request from docker-py client to docker daemon, in seconds.
 DOCKER_D_REQUEST_TIMEOUT = 300
 
-DOCKER_IMAGE_NAME_FORMAT = '{display}.{module}.{version}'
-DOCKER_IMAGE_NAME_DOMAIN_FORMAT = '{domain}.{display}.{module}.{version}'
+DOCKER_IMAGE_NAME_FORMAT = 'us.gcr.io/{display}/appengine/{service}.{version}'
+DOCKER_IMAGE_TAG = 'latest'
+DOCKER_IMAGE_NAME_DOMAIN_FORMAT = (
+    'us.gcr.io/{domain}/{display}/appengine/{service}.{version}')
 
 # Name of the a Dockerfile.
 DOCKERFILE = 'Dockerfile'
