@@ -11,17 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Base classes for abstracting away common logic."""
+
 import abc
-import argparse
 import cStringIO
 import json
-import sys
 import textwrap
 
-from enum import Enum
+import enum
+from googlecloudsdk.api_lib.compute import client_adapter
 from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.api_lib.compute import lister
+from googlecloudsdk.api_lib.compute import managed_instance_groups_utils
 from googlecloudsdk.api_lib.compute import metadata_utils
 from googlecloudsdk.api_lib.compute import path_simplifier
 from googlecloudsdk.api_lib.compute import property_selector
@@ -33,13 +35,13 @@ from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
-from googlecloudsdk.core import log
+from googlecloudsdk.command_lib.compute import flags
+from googlecloudsdk.core import apis as core_apis
 from googlecloudsdk.core import properties
-from googlecloudsdk.core import resource_printer
-from googlecloudsdk.core import resources as resource_exceptions
+from googlecloudsdk.core import resolvers
+from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import edit
-from googlecloudsdk.third_party.apis.compute.v1 import compute_v1_messages
 from googlecloudsdk.third_party.apitools.base.protorpclite import messages
 from googlecloudsdk.third_party.apitools.base.py import encoding
 from googlecloudsdk.third_party.py27 import py27_collections as collections
@@ -47,48 +49,90 @@ from googlecloudsdk.third_party.py27 import py27_copy as copy
 import yaml
 
 
-def PrintTable(resources, table_cols):
-  """Prints a table of the given resources."""
-  # TODO(user): Switch over to console_io.TablePrinter once the
-  # class is refactored to support tables without ASCII borders.
-  printer = resource_printer.TablePrinter(out=log.out)
+class ComputeApiHolder(object):
+  """Convenience class to hold lazy initialized client and resources."""
 
-  header = []
-  for name, _ in table_cols:
-    header.append(name)
-  printer.AddRow(header)
+  def __init__(self, release_track):
+    if release_track == base.ReleaseTrack.ALPHA:
+      self._api_version = 'alpha'
+    elif release_track == base.ReleaseTrack.BETA:
+      self._api_version = 'beta'
+    else:
+      self._api_version = 'v1'
+    self._client = None
+    self._resources = None
 
-  try:
-    for resource in resources:
-      row = []
-      for _, action in table_cols:
-        if isinstance(action, property_selector.PropertyGetter):
-          row.append(action.Get(resource) or '')
-        elif callable(action):
-          row.append(action(resource))
-      printer.AddRow(row)
-  finally:
-    printer.Print()
+  @property
+  def client(self):
+    """Specifies the compute client."""
+    if self._client is None:
+      self._client = client_adapter.ClientAdapter(self._api_version)
+    return self._client
+
+  @property
+  def resources(self):
+    """Specifies the resources parser for compute resources."""
+    if self._resources is None:
+      _SetResourceParamDefaults()
+      self._resources = resources.REGISTRY.CloneAndSwitchAPIs(
+          self.client.apitools_client)
+    return self._resources
+
+
+class ComputeUserAccountsApiHolder(object):
+  """Convenience class to hold lazy initialized client and resources."""
+
+  def __init__(self, release_track):
+    if release_track == base.ReleaseTrack.ALPHA:
+      self._api_version = 'alpha'
+    else:
+      self._api_version = 'beta'
+    self._client = None
+    self._resources = None
+
+  @property
+  def client(self):
+    """Specifies the compute client."""
+    if self._client is None:
+      self._client = core_apis.GetClientInstance(
+          'clouduseraccounts', self._api_version)
+    return self._client
+
+  @property
+  def resources(self):
+    """Specifies the resources parser for compute resources."""
+    if self._resources is None:
+      resources.SetParamDefault(
+          api='clouduseraccounts',
+          collection=None,
+          param='project',
+          resolver=resolvers.FromProperty(properties.VALUES.core.project))
+      self._resources = resources.REGISTRY.CloneAndSwitchAPIs(self.client)
+    return self._resources
 
 
 class BaseCommand(base.Command, scope_prompter.ScopePrompter):
   """Base class for all compute subcommands."""
 
-  __metaclass__ = abc.ABCMeta
-
   def __init__(self, *args, **kwargs):
     super(BaseCommand, self).__init__(*args, **kwargs)
 
-    # Set the default project for resource resolution
+    self.__resource_spec = None
+    self._project = properties.VALUES.core.project.Get(required=True)
+    self._compute_holder = ComputeApiHolder(self.ReleaseTrack())
+    self._user_accounts_holder = ComputeUserAccountsApiHolder(
+        self.ReleaseTrack())
 
-    if self.resource_type:
+  @property
+  def _resource_spec(self):
+    if not self.resource_type:
+      return None
+    if self.__resource_spec is None:
       # Constructing the spec can be potentially expensive (e.g.,
       # generating the set of valid fields from the protobuf message),
-      # so we fetch it once in the constructor.
-      self._resource_spec = resource_specs.GetSpec(
-          self.resource_type, self.messages, self.context['api-version'])
-    else:
-      self._resource_spec = None
+      self.__resource_spec = resource_specs.GetSpec(
+          self.resource_type, self.messages, self.compute_client.api_version)
+    return self.__resource_spec
 
   @property
   def transformations(self):
@@ -105,65 +149,56 @@ class BaseCommand(base.Command, scope_prompter.ScopePrompter):
   @property
   def http(self):
     """Specifies the http client to be used for requests."""
-    return self.context['http']
+    return self.compute_client.apitools_client.http
 
   @property
   def project(self):
     """Specifies the user's project."""
-    return self.context['project']
+    return self._project
 
   @property
   def batch_url(self):
     """Specifies the API batch URL."""
-    return self.context['batch-url']
+    return self.compute_client.batch_url
+
+  @property
+  def compute_client(self):
+    """Specifies the compute client."""
+    return self._compute_holder.client
 
   @property
   def compute(self):
     """Specifies the compute client."""
-    return self.context['compute']
+    return self.compute_client.apitools_client
 
   @property
   def resources(self):
     """Specifies the resources parser for compute resources."""
-    return self.context['resources']
+    return self._compute_holder.resources
 
   @property
   def clouduseraccounts(self):
-    return self.context['clouduseraccounts']
+    return self._user_accounts_holder.client
 
   @property
   def clouduseraccounts_resources(self):
-    return self.context['clouduseraccounts-resources']
+    return self._user_accounts_holder.resources
 
   @property
   def messages(self):
     """Specifies the API message classes."""
-    return self.compute.MESSAGES_MODULE
+    return self.compute_client.messages
 
-  def Format(self, args):
-    return 'default'
+  def Collection(self):
+    """Returns the resource collection path."""
+    return 'compute.' + self.resource_type if self.resource_type else None
 
 
-class BaseLister(BaseCommand):
+class BaseLister(base.ListCommand, BaseCommand):
   """Base class for the list subcommands."""
 
-  __metaclass__ = abc.ABCMeta
-
   @staticmethod
   def Args(parser):
-    parser.add_argument(
-        '--limit',
-        type=arg_parsers.BoundedInt(1, sys.maxint),
-        help='The maximum number of results.')
-
-    sort_by = parser.add_argument(
-        '--sort-by',
-        help='A field to sort by.')
-    sort_by.detailed_help = """\
-        A field to sort by. To perform a descending-order sort, prefix
-        the value of this flag with a tilde (``~'').
-        """
-
     parser.add_argument(
         'names',
         metavar='NAME',
@@ -173,17 +208,6 @@ class BaseLister(BaseCommand):
         help=('If provided, show details for the specified names and/or URIs '
               'of resources.'))
 
-    uri = parser.add_argument(
-        '--uri',
-        action='store_true',
-        help='If provided, a list of URIs is printed instead of a table.')
-    uri.detailed_help = """\
-        If provided, the list command will only print URIs for the
-        resources returned.  If this flag is not provided, the list
-        command will print a human-readable table of useful resource
-        data.
-        """
-
     regexp = parser.add_argument(
         '--regexp', '-r',
         help='A regular expression to filter the names of the results on.')
@@ -226,7 +250,7 @@ class BaseLister(BaseCommand):
         self.self_links.add(ref.SelfLink())
         self.resource_refs.append(ref)
         continue
-      except resource_exceptions.UserError:
+      except resources.UserError:
         pass
 
       self.names.add(name)
@@ -256,38 +280,12 @@ class BaseLister(BaseCommand):
     self.names = set()
     self.resource_refs = []
 
-    if args.uri:
-      field_selector = None
-    else:
-      # The field selector should be constructed before any resources
-      # are fetched, so if there are any syntactic errors with the
-      # fields, we can fail fast.
-      field_selector = property_selector.PropertySelector(
-          properties=None,
-          transformations=self.transformations)
-
-    if args.sort_by:
-      if args.sort_by.startswith('~'):
-        sort_by = args.sort_by[1:]
-        descending = True
-      else:
-        sort_by = args.sort_by
-        descending = False
-
-      for col_name, path in self._resource_spec.table_cols:
-        if sort_by == col_name:
-          sort_by = path
-          break
-
-      if isinstance(sort_by, property_selector.PropertyGetter):
-        property_getter = sort_by
-      else:
-        property_getter = property_selector.PropertyGetter(sort_by)
-      sort_key_fn = property_getter.Get
-
-    else:
-      sort_key_fn = None
-      descending = False
+    # The field selector should be constructed before any resources
+    # are fetched, so if there are any syntactic errors with the
+    # fields, we can fail fast.
+    field_selector = property_selector.PropertySelector(
+        properties=None,
+        transformations=self.transformations)
 
     errors = []
 
@@ -296,28 +294,15 @@ class BaseLister(BaseCommand):
     items = lister.ProcessResults(
         resources=items,
         field_selector=field_selector,
-        sort_key_fn=sort_key_fn,
-        reverse_sort=descending,
         limit=args.limit)
     items = self.ComputeDynamicProperties(args, items)
 
     for item in items:
-      if args.uri:
-        yield item['selfLink']
-      else:
-        yield item
+      yield item
 
     if errors:
       utils.RaiseToolException(errors)
 
-  def Display(self, args, resources):
-    """Prints the given resources."""
-    if args.uri:
-      for resource in resources:
-        log.out.Print(resource)
-    else:
-      PrintTable(resources, self._resource_spec.table_cols)
-
 
 class GlobalLister(BaseLister):
   """Base class for listing global resources."""
@@ -463,7 +448,7 @@ def GetZonalListerHelp(resource):
   }
 
 
-class ScopeType(Enum):
+class ScopeType(enum.Enum):
   """Scope type of compute resource."""
   global_scope = 1
   regional_scope = 2
@@ -473,8 +458,6 @@ class ScopeType(Enum):
 class MultiScopeLister(BaseLister):
   """Base class for listing global and regional resources."""
 
-  __metaclass__ = abc.ABCMeta
-
   @staticmethod
   def AddScopeArgs(parser, scopes):
     BaseLister.Args(parser)
@@ -536,21 +519,10 @@ class MultiScopeLister(BaseLister):
     max_results = constants.MAX_RESULTS_PER_PAGE
     project = self.project
 
-    # If --global is present OR no scope flags are present then we have to fetch
-    # the global resources.
-    if has_global:
-      requests.append(
-          (self.global_service,
-           'List',
-           self.global_service.GetRequestType('List')(
-               filter=filter_expr,
-               maxResults=max_results,
-               project=project)))
-
     # If --regions is present with no arguments OR no scope flags are present
     # then we have to do an aggregated list
     # pylint:disable=g-explicit-bool-comparison
-    if no_scope_flags:
+    if no_scope_flags and self.aggregation_service:
       requests.append(
           (self.aggregation_service,
            'AggregatedList',
@@ -586,6 +558,16 @@ class MultiScopeLister(BaseLister):
                  maxResults=max_results,
                  zone=zone_name,
                  project=project)))
+    else:
+      # Either --global was specified or we do not have aggregation service.
+      # Note that --global, --region and --zone are mutually exclusive.
+      requests.append(
+          (self.global_service,
+           'List',
+           self.global_service.GetRequestType('List')(
+               filter=filter_expr,
+               maxResults=max_results,
+               project=project)))
 
     return request_helper.MakeRequests(
         requests=requests,
@@ -686,8 +668,6 @@ def GetGlobalRegionalListerHelp(resource):
 class BaseDescriber(BaseCommand):
   """Base class for the describe subcommands."""
 
-  __metaclass__ = abc.ABCMeta
-
   @staticmethod
   def Args(parser, resource=None, list_command_path=None):
     BaseDescriber.AddArgs(parser, resource, list_command_path)
@@ -724,10 +704,6 @@ class BaseDescriber(BaseCommand):
 
   def Run(self, args):
     """Yields JSON-serializable dicts of resources."""
-    # The field selector should be constructed before any resources
-    # are fetched, so if there are any syntactic errors with the
-    # fields, we can fail fast.
-    field_selector = property_selector.PropertySelector(properties=args.fields)
     ref = self.CreateReference(args)
 
     get_request_class = self.service.GetRequestType(self.method)
@@ -746,14 +722,14 @@ class BaseDescriber(BaseCommand):
         errors=errors,
         custom_get_requests=None)
 
-    resources = lister.ProcessResults(objects, field_selector)
-    resources = list(self.ComputeDynamicProperties(args, resources))
+    resource_list = lister.ProcessResults(objects, field_selector=None)
+    resource_list = list(self.ComputeDynamicProperties(args, resource_list))
 
     if errors:
       utils.RaiseToolException(
           errors,
           error_message='Could not fetch resource:')
-    return resources[0]
+    return resource_list[0]
 
 
 class GlobalDescriber(BaseDescriber):
@@ -769,7 +745,7 @@ class RegionalDescriber(BaseDescriber):
   @staticmethod
   def Args(parser, resource=None, command=None):
     BaseDescriber.AddArgs(parser, resource, command)
-    utils.AddRegionFlag(
+    flags.AddRegionFlag(
         parser,
         resource_type='resource',
         operation_type='fetch')
@@ -787,7 +763,7 @@ class ZonalDescriber(BaseDescriber):
   @staticmethod
   def Args(parser, resource=None, command=None):
     BaseDescriber.AddArgs(parser, resource, command)
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser,
         resource_type='resource',
         operation_type='fetch')
@@ -802,13 +778,10 @@ class ZonalDescriber(BaseDescriber):
 class MultiScopeDescriber(BaseDescriber):
   """Base class for describing global or regional resources."""
 
-  __metaclass__ = abc.ABCMeta
-
   @staticmethod
   def AddScopeArgs(parser, resource_type, scope_types, command=None):
     resource = resource_type
     BaseDescriber.AddArgs(parser, 'compute.' + resource, command)
-    AddFieldsFlag(parser, resource_type)
 
     scope = parser.add_mutually_exclusive_group()
 
@@ -859,7 +832,7 @@ class MultiScopeDescriber(BaseDescriber):
   def service(self):
     return self._service
 
-  def CreateReference(self, args):
+  def CreateReference(self, args, default=None):
     has_region = hasattr(args, 'region') and args.region
     has_zone = hasattr(args, 'zone') and args.zone
     has_global = hasattr(args, 'global') and getattr(args, 'global')
@@ -867,6 +840,14 @@ class MultiScopeDescriber(BaseDescriber):
     only_zone_prompt = hasattr(args, 'zone') and not hasattr(args, 'region')
     only_region_prompt = hasattr(args, 'region') and not hasattr(args, 'zone')
 
+    if not (has_region or has_zone or has_global):
+      if default == ScopeType.global_scope:
+        has_global = True
+      elif default == ScopeType.regional_scope:
+        has_region = True
+      elif default == ScopeType.zonal_scope:
+        has_zone = True
+
     ref = None
     try:
       params = {}
@@ -875,7 +856,7 @@ class MultiScopeDescriber(BaseDescriber):
       if has_zone:
         params['zone'] = args.zone
       ref = self.resources.Parse(args.name, params=params)
-    except resource_exceptions.UnknownCollectionException:
+    except resources.UnknownCollectionException:
       ref = None
 
     if ref is None:
@@ -979,50 +960,9 @@ class GlobalRegionalDescriber(MultiScopeDescriber):
     return None
 
 
-def AddFieldsFlag(parser, resource_type):
-  """Adds the --fields flag to the given parser.
-
-  This function is to be called from implementations of describe/list
-  subcommands. The resulting help text of --fields will contain all
-  valid values for the flag. We need this function becasue Args() is a
-  static method so the only way to communicate the resource type is by
-  having the subclass pass it in.
-
-  Args:
-    parser: The parser to add --fields to.
-    resource_type: The resource type as defined in the resource_specs
-      module.
-  """
-
-  def GenerateDetailedHelp():
-    return ('Fields to display. Possible values are:\n+\n  ' +
-            '\n  '.join(resource_specs.GetSpec(
-                resource_type, compute_v1_messages, 'v1').fields))
-
-  fields = parser.add_argument(
-      '--fields',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='FIELD',
-      action=arg_parsers.FloatingListValuesCatcher(),
-
-      # We have not reached an agreement over the --fields flag for
-      # Cloud SDK tools. It has been agreed that the compute component
-      # will keep --fields but will keep it undocumented until
-      # consensus can be reached over the flag's fate.
-      help=argparse.SUPPRESS)
-
-  # Note that we do not actually call GenerateDetailedHelp, the help
-  # generator does that. This is important because getting the set of
-  # fields is a potentially expensive operation, so we only want to do
-  # it when needed.
-  fields.detailed_help = GenerateDetailedHelp
-
-
 class BaseAsyncMutator(BaseCommand):
   """Base class for subcommands that mutate resources."""
 
-  __metaclass__ = abc.ABCMeta
-
   @abc.abstractproperty
   def service(self):
     """The service that can mutate resources."""
@@ -1056,9 +996,19 @@ class BaseAsyncMutator(BaseCommand):
       args: The command-line arguments.
 
     Returns:
-      A list of request protobufs.
+      A list of requests. Request could be one of:
+        * protobuf - protobuf will be sent to self.service and self.method
+        * (method, protobuf) - protobuf will be sent to self.service and
+            provided method
+        * (service, method, protobuf) - protobuf will be sent to
+            provided service and method
     """
 
+  def ComputeDynamicProperties(self, args, items):
+    """Computes dynamic properties, which are not returned by GCE API."""
+    _ = args
+    return items
+
   def Run(self, args, request_protobufs=None, service=None):
     if request_protobufs is None:
       request_protobufs = self.CreateRequests(args)
@@ -1069,8 +1019,10 @@ class BaseAsyncMutator(BaseCommand):
     # default
     for request in request_protobufs:
       if isinstance(request, tuple):
-        method = request[0]
-        proto = request[1]
+        if len(request) == 2:
+          method, proto = request
+        elif len(request) == 3:
+          service, method, proto = request
       else:
         method = self.method
         proto = request
@@ -1079,26 +1031,25 @@ class BaseAsyncMutator(BaseCommand):
     errors = []
     # We want to run through the generator that MakeRequests returns in order to
     # actually make the requests, since these requests mutate resources.
-    resources = list(request_helper.MakeRequests(
+    resource_list = list(request_helper.MakeRequests(
         requests=requests,
         http=self.http,
         batch_url=self.batch_url,
         errors=errors,
         custom_get_requests=self.custom_get_requests))
 
-    resources = lister.ProcessResults(
-        resources=resources,
+    resource_list = lister.ProcessResults(
+        resources=resource_list,
         field_selector=property_selector.PropertySelector(
             properties=None,
             transformations=self.transformations))
 
+    resource_list = self.ComputeDynamicProperties(args, resource_list)
+
     if errors:
       utils.RaiseToolException(errors)
 
-    return resources
-
-  def Format(self, args):
-    return 'default'
+    return resource_list
 
 
 class NoOutputAsyncMutator(BaseAsyncMutator):
@@ -1108,16 +1059,32 @@ class NoOutputAsyncMutator(BaseAsyncMutator):
     return 'none'
 
 
-class InstanceGroupFilteringMode(Enum):
-  """Filtering mode for Instance Groups based on dynamic properties."""
-  all_groups = 1
-  only_managed_groups = 2
-  only_unmanaged_groups = 3
-
-
 class InstanceGroupManagerDynamicProperiesMixin(object):
   """Mixin class to compute dynamic information for instance groups."""
 
+  def ComputeDynamicProperties(self, args, items):
+    """Add Autoscaler information if Autoscaler is defined for the item."""
+    _ = args
+    # Items are expected to be IGMs.
+    items = list(items)
+    for mig in managed_instance_groups_utils.AddAutoscalersToMigs(
+        migs_iterator=self.ComputeInstanceGroupSize(items=items),
+        project=self.project,
+        compute=self.compute,
+        http=self.http,
+        batch_url=self.batch_url,
+        fail_when_api_not_supported=False):
+      if 'autoscaler' in mig and mig['autoscaler'] is not None:
+        if (hasattr(mig['autoscaler'], 'status') and mig['autoscaler'].status ==
+            self.messages.Autoscaler.StatusValueValuesEnum.ERROR):
+          mig['autoscaled'] = 'yes (*)'
+          self._had_errors = True
+        else:
+          mig['autoscaled'] = 'yes'
+      else:
+        mig['autoscaled'] = 'no'
+      yield mig
+
   def ComputeInstanceGroupSize(self, items):
     """Add information about Instance Group size."""
     errors = []
@@ -1174,68 +1141,14 @@ class InstanceGroupManagerDynamicProperiesMixin(object):
       yield item
 
 
-class InstanceGroupDynamicProperiesMixin(object):
-  """Mixin class to compute dynamic information for instance groups."""
+class BaseAsyncCreator(BaseAsyncMutator):
+  """Base class for subcommands that create resources.
 
-  def ComputeInstanceGroupManagerMembership(self, items,
-                                            filter_mode=(
-                                                InstanceGroupFilteringMode
-                                                .all_groups)):
-    """Add information if instance group is managed."""
-    errors = []
-    items = list(items)
-    zone_names = set([path_simplifier.Name(result['zone']) for result in items])
+  This class is a base.Command with base.ListCommand formatting.
+  """
 
-    instance_group_managers = lister.GetZonalResources(
-        service=self.compute.instanceGroupManagers,
-        project=self.project,
-        requested_zones=zone_names,
-        filter_expr=None,
-        http=self.http,
-        batch_url=self.batch_url,
-        errors=errors)
-    instance_group_managers_refs = set([
-        path_simplifier.ScopedSuffix(igm.selfLink)
-        for igm in instance_group_managers])
-
-    if errors:
-      utils.RaiseToolException(errors)
-
-    for item in items:
-      self_link = item['selfLink']
-      igm_self_link = self_link.replace(
-          '/instanceGroups/', '/instanceGroupManagers/')
-      scoped_suffix = path_simplifier.ScopedSuffix(igm_self_link)
-      is_managed = scoped_suffix in instance_group_managers_refs
-
-      if (is_managed and
-          filter_mode == InstanceGroupFilteringMode.only_unmanaged_groups):
-        continue
-      elif (not is_managed and
-            filter_mode == InstanceGroupFilteringMode.only_managed_groups):
-        continue
-
-      item['isManaged'] = ('Yes' if is_managed else 'No')
-      if is_managed:
-        item['instanceGroupManagerUri'] = igm_self_link
-      yield item
-
-
-class ListOutputMixin(object):
-  """Mixin class to display a list by default."""
-
-  def ComputeDynamicProperties(self, args, items):
-    """Computes dynamic properties, which are not returned by GCE API."""
-    _ = args
-    return items
-
-  def Display(self, args, resources):
-    PrintTable(self.ComputeDynamicProperties(args, resources),
-               self._resource_spec.table_cols)
-
-
-class BaseAsyncCreator(ListOutputMixin, BaseAsyncMutator):
-  """Base class for subcommands that create resources."""
+  def Format(self, args):
+    return self.ListFormat(args)
 
 
 class BaseDeleter(BaseAsyncMutator):
@@ -1300,7 +1213,7 @@ class ZonalDeleter(BaseDeleter):
   @staticmethod
   def Args(parser, resource=None, command=None):
     BaseDeleter.AddArgs(parser, resource, command)
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser, resource_type='resources', operation_type='delete')
 
   @property
@@ -1321,7 +1234,7 @@ class RegionalDeleter(BaseDeleter):
   @staticmethod
   def Args(parser, resource=None, command=None):
     BaseDeleter.AddArgs(parser, resource, command)
-    utils.AddRegionFlag(
+    flags.AddRegionFlag(
         parser, resource_type='resources', operation_type='delete')
 
   @property
@@ -1356,8 +1269,6 @@ class GlobalDeleter(BaseDeleter):
 class ReadWriteCommand(BaseCommand):
   """Base class for read->update->write subcommands."""
 
-  __metaclass__ = abc.ABCMeta
-
   @abc.abstractproperty
   def service(self):
     pass
@@ -1409,19 +1320,19 @@ class ReadWriteCommand(BaseCommand):
         yield resource
       return
 
-    resources = request_helper.MakeRequests(
+    resource_list = request_helper.MakeRequests(
         requests=[self.GetSetRequest(args, new_object, objects[0])],
         http=self.http,
         batch_url=self.batch_url,
         errors=errors,
         custom_get_requests=None)
 
-    resources = lister.ProcessResults(
-        resources=resources,
+    resource_list = lister.ProcessResults(
+        resources=resource_list,
         field_selector=property_selector.PropertySelector(
             properties=None,
             transformations=self.transformations))
-    for resource in resources:
+    for resource in resource_list:
       yield resource
 
     if errors:
@@ -1518,7 +1429,7 @@ class InstanceMetadataMutatorMixin(ReadWriteCommand):
 
   @staticmethod
   def Args(parser):
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser,
         resource_type='instance',
         operation_type='set metadata on')
@@ -1566,7 +1477,7 @@ class InstanceTagsMutatorMixin(ReadWriteCommand):
 
   @staticmethod
   def Args(parser):
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser,
         resource_type='instance',
         operation_type='set tags on')
@@ -1692,8 +1603,6 @@ def _WriteResourceInCommentBlock(serialized_resource, title, buf):
 class BaseEdit(BaseCommand):
   """Base class for modifying resources using $EDITOR."""
 
-  __metaclass__ = abc.ABCMeta
-
   DEFAULT_FORMAT = 'yaml'
 
   @abc.abstractmethod
@@ -1763,7 +1672,7 @@ class BaseEdit(BaseCommand):
       return [self.original_object]
 
     errors = []
-    resources = list(request_helper.MakeRequests(
+    resource_list = list(request_helper.MakeRequests(
         requests=[self.GetSetRequest(args, new_object, self.original_object)],
         http=self.http,
         batch_url=self.batch_url,
@@ -1774,7 +1683,7 @@ class BaseEdit(BaseCommand):
           errors,
           error_message='Could not update resource:')
 
-    return resources
+    return resource_list
 
   def Run(self, args):
     self.ref = self.CreateReference(args)
@@ -1831,7 +1740,7 @@ class BaseEdit(BaseCommand):
       except edit.NoSaveException:
         raise calliope_exceptions.ToolException('Edit aborted by user.')
       try:
-        resources = self.ProcessEditedResource(file_contents, args)
+        resource_list = self.ProcessEditedResource(file_contents, args)
         break
       except (ValueError, yaml.error.YAMLError,
               messages.ValidationError,
@@ -1853,13 +1762,28 @@ class BaseEdit(BaseCommand):
             prompt_string='Would you like to edit the resource again?'):
           raise calliope_exceptions.ToolException('Edit aborted by user.')
 
-    resources = lister.ProcessResults(
-        resources=resources,
+    resource_list = lister.ProcessResults(
+        resources=resource_list,
         field_selector=property_selector.PropertySelector(
             properties=None,
             transformations=self.transformations))
-    for resource in resources:
+    for resource in resource_list:
       yield resource
 
-  def Format(self, args):
-    return 'default'
+
+def _SetResourceParamDefaults():
+  """Sets resource parsing default parameters to point to properties."""
+  core_values = properties.VALUES.core
+  compute_values = properties.VALUES.compute
+  for api, param, prop in (
+      ('compute', 'project', core_values.project),
+      ('resourceviews', 'projectName', core_values.project),
+      ('compute', 'zone', compute_values.zone),
+      ('resourceviews', 'zone', compute_values.zone),
+      ('compute', 'region', compute_values.region),
+      ('resourceviews', 'region', compute_values.region)):
+    resources.SetParamDefault(
+        api=api,
+        collection=None,
+        param=param,
+        resolver=resolvers.FromProperty(prop))
