@@ -36,45 +36,81 @@ DEFAULT_NETWORK = 'default'
 DEFAULT_IMAGE = 'debian-8'
 
 ImageAlias = collections.namedtuple(
-    'ImageAlias', ['project', 'name_prefix'])
+    'ImageAlias', ['project', 'name_prefix', 'family'])
 
 IMAGE_ALIASES = {
-    'centos-6': ImageAlias(project='centos-cloud', name_prefix='centos-6'),
-    'centos-7': ImageAlias(project='centos-cloud', name_prefix='centos-7'),
+    'centos-6': ImageAlias(
+        project='centos-cloud',
+        name_prefix='centos-6',
+        family='centos-6'),
+    'centos-7': ImageAlias(
+        project='centos-cloud',
+        name_prefix='centos-7',
+        family='centos-7'),
     'container-vm': ImageAlias(
-        project='google-containers', name_prefix='container-vm'),
-    'coreos': ImageAlias(project='coreos-cloud', name_prefix='coreos-stable'),
-    'debian-7':
-        ImageAlias(project='debian-cloud', name_prefix='debian-7-wheezy'),
-    'debian-7-backports': ImageAlias(
-        project='debian-cloud', name_prefix='backports-debian-7-wheezy'),
-    'debian-8':
-        ImageAlias(project='debian-cloud', name_prefix='debian-8-jessie'),
+        project='google-containers',
+        name_prefix='container-vm',
+        family='container-vm'),
+    'coreos': ImageAlias(
+        project='coreos-cloud',
+        name_prefix='coreos-stable',
+        family='coreos-stable'),
+    'debian-8': ImageAlias(
+        project='debian-cloud',
+        name_prefix='debian-8-jessie',
+        family='debian-8'),
     'opensuse-13': ImageAlias(
-        project='opensuse-cloud', name_prefix='opensuse-13'),
-    'rhel-6': ImageAlias(project='rhel-cloud', name_prefix='rhel-6'),
-    'rhel-7': ImageAlias(project='rhel-cloud', name_prefix='rhel-7'),
-    'sles-11': ImageAlias(project='suse-cloud', name_prefix='sles-11'),
-    'sles-12': ImageAlias(project='suse-cloud', name_prefix='sles-12'),
+        project='opensuse-cloud',
+        name_prefix='opensuse-13',
+        family=None),
+    'rhel-6': ImageAlias(
+        project='rhel-cloud',
+        name_prefix='rhel-6',
+        family='rhel-6'),
+    'rhel-7': ImageAlias(
+        project='rhel-cloud',
+        name_prefix='rhel-7',
+        family='rhel-7'),
+    'sles-11': ImageAlias(
+        project='suse-cloud',
+        name_prefix='sles-11',
+        family=None),
+    'sles-12': ImageAlias(
+        project='suse-cloud',
+        name_prefix='sles-12',
+        family=None),
     'ubuntu-12-04': ImageAlias(
-        project='ubuntu-os-cloud', name_prefix='ubuntu-1204-precise'),
+        project='ubuntu-os-cloud',
+        name_prefix='ubuntu-1204-precise',
+        family='ubuntu-1204-lts'),
     'ubuntu-14-04': ImageAlias(
-        project='ubuntu-os-cloud', name_prefix='ubuntu-1404-trusty'),
+        project='ubuntu-os-cloud',
+        name_prefix='ubuntu-1404-trusty',
+        family='ubuntu-1404-lts'),
     'ubuntu-15-04': ImageAlias(
-        project='ubuntu-os-cloud', name_prefix='ubuntu-1504-vivid'),
+        project='ubuntu-os-cloud',
+        name_prefix='ubuntu-1504-vivid',
+        family=None),
     'ubuntu-15-10': ImageAlias(
-        project='ubuntu-os-cloud', name_prefix='ubuntu-1510-wily'),
+        project='ubuntu-os-cloud',
+        name_prefix='ubuntu-1510-wily',
+        family='ubuntu-1510'),
     'windows-2008-r2': ImageAlias(
-        project='windows-cloud', name_prefix='windows-server-2008-r2'),
+        project='windows-cloud',
+        name_prefix='windows-server-2008-r2',
+        family='windows-2008-r2'),
     'windows-2012-r2': ImageAlias(
-        project='windows-cloud', name_prefix='windows-server-2012-r2'),
+        project='windows-cloud',
+        name_prefix='windows-server-2012-r2',
+        family='windows-2012-r2'),
 }
 
 # These are like IMAGE_ALIASES, but don't show up in the alias list.
 HIDDEN_IMAGE_ALIASES = {
-    'gae-builder-vm':
-        ImageAlias(project='goog-vmruntime-images',
-                   name_prefix='gae-builder-vm'),
+    'gae-builder-vm': ImageAlias(
+        project='goog-vmruntime-images',
+        name_prefix='gae-builder-vm',
+        family=None),
 }
 
 WINDOWS_IMAGE_PROJECT = 'windows-cloud'
@@ -82,7 +118,6 @@ PUBLIC_IMAGE_PROJECTS = [
     'centos-cloud',
     'coreos-cloud',
     'debian-cloud',
-    'google-containers',
     'opensuse-cloud',
     'rhel-cloud',
     'suse-cloud',
@@ -94,18 +129,24 @@ PREVIEW_IMAGE_PROJECTS = []
 # SSH-related constants.
 DEFAULT_SSH_KEY_FILE = os.path.join('~', '.ssh', 'google_compute_engine')
 SSH_KEYS_METADATA_KEY = 'sshKeys'
-SSH_KEYS_INSTANCE_RESTRICTED_METADATA_KEY = 'additional-ssh-keys'
+SSH_KEYS_INSTANCE_RESTRICTED_METADATA_KEY = 'ssh-keys'
+SSH_KEYS_BLOCK_METADATA_KEY = 'block-project-ssh-keys'
 MAX_METADATA_VALUE_SIZE_IN_BYTES = 32768
 PER_USER_SSH_CONFIG_FILE = os.path.join('~', '.ssh', 'config')
+GOOGLE_SSH_KNOWN_HOSTS_FILE = os.path.join('~', '.ssh',
+                                           'google_compute_known_hosts')
 
 _STORAGE_RO = 'https://www.googleapis.com/auth/devstorage.read_only'
 _USERACCOUNTS_RO = 'https://www.googleapis.com/auth/cloud.useraccounts.readonly'
 _LOGGING_WRITE = 'https://www.googleapis.com/auth/logging.write'
 _MONITORING_WRITE = 'https://www.googleapis.com/auth/monitoring.write'
+_SERVICE_CONTROL_SCOPE = 'https://www.googleapis.com/auth/servicecontrol'
+_SERVICE_MANAGEMENT_SCOPE = 'https://www.googleapis.com/auth/service.management'
 
-DEFAULT_SCOPES = [
-    _STORAGE_RO, _USERACCOUNTS_RO, _LOGGING_WRITE, _MONITORING_WRITE
-]
+DEFAULT_SCOPES = sorted([
+    _STORAGE_RO, _USERACCOUNTS_RO, _LOGGING_WRITE, _MONITORING_WRITE,
+    _SERVICE_CONTROL_SCOPE, _SERVICE_MANAGEMENT_SCOPE,
+])
 
 SCOPES = {
     'bigquery': 'https://www.googleapis.com/auth/bigquery',
@@ -118,6 +159,8 @@ SCOPES = {
     'logging-write': _LOGGING_WRITE,
     'monitoring': 'https://www.googleapis.com/auth/monitoring',
     'monitoring-write': _MONITORING_WRITE,
+    'service-control': _SERVICE_CONTROL_SCOPE,
+    'service-management': _SERVICE_MANAGEMENT_SCOPE,
     'sql': 'https://www.googleapis.com/auth/sqlservice',
     'sql-admin': 'https://www.googleapis.com/auth/sqlservice.admin',
     'storage-full': 'https://www.googleapis.com/auth/devstorage.full_control',
@@ -127,42 +170,3 @@ SCOPES = {
     'userinfo-email': 'https://www.googleapis.com/auth/userinfo.email',
 }
 
-ZONE_PROPERTY_EXPLANATION = """\
-If not specified, you will be prompted to select a zone.
-
-To avoid prompting when this flag is omitted, you can set the
-``compute/zone'' property:
-
-  $ gcloud config set compute/zone ZONE
-
-A list of zones can be fetched by running:
-
-  $ gcloud compute zones list
-
-To unset the property, run:
-
-  $ gcloud config unset compute/zone
-
-Alternatively, the zone can be stored in the environment variable
-``CLOUDSDK_COMPUTE_ZONE''.
-"""
-
-REGION_PROPERTY_EXPLANATION = """\
-If not specified, you will be prompted to select a region.
-
-To avoid prompting when this flag is omitted, you can set the
-``compute/region'' property:
-
-  $ gcloud config set compute/region REGION
-
-A list of regions can be fetched by running:
-
-  $ gcloud compute regions list
-
-To unset the property, run:
-
-  $ gcloud config unset compute/region
-
-Alternatively, the region can be stored in the environment
-variable ``CLOUDSDK_COMPUTE_REGION''.
-"""
