@@ -19,6 +19,8 @@ from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
 
+FAMILY_PREFIX = 'family/'
+
 
 class ImageResourceFetcher(object):
   """Mixin class for displaying images."""
@@ -55,12 +57,23 @@ class ImageExpander(object):
   def GetImage(self, image_ref):
     """Returns the image resource corresponding to the given reference."""
     errors = []
+    requests = []
+    name = image_ref.Name()
+    if name.startswith(FAMILY_PREFIX):
+      requests.append((self.compute.images,
+                       'GetFromFamily',
+                       self.messages.ComputeImagesGetFromFamilyRequest(
+                           family=name[len(FAMILY_PREFIX):],
+                           project=image_ref.project)))
+    else:
+      requests.append((self.compute.images,
+                       'Get',
+                       self.messages.ComputeImagesGetRequest(
+                           image=name,
+                           project=image_ref.project)))
+
     res = list(request_helper.MakeRequests(
-        requests=[(self.compute.images,
-                   'Get',
-                   self.messages.ComputeImagesGetRequest(
-                       image=image_ref.Name(),
-                       project=image_ref.project))],
+        requests=requests,
         http=self.http,
         batch_url=self.batch_url,
         errors=errors,
@@ -72,7 +85,7 @@ class ImageExpander(object):
     return res[0]
 
   def ExpandImageFlag(self, args, return_image_resource=False):
-    """Resolves the --image flag value.
+    """Resolves the --image or --image-family flag value.
 
     If the value of --image is one of the aliases defined in the
     constants module, both the user's project and the public image
@@ -81,8 +94,8 @@ class ImageExpander(object):
     provided, only the given project is queried.
 
     Args:
-      args: The command-line flags. The flags accessed are --image and
-        --image-project.
+      args: The command-line flags. The flags accessed are --image,
+        --image-family and --image-project.
       return_image_resource: If True, always makes an API call to also
         fetch the image resource.
 
@@ -91,10 +104,29 @@ class ImageExpander(object):
         return_image_resource is False, the second element is None, otherwise
         it is the image resource.
     """
-    image_ref = self.resources.Parse(
-        args.image or constants.DEFAULT_IMAGE,
-        collection='compute.images',
-        resolve=False)
+
+    image_ref = None
+    warn_alias = True
+
+    if args.image:
+      image_ref = self.resources.Parse(
+          args.image,
+          collection='compute.images',
+          resolve=False)
+    elif args.image_family is not None:
+      image_ref = self.resources.Parse(
+          args.image_family,
+          collection='compute.images',
+          resolve=False)
+      if not image_ref.image.startswith(FAMILY_PREFIX):
+        image_ref.image = FAMILY_PREFIX + image_ref.image
+    else:
+      # TODO(b/29177598): Stop using an alias for the default image
+      warn_alias = False
+      image_ref = self.resources.Parse(
+          constants.DEFAULT_IMAGE,
+          collection='compute.images',
+          resolve=False)
 
     # If an image project was specified, then assume that image refers
     # to an image in that project.
@@ -125,6 +157,9 @@ class ImageExpander(object):
     # latest one among the public image project and the user's
     # project.
 
+    if warn_alias:
+      WarnAlias(alias)
+
     errors = []
     images = self.GetMatchingImages(image_ref.Name(), alias, errors)
 
@@ -191,35 +226,19 @@ def AddImageProjectFlag(parser):
       '--image-project',
       help='The project against which all image references will be resolved.')
   image_project.detailed_help = """\
-      The project against which all image references will be
+      The project against which all image and image family references will be
       resolved. See ``--image'' for more details.
       """
 
 
-def GetImageAliasTable(indent=0):
-  """Returns help text that explains the image aliases."""
-  # Note: The leading spaces are very important in this string. The
-  # final help text is dedented, so if the leading spaces are off, the
-  # help will not be generated properly.
-  table = ('\n' + indent * ' ').join(
-      ','.join([alias, project, image_name])
-      for alias, (project, image_name) in
-      sorted(constants.IMAGE_ALIASES.iteritems()))
-  return ('\n' + indent * ' ').join([
-      'The value for this option can be the name of an image or an',
-      'alias from the table below.\n',
-      '[options="header",format="csv",grid="none",frame="none"]',
-      '|========',
-      'Alias,Project,Image Name',
-      '{0}',
-      '|========\n',
-      'When the value is an alias, this tool will query the public',
-      'image project that contains the image type to find the',
-      'latest image matching the alias. The user\'s project is also',
-      'queried for an image with the same name as the alias. If a',
-      'conflict exists, the user will be prompted to resolve the',
-      'conflict.\n',
-      'To specify an image in another project for which there is no',
-      'alias, use `--image-project`. When `--image-project` is',
-      'present, no API calls are made to resolve the image. This',
-      'property is useful for scripts.']).format(table)
+def WarnAlias(alias):
+  """WarnAlias outputs a warning telling users to not use the given alias."""
+  msg = ('Image aliases are deprecated and will be removed in a future '
+         'version. ')
+  if alias.family is not None:
+    msg += ('Please use --image-family={family} and --image-project={project} '
+            'instead.').format(family=alias.family, project=alias.project)
+  else:
+    msg += 'Please use --image-family and --image-project instead.'
+
+  log.warn(msg)
