@@ -61,17 +61,6 @@ def _FirewallTargetTagsToCell(firewall):
   return ','.join(firewall.get('targetTags', []))
 
 
-def _NextMaintenanceToCell(zone):
-  """Returns the timestamps of the next maintenance or ''."""
-  maintenance_events = zone.get('maintenanceWindows', [])
-  if maintenance_events:
-    next_event = min(maintenance_events, key=lambda x: x.get('beginTime'))
-    return '{0}--{1}'.format(next_event.get('beginTime'),
-                             next_event.get('endTime'))
-  else:
-    return ''
-
-
 def _StatusToCell(zone_or_region):
   """Returns status of a machine with deprecation information if applicable."""
   deprecated = zone_or_region.get('deprecated', '')
@@ -114,6 +103,26 @@ def _QuotaToCell(metric, is_integer=True):
   return QuotaToCell
 
 
+def _LocationName(instance_group):
+  """Returns a location name, could be region name or zone name."""
+  if 'zone' in instance_group:
+    return path_simplifier.Name(instance_group['zone'])
+  elif 'region' in instance_group:
+    return path_simplifier.Name(instance_group['region'])
+  else:
+    return None
+
+
+def _LocationScopeType(instance_group):
+  """Returns a location scope type, could be region or zone."""
+  if 'zone' in instance_group:
+    return 'zone'
+  elif 'region' in instance_group:
+    return 'region'
+  else:
+    return None
+
+
 def _MachineTypeMemoryToCell(machine_type):
   """Returns the memory of the given machine type in GB."""
   memory = machine_type.get('memoryMb')
@@ -249,14 +258,16 @@ def _RoutesNextHopToCell(route):
     return path_simplifier.ScopedSuffix(route.get('nextHopGateway'))
   elif route.get('nextHopIp'):
     return route.get('nextHopIp')
+  elif route.get('nextHopVpnTunnel'):
+    return path_simplifier.ScopedSuffix(route.get('nextHopVpnTunnel'))
   else:
     return ''
 
 
-def _TargetHttpsProxySslCertificatesToCell(target_https_proxy):
-  """Joins the names of ssl certificates of the given target https proxy."""
+def _TargetProxySslCertificatesToCell(target_proxy):
+  """Joins the names of ssl certificates of the given HTTPS or SSL proxy."""
   return ','.join(path_simplifier.Name(cert) for cert in
-                  target_https_proxy.get('sslCertificates', []))
+                  target_proxy.get('sslCertificates', []))
 
 
 def _ProtobufDefinitionToFields(message_class):
@@ -473,12 +484,11 @@ _SPECS_V1 = {
         table_cols=[
             ('NAME', 'name'),
             ('PROJECT', _ProjectToCell),
-            ('ALIAS', _AliasToCell),
+            ('FAMILY', 'family'),
             ('DEPRECATED', 'deprecated.state'),
             ('STATUS', 'status'),
         ],
-        transformations=[
-            ],
+        transformations=[],
         editables=None,
     ),
 
@@ -705,7 +715,7 @@ _SPECS_V1 = {
         message_class_name='TargetHttpsProxy',
         table_cols=[
             ('NAME', 'name'),
-            ('SSL_CERTIFICATES', _TargetHttpsProxySslCertificatesToCell),
+            ('SSL_CERTIFICATES', _TargetProxySslCertificatesToCell),
             ('URL_MAP', 'urlMap'),
         ],
         transformations=[
@@ -809,7 +819,6 @@ _SPECS_V1 = {
             ('NAME', 'name'),
             ('REGION', 'region'),
             ('STATUS', _StatusToCell),
-            ('NEXT_MAINTENANCE', _NextMaintenanceToCell),
             ('TURNDOWN_DATE', _DeprecatedDateTimeToCell),
         ],
         transformations=[
@@ -831,12 +840,41 @@ _SPECS_V1 = {
             ('targetVpnGateway', path_simplifier.Name)],
         editables=None
     ),
+
+    'routers': _InternalSpec(
+        message_class_name='Router',
+        table_cols=[
+            ('NAME', 'name'),
+            ('REGION', 'region'),
+            ('NETWORK', 'network'),
+        ],
+        transformations=[
+            ('network', path_simplifier.Name),
+            ('region', path_simplifier.Name),
+        ],
+        editables=None,
+    ),
 }
 
 
 _SPECS_BETA = _SPECS_V1.copy()
 
+
 _SPECS_ALPHA = _SPECS_BETA.copy()
+_SPECS_ALPHA['targetSslProxies'] = _InternalSpec(
+    message_class_name='TargetSslProxy',
+    table_cols=[
+        ('NAME', 'name'),
+        ('PROXY_HEADER', 'proxyHeader'),
+        ('SERVICE', 'service'),
+        ('SSL_CERTIFICATES', _TargetProxySslCertificatesToCell)
+    ],
+    transformations=[
+        ('sslCertificates[]', path_simplifier.Name),
+        ('service', path_simplifier.Name),
+    ],
+    editables=None,
+    )
 _SPECS_ALPHA['backendBuckets'] = _InternalSpec(
     message_class_name='BackendBucket',
     table_cols=[
@@ -857,19 +895,101 @@ _SPECS_ALPHA['healthChecks'] = _InternalSpec(
     transformations=[],
     editables=None,
 )
-_SPECS_ALPHA['routers'] = _InternalSpec(
-    message_class_name='Router',
+_SPECS_ALPHA['instanceGroups'] = _InternalSpec(
+    message_class_name='InstanceGroup',
     table_cols=[
         ('NAME', 'name'),
-        ('REGION', 'region'),
+        ('LOCATION', _LocationName),
+        ('SCOPE', _LocationScopeType),
         ('NETWORK', 'network'),
+        ('MANAGED', 'isManaged'),
+        ('INSTANCES', 'size'),
     ],
     transformations=[
         ('network', path_simplifier.Name),
-        ('region', path_simplifier.Name),
+        ('size', str),
     ],
     editables=None,
 )
+_SPECS_ALPHA['instanceGroupManagers'] = _InternalSpec(
+    message_class_name='InstanceGroupManager',
+    table_cols=[
+        ('NAME', 'name'),
+        ('LOCATION', _LocationName),
+        ('SCOPE', _LocationScopeType),
+        ('BASE_INSTANCE_NAME', 'baseInstanceName'),
+        ('SIZE', 'size'),
+        ('TARGET_SIZE', 'targetSize'),
+        ('INSTANCE_TEMPLATE', 'instanceTemplate'),
+        ('AUTOSCALED', 'autoscaled'),
+    ],
+    transformations=[
+        ('instanceGroup', path_simplifier.Name),
+        ('instanceTemplate', path_simplifier.Name),
+    ],
+    editables=None,
+    )
+_SPECS_ALPHA['backendServices'] = _InternalSpec(
+    message_class_name='BackendService',
+    table_cols=[
+        ('NAME', 'name'),
+        ('BACKENDS', _BackendsToCell),
+        ('PROTOCOL', 'protocol'),
+    ],
+    transformations=[
+        ('backends[].group', path_simplifier.ScopedSuffix),
+    ],
+    editables=[
+        'backends',
+        'description',
+        'enableCDN',
+        'sessionAffinity',
+        'affinityCookieTTL',
+        'healthChecks',
+        'port',
+        'portName',
+        'protocol',
+        'timeoutSec',
+    ],)
+_SPECS_ALPHA['regionBackendServices'] = _InternalSpec(
+    message_class_name='BackendService',
+    table_cols=[
+        ('NAME', 'name'),
+        ('BACKENDS', _BackendsToCell),
+        ('PROTOCOL', 'protocol'),
+    ],
+    transformations=[
+        ('backends[].group', path_simplifier.ScopedSuffix),
+    ],
+    editables=[
+        'backends',
+        'description',
+        'enableCDN',
+        'healthChecks',
+        'port',
+        'portName',
+        'protocol',
+        'timeoutSec',
+    ],)
+_SPECS_ALPHA['urlMaps'] = _InternalSpec(
+    message_class_name='UrlMap',
+    table_cols=[
+        ('NAME', 'name'),
+        ('DEFAULT_SERVICE', 'defaultService'),
+    ],
+    transformations=[
+        ('defaultService', path_simplifier.TypeSuffix),
+        ('pathMatchers[].defaultService', path_simplifier.TypeSuffix),
+        ('pathMatchers[].pathRules[].service', path_simplifier.TypeSuffix),
+        ('tests[].service', path_simplifier.TypeSuffix),
+    ],
+    editables=[
+        'defaultService',
+        'description',
+        'hostRules',
+        'pathMatchers',
+        'tests',
+    ])
 
 
 def _GetSpecsForVersion(api_version):
@@ -886,7 +1006,7 @@ def _GetSpecsForVersion(api_version):
   """
   if api_version == 'v1' or api_version == 'v2beta1':
     return _SPECS_V1
-  if api_version == 'alpha':
+  if 'alpha' in api_version:
     return _SPECS_ALPHA
   return _SPECS_BETA
 
