@@ -13,8 +13,11 @@
 # limitations under the License.
 
 """Utilities for subcommands that need to SSH into virtual machine guests."""
+import errno
+import getpass
 import logging
 import os
+import re
 
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import constants
@@ -44,6 +47,11 @@ _SSH_KEY_PROPAGATION_TIMEOUT_SEC = 60
 # successful `ssh` execution where the *command* errored).
 _SSH_ERROR_EXIT_CODE = 255
 
+# Normally, all SSH output is simply returned to the user (or sent to
+# /dev/null if user output is disabled). For testing, this value can be
+# overridden with a file path.
+SSH_OUTPUT_FILE = None
+
 
 class SetProjectMetadataError(core_exceptions.Error):
   pass
@@ -70,6 +78,153 @@ class SshLikeCmdFailed(core_exceptions.Error):
         exit_code=return_code)
 
 
+def _IsValidSshUsername(user):
+  # All characters must be ASCII, and no spaces are allowed
+  # This may grant false positives, but will prevent backwards-incompatible
+  # behavior.
+  return all(ord(c) < 128 and c != ' ' for c in user)
+
+
+def _WarnOrReadFirstKeyLine(path, kind):
+  """Returns the first line from the key file path.
+
+  A None return indicates an error and is always accompanied by a log.warn
+  message.
+
+  Args:
+    path: The path of the file to read from.
+    kind: The kind of key file, 'private' or 'public'.
+
+  Returns:
+    None (and prints a log.warn message) if the file does not exist, is not
+    readable, or is empty. Otherwise returns the first line utf8 decoded.
+  """
+  try:
+    with open(path) as f:
+      # Decode to utf8 to handle any unicode characters. Key data is base64
+      # encoded so it cannot contain any unicode. Comments may contain unicode,
+      # but they are ignored in the key file analysis here, so replacing invalid
+      # chars with ? is OK.
+      line = f.readline().strip().decode('utf8', 'replace')
+      if line:
+        return line
+      msg = 'is empty'
+  except IOError as e:
+    if e.errno == errno.ENOENT:
+      msg = 'does not exist'
+    else:
+      msg = 'is not readable'
+  log.warn('The %s SSH key file for Google Compute Engine %s.', kind, msg)
+  return None
+
+
+# TODO(user): This function can be dropped 1Q2017.
+def _IsPublicKeyCorrupt95Through97(key):
+  """Returns True if the encoded public key has the release 95.0.0 corruption.
+
+  Windows corruption checks for release 95.0.0 through 97.0.0.
+  Corrupt Windows encoded keys have these properties:
+    type:       'ssh-rsa'
+    exponent:   65537
+    length:     256
+    next byte:  bit 0x80 set
+  A valid key either has exponent != 65537 or:
+    type:       'ssh-rsa'
+    exponent:   65537
+    length:     257
+    next byte:  0
+
+  Args:
+    key: The base64 encoded public key.
+
+  Returns:
+    True if the encoded public key has the release 95.0.0 corruption.
+  """
+  # The corruption only happened on Windows.
+  if not IsRunningOnWindows():
+    return False
+
+  # All corrupt encodings have the same encoded prefix (up to the second to
+  # last byte of the modulus size).
+  prefix = 'AAAAB3NzaC1yc2EAAAADAQABAAAB'
+  if not key.startswith(prefix):
+    return False
+
+  # The next 3 base64 chars determine the next 2 encoded bytes.
+  modulus = key[len(prefix):len(prefix) + 3]
+  # The last byte of the size must be 01 and the first byte of the modulus must
+  # be 00, and that corresponds to one of two base64 encodings:
+  if modulus in ('AQC', 'AQD'):
+    return False
+
+  # Looks bad.
+  return True
+
+
+def _KeyFilesAreValid(private=None, public=None):
+  """Returns True if private and public pass minimum key file requirements.
+
+  Args:
+    private: The private key file path.
+    public: The public key file path.
+
+  Returns:
+    True if private and public meet minumum key file requirements.
+  """
+  # The private key file must be readable and non-empty.
+  if not _WarnOrReadFirstKeyLine(private, 'private'):
+    return False
+
+  # The PuTTY PPK key file must be readable and non-empty.
+  if IsRunningOnWindows() and not _WarnOrReadFirstKeyLine(private + '.ppk',
+                                                          'PuTTY PPK'):
+    return False
+
+  # The public key file must be readable and non-empty.
+  public_line = _WarnOrReadFirstKeyLine(public, 'public')
+  if not public_line:
+    return False
+
+  # The remaining checks are for the public key file.
+
+  # Must have at least 2 space separated fields.
+  fields = public_line.split(' ')
+  if len(fields) < 2 or _IsPublicKeyCorrupt95Through97(fields[1]):
+    log.warn('The public SSH key file for Google Compute Engine is corrupt.')
+    return False
+
+  # Looks OK.
+  return True
+
+
+def GetDefaultSshUsername(warn_on_account_user=False):
+  """Returns the default username for ssh.
+
+  The default username is the local username, unless that username is invalid.
+  In that case, the default username is the username portion of the current
+  account.
+
+  Emits a warning if it's not using the local account username.
+
+  Args:
+    warn_on_account_user: bool, whether to warn if using the current account
+      instead of the local username.
+
+  Returns:
+    str, the default SSH username.
+  """
+  user = getpass.getuser()
+  if not _IsValidSshUsername(user):
+    full_account = properties.VALUES.core.account.Get(required=True)
+    account_user = gaia_utils.MapGaiaEmailToDefaultAccountName(full_account)
+    if warn_on_account_user:
+      log.warn('Invalid characters in local username [{0}]. '
+               'Using username corresponding to active account: [{1}]'.format(
+                   user, account_user))
+    user = account_user
+  return user
+
+
 def UserHost(user, host):
   """Returns a string of the form user@host."""
   if user:
@@ -132,32 +287,50 @@ def _RunExecutable(cmd_args, strict_error_checking=True):
     SshLikeCmdFailed: if the command failed (based on the command exit code and
       the strict_error_checking flag)
   """
-  with open(os.devnull, 'w') as devnull:
-    if log.IsUserOutputEnabled():
+  outfile = SSH_OUTPUT_FILE or os.devnull
+  with open(outfile, 'w') as output_file:
+    if log.IsUserOutputEnabled() and not SSH_OUTPUT_FILE:
       stdout, stderr = None, None
     else:
-      stdout, stderr = devnull, devnull
+      stdout, stderr = output_file, output_file
+    if IsRunningOnWindows() and not cmd_args[0].endswith('winkeygen.exe'):
+      # TODO(user): b/25126583 will drop StrictHostKeyChecking=no and 'y'.
+      # PuTTY and friends always prompt on fingerprint mismatch. A 'y' response
+      # adds/updates the fingerprint registry entry and proceeds. The prompt
+      # will appear once for each new/changed host. Redirecting stdin is not a
+      # problem. Even interactive ssh is not a problem because a separate PuTTY
+      # term is used and it ignores the calling process stdin.
+      stdin = subprocess.PIPE
+    else:
+      stdin = None
     try:
-      subprocess.check_call(cmd_args, stdout=stdout, stderr=stderr)
+      proc = subprocess.Popen(
+          cmd_args, stdin=stdin, stdout=stdout, stderr=stderr)
+      if stdin == subprocess.PIPE:
+        # Max one prompt per host and there can't be more hosts than args.
+        proc.communicate('y\n' * len(cmd_args))
+      returncode = proc.wait()
     except OSError as e:
       raise SshLikeCmdFailed(cmd_args[0], message=e.strerror)
-    except subprocess.CalledProcessError as e:
-      if strict_error_checking or e.returncode == _SSH_ERROR_EXIT_CODE:
-        raise SshLikeCmdFailed(cmd_args[0], return_code=e.returncode)
-      return e.returncode
-    return 0
+    if ((returncode and strict_error_checking) or
+        returncode == _SSH_ERROR_EXIT_CODE):
+      raise SshLikeCmdFailed(cmd_args[0], return_code=returncode)
+    return returncode
 
 
 def _GetMetadataKey(iam_ssh_keys):
   """Get the metadata key name for the desired SSH key metadata.
 
-  There are three SSH key related metadata pairs:
+  There are four SSH key related metadata pairs:
   * Per-project 'sshKeys': this grants SSH access to VMs project-wide.
   * Per-instance 'sshKeys': this is used to grant access to an individual
     instance. For historical reasons, it acts as an override to the
     project-global value.
-  * Per-instance 'additional-ssh-keys': this also grants access to an individual
-   instance, but acts in addition to the per-project 'sshKeys'.
+  * Per-instance 'block-project-ssh-keys': this determines whether 'ssh-keys'
+    overrides or adds to the per-project 'sshKeys'
+  * Per-instance 'ssh-keys': this also grants access to an individual
+     instance, but acts in addition or as an override to the per-project
+     'sshKeys' depending on 'block-project-ssh-keys'
 
   Args:
     iam_ssh_keys: bool. If False, give the name of the original SSH metadata key
@@ -237,17 +410,81 @@ def _AddSSHKeyToMetadataMessage(message_classes, user, public_key, metadata,
         existing_metadata=metadata)
 
 
-def _IsRunningOnWindows():
+def IsRunningOnWindows():
   """Returns True if the current os is Windows."""
   current_os = platforms.OperatingSystem.Current()
   return current_os is platforms.OperatingSystem.WINDOWS
 
 
+def ReadFile(file_path):
+  """Returns the contents of the file or ''."""
+  try:
+    with open(file_path) as f:
+      return f.read()
+  except IOError as e:
+    if e.errno == errno.ENOENT:
+      return ''
+    else:
+      raise exceptions.ToolException('There was a problem reading [{0}]: {1}'
+                                     .format(file_path, e.message))
+
+
+def UpdateKnownHostsFile(known_hosts_file, hostname, host_key,
+                         overwrite_keys=False):
+  """Update the known_hosts file entry for the given hostname.
+
+  If there is no entry for the give hostname, it will be added. If there is
+  an entry already and overwrite_keys is False, nothing will be changed. If
+  there is an entry and overwrite_keys is True, the key will be updated if it
+  has changed.
+
+  Args:
+    known_hosts_file: str, The full path of the known_hosts file to update.
+    hostname: str, The hostname for the known_hosts entry.
+    host_key: str, The host key for the given hostname.
+    overwrite_keys: bool, If true, will overwrite the entry corresponding to
+      hostname with the new host_key if it already exists. If false and an
+      entry already exists for hostname, will ignore the new host_key value.
+  """
+  known_hosts_contents = ReadFile(known_hosts_file)
+  key_list = known_hosts_contents.splitlines()
+  found_key_entry = None
+  new_key_entry = '{0} {1}'.format(hostname, host_key)
+
+  for key in key_list:
+    if key.startswith(hostname):
+      found_key_entry = key
+      break
+
+  if overwrite_keys and found_key_entry:
+    if found_key_entry != new_key_entry:
+      key_list.remove(found_key_entry)
+      found_key_entry = None
+
+  if not found_key_entry:
+    key_list.append(new_key_entry)
+
+  new_contents = '\n'.join(key_list) + '\n'
+  with files.OpenForWritingPrivate(known_hosts_file) as f:
+    f.write(new_contents)
+
+
 def _SdkHelperBin():
   """Returns the SDK helper executable bin directory."""
   return os.path.join(config.Paths().sdk_root, 'bin', 'sdk')
 
 
+def _MetadataHasBlockProjectSshKeys(metadata):
+  """Return true if the metadata has 'block-project-ssh-keys' set and 'true'."""
+  if not (metadata and metadata.items):
+    return False
+  matching_values = [item.value for item in metadata.items
+                     if item.key == constants.SSH_KEYS_BLOCK_METADATA_KEY]
+  if not matching_values:
+    return False
+  return matching_values[0].lower() == 'true'
+
+
 class BaseSSHCommand(base_classes.BaseCommand,
                      user_utils.UserResourceFetcher):
   """Base class for subcommands that need to connect to instances using SSH.
@@ -419,10 +656,10 @@ class BaseSSHCommand(base_classes.BaseCommand,
     return True
 
   def GetPublicKey(self):
-    """Generates an SSH key using ssh-key (if necessary) and returns it."""
+    """Generates an SSH key using ssh-keygen (if necessary) and returns it."""
     public_ssh_key_file = self.ssh_key_file + '.pub'
-    if (not os.path.exists(self.ssh_key_file) or
-        not os.path.exists(public_ssh_key_file)):
+    if not _KeyFilesAreValid(private=self.ssh_key_file,
+                             public=public_ssh_key_file):
       log.warn('You do not have an SSH key for Google Compute Engine.')
       log.warn('[%s] will be executed to generate a key.',
                self.ssh_keygen_executable)
@@ -436,8 +673,14 @@ class BaseSSHCommand(base_classes.BaseCommand,
         else:
           raise exceptions.ToolException('SSH key generation aborted by user.')
 
+      # Remove the private key file to avoid interactive prompts.
+      try:
+        os.remove(self.ssh_key_file)
+      except OSError:
+        pass
+
       keygen_args = [self.ssh_keygen_executable]
-      if _IsRunningOnWindows():
+      if IsRunningOnWindows():
         # No passphrase in the current implementation.
         keygen_args.append(self.ssh_key_file)
       else:
@@ -463,7 +706,7 @@ class BaseSSHCommand(base_classes.BaseCommand,
 
   def Run(self, args):
     """Subclasses must call this in their Run() before continuing."""
-    if _IsRunningOnWindows():
+    if IsRunningOnWindows():
       scp_command = 'pscp'
       ssh_command = 'plink'
       ssh_keygen_command = 'winkeygen'
@@ -491,6 +734,8 @@ class BaseSSHCommand(base_classes.BaseCommand,
 
     self.ssh_key_file = os.path.realpath(os.path.expanduser(
         args.ssh_key_file or constants.DEFAULT_SSH_KEY_FILE))
+    self.known_hosts_file = os.path.realpath(os.path.expanduser(
+        constants.GOOGLE_SSH_KNOWN_HOSTS_FILE))
 
 
 class BaseSSHCLICommand(BaseSSHCommand):
@@ -513,17 +758,27 @@ class BaseSSHCLICommand(BaseSSHCommand):
     plain.detailed_help = """\
         Suppresses the automatic addition of *ssh(1)*/*scp(1)* flags. This flag
         is useful if you want to take care of authentication yourself or
-        re-enable strict host checking.
+        use specific ssh/scp features.
+        """
+
+    strict_host_key = parser.add_argument(
+        '--strict-host-key-checking',
+        choices=['yes', 'no', 'ask'],
+        help='Override the default behavior for ssh/scp StrictHostKeyChecking')
+    strict_host_key.detailed_help = """\
+        Override the default behavior of StrictHostKeyChecking. By default,
+        StrictHostKeyChecking is set to 'no' the first time you connect to an
+        instance and will be set to 'yes' for all subsequent connections. Use
+        this flag to specify a value for the connection.
         """
 
   def GetDefaultFlags(self):
     """Returns a list of default commandline flags."""
     return [
         '-i', self.ssh_key_file,
-        '-o', 'UserKnownHostsFile=/dev/null',
+        '-o', 'UserKnownHostsFile={0}'.format(self.known_hosts_file),
         '-o', 'IdentitiesOnly=yes',  # ensure our SSH key trumps any ssh_agent
-        '-o', 'CheckHostIP=no',
-        '-o', 'StrictHostKeyChecking=no',
+        '-o', 'CheckHostIP=no'
     ]
 
   def GetInstance(self, instance_ref):
@@ -548,10 +803,13 @@ class BaseSSHCLICommand(BaseSSHCommand):
           error_message='Could not fetch instance:')
     return objects[0]
 
-  def WaitUntilSSHable(self, user, external_ip_address):
+  def WaitUntilSSHable(self, user, args, instance):
     """Blocks until SSHing to the given host succeeds."""
+    external_ip_address = GetExternalIPAddress(instance)
     ssh_args_for_polling = [self.ssh_executable]
     ssh_args_for_polling.extend(self.GetDefaultFlags())
+    ssh_args_for_polling.extend(self.GetHostKeyArgs(args, instance))
+
     ssh_args_for_polling.append(UserHost(user, external_ip_address))
     ssh_args_for_polling.append('true')
     ssh_args_for_polling = self.LocalizeCommand(ssh_args_for_polling)
@@ -581,7 +839,7 @@ class BaseSSHCLICommand(BaseSSHCommand):
     Returns:
       Returns new_cmd_args, the localized command line.
     """
-    if not _IsRunningOnWindows():
+    if not IsRunningOnWindows():
       return cmd_args
     args = [cmd_args[0]]
     i = 1
@@ -591,13 +849,13 @@ class BaseSSHCLICommand(BaseSSHCommand):
       arg = cmd_args[i]
       i += 1
       if arg == '-i' and i < n:
-        # -i private_key_file -- use private_key_file.ppk if it exists.
+        # -i private_key_file -- use private_key_file.ppk -- if it doesn't exist
+        # then winkeygen will be called to generate it before attempting to
+        # connect.
         args.append(arg)
         arg = cmd_args[i]
         i += 1
-        ppk_file = arg + '.ppk'
-        if os.path.exists(ppk_file):
-          arg = ppk_file
+        arg += '.ppk'
       elif arg == '-o' and i < n:
         arg = cmd_args[i]
         i += 1
@@ -624,8 +882,41 @@ class BaseSSHCLICommand(BaseSSHCommand):
       args[0] = self.ssh_term_executable
     return args
 
+  def IsHostKeyAliasInKnownHosts(self, host_key_alias):
+    known_hosts = ReadFile(self.known_hosts_file)
+    if known_hosts:
+      return host_key_alias in known_hosts
+    else:
+      return False
+
+  def GetHostKeyArgs(self, args, instance):
+    """Returns default values for HostKeyAlias and StrictHostKeyChecking.
+
+    Args:
+      args: argparse.Namespace, The calling command invocation args.
+      instance: Instance resource that ssh/scp is connecting to.
+
+    Returns:
+      list, list of arguments to add to the ssh command line.
+    """
+    if args.plain or IsRunningOnWindows():
+      return []
+    host_key_alias = 'compute.{0}'.format(instance.id)
+
+    if args.strict_host_key_checking:
+      strict_host_key_value = args.strict_host_key_checking
+    elif self.IsHostKeyAliasInKnownHosts(host_key_alias):
+      strict_host_key_value = 'yes'
+    else:
+      strict_host_key_value = 'no'
+
+    cmd_args = ['-o', 'HostKeyAlias={0}'.format(host_key_alias), '-o',
+                'StrictHostKeyChecking={0}'.format(strict_host_key_value)]
+    return cmd_args
+
   def ActuallyRun(self, args, cmd_args, user, instance,
-                  strict_error_checking=True, use_account_service=False):
+                  strict_error_checking=True, use_account_service=False,
+                  wait_for_sshable=True):
     """Runs the scp/ssh command specified in cmd_args.
 
     If the scp/ssh command exits non-zero, this command will exit with the same
@@ -639,6 +930,7 @@ class BaseSSHCLICommand(BaseSSHCommand):
       strict_error_checking: bool, whether to fail on a non-zero, non-255 exit
         code (alternative behavior is to return the exit code
       use_account_service: bool, when false upload ssh keys to project metadata.
+      wait_for_sshable: bool, when false skip the sshability check.
 
     Returns:
       int, the exit code of the command that was run
@@ -648,22 +940,51 @@ class BaseSSHCLICommand(BaseSSHCommand):
       log.out.Print(' '.join(cmd_args))
       return
 
-    if use_account_service:
+    if args.plain:
+      keys_newly_added = []
+    elif use_account_service:
       keys_newly_added = self.EnsureSSHKeyExistsForUser(user)
     else:
       # There are two kinds of metadata: project-wide metadata and per-instance
-      # metadata. SSH-like commands work by copying a relevant SSH key to
-      # the 'sshKeys' metadata value. A VM first checks the per-instance
-      # metadata, then the project-wide metadata (if the former exists, it
-      # doesn't check the latter), so the per-instance metadata functions as an
-      # "override" for the project-wide metadata.
-      # If we add a key to project-wide metadata but the per-instance metadata
-      # exists, we won't be able to ssh in because the VM won't check the
-      # project-wide metadata. To avoid this, if the instance has per-instance
-      # SSH key metadata, we add the key there instead.
+      # metadata. There are four SSH-key related metadata keys:
+      #
+      # * project['sshKeys']: shared project-wide
+      # * instance['sshKeys']: legacy. Acts as an override to project['sshKeys']
+      # * instance['block-project-ssh-keys']: If true, instance['ssh-keys']
+      #     overrides project['sshKeys']. Otherwise, keys from both metadata
+      #     pairs are valid.
+      # * instance['ssh-keys']: Acts either in conjunction with or as an
+      #     override to project['sshKeys'], depending on
+      #     instance['block-project-ssh-keys']
+      #
+      # SSH-like commands work by copying a relevant SSH key to
+      # the appropriate metadata value. The VM grabs keys from the metadata as
+      # follows (pseudo-Python):
+      #
+      #   def GetAllSshKeys(project, instance):
+      #       if 'sshKeys' in instance.metadata:
+      #           return (instance.metadata['sshKeys'] +
+      #                   instance.metadata['ssh-keys'])
+      #       elif instance.metadata['block-project-ssh-keys'] == 'true':
+      #           return instance.metadata['ssh-keys']
+      #       else:
+      #           return (instance.metadata['ssh-keys'] +
+      #                   project.metadata['sshKeys'])
+      #
       if _GetSSHKeysFromMetadata(instance.metadata):
+        # If we add a key to project-wide metadata but the per-instance
+        # 'sshKeys' metadata exists, we won't be able to ssh in because the VM
+        # won't check the project-wide metadata. To avoid this, if the instance
+        # has per-instance SSH key metadata, we add the key there instead.
         keys_newly_added = self.EnsureSSHKeyIsInInstance(user, instance)
+      elif _MetadataHasBlockProjectSshKeys(instance.metadata):
+        # If the instance 'ssh-keys' metadata overrides the project-wide
+        # 'sshKeys' metadata, we should put our key there.
+        keys_newly_added = self.EnsureSSHKeyIsInInstance(user, instance,
+                                                         iam_keys=True)
       else:
+        # Otherwise, try to add to the project-wide metadata. If we don't have
+        # permissions to do that, add to the instance 'ssh-keys' metadata.
         try:
           keys_newly_added = self.EnsureSSHKeyIsInProject(user)
         except SetProjectMetadataError:
@@ -679,9 +1000,43 @@ class BaseSSHCLICommand(BaseSSHCommand):
           keys_newly_added = self.EnsureSSHKeyIsInInstance(user, instance,
                                                            iam_keys=True)
 
-    if keys_newly_added:
-      self.WaitUntilSSHable(user, GetExternalIPAddress(instance))
+    if keys_newly_added and wait_for_sshable:
+      self.WaitUntilSSHable(user, args, instance)
 
     logging.debug('%s command: %s', cmd_args[0], ' '.join(cmd_args))
 
     return _RunExecutable(cmd_args, strict_error_checking=strict_error_checking)
+
+
+# A remote path has three parts host[@user]:[path], where @user and path are
+# optional.
+#   A host:
+#   - cannot start with '.'
+#   - cannot contain ':', '/', '\\', '@'
+#   A user:
+#   - cannot contain ':'.
+#   A path:
+#   - can be anything
+
+_SSH_REMOTE_PATH_REGEX = r'[^.:/\\@][^:/\\@]*(@[^:]*)?:'
+
+
+def IsScpLocalPath(path):
+  """Checks if path is an scp local file path.
+
+  Args:
+    path: The path name to check.
+
+  Returns:
+    True if path is an scp local path, false if it is a remote path.
+  """
+  # Paths that start with a drive are local. _SSH_REMOTE_PATH_REGEX could match
+  # path for some os implementations, so the drive test must be done before the
+  # pattern match.
+  if os.path.splitdrive(path)[0]:
+    return True
+  # Paths that match _SSH_REMOTE_PATH_REGEX are not local.
+  if re.match(_SSH_REMOTE_PATH_REGEX, path):
+    return False
+  # Otherwise the path is local.
+  return True
