@@ -19,9 +19,11 @@ import re
 from dns import rdatatype
 from dns import zone
 from googlecloudsdk.calliope import exceptions
-from googlecloudsdk.third_party.apis.dns.v1 import dns_v1_messages as messages
+from googlecloudsdk.core import apis as core_apis
 import yaml
 
+messages = core_apis.GetMessagesModule('dns', 'v1')
+
 
 def _AddressTranslation(rdata, unused_origin):
   """Returns the address of the given rdata.
