@@ -13,6 +13,4 @@
 # limitations under the License.
 """A helper library for this command group."""
 
-GENOMICS_MESSAGES_MODULE_KEY = 'genomics_messages'
-GENOMICS_APITOOLS_CLIENT_KEY = 'genomics_client'
-GENOMICS_RESOURCES_KEY = 'genomics_resources'
+STORAGE_V1_CLIENT_KEY = 'storage_v1_client'
