@@ -38,9 +38,8 @@ Example usage:
       default='10GB',
       type=arg_parsers.BinarySize(lower_bound='1GB', upper_bound='10TB')
 
-  # will emit a warning about space-separated metadata
   res = parser.parse_args(
-      '--names --metadata x=y,a=b c=d --delay 1s --disk-size 10gb'.split())
+      '--names --metadata x=y,a=b,c=d --delay 1s --disk-size 10gb'.split())
 
   assert res.metadata == {'a': 'b', 'c': 'd', 'x': 'y'}
   assert res.delay == 1
@@ -49,11 +48,14 @@ Example usage:
 """
 
 import argparse
+import copy
 import datetime
 import re
+import sys
 
 from googlecloudsdk.core import log
 
+
 __all__ = ['Duration', 'BinarySize']
 
 
@@ -105,6 +107,8 @@ _VALUE_PATTERN = r"""
     $                       # End of input marker.
 """
 
+_RANGE_PATTERN = r'^(?P<start>[0-9]+)(-(?P<end>[0-9]+))?$'
+
 _SECOND = 1
 _MINUTE = 60 * _SECOND
 _HOUR = 60 * _MINUTE
@@ -154,7 +158,8 @@ def GetMultiCompleter(individual_completer):
   return MultiCompleter
 
 
-def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None):
+def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None,
+                 strict_case=True, suggested_binary_size_scales=None):
   """A helper that returns a function that can parse values with units.
 
   Casing for all units matters.
@@ -166,15 +171,21 @@ def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None):
       missing unit.
     lower_bound: str, An inclusive lower bound.
     upper_bound: str, An inclusive upper bound.
+    strict_case: bool, whether to be strict on case-checking
+    suggested_binary_size_scales: list, A list of strings with units that will
+                                    be recommended to user.
 
   Returns:
     A function that can parse values.
   """
 
-  def UnitsByMagnitude():
+  def UnitsByMagnitude(suggested_binary_size_scales=None):
     """Returns a list of the units in scales sorted by magnitude."""
-    return [key for key, _
-            in sorted(scales.iteritems(), key=lambda value: value[1])]
+    scale_items = sorted(scales.iteritems(), key=lambda value: value[1])
+    if suggested_binary_size_scales is None:
+      return [key for key, _ in scale_items]
+    return [key for key, _ in scale_items
+            if key in suggested_binary_size_scales]
 
   def Parse(value):
     """Parses value that can contain a unit."""
@@ -183,15 +194,24 @@ def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None):
       raise ArgumentTypeError(_GenerateErrorMessage(
           'given value must be of the form INTEGER[UNIT] where units '
           'can be one of {0}'
-          .format(', '.join(UnitsByMagnitude())),
+          .format(', '.join(UnitsByMagnitude(suggested_binary_size_scales))),
           user_input=value))
 
     amount = int(match.group('amount'))
     unit = match.group('unit')
-    if unit is None:
-      return amount * scales[default_unit]
-    elif unit in scales:
-      return amount * scales[unit]
+    if strict_case:
+      unit_case = unit
+      default_unit_case = default_unit
+      scales_case = scales
+    else:
+      unit_case = unit and unit.upper()
+      default_unit_case = default_unit.upper()
+      scales_case = dict([(k.upper(), v) for k, v in scales.items()])
+
+    if unit_case is None:
+      return amount * scales_case[default_unit_case]
+    elif unit_case in scales_case:
+      return amount * scales_case[unit_case]
     else:
       raise ArgumentTypeError(_GenerateErrorMessage(
           'unit must be one of {0}'.format(', '.join(UnitsByMagnitude())),
@@ -264,7 +284,8 @@ def Duration(lower_bound=None, upper_bound=None):
                       lower_bound=lower_bound, upper_bound=upper_bound)
 
 
-def BinarySize(lower_bound=None, upper_bound=None):
+def BinarySize(lower_bound=None, upper_bound=None,
+               suggested_binary_size_scales=None):
   """Returns a function that can parse binary sizes.
 
   Binary sizes are defined as base-2 values representing number of
@@ -286,6 +307,8 @@ def BinarySize(lower_bound=None, upper_bound=None):
   Args:
     lower_bound: str, An inclusive lower bound for values.
     upper_bound: str, An inclusive upper bound for values.
+    suggested_binary_size_scales: list, A list of strings with units that will
+                                    be recommended to user.
 
   Raises:
     ArgumentTypeError: If either the lower_bound or upper_bound
@@ -298,28 +321,90 @@ def BinarySize(lower_bound=None, upper_bound=None):
     A function that accepts a single binary size as input to be
       parsed.
   """
-  return _ValueParser(_BINARY_SIZE_SCALES, default_unit='GB',
-                      lower_bound=lower_bound, upper_bound=upper_bound)
+  return _ValueParser(
+      _BINARY_SIZE_SCALES, default_unit='GB',
+      lower_bound=lower_bound, upper_bound=upper_bound,
+      strict_case=False,
+      suggested_binary_size_scales=suggested_binary_size_scales)
 
 
 _KV_PAIR_DELIMITER = '='
 
 
+class Range(object):
+  """Range of integer values."""
+
+  def __init__(self, start, end):
+    self.start = start
+    self.end = end
+
+  @staticmethod
+  def Parse(string_value):
+    """Creates Range object out of given string value."""
+    match = re.match(_RANGE_PATTERN, string_value)
+    if not match:
+      raise ArgumentTypeError('Expected a non-negative integer value or a '
+                              'range of such values instead of "{0}"'
+                              .format(string_value))
+    start = int(match.group('start'))
+    end = match.group('end')
+    if end is None:
+      end = start
+    else:
+      end = int(end)
+    if end < start:
+      raise ArgumentTypeError('Expected range start {0} smaller or equal to '
+                              'range end {1} in "{2}"'.format(
+                                  start, end, string_value))
+    return Range(start, end)
+
+  def Combine(self, other):
+    """Combines two overlapping or adjacent ranges, raises otherwise."""
+    if self.end + 1 < other.start or self.start > other.end + 1:
+      raise Error('Cannot combine non-overlapping or non-adjacent ranges '
+                  '{0} and {1}'.format(self, other))
+    return Range(min(self.start, other.start), max(self.end, other.end))
+
+  def __eq__(self, other):
+    if isinstance(other, Range):
+      return self.start == other.start and self.end == other.end
+    return False
+
+  def __lt__(self, other):
+    if self.start == other.start:
+      return self.end < other.end
+    return self.start < other.start
+
+  def __str__(self):
+    if self.start == self.end:
+      return str(self.start)
+    return '{0}-{1}'.format(self.start, self.end)
+
+
 class HostPort(object):
   """A class for holding host and port information."""
 
+  IPV4_OR_HOST_PATTERN = r'^(?P<address>[\w\d\.-]+)?(:|:(?P<port>[\d]+))?$'
+  # includes hostnames
+  IPV6_PATTERN = r'^(\[(?P<address>[\w\d:]+)\])(:|:(?P<port>[\d]+))?$'
+
   def __init__(self, host, port):
     self.host = host
     self.port = port
 
   @staticmethod
-  def Parse(s):
+  def Parse(s, ipv6_enabled=False):
     """Parse the given string into a HostPort object.
 
     This can be used as an argparse type.
 
     Args:
-      s: str, The string to parse.
+      s: str, The string to parse. If ipv6_enabled and host is an IPv6 address,
+      it should be placed in square brackets: e.g.
+        [2001:db8:0:0:0:ff00:42:8329]
+        or
+        [2001:db8:0:0:0:ff00:42:8329]:8080
+      ipv6_enabled: boolean, If True then accept IPv6 addresses.
 
     Raises:
       ArgumentTypeError: If the string is not valid.
@@ -329,13 +414,25 @@ class HostPort(object):
     """
     if not s:
       return HostPort(None, None)
-    if ':' not in s:
-      return HostPort(s, None)
-    parts = s.split(':')
-    if len(parts) > 2:
-      raise ArgumentTypeError(
-          _GenerateErrorMessage('Failed to parse host and port', user_input=s))
-    return HostPort(parts[0] or None, parts[1] or None)
+
+    match = re.match(HostPort.IPV4_OR_HOST_PATTERN, s, re.UNICODE)
+    if ipv6_enabled and not match:
+      match = re.match(HostPort.IPV6_PATTERN, s, re.UNICODE)
+      if not match:
+        raise ArgumentTypeError(_GenerateErrorMessage(
+            'Failed to parse host and port. Expected format \n\n'
+            '  IPv4_ADDRESS_OR_HOSTNAME:PORT\n\n'
+            'or\n\n'
+            '  [IPv6_ADDRESS]:PORT\n\n'
+            '(where :PORT is optional).',
+            user_input=s))
+    elif not match:
+      raise ArgumentTypeError(_GenerateErrorMessage(
+          'Failed to parse host and port. Expected format \n\n'
+          '  IPv4_ADDRESS_OR_HOSTNAME:PORT\n\n'
+          '(where :PORT is optional).',
+          user_input=s))
+    return HostPort(match.group('address'), match.group('port'))
 
 
 class Day(object):
@@ -376,28 +473,46 @@ class Datetime(object):
             user_input=s))
 
 
-def BoundedInt(lower_bound=None, upper_bound=None, unlimited=False):
-  """Returns a function that can parse integers within some bound.
+def _BoundedType(type_builder, type_description,
+                 lower_bound=None, upper_bound=None, unlimited=False):
+  """Returns a function that can parse given type within some bound.
 
   Args:
-    lower_bound: int, The value must be >= lower_bound.
-    upper_bound: int, The value must be <= upper_bound.
+    type_builder: A callable for building the requested type from the value
+        string.
+    type_description: str, Description of the requested type (for verbose
+        messages).
+    lower_bound: of type compatible with type_builder,
+        The value must be >= lower_bound.
+    upper_bound: of type compatible with type_builder,
+        The value must be <= upper_bound.
     unlimited: bool, If True then a value of 'unlimited' means no limit.
 
   Returns:
-    A function that can parse integers within some bound.
+    A function that can parse given type within some bound.
   """
 
-  def _Parse(value):
-    """Parses value as an int, raising ArgumentTypeError if out of bounds."""
+  def Parse(value):
+    """Parses value as a type constructed by type_builder.
+
+    Args:
+      value: str, Value to be converted to the requested type.
+
+    Raises:
+      ArgumentTypeError: If the provided value is out of bounds or unparsable.
+
+    Returns:
+      Value converted to the requested type.
+    """
     if unlimited and value == 'unlimited':
       return None
 
     try:
-      v = int(value)
+      v = type_builder(value)
     except ValueError:
       raise ArgumentTypeError(
-          _GenerateErrorMessage('Value must be an integer', user_input=value))
+          _GenerateErrorMessage('Value must be {0}'.format(type_description),
+                                user_input=value))
 
     if lower_bound is not None and v < lower_bound:
       raise ArgumentTypeError(
@@ -413,7 +528,15 @@ def BoundedInt(lower_bound=None, upper_bound=None, unlimited=False):
 
     return v
 
-  return _Parse
+  return Parse
+
+
+def BoundedInt(*args, **kwargs):
+  return _BoundedType(int, 'an integer', *args, **kwargs)
+
+
+def BoundedFloat(*args, **kwargs):
+  return _BoundedType(float, 'a floating point number', *args, **kwargs)
 
 
 def _TokenizeQuotedList(arg_value, delim=','):
@@ -598,8 +721,8 @@ def FloatingListValuesCatcher(
   class FloatingListValuesCatcherAction(action):
     """This is to assist with refactoring argument lists.
 
-    Provides a error for users who type (or have a script) that specifies a list
-    with the elements in different arguments. eg.
+    Provides an error for users who type (or have a script) that specifies a
+    list with the elements in different arguments. eg.
      $ gcloud sql instances create foo --authorized-networks x y
      usage: gcloud sql instances create  INSTANCE [optional flags]
      ERROR: (gcloud.sql.instances.create) argument --authorized-networks: lists
@@ -632,6 +755,12 @@ def FloatingListValuesCatcher(
 
     def __call__(self, parser, namespace, values, option_string=None):
       if not values and switch_value is not None:
+        msg = (
+            'We noticed that you provided no value for flag [{flag}]. This '
+            'behavior is deprecated.\nInstead, please provide an empty string '
+            'as the explicit value (try [{flag} \'\']).').format(
+                flag=option_string)
+        log.warn(msg)
         super(FloatingListValuesCatcherAction, self).__call__(
             parser, namespace, switch_value, option_string=option_string)
         return
@@ -670,18 +799,222 @@ def FloatingListValuesCatcher(
                 values=','.join(suggestions),
                 extras=', '.join(extras))
 
-        # TODO(user): stop warning when we're ready
-        warn_only = True
-
-        if not warn_only:
-          raise argparse.ArgumentError(ArgShell(option_string), msg)
-        else:
-          log.warn(msg)
-
-        super(FloatingListValuesCatcherAction, self).__call__(
-            parser, namespace, aggregate_value, option_string=option_string)
+        raise argparse.ArgumentError(ArgShell(option_string), msg)
       else:
         super(FloatingListValuesCatcherAction, self).__call__(
             parser, namespace, values[0], option_string=option_string)
 
   return FloatingListValuesCatcherAction
+
+
+class UpdateAction(argparse.Action):
+  r"""Create a single dict value from delimited or repeated flags.
+
+  This class is intended to be a more flexible version of
+  argparse._AppendAction.
+
+  For example, with the following flag definition:
+
+      parser.add_argument(
+        '--inputs',
+        type=arg_parsers.ArgDict(),
+        action=arg_parsers.FloatingListValuesCatcher(
+            arg_parsers._AppendAction),
+
+  a caller can specify on the command line flags such as:
+
+    --inputs k1=v1,k2=v2
+
+  and the result will be a list of one dict:
+
+    [{ 'k1': 'v1', 'k2': 'v2' }]
+
+  Specifying two separate command line flags such as:
+
+    --inputs k1=v1 \
+    --inputs k2=v2
+
+  will produce a list of dicts:
+
+    [{ 'k1': 'v1'}, 'k2': 'v2' }]
+
+  The UpdateAction class allows for both of the above user inputs to result
+  in the same: a single dictionary:
+
+    { 'k1': 'v1', 'k2': 'v2' }
+
+  This gives end-users a lot more flexibility in constructing their command
+  lines, especially when scripting calls.
+
+  Note that this class will raise an exception if a key value is specified
+  more than once. To allow for a key value to be specified multiple times,
+  use UpdateActionWithAppend.
+  """
+
+  def OnDuplicateKeyRaiseError(self, key, existing_value, new_value):
+    raise argparse.ArgumentError(self, _GenerateErrorMessage(
+        '"{0}" cannot be specified multiple times'
+        .format(key), user_input=', '.join([existing_value, new_value])))
+
+  def __init__(self,
+               option_strings,
+               dest,
+               nargs=None,
+               const=None,
+               default=None,
+               type=None,  # pylint:disable=redefined-builtin
+               choices=None,
+               required=False,
+               help=None,  # pylint:disable=redefined-builtin
+               metavar=None,
+               onduplicatekey_handler=OnDuplicateKeyRaiseError):
+    if nargs == 0:
+      raise ValueError('nargs for append actions must be > 0; if arg '
+                       'strings are not supplying the value to append, '
+                       'the append const action may be more appropriate')
+    if const is not None and nargs != argparse.OPTIONAL:
+      raise ValueError('nargs must be %r to supply const' % argparse.OPTIONAL)
+    super(UpdateAction, self).__init__(
+        option_strings=option_strings,
+        dest=dest,
+        nargs=nargs,
+        const=const,
+        default=default,
+        type=type,
+        choices=choices,
+        required=required,
+        help=help,
+        metavar=metavar)
+    self.onduplicatekey_handler = onduplicatekey_handler
+
+  def __call__(self, parser, namespace, values, option_string=None):
+
+    # Get the existing dictionary (if any)
+    items = copy.copy(argparse._ensure_value(namespace, self.dest, {}))
+
+    # Merge the new key/value pair(s) in
+    for k, v in values.iteritems():
+      if k in items:
+        items[k] = self.onduplicatekey_handler(self, k, items[k], v)
+      else:
+        items[k] = v
+
+    # Saved the merged dictionary
+    setattr(namespace, self.dest, items)
+
+
+class UpdateActionWithAppend(UpdateAction):
+  """Create a single dict value from delimited or repeated flags.
+
+  This class provides a variant of UpdateAction, which allows for users to
+  append, rather than reject, duplicate key values. For example, the user
+  can specify:
+
+    --inputs k1=v1a --inputs k1=v1b --inputs k2=v2
+
+  and the result will be:
+
+     { 'k1': ['v1a', 'v1b'], 'k2': 'v2' }
+  """
+
+  def OnDuplicateKeyAppend(self, unused_key, existing_value, new_value):
+    if isinstance(existing_value, list):
+      return existing_value + [new_value]
+    else:
+      return [existing_value, new_value]
+
+  def __init__(self,
+               option_strings,
+               dest,
+               nargs=None,
+               const=None,
+               default=None,
+               type=None,  # pylint:disable=redefined-builtin
+               choices=None,
+               required=False,
+               help=None,  # pylint:disable=redefined-builtin
+               metavar=None,
+               onduplicatekey_handler=OnDuplicateKeyAppend):
+    super(UpdateActionWithAppend, self).__init__(
+        option_strings=option_strings,
+        dest=dest,
+        nargs=nargs,
+        const=const,
+        default=default,
+        type=type,
+        choices=choices,
+        required=required,
+        help=help,
+        metavar=metavar,
+        onduplicatekey_handler=onduplicatekey_handler)
+
+
+class BufferedFileInput(object):
+  """Creates an argparse type that reads and buffers file or stdin contents.
+
+  This is similar to argparse.FileType, but unlike FileType it does not leave
+  a dangling file handle open. The argument stored in the argparse Namespace
+  is the file's contents.
+
+  Args:
+    max_bytes: int, The maximum file size in bytes, or None to specify no
+        maximum.
+    chunk_size: int, When max_bytes is not None, the buffer size to use when
+        reading chunks from the input file.
+
+  Returns:
+    A function that accepts a filename, or "-" representing that stdin should be
+    used as input.
+  """
+
+  def __init__(self, max_bytes=None, chunk_size=16*1024):
+    self.max_bytes = max_bytes
+    self.chunk_size = chunk_size
+
+  def _ReadFile(self, f):
+    # Unbounded
+    if self.max_bytes is None:
+      return f.read()
+    else:
+      contents = ''
+      while True:
+        chunk = f.read(self.chunk_size)
+
+        # Check for EOF
+        if not chunk:
+          return contents
+        # Fail if the file is too large.
+        if len(contents) + len(chunk) > self.max_bytes:
+          if hasattr(f, 'name'):
+            raise ArgumentTypeError("File '{0}' is too large.".format(f.name))
+          else:
+            raise ArgumentTypeError('File is too large.')
+
+        contents += chunk
+
+  def __call__(self, name):
+    """Return the contents of the file with the specified name.
+
+    If name is "-", stdin is read until EOF. Otherwise, the named file is read.
+    If max_bytes is provided when calling BufferedFileInput, this function will
+    raise an ArgumentTypeError if the specified file is too large.
+
+    Args:
+      name: str, The file name, or '-' to indicate stdin.
+
+    Returns:
+      The contents of the file.
+
+    Raises:
+      ArgumentTypeError: If the file cannot be read or is too large.
+    """
+    # Handle stdin
+    if name == '-':
+      return self._ReadFile(sys.stdin)
+
+    try:
+      with open(name, 'r') as f:
+        return self._ReadFile(f)
+    except (IOError, OSError) as e:
+      raise ArgumentTypeError(
+          "Can't open '{0}': {1}".format(name, e))
