@@ -29,10 +29,8 @@ from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import display
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.calliope import usage_text
-from googlecloudsdk.core import cli as core_cli
 from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
-from googlecloudsdk.core import properties
 from googlecloudsdk.core import remote_completion
 from googlecloudsdk.core.updater import update_manager
 from googlecloudsdk.core.util import pkg_resources
@@ -252,7 +250,14 @@ class ArgumentParser(argparse.ArgumentParser):
     Args:
       message: str, The error message to print.
     """
-    if self._is_group:
+    # No need to output help/usage text if we are in completion mode. However,
+    # we do need to populate group/command level choices. These choices are not
+    # loaded when there is a parser error since we do lazy loading.
+    if '_ARGCOMPLETE' in os.environ:
+      # pylint:disable=protected-access
+      if self._calliope_command._sub_parser:
+        self._calliope_command.LoadAllSubElements()
+    elif self._is_group:
       shorthelp = usage_text.ShortHelpText(
           self._calliope_command, self._calliope_command.ai)
       # pylint:disable=protected-access
@@ -317,6 +322,10 @@ class ArgumentParser(argparse.ArgumentParser):
         # ones for which a value was not actually provided.  If it is provided,
         # save the metavar name or the destination name.
         name = action.metavar if action.metavar else action.dest
+        if action.nargs and action.nargs != '?':
+          # This arg takes in multiple values, record how many were provided.
+          # (? means 0 or 1, so treat that as an arg that takes a single value.
+          name += ':' + str(len(arg_strings))
       if name:
         self._flag_collection.append(name)
     return super(ArgumentParser, self)._get_values(action, arg_strings)
@@ -478,8 +487,10 @@ class ArgumentInterceptor(object):
     # added at the root of the CLI tree, or if it is explicitly added to every
     # command level.
     is_global = self.is_root or is_replicated
-    # True if this should be marked as a commonly used flag.
-    is_common = kwargs.pop('is_common', False)
+    # The flag category name, None for no category. This is also used for help
+    # printing. Flags in the same category are grouped together in a section
+    # named "{category} FLAGS".
+    category = kwargs.pop('category', None)
     # Any alias this flag has for the purposes of the "did you mean"
     # suggestions.
     suggestion_aliases = kwargs.pop('suggestion_aliases', [])
@@ -488,6 +499,9 @@ class ArgumentInterceptor(object):
     # An explicit command to run for remote completion instead of the default
     # for this resource type.
     list_command_path = kwargs.pop('list_command_path', None)
+    # Callback function that receives the currently entered args at the time of
+    # remote completion processing, and returns the command to run.
+    list_command_callback_fn = kwargs.pop('list_command_callback_fn', None)
 
     positional = not name.startswith('-')
     if positional:
@@ -501,9 +515,9 @@ class ArgumentInterceptor(object):
         raise ArgumentException(
             "Positional arguments cannot contain a '-'. Illegal argument [{0}] "
             'for command [{1}]'.format(name, self.data.command_name))
-      if is_common:
+      if category:
         raise ArgumentException(
-            'Positional argument [{0}] cannot be marked as a common flag in '
+            'Positional argument [{0}] cannot have a category in '
             'command [{1}]'.format(name, self.data.command_name))
       if suggestion_aliases:
         raise ArgumentException(
@@ -522,25 +536,37 @@ class ArgumentInterceptor(object):
 
     added_argument = self.parser.add_argument(*args, **kwargs)
     self._AddRemoteCompleter(added_argument, completion_resource,
-                             list_command_path)
+                             list_command_path, list_command_callback_fn)
 
     if positional:
+      if category:
+        raise ArgumentException(
+            'Positional argument [{0}] cannot have a category in '
+            'command [{1}]'.format(name, self.data.command_name))
       self.positional_args.append(added_argument)
     else:
+      if category and required:
+        raise ArgumentException(
+            'Required flag [{0}] cannot have a category in '
+            'command [{1}]'.format(name, self.data.command_name))
+      if category == 'REQUIRED':
+        raise ArgumentException(
+            "Flag [{0}] cannot have category='REQUIRED' in "
+            'command [{1}]'.format(name, self.data.command_name))
+      added_argument.category = category
       added_argument.do_not_propagate = do_not_propagate
       added_argument.is_replicated = is_replicated
       added_argument.is_global = is_global
-      added_argument.is_common = is_common
       added_argument.suggestion_aliases = suggestion_aliases
       self.flag_args.append(added_argument)
 
       inverted_flag = self._AddInvertedBooleanFlagIfNecessary(
           added_argument, name, dest, kwargs)
       if inverted_flag:
+        inverted_flag.category = category
         inverted_flag.do_not_propagate = do_not_propagate
         inverted_flag.is_replicated = is_replicated
         inverted_flag.is_global = is_global
-        inverted_flag.is_common = is_common
         # Don't add suggestion aliases for the inverted flag.  It can only map
         # to one or the other.
         self.flag_args.append(inverted_flag)
@@ -617,14 +643,18 @@ class ArgumentInterceptor(object):
     # have the same dest and mutually exclusive group as the original flag.
     inverted_name = '--no-' + name[2:]
     # Explicit default=None yields the 'Use to disable.' text.
+    show_inverted = False
     if prop or (default in (True, None) and help_str != argparse.SUPPRESS):
       if prop:
+        if prop.default:
+          show_inverted = True
         inverted_help = (' Overrides the default *{0}* property value'
                          ' for this command invocation. Use *{1}* to'
                          ' disable.'.format(prop.name, inverted_name))
       elif default:
         inverted_help = ' Enabled by default, use *{0}* to disable.'.format(
             inverted_name)
+        show_inverted = True
       else:
         inverted_help = ' Use *{0}* to disable.'.format(inverted_name)
       # calliope.markdown.MarkdownGenerator._Details() checks and appends
@@ -642,7 +672,10 @@ class ArgumentInterceptor(object):
       kwargs['dest'] = dest
     kwargs['help'] = argparse.SUPPRESS
 
-    return self.parser.add_argument(inverted_name, **kwargs)
+    inverted_argument = self.parser.add_argument(inverted_name, **kwargs)
+    if show_inverted:
+      setattr(inverted_argument, 'show_inverted', show_inverted)
+    return inverted_argument
 
   def _ShouldInvertBooleanFlag(self, name, action):
     """Checks if flag name with action is a Boolean flag to invert.
@@ -673,7 +706,7 @@ class ArgumentInterceptor(object):
     return False, None
 
   def _AddRemoteCompleter(self, added_argument, completion_resource,
-                          list_command_path):
+                          list_command_path, list_command_callback_fn):
     """Adds a remote completer to the given argument if necessary.
 
     Args:
@@ -682,16 +715,19 @@ class ArgumentInterceptor(object):
         corresponds to.
       list_command_path: str, The explicit calliope command to run to get the
         completions if you want to override the default for the given resource
-        type.
+        type. list_command_callback_fn takes precedence.
+      list_command_callback_fn: function, Callback function to be called to get
+        the list command. Takes precedence over list_command_path.
     """
     if not completion_resource:
       return
 
     if not list_command_path:
       list_command_path = completion_resource
-      # alpha and beta commands need to specify list_command_path
+      # alpha, beta, and preview commands need to specify list_command_path
       if (list_command_path.startswith('alpha') or
-          list_command_path.startswith('beta')):
+          list_command_path.startswith('beta') or
+          list_command_path.startswith('preview')):
         # if list_command_path not specified don't add the completer
         completion_resource = None
       else:
@@ -703,7 +739,8 @@ class ArgumentInterceptor(object):
           remote_completion.RemoteCompletion.GetCompleterForResource(
               completion_resource,
               self.cli_generator.Generate,
-              command_line=list_command_path))
+              command_line=list_command_path,
+              list_command_callback_fn=list_command_callback_fn))
       added_argument.completion_resource = completion_resource
 
   def _LowerCaseWithDashes(self, name):
@@ -713,53 +750,6 @@ class ArgumentInterceptor(object):
     return s2
 
 
-class ConfigHooks(object):
-  """This class holds function hooks for context and config loading/saving."""
-
-  def __init__(
-      self,
-      load_context=None,
-      context_filters=None,
-      group_class=None):
-    """Create a new object with the given hooks.
-
-    Args:
-      load_context: a function returns the context to be sent to commands.
-      context_filters: a list of functions that take (contex, args),
-          that will be called in order before a command is run. They are
-          described in the README under the heading GROUP SPECIFICATION.
-      group_class: base.Group, The class that this config hooks object is for.
-    """
-    self.load_context = load_context if load_context else lambda: {}
-    self.context_filters = context_filters if context_filters else []
-    self.group_class = group_class
-
-  def OverrideWithBase(self, group_base):
-    """Get a new ConfigHooks object with overridden functions based on module.
-
-    If module defines any of the function, they will be used instead of what
-    is in this object.  Anything that is not defined will use the existing
-    behavior.
-
-    Args:
-      group_base: The base.Group class corresponding to the group.
-
-    Returns:
-      A new ConfigHooks object updated with any newly found hooks
-    """
-
-    def ContextFilter(context, http_func, args):
-      group = group_base(http_func)
-      group.Filter(context, args)
-      return group
-    # We want the new_context_filters to be a completely new list, if there is
-    # a change.
-    new_context_filters = self.context_filters + [ContextFilter]
-    return ConfigHooks(load_context=self.load_context,
-                       context_filters=new_context_filters,
-                       group_class=group_base)
-
-
 class CommandCommon(object):
   """A base class for CommandGroup and Command.
 
@@ -768,7 +758,7 @@ class CommandCommon(object):
   """
 
   def __init__(self, common_type, path, release_track, cli_generator,
-               config_hooks, parser_group, allow_positional_args, parent_group):
+               parser_group, allow_positional_args, parent_group):
     """Create a new CommandCommon.
 
     Args:
@@ -778,10 +768,10 @@ class CommandCommon(object):
         group with respect to the CLI itself.  This path should be used for
         things like error reporting when a specific element in the tree needs
         to be referenced.
-      release_track: base.ReleaseTrack, The release track (ga, beta, alpha) that
-        this command group is in.  This will apply to all commands under it.
+      release_track: base.ReleaseTrack, The release track (ga, beta, alpha,
+        preview) that this command group is in.  This will apply to all commands
+        under it.
       cli_generator: cli.CLILoader, The builder used to generate this CLI.
-      config_hooks: a ConfigHooks object to use for loading context.
       parser_group: argparse.Parser, The parser that this command or group will
         live in.
       allow_positional_args: bool, True if this command can have positional
@@ -789,7 +779,6 @@ class CommandCommon(object):
       parent_group: CommandGroup, The parent of this command or group. None if
         at the root.
     """
-    self._config_hooks = config_hooks
     self._parent_group = parent_group
 
     self.name = path[-1]
@@ -802,7 +791,6 @@ class CommandCommon(object):
     self._cli_generator = cli_generator
 
     self._common_type = common_type
-    self._common_type.group_class = config_hooks.group_class
     self._common_type._cli_generator = cli_generator
     self._common_type._release_track = release_track
 
@@ -810,13 +798,6 @@ class CommandCommon(object):
       # Propagate down the hidden attribute.
       if parent_group.IsHidden():
         self._common_type._is_hidden = True
-      # TODO(user): This is going to go away once we remove the explicit
-      # Alpha and Beta decorators for commands.  Once the commands show up
-      # under the correct track, the help will use the regular release track
-      # for annotations (b/19406151).
-      legacy_release_track = parent_group._common_type._legacy_release_track
-      if legacy_release_track and not self._common_type._legacy_release_track:
-        self._common_type._legacy_release_track = legacy_release_track
 
     self.detailed_help = getattr(self._common_type, 'detailed_help', {})
     self._ExtractHelpStrings(self._common_type.__doc__)
@@ -825,9 +806,9 @@ class CommandCommon(object):
         parser_group=parser_group,
         allow_positional_args=allow_positional_args)
 
-  def ReleaseTrack(self, for_help=False):
+  def ReleaseTrack(self):
     """Gets the release track of this command or group."""
-    return self._common_type.ReleaseTrack(for_help=for_help)
+    return self._common_type.ReleaseTrack()
 
   def IsHidden(self):
     """Gets the hidden status of this command or group."""
@@ -867,12 +848,12 @@ class CommandCommon(object):
         self.index_help = self.index_help[:-1]
 
     # Add an annotation to the help strings to mark the release stage.
-    tag = self.ReleaseTrack(for_help=True).help_tag
+    tag = self.ReleaseTrack().help_tag
     if tag:
       self.short_help = tag + self.short_help
       self.long_help = tag + self.long_help
       # TODO(user):b/21208128: Drop these 4 lines.
-      prefix = self.ReleaseTrack(for_help=True).prefix
+      prefix = self.ReleaseTrack().prefix
       if len(self._path) < 2 or self._path[1] != prefix:
         self.index_help = tag + self.index_help
 
@@ -916,12 +897,12 @@ class CommandCommon(object):
     self.ai.add_argument(
         '-h', action=actions.ShortHelpAction(self),
         is_replicated=True,
-        is_common=True,
+        category=base.COMMONLY_USED_FLAGS,
         help='Print a summary help and exit.')
     self.ai.add_argument(
         '--help', action=actions.RenderDocumentAction(self, '--help'),
         is_replicated=True,
-        is_common=True,
+        category=base.COMMONLY_USED_FLAGS,
         help='Display detailed help.')
     self.ai.add_argument(
         '--document', action=actions.RenderDocumentAction(self),
@@ -1062,8 +1043,8 @@ class CommandGroup(CommandCommon):
   """A class to encapsulate a group of commands."""
 
   def __init__(self, module_dir, module_path, path, release_track,
-               construction_id, cli_generator, parser_group, config_hooks,
-               parent_group=None, allow_empty=False):
+               construction_id, cli_generator, parser_group, parent_group=None,
+               allow_empty=False):
     """Create a new command group.
 
     Args:
@@ -1082,7 +1063,6 @@ class CommandGroup(CommandCommon):
       parser_group: the current argparse parser, or None if this is the root
         command group.  The root command group will allocate the initial
         top level argparse parser.
-      config_hooks: a ConfigHooks object to use for loading context
       parent_group: CommandGroup, The parent of this group. None if at the
         root.
       allow_empty: bool, True to allow creating this group as empty to start
@@ -1103,7 +1083,6 @@ class CommandGroup(CommandCommon):
         path=path,
         release_track=release_track,
         cli_generator=cli_generator,
-        config_hooks=config_hooks,
         allow_positional_args=False,
         parser_group=parser_group,
         parent_group=parent_group)
@@ -1112,7 +1091,6 @@ class CommandGroup(CommandCommon):
     self._module_path = module_path
     self._construction_id = construction_id
 
-    self._config_hooks = self._config_hooks.OverrideWithBase(self._common_type)
     # find sub groups and commands
     self.groups = {}
     self.commands = {}
@@ -1159,7 +1137,7 @@ class CommandGroup(CommandCommon):
       CommandGroup and Command.
     """
     return [(self._module_dir, self._module_path + [name], name,
-             self.ReleaseTrack(for_help=False))
+             self.ReleaseTrack())
             for name in names]
 
   def AddSubGroup(self, group_info):
@@ -1207,7 +1185,7 @@ class CommandGroup(CommandCommon):
           continue
         (module_dir, module_path, name, unused_track) = info
         dst[name] = (module_dir, module_path, name,
-                     other_group.ReleaseTrack(for_help=False))
+                     other_group.ReleaseTrack())
 
   def SubParser(self):
     """Gets or creates the argparse sub parser for this group.
@@ -1278,14 +1256,14 @@ class CommandGroup(CommandCommon):
         element = CommandGroup(
             module_dir, module_path, self._path + [name], track,
             self._construction_id, self._cli_generator, self.SubParser(),
-            self._config_hooks, parent_group=self, allow_empty=allow_empty)
+            parent_group=self, allow_empty=allow_empty)
         self.groups[element.name] = element
       elif name in self._commands_to_load:
         (module_dir, module_path, name, track) = self._commands_to_load[name]
         element = Command(
             module_dir, module_path, self._path + [name], track,
-            self._construction_id, self._cli_generator, self._config_hooks,
-            self.SubParser(), parent_group=self)
+            self._construction_id, self._cli_generator, self.SubParser(),
+            parent_group=self)
         self.commands[element.name] = element
     except base.ReleaseTrackNotImplementedException as e:
       self._unloadable_elements.add(name)
@@ -1305,16 +1283,29 @@ class CommandGroup(CommandCommon):
         (item.cli_name,
          usage_text.HelpInfo(help_text=item.short_help,
                              is_hidden=item.IsHidden(),
-                             release_track=item.ReleaseTrack(for_help=True)))
+                             release_track=item.ReleaseTrack()))
         for item in self.groups.values())
 
+  def RunGroupFilter(self, context, args):
+    """Constructs and runs the Filter() method of all parent groups.
+
+    This recurses up to the root group and then constructs each group and runs
+    its Filter() method down the tree.
+
+    Args:
+      context: {}, The context dictionary that Filter() can modify.
+      args: The argparse namespace.
+    """
+    if self._parent_group:
+      self._parent_group.RunGroupFilter(context, args)
+    self._common_type().Filter(context, args)
+
 
 class Command(CommandCommon):
   """A class that encapsulates the configuration for a single command."""
 
   def __init__(self, module_dir, module_path, path, release_track,
-               construction_id, cli_generator, config_hooks, parser_group,
-               parent_group=None):
+               construction_id, cli_generator, parser_group, parent_group=None):
     """Create a new command.
 
     Args:
@@ -1329,7 +1320,6 @@ class Command(CommandCommon):
       construction_id: str, A unique identifier for the CLILoader that is
         being constructed.
       cli_generator: cli.CLILoader, The builder used to generate this CLI.
-      config_hooks: a ConfigHooks object to use for loading context
       parser_group: argparse.Parser, The parser to be used for this command.
       parent_group: CommandGroup, The parent of this command.
     """
@@ -1345,7 +1335,6 @@ class Command(CommandCommon):
         path=path,
         release_track=release_track,
         cli_generator=cli_generator,
-        config_hooks=config_hooks,
         allow_positional_args=True,
         parser_group=parser_group,
         parent_group=parent_group)
@@ -1369,38 +1358,15 @@ class Command(CommandCommon):
     """
     metrics.Loaded()
 
-    def Http(**kwargs):
-      # Possibly override timeout, making sure to leave kwargs[timeout]
-      # undefined (as opposed to None) if args.http_timout is not set.
-      if args.http_timeout:
-        kwargs['timeout'] = args.http_timeout
+    tool_context = {}
+    if self._parent_group:
+      self._parent_group.RunGroupFilter(tool_context, args)
 
-      return core_cli.Http(
-          cmd_path=self.dotted_name,
-          trace_token=args.trace_token,
-          trace_email=args.trace_email,
-          trace_log=args.trace_log,
-          log_http=properties.VALUES.core.log_http.GetBool(),
-          authority_selector=properties.VALUES.auth.authority_selector.Get(),
-          authorization_token_file=(
-              properties.VALUES.auth.authorization_token_file.Get()),
-          **kwargs)
-
-    tool_context = self._config_hooks.load_context()
-    last_group = None
-    for context_filter in self._config_hooks.context_filters:
-      last_group = context_filter(tool_context, Http, args)
-
-    command_instance = self._common_type(
-        cli=cli,
-        context=tool_context,
-        group=last_group,
-        http_func=Http,
-        format_string=args.format or 'yaml')
+    command_instance = self._common_type(cli=cli, context=tool_context)
 
     log.debug('Running %s with %s.', self.dotted_name, args)
     resources = command_instance.Run(args)
-    display.Displayer(command_instance, args, resources).Display()
+    resources = display.Displayer(command_instance, args, resources).Display()
     metrics.Ran()
 
     if command_instance.exit_code != 0:
