@@ -19,14 +19,21 @@ import abc
 import sys
 
 from googlecloudsdk.calliope import arg_parsers
-from googlecloudsdk.calliope import usage_text
+from googlecloudsdk.calliope import display
 from googlecloudsdk.core import log
+from googlecloudsdk.core import remote_completion
 from googlecloudsdk.core.resource import resource_exceptions
-from googlecloudsdk.core.resource import resource_printer
 from googlecloudsdk.core.resource import resource_registry
 
 
-ADD_TO_URI_CACHE, DELETE_FROM_URI_CACHE, REPLACE_URI_CACHE = range(1, 4)
+# Common markdown.
+MARKDOWN_BOLD = '*'
+MARKDOWN_ITALIC = '_'
+MARKDOWN_CODE = '`'
+
+
+# Common flag categories.
+COMMONLY_USED_FLAGS = 'COMMONLY USED'
 
 
 class LayoutException(Exception):
@@ -42,8 +49,9 @@ class ReleaseTrack(object):
   """An enum representing the release track of a command or command group.
 
   The release track controls where a command appears.  The default of GA means
-  it will show up under gcloud.  If you enable a command or group for the alpha
-  or beta tracks, those commands will be duplicated under those groups as well.
+  it will show up under gcloud.  If you enable a command or group for the alpha,
+  beta, or preview tracks, those commands will be duplicated under those groups
+  as well.
   """
 
   class _TRACK(object):
@@ -65,13 +73,18 @@ class ReleaseTrack(object):
   GA = _TRACK('GA', None, None, None)
   BETA = _TRACK(
       'BETA', 'beta',
-      '{0}(BETA){0} '.format(usage_text.MARKDOWN_BOLD),
+      '{0}(BETA){0} '.format(MARKDOWN_BOLD),
       'This command is currently in BETA and may change without notice.')
   ALPHA = _TRACK(
       'ALPHA', 'alpha',
-      '{0}(ALPHA){0} '.format(usage_text.MARKDOWN_BOLD),
+      '{0}(ALPHA){0} '.format(MARKDOWN_BOLD),
       'This command is currently in ALPHA and may change without notice.')
-  _ALL = [GA, BETA, ALPHA]
+  PREVIEW = _TRACK(
+      'PREVIEW', 'preview',
+      '{0}(PREVIEW){0} '.format(MARKDOWN_BOLD),
+      'This command is currently in DEVELOPER PREVIEW and may change without '
+      'notice.')
+  _ALL = [GA, BETA, ALPHA, PREVIEW]
 
   @staticmethod
   def AllValues():
@@ -113,6 +126,14 @@ class Argument(object):
     self.__args = args
     self.__kwargs = kwargs
 
+  def __GetFlag(self, parser):
+    """Returns the flag object in parser."""
+    name = self.__args[0]
+    for flag in parser.flag_args:
+      if name in flag.option_strings:
+        return flag
+    return None
+
   def AddToParser(self, parser):
     """Adds this argument to the given parser.
 
@@ -127,9 +148,48 @@ class Argument(object):
       arg.detailed_help = self.__detailed_help
     return arg
 
+  def RemoveFromParser(self, parser):
+    """Removes this flag from the given parser.
+
+    Args:
+      parser: The argparse parser.
+    """
+    flag = self.__GetFlag(parser)
+    if flag:
+      # Remove the flag and its inverse, if it exists, from its container.
+      name = flag.option_strings[0]
+      conflicts = [(name, flag)]
+      no_name = '--no-' + name[2:]
+      for no_flag in parser.flag_args:
+        if no_name in no_flag.option_strings:
+          conflicts.append((no_name, no_flag))
+      # pylint: disable=protected-access, argparse, why can't we be friends
+      flag.container._handle_conflict_resolve(flag, conflicts)
+      # Remove the conflict flags from the calliope argument interceptor.
+      for _, flag in conflicts:
+        parser.defaults.pop(flag.dest, None)
+        if flag.dest in parser.dests:
+          parser.dests.remove(flag.dest)
+        if flag in parser.flag_args:
+          parser.flag_args.remove(flag)
+
+  def SetDefault(self, parser, default):
+    """Sets the default value for this flag in the given parser.
+
+    Args:
+      parser: The argparse parser.
+      default: The default flag value.
+    """
+    flag = self.__GetFlag(parser)
+    if flag:
+      kwargs = {flag.dest: default}
+      parser.set_defaults(**kwargs)
+
 
 # Common flag definitions for consistency.
 
+LIST_COMMAND_FLAGS = 'LIST COMMAND'
+
 ASYNC_FLAG = Argument(
     '--async',
     action='store_true',
@@ -141,6 +201,7 @@ ASYNC_FLAG = Argument(
 FILTER_FLAG = Argument(
     '--filter',
     metavar='EXPRESSION',
+    category=LIST_COMMAND_FLAGS,
     help='Apply _EXPRESSION_ to select resource items to list.',
     detailed_help="""\
     Apply a Boolean filter _EXPRESSION_ to each resource item to be listed.
@@ -148,29 +209,47 @@ FILTER_FLAG = Argument(
     details run $ gcloud topic filters. If *--limit* is also specified
     then it is applied after *--filter*.""")
 
+FLATTEN_FLAG = Argument(
+    '--flatten',
+    metavar='KEY',
+    type=arg_parsers.ArgList(),
+    category=LIST_COMMAND_FLAGS,
+    help=('Flatten _name_[] resource slices in _KEY_ to separate records '
+          'for each item in each slice.'),
+    detailed_help="""\
+    Flatten _name_[] resource slices in _KEY_ to separate records for each
+    item in each slice. Multiple keys and slices may be specified. This also
+    flattens keys for *--format* and *--filter*. For example,
+    *--flatten=abc.def[]* flattens *abc.def[].ghi* references to
+    *abc.def.ghi*.""")
+
 LIMIT_FLAG = Argument(
     '--limit',
     type=arg_parsers.BoundedInt(1, sys.maxint, unlimited=True),
+    category=LIST_COMMAND_FLAGS,
     help='The maximum number of resources to list.',
     detailed_help="""\
     The maximum number of resources to list. The default is *unlimited*.
     If *--filter* is also specified then it is applied before *--limit*.
     """)
 
-PAGE_FLAG = Argument(
-    '--page',
+PAGE_SIZE_FLAG = Argument(
+    '--page-size',
     type=arg_parsers.BoundedInt(1, sys.maxint, unlimited=True),
-    help='The maximum resources list page size.',
+    category=LIST_COMMAND_FLAGS,
+    help='The service resource list page size.',
     detailed_help="""\
-    The maximum resources list page size. The default is determined by
-    the service if it supports paging, otherwise it is *unlimited*. If
-    *--filter* or *--limit** are also specified then less than the maximum may
-    be listed in each resource page.
+    Some services group resource list output into pages. This flag specifies
+    the maximum number of resources per page. The default is determined by the
+    service if it supports paging, otherwise it is *unlimited* (no paging).
+    Paging may be applied before or after *--filter* and *--limit* depending
+    on the service.
     """)
 
 SORT_BY_FLAG = Argument(
     '--sort-by',
     metavar='FIELDS',
+    category=LIST_COMMAND_FLAGS,
     help='A comma-separated list of field key names to sort by.',
     detailed_help="""\
     A comma-separated list of resource field key names to sort by. The
@@ -181,13 +260,8 @@ SORT_BY_FLAG = Argument(
 URI_FLAG = Argument(
     '--uri',
     action='store_true',
-    help='If provided, a list of URIs is printed instead of a table.',
-    detailed_help="""\
-    If provided, the list command will only print URIs for the resources
-    returned.  If this flag is not provided, the list
-    command will print a human-readable table of useful resource
-    data.
-    """)
+    category=LIST_COMMAND_FLAGS,
+    help='Print a list of resource URIs instead of the default output.')
 
 
 class _Common(object):
@@ -205,13 +279,9 @@ class _Common(object):
   _cli_generator = None
   _is_hidden = False
   _release_track = None
-  # TODO(user): Remove this once commands are only allowed to show up under
-  # the correct track (b/19406151)
-  _legacy_release_track = None
   _valid_release_tracks = None
 
-  def __init__(self, http_func):
-    self._http_func = http_func
+  def __init__(self):
     self.exit_code = 0
 
   @staticmethod
@@ -338,11 +408,7 @@ class _Common(object):
     return cls._is_hidden
 
   @classmethod
-  def ReleaseTrack(cls, for_help=False):
-    # TODO(user): Remove for_help once commands are only allowed to show up
-    # under the correct track (b/19406151).
-    if for_help and cls._legacy_release_track:
-      return cls._legacy_release_track
+  def ReleaseTrack(cls):
     return cls._release_track
 
   @classmethod
@@ -382,20 +448,6 @@ class _Common(object):
       return cls._cli_generator.Generate
     return None
 
-  def Http(self, auth=True, creds=None, **kwargs):
-    """Get the http object to be used during service requests.
-
-    Args:
-      auth: bool, True if the http object returned should be authorized.
-      creds: oauth2client.client.Credentials, If auth is True and creds is not
-          None, use those credentials to authorize the httplib2.Http object.
-      **kwargs: keyword arguments to forward to httplib2.Http()
-
-    Returns:
-      httplib2.Http, http object to be used during service requests.
-    """
-    return self._http_func(auth=auth, creds=creds, **kwargs)
-
 
 class Group(_Common):
   """Group is a base class for groups to implement.
@@ -405,8 +457,8 @@ class Group(_Common):
         service requests.
   """
 
-  def __init__(self, http_func):
-    super(Group, self).__init__(http_func)
+  def __init__(self):
+    super(Group, self).__init__()
 
   def Filter(self, context, args):
     """Modify the context that will be given to this group's commands when run.
@@ -427,23 +479,18 @@ class Command(_Common):
     cli: calliope.cli.CLI, The CLI object representing this command line tool.
     context: {str:object}, A set of key-value pairs that can be used for
         common initialization among commands.
-    group: base.Group, The instance of the group class above this command.  You
-        can use this to access common methods within a group.
     http_func: function that returns an http object that can be used during
         service requests.
-    __format_string: str, The default resource printer format string.
-    __uri_cache_enabled: bool, The URI cache enabled state.
+    _uri_cache_enabled: bool, The URI cache enabled state.
   """
 
   __metaclass__ = abc.ABCMeta
 
-  def __init__(self, cli, context, group, http_func, format_string):
-    super(Command, self).__init__(http_func)
+  def __init__(self, cli, context):
+    super(Command, self).__init__()
     self.cli = cli
     self.context = context
-    self.group = group
-    self.__format_string = format_string
-    self.__uri_cache_enabled = False
+    self._uri_cache_enabled = False
 
   def ExecuteCommand(self, args):
     self.cli.Execute(args, call_arg_complete=False)
@@ -461,21 +508,15 @@ class Command(_Common):
     """
     pass
 
-  @staticmethod
-  def Collection(args):
+  def Collection(self):
     """Returns the default collection path string.
 
     Should handle all command-specific args. --async is handled by
     ResourceInfo().
 
-    Args:
-      args: argparse.Namespace, An object that contains the values for the
-          arguments specified in the ._Flags() and .Args() methods.
-
     Returns:
       The default collection path string.
     """
-    _ = args
     return None
 
   def ResourceInfo(self, args):
@@ -496,45 +537,61 @@ class Command(_Common):
     Returns:
       A resource object dispatched by display.Displayer().
     """
-    collection = self.Collection(args)  # pylint: disable=assignment-from-none
+    collection = self.Collection()  # pylint: disable=assignment-from-none
     if not collection:
       return None
     info = resource_registry.Get(collection)
     if not getattr(args, 'async', False):
       return info
-    async_collection = info.async_collection
-    if not async_collection:
+    if not info.async_collection:
       raise resource_exceptions.ResourceRegistryAttributeError(
           'Collection [{collection}] does not have an async_collection '
           'attribute.'.format(collection=collection))
-    return resource_registry.Get(async_collection)
+    info = resource_registry.Get(info.async_collection)
+    # One more indirection allowed for commands that have a different operations
+    # format for --async and operations list.
+    if info.async_collection:
+      info = resource_registry.Get(info.async_collection)
+    return info
 
   def Format(self, unused_args):
     """Returns the default format string."""
     return 'default'
 
-  def GetUriCacheUpdateOp(self):
-    """Returns the URI cache update OP."""
-    return None
+  def ListFormat(self, args):
+    info = self.ResourceInfo(args)
+    if info and info.list_format:
+      return info.list_format
+    return 'default'
 
-  def DisableUriCache(self):
-    """Disables URI caching for this command."""
-    self.__uri_cache_enabled = False
-
-  def IsUriCacheEnabled(self):
-    """Returns True if URI caching is enabled for this command."""
-    return self.__uri_cache_enabled
-
-  # TODO(user): Drop format, __format_string after all use resource_printer.
-  # pylint: disable=invalid-name
-  def format(self, obj):
-    """Prints out the given object using the format decided by the format flag.
+  def Epilog(self, resources_were_displayed):
+    """Called after resources are displayed if the default format was used.
 
     Args:
-      obj: Object, The object to print.
+      resources_were_displayed: True if resources were displayed.
     """
-    if obj:
-      resource_printer.Print(obj, self.__format_string, out=log.out)
+    _ = resources_were_displayed
+
+  def Defaults(self):
+    """Returns the command projection defaults."""
+    return None
+
+  def GetReferencedKeyNames(self, args):
+    """Returns the key names referenced by the filter and format expressions."""
+    return display.Displayer(self, args, None).GetReferencedKeyNames()
+
+  def GetUriFunc(self):
+    """Returns a function that transforms a command resource item to a URI.
+
+    Returns:
+      func(resource) that transforms resource into a URI.
+    """
+    return None
+
+  @staticmethod
+  def GetUriCacheUpdateOp():
+    """Returns the URI cache update OP."""
+    return None
 
 
 class SilentCommand(Command):
@@ -559,10 +616,11 @@ class CacheCommand(Command):
 
   def __init__(self, *args, **kwargs):
     super(CacheCommand, self).__init__(*args, **kwargs)
-    self.__uri_cache_enabled = True
+    self._uri_cache_enabled = True
 
+  @staticmethod
   @abc.abstractmethod
-  def GetUriCacheUpdateOp(self):
+  def GetUriCacheUpdateOp():
     """Returns the URI cache update OP."""
     pass
 
@@ -574,18 +632,34 @@ class ListCommand(CacheCommand):
 
   @staticmethod
   def _Flags(parser):
-    """Adds the default flags for all ListCommand commands."""
+    """Adds the default flags for all ListCommand commands.
+
+    Args:
+      parser: The argparse parser.
+    """
 
     FILTER_FLAG.AddToParser(parser)
+    FLATTEN_FLAG.AddToParser(parser)
     LIMIT_FLAG.AddToParser(parser)
+    PAGE_SIZE_FLAG.AddToParser(parser)
     SORT_BY_FLAG.AddToParser(parser)
+    URI_FLAG.AddToParser(parser)
+
+  def Epilog(self, resources_were_displayed):
+    """Called after resources are displayed if the default format was used.
+
+    Args:
+      resources_were_displayed: True if resources were displayed.
+    """
+    if not resources_were_displayed:
+      log.status.Print('Listed 0 items.')
 
   def Format(self, args):
-    info = self.ResourceInfo(args)
-    return info.list_format or 'default'
+    return self.ListFormat(args)
 
-  def GetUriCacheUpdateOp(self):
-    return REPLACE_URI_CACHE
+  @staticmethod
+  def GetUriCacheUpdateOp():
+    return remote_completion.ReplaceCacheOp
 
 
 class CreateCommand(CacheCommand):
@@ -593,17 +667,19 @@ class CreateCommand(CacheCommand):
 
   __metaclass__ = abc.ABCMeta
 
-  def GetUriCacheUpdateOp(self):
-    return ADD_TO_URI_CACHE
+  @staticmethod
+  def GetUriCacheUpdateOp():
+    return remote_completion.AddToCacheOp
 
 
-class DeleteCommand(CacheCommand):
+class DeleteCommand(CacheCommand, SilentCommand):
   """A command that deletes resources."""
 
   __metaclass__ = abc.ABCMeta
 
-  def GetUriCacheUpdateOp(self):
-    return DELETE_FROM_URI_CACHE
+  @staticmethod
+  def GetUriCacheUpdateOp():
+    return remote_completion.DeleteFromCacheOp
 
 
 def Hidden(cmd_class):
@@ -623,38 +699,6 @@ def Hidden(cmd_class):
   return cmd_class
 
 
-# TODO(user): Remove this once commands are only allowed to show up under
-# the correct track (b/19406151).
-def Alpha(cmd_class):
-  """Decorator for annotating a command or group as ALPHA.
-
-  Args:
-    cmd_class: base._Common, A calliope command or group.
-
-  Returns:
-    A modified version of the provided class.
-  """
-  # pylint: disable=protected-access
-  cmd_class._legacy_release_track = ReleaseTrack.ALPHA
-  return cmd_class
-
-
-# TODO(user): Remove this once commands are only allowed to show up under
-# the correct track (b/19406151)
-def Beta(cmd_class):
-  """Decorator for annotating a command or group as BETA.
-
-  Args:
-    cmd_class: base._Common, A calliope command or group.
-
-  Returns:
-    A modified version of the provided class.
-  """
-  # pylint: disable=protected-access
-  cmd_class._legacy_release_track = ReleaseTrack.BETA
-  return cmd_class
-
-
 def ReleaseTracks(*tracks):
   """Mark this class as the command implementation for the given release tracks.
 
