@@ -17,7 +17,6 @@
 import argparse
 import os
 import re
-import ssl
 import sys
 import uuid
 
@@ -30,24 +29,12 @@ from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions as core_exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
-from googlecloudsdk.core import named_configs
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.configurations import named_configs
 from googlecloudsdk.core.resource import resource_printer
-from googlecloudsdk.core.util import files
 from googlecloudsdk.core.util import pkg_resources
 
 
-# There are some error classes that want to be handled as recoverable errors,
-# but cannot import the core_exceptions module (and therefore the Error class)
-# for various reasons (e.g. circular dependencies). To work around this, we keep
-# a list of known "friendly" error types, which we handle in the same way.
-# Additionally, we provide useful message suffixes for these error types.
-KNOWN_ERRORS = {
-    files.Error: '',
-    ssl.SSLError: 'This may be due to network connectivity issues. '
-                  'Please check your network settings, '
-                  'and the status of the service you are trying to reach.',
-}
 _COMMAND_SUFFIX = '.py'
 
 
@@ -98,7 +85,7 @@ class CLILoader(object):
   PATH_RE = re.compile(r'(?:([\w\.]+)\.)?([^\.]+)')
 
   def __init__(self, name, command_root_directory,
-               allow_non_existing_modules=False, load_context=None,
+               allow_non_existing_modules=False,
                logs_dir=config.Paths().logs_dir, version_func=None):
     """Initialize Calliope.
 
@@ -109,8 +96,6 @@ class CLILoader(object):
         CLI module.
       allow_non_existing_modules: True to allow extra module directories to not
         exist, False to raise an exception if a module does not exist.
-      load_context: A function that returns a context dict, or None for a
-        default which always returns {}.
       logs_dir: str, The path to the root directory to store logs in, or None
         for no log files.
       version_func: func, A function to call for a top-level -v and
@@ -127,7 +112,6 @@ class CLILoader(object):
 
     self.__allow_non_existing_modules = allow_non_existing_modules
 
-    self.__config_hooks = backend.ConfigHooks(load_context=load_context)
     self.__logs_dir = logs_dir
     self.__version_func = version_func
 
@@ -325,7 +309,7 @@ class CLILoader(object):
           cmd_or_grp_info = self.__GetCommandOrGroupInfo(
               module_dir_path=module_dir_path,
               name=cmd_or_grp_name,
-              release_track=(parent_group.ReleaseTrack(for_help=False)
+              release_track=(parent_group.ReleaseTrack()
                              if parent_group else None),
               allow_non_existing_modules=self.__allow_non_existing_modules,
               exception_if_present=exception_if_present)
@@ -378,8 +362,9 @@ class CLILoader(object):
     Args:
       module_dir_path: str, The path to the location of the module.
       name: str, The name that this group will appear as in the CLI.
-      release_track: base.ReleaseTrack, The release track (ga, beta, alpha) that
-        this command group is in.  This will apply to all commands under it.
+      release_track: base.ReleaseTrack, The release track (ga, beta, alpha,
+        preview) that this command group is in.  This will apply to all commands
+        under it.
       allow_non_existing_modules: True to allow this module directory to not
         exist, False to raise an exception if this module does not exist.
       exception_if_present: Exception, An exception to throw if the module
@@ -397,8 +382,8 @@ class CLILoader(object):
       group is found under.  The module_path is the relative path of the root
       of the command group from the module_dir. name is the user facing name
       this group will appear under wherever this command group is mounted.  The
-      release_track is the release track (ga, beta, alpha) that this command
-      group is in.
+      release_track is the release track (ga, beta, alpha, preview) that this
+      command group is in.
     """
     module_root, module = os.path.split(module_dir_path)
     if not pkg_resources.IsImportable(module, module_root):
@@ -426,7 +411,7 @@ class CLILoader(object):
     (module_root, module, name, release_track) = group_info
     return backend.CommandGroup(
         module_root, module, [name], release_track, uuid.uuid4().hex, self,
-        None, self.__config_hooks)
+        None)
 
   def __AddBuiltinGlobalFlags(self, top_element):
     """Adds in calliope builtin global flags.
@@ -442,7 +427,7 @@ class CLILoader(object):
       top_element.ai.add_argument(
           '-v', '--version',
           do_not_propagate=True,
-          is_common=True,
+          category=calliope_base.COMMONLY_USED_FLAGS,
           action=actions.FunctionExitAction(self.__version_func),
           help='Print version information and exit. This flag is only available'
           ' at the global level.')
@@ -450,7 +435,7 @@ class CLILoader(object):
     configuration_flag = top_element.ai.add_argument(
         '--configuration',
         metavar='CONFIGURATION',
-        is_common=True,
+        category=calliope_base.COMMONLY_USED_FLAGS,
         help='The configuration to use for this command invocation.')
     configuration_flag.detailed_help = (
         'The configuration to use for this command invocation. For more '
@@ -463,7 +448,7 @@ class CLILoader(object):
         '--verbosity',
         choices=log.OrderedVerbosityNames(),
         default=None,
-        is_common=True,
+        category=calliope_base.COMMONLY_USED_FLAGS,
         help=(
             'Override the default verbosity for this command.  This must be '
             'a standard logging verbosity level: [{values}] (Default: '
@@ -490,7 +475,7 @@ class CLILoader(object):
     format_flag = top_element.ai.add_argument(
         '--format',
         default=None,
-        is_common=True,
+        category=calliope_base.COMMONLY_USED_FLAGS,
         help='The format for printing command output resources.')
     format_flag.detailed_help = """\
         Sets the format for printing command output resources. The default is a
@@ -500,7 +485,6 @@ class CLILoader(object):
 
     top_element.ai.add_argument(
         '--log-http',
-        nargs=0,
         default=None,  # Tri-valued, None => don't override the property.
         action=actions.StoreBooleanProperty(properties.VALUES.core.log_http),
         help='Log all HTTP server requests and responses to stderr.')
@@ -518,11 +502,17 @@ class CLILoader(object):
             properties.VALUES.auth.authorization_token_file),
         help=argparse.SUPPRESS)
 
+    top_element.ai.add_argument(
+        '--credential-file-override',
+        action=actions.StoreProperty(
+            properties.VALUES.auth.credential_file_override),
+        help=argparse.SUPPRESS)
+
     # Timeout value for HTTP requests.
     top_element.ai.add_argument(
         '--http-timeout',
         default=None,
-        type=float,
+        action=actions.StoreProperty(properties.VALUES.core.http_timeout),
         help=argparse.SUPPRESS)
 
   def __MakeCLI(self, top_element):
@@ -565,6 +555,10 @@ class CLI(object):
   def _TopElement(self):
     return self.__top_element
 
+  @property
+  def name(self):
+    return self.__name
+
   def IsValidCommand(self, cmd):
     """Checks if given command exists.
 
@@ -599,44 +593,45 @@ class CLI(object):
     if not args:
       args = sys.argv[1:]
 
+    # help ... is the same as ... --help or ... --document=style=help. We use
+    # --document=style=help to signal the metrics.Help() 'help' label in
+    # actions.RenderDocumentAction().Action(). It doesn't matter if we append
+    # to a command that already has --help or --document=style=... because the
+    # first --help/--document from the left takes effect. Note that
+    # `help ... --help` produces help for the help command itself.
+    if args and args[0] == 'help' and '--help' not in args:
+      args = args[1:] + ['--document=style=help']
+
     # Set the command name in case an exception happens before the command name
     # is finished parsing.
     command_path_string = self.__name
 
-    named_configs.FLAG_OVERRIDE_STACK.AllocateFrame()
+    # Look for a --configuration flag and update property state based on
+    # that before proceeding to the main argparse parse step.
+    named_configs.FLAG_OVERRIDE_STACK.PushFromArgs(args)
     properties.VALUES.PushInvocationValues()
+
     flag_names = None
     try:
       for s in args:
         try:
           s.decode('ascii')
         except UnicodeDecodeError:
-          raise exceptions.InvalidCharacterInArgException([sys.argv[0]] + args,
-                                                          s)
-      # Look for a --configuration flag and update property state based on
-      # that before proceeding to the main argparse parse step.
-      named_configs.FLAG_OVERRIDE_STACK.ReplaceTop(
-          named_configs.AdhocConfigFlagParse(args),
-          properties.PropertiesFile.Invalidate)
+          raise exceptions.InvalidCharacterInArgException(
+              [sys.argv[0]] + args, s)
+
       args = self.__parser.parse_args(args)
       flag_names = self.__parser.GetFlagCollection()
-
       # -h|--help|--document are dispatched by parse_args and never get here.
 
-      # In principle it's possible that the initial round of ad-hoc parsing of
-      # --configuration would not agree with the later parse by argparse.  The
-      # following warning is intended to be dead code, but just in case...
-      if named_configs.FLAG_OVERRIDE_STACK.Peek() != args.configuration:
-        log.warn('Problem parsing --configration flag.  Using named '
-                 'flag value --configuration=[{0}].'.format(
-                     named_configs.FLAG_OVERRIDE_STACK.Peek()))
-
       # Now that we have parsed the args, reload the settings so the flags will
       # take effect.  These will use the values from the properties.
       log.SetUserOutputEnabled(None)
       log.SetVerbosity(None)
 
       command_path_string = '.'.join(args.command_path)
+      properties.VALUES.SetInvocationValue(
+          properties.VALUES.metrics.command_name, command_path_string, None)
       metrics.Commands(command_path_string, config.CLOUD_SDK_VERSION,
                        flag_names)
 
@@ -657,10 +652,7 @@ class CLI(object):
       self._HandleKnownError(command_path_string, exc, flag_names,
                              print_error=True)
     except Exception as exc:
-      if type(exc) in KNOWN_ERRORS:
-        self._HandleKnownError(command_path_string, exc, flag_names,
-                               print_error=True)
-      else:
+      if not self._MaybeHandleKnownError(command_path_string, exc, flag_names):
         # Make sure any uncaught exceptions still make it into the log file.
         exc_printable = self.SafeExceptionToString(exc)
         log.debug(exc_printable, exc_info=sys.exc_info())
@@ -685,32 +677,51 @@ class CLI(object):
     """
     try:
       return str(exc)
-    except UnicodeEncodeError:
+    except UnicodeError:
       # Handle errors that have unprintable unicode BOM markers (eg html).
       # See http://bugs.python.org/issue2517.
       # See http://stackoverflow.com/questions/3715865/unicodeencodeerror-ascii.
-      return unicode(exc).encode('ascii', 'ignore')
+      return unicode(exc).encode('utf-8', 'replace')
 
-  def _HandleKnownError(self, command_path_string, exc, flag_names,
-                        print_error=True):
-    """For exceptions we know about, just print the error and exit.
+  def _MaybeHandleKnownError(self, command_path_string, exc, flag_names):
+    """Attempt to recognize the given exception as a known type and handle it.
 
     Args:
       command_path_string: str, The command that was run.
-      exc: Exception, The exeption that was raised.
+      exc: Exception, The exception that was raised.
+      flag_names: [str], The names of the flags that were used during this
+        execution.
+
+    Returns:
+      True if it was handled, False otherwise.
+    """
+    known_exc = exceptions.ConvertKnownError(exc)
+    if not known_exc:
+      return False
+    self._HandleKnownError(command_path_string, known_exc, flag_names,
+                           print_error=True, orig_exc=exc)
+    return True
+
+  def _HandleKnownError(self, command_path_string, exc, flag_names,
+                        print_error=True, orig_exc=None):
+    """Print the error and exit for exceptions of known type.
+
+    Args:
+      command_path_string: str, The command that was run.
+      exc: Exception, The exception that was raised.
       flag_names: [str], The names of the flags that were used during this
         execution.
       print_error: bool, True to print an error message, False to just exit with
         the given error code.
+      orig_exc: Exception or None, The original exception for metrics purposes
+        if the exception was converted to a new type.
     """
-    # If this is a known error, we may have an additional message suffix.
-    message_suffix = KNOWN_ERRORS.get(type(exc), '')
-    msg = u'({0}) {1} {2}'.format(command_path_string, unicode(exc),
-                                  message_suffix)
+    msg = '({0}) {1}'.format(
+        command_path_string, self.SafeExceptionToString(exc))
     log.debug(msg, exc_info=sys.exc_info())
     if print_error:
       log.error(msg)
-    metrics.Error(command_path_string, exc, flag_names)
+    metrics.Error(command_path_string, orig_exc or exc, flag_names)
     self._Exit(exc)
 
   def _Exit(self, exc):
