@@ -19,19 +19,122 @@ import re
 import StringIO
 import textwrap
 
+from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import usage_text
 from googlecloudsdk.third_party.py27 import py27_collections as collections
 
 
-def FilterOutSuppressed(args):
-  """Returns a copy of args containing only non-suppressed arguments."""
-  return [a for a in args if a.help != argparse.SUPPRESS]
-
-
 class Error(Exception):
   """Exceptions for the markdown module."""
 
 
+_SPLIT = 78  # Split lines longer than this.
+_SECTION_INDENT = 8  # Section or list within section indent.
+_FIRST_INDENT = 2  # First line indent.
+_SUBSEQUENT_INDENT = 6  # Subsequent line indent.
+
+
+class ExampleCommandLineSplitter(object):
+  """Example command line splitter.
+
+  Attributes:
+    max_index: int, The max index to check in line.
+    quote_char: str, The current quote char for quotes split across lines.
+    quote_index: int, The index of quote_char in line or 0 if in previous line.
+  """
+
+  def __init__(self):
+    self._max_index = _SPLIT - _SECTION_INDENT - _FIRST_INDENT
+    self._quote_char = None
+    self._quote_index = 0
+
+  def _SplitInTwo(self, line):
+    """Splits line into before and after, len(before) < self._max_index.
+
+    Args:
+      line: str, The line to split.
+
+    Returns:
+      (before, after)
+        The line split into two parts. <before> is a list of strings that forms
+        the first line of the split and <after> is a string containing the
+        remainder of the line to split. The display width of <before> is
+        < self._max_index. <before> contains the separator chars, including a
+        newline.
+    """
+    punct_index = 0
+    quoted_space_index = 0
+    quoted_space_quote = None
+    space_index = 0
+    space_flag = False
+    i = 0
+    while i < self._max_index:
+      c = line[i]
+      i += 1
+      if c == self._quote_char:
+        self._quote_char = None
+      elif self._quote_char:
+        if c == ' ':
+          quoted_space_index = i - 1
+          quoted_space_quote = self._quote_char
+      elif c in ('"', "'"):
+        self._quote_char = c
+        self._quote_index = i
+        quoted_space_index = 0
+      elif c == '\\':
+        i += 1
+      elif i < self._max_index:
+        if c == ' ':
+          # Split before a flag instead of the next arg; it could be the flag
+          # value.
+          if line[i] == '-':
+            space_flag = True
+            space_index = i
+          elif space_flag:
+            space_flag = False
+          else:
+            space_index = i
+        elif c in (',', ';', '/', '|'):
+          punct_index = i
+        elif c == '=':
+          space_flag = False
+    separator = '\\\n'
+    indent = _FIRST_INDENT
+    if space_index:
+      split_index = space_index
+      indent = _SUBSEQUENT_INDENT
+    elif quoted_space_index:
+      split_index = quoted_space_index
+      if quoted_space_quote == "'":
+        separator = '\n'
+    elif punct_index:
+      split_index = punct_index
+    else:
+      split_index = self._max_index
+    if split_index <= self._quote_index:
+      self._quote_char = None
+    else:
+      self._quote_index = 0
+    self._max_index = _SPLIT - _SECTION_INDENT - indent
+    return [line[:split_index], separator, ' ' * indent], line[split_index:]
+
+  def Split(self, line):
+    """Splits a long example command line by inserting newlines.
+
+    Args:
+      line: str, The command line to split.
+
+    Returns:
+      str, The command line with newlines inserted.
+    """
+    lines = []
+    while len(line) > self._max_index:
+      before, line = self._SplitInTwo(line)
+      lines.extend(before)
+    lines.append(line)
+    return ''.join(lines)
+
+
 class MarkdownGenerator(object):
   """Command help markdown document generator.
 
@@ -52,11 +155,6 @@ class MarkdownGenerator(object):
     _subgroup: The list of subgroup instances or None.
   """
 
-  SPLIT = 78  # Split lines longer than this.
-  SECTION_INDENT = 10  # Section or list within section indent.
-  FIRST_INDENT = 2  # First line indent.
-  SUBSEQUENT_INDENT = 6  # Subsequent line indent.
-
   def __init__(self, command):
     """Constructor.
 
@@ -72,7 +170,7 @@ class MarkdownGenerator(object):
     self._command_path = command.GetPath()
     self._command_name = ' '.join(self._command_path)
     self._file_name = '_'.join(self._command_path)
-    self._track = command.ReleaseTrack(for_help=True).prefix
+    self._track = command.ReleaseTrack().prefix
     command_index = (2 if self._track and len(self._command_path) >= 3 and
                      self._command_path[1] == self._track else 1)
     self._is_topic = (len(self._command_path) >= (command_index + 1) and
@@ -83,17 +181,6 @@ class MarkdownGenerator(object):
     self._subcommands = command.GetSubCommandHelps()
     self._subgroups = command.GetSubGroupHelps()
 
-  def _IsSuppressed(self, arg):
-    """Checks if arg help is suppressed.
-
-    Args:
-      arg: The argparse arg to check.
-
-    Returns:
-      True if arg help is suppressed.
-    """
-    return arg.help == argparse.SUPPRESS
-
   def _UserInput(self, msg):
     """Returns msg with user input markdown.
 
@@ -103,9 +190,9 @@ class MarkdownGenerator(object):
     Returns:
       The msg string with embedded user input markdown.
     """
-    return (usage_text.MARKDOWN_CODE + usage_text.MARKDOWN_ITALIC +
+    return (base.MARKDOWN_CODE + base.MARKDOWN_ITALIC +
             msg +
-            usage_text.MARKDOWN_ITALIC + usage_text.MARKDOWN_CODE)
+            base.MARKDOWN_ITALIC + base.MARKDOWN_CODE)
 
   def _Section(self, name, sep=True):
     """Prints the section header markdown for name.
@@ -121,8 +208,8 @@ class MarkdownGenerator(object):
   def _PrintSynopsisSection(self):
     """Prints the command line synopsis section."""
     # MARKDOWN_CODE is the default SYNOPSIS font style.
-    code = usage_text.MARKDOWN_CODE
-    em = usage_text.MARKDOWN_ITALIC
+    code = base.MARKDOWN_CODE
+    em = base.MARKDOWN_ITALIC
     self._Section('SYNOPSIS')
     self._out('{code}{command}{code}'.format(code=code,
                                              command=self._command_name))
@@ -130,9 +217,9 @@ class MarkdownGenerator(object):
     # Output the positional args up to the first REMAINDER or '-- *' args. The
     # rest will be picked up after the flag args are output. argparse does not
     # have an explicit '--' arg intercept, so we use the metavar value as a '--'
-    # sentinel.
-    # Any SUPPRESSed args are ingnored by a pre-pass.
-    positional_args = FilterOutSuppressed(self._command.ai.positional_args[:])
+    # sentinel. Any suppressed args are ingnored by a pre-pass.
+    positional_args = usage_text.FilterOutSuppressed(
+        self._command.ai.positional_args)
     while positional_args:
       arg = positional_args[0]
       if arg.nargs == argparse.REMAINDER or arg.metavar.startswith('-- '):
@@ -160,31 +247,48 @@ class MarkdownGenerator(object):
         group_id = self._command.ai.mutex_groups.get(flag.dest, flag.dest)
         groups[group_id].append(flag)
 
-    # Split the groups dict into required, common, and then the rest of the
-    # flags.  A group is required if any flag in it is required and common if
-    # any flag in it is common.  Required takes precedence over common.
-    required_groups = {}
-    common_groups = {}
+    # Partition the non-GLOBAL flag groups dict into categorized sections. A
+    # group is REQUIRED if any flag in it is required, categorized if any flag
+    # in it is categorized, otherwise its OTHER.  REQUIRED takes precedence
+    # over categorized.
+    categorized_groups = {}
     for group_id, flags in groups.iteritems():
       for f in flags:
         if f.required:
-          required_groups[group_id] = flags
-          break
-        elif f.is_common:
-          common_groups[group_id] = flags
-          break
-    for g in required_groups:
-      del groups[g]
-    for g in common_groups:
-      del groups[g]
+          category = 'REQUIRED'
+        elif f.category:
+          category = f.category
+        else:
+          continue
+        if category not in categorized_groups:
+          categorized_groups[category] = {}
+        categorized_groups[category][group_id] = flags
+        break
+    # Delete the categorized groups to get OTHER.
+    for v in categorized_groups.values():
+      for g in v:
+        del groups[g]
+    category = 'OTHER'
+    if category not in categorized_groups:
+      categorized_groups[category] = {}
+    for group_id, flags in groups.iteritems():
+      categorized_groups[category][group_id] = flags
 
-    # Generate the flag usage string with required flags first, then common
-    # flags, then the rest of the flags.
-    for section in [required_groups, common_groups, groups]:
+    # Collect the sections in order: REQUIRED, COMMON, OTHER, and categorized.
+    sections = []
+    for category in ('REQUIRED', base.COMMONLY_USED_FLAGS, 'OTHER'):
+      if category in categorized_groups:
+        sections.append(categorized_groups[category])
+        del categorized_groups[category]
+    for _, v in sorted(categorized_groups.iteritems()):
+      sections.append(v)
+
+    # Generate the flag usage string with flags in section order.
+    for section in sections:
       for group in sorted(section.values(), key=lambda g: g[0].option_strings):
         if len(group) == 1:
           arg = group[0]
-          if self._IsSuppressed(arg):
+          if usage_text.IsSuppressed(arg):
             continue
           msg = usage_text.FlagDisplayString(arg, markdown=True)
           if not msg:
@@ -194,10 +298,22 @@ class MarkdownGenerator(object):
           else:
             self._out(' [{msg}]'.format(msg=msg))
         else:
-          group.sort(key=lambda f: f.option_strings)
-          group = [flag for flag in group if not self._IsSuppressed(flag)]
-          msg = ' | '.join(usage_text.FlagDisplayString(arg, markdown=True)
-                           for arg in group)
+          # Check if the inverted boolean name should be displayed.
+          inverted = None
+          if len(group) == 2:
+            for arg in group:
+              if getattr(arg, 'show_inverted', False):
+                inverted = arg
+                break
+          if inverted:
+            # The inverted arg replaces the boolean group which only contains
+            # the arg and inverted arg.
+            msg = usage_text.FlagDisplayString(inverted, markdown=True)
+          else:
+            group = usage_text.FilterOutSuppressed(group)
+            group.sort(key=lambda f: f.option_strings)
+            msg = ' | '.join(usage_text.FlagDisplayString(arg, markdown=True)
+                             for arg in group)
           if not msg:
             continue
           self._out(' [{msg}]'.format(msg=msg))
@@ -206,12 +322,10 @@ class MarkdownGenerator(object):
       self._out(' [' + em + 'GLOBAL-FLAG ...' + em + ']')
 
     # positional_args will only be non-empty if we had -- ... or REMAINDER left.
-    for arg in FilterOutSuppressed(positional_args):
+    for arg in usage_text.FilterOutSuppressed(positional_args):
       self._out(usage_text.PositionalDisplayString(arg, markdown=True))
 
-  def _PrintFlagSection(self, flags, section):
-    if not flags:
-      return
+  def _PrintFlagSection(self, section, flags):
     self._Section(section, sep=False)
     for flag in sorted(flags, key=lambda f: f.option_strings):
       self._out('\n{0}::\n'.format(
@@ -220,7 +334,8 @@ class MarkdownGenerator(object):
 
   def _PrintPositionalsAndFlagsSections(self):
     """Prints the positionals and flags sections."""
-    visible_positionals = FilterOutSuppressed(self._command.ai.positional_args)
+    visible_positionals = usage_text.FilterOutSuppressed(
+        self._command.ai.positional_args)
     if visible_positionals:
       self._Section('POSITIONAL ARGUMENTS', sep=False)
       for arg in visible_positionals:
@@ -228,34 +343,12 @@ class MarkdownGenerator(object):
             usage_text.PositionalDisplayString(arg, markdown=True).lstrip()))
         self._out('\n{arghelp}\n'.format(arghelp=self._Details(arg)))
 
-    # Partition the flags into REQUIRED FLAGS, COMMON FLAGS, OPTIONAL_FLAGS and
-    # GLOBAL FLAGS subsections.
-    required_flags = []
-    common_flags = []
-    other_flags = []
-    has_global_flags = False
-    for arg in (self._command.ai.flag_args +
-                self._command.ai.ancestor_flag_args):
-      if not self._IsSuppressed(arg):
-        if arg.is_global and not self._is_top_element:
-          has_global_flags = True
-        elif arg.required:
-          required_flags.append(arg)
-        elif arg.is_common:
-          common_flags.append(arg)
-        else:
-          other_flags.append(arg)
+    sections, has_global_flags = usage_text.GetFlagSections(
+        self._command, self._command.ai)
 
-    other_flags_heading = 'FLAGS'
-    if required_flags:
-      other_flags_heading = 'OPTIONAL FLAGS'
-    if common_flags:
-      other_flags_heading = 'OTHER FLAGS'
-
-    for flags, section in ((required_flags, 'REQUIRED FLAGS'),
-                           (common_flags, 'COMMONLY USED FLAGS'),
-                           (other_flags, other_flags_heading)):
-      self._PrintFlagSection(flags, section)
+    # List the sections in order.
+    for section, _, flags in sections:
+      self._PrintFlagSection(section, flags)
 
     if has_global_flags:
       self._Section('GLOBAL FLAGS', sep=False)
@@ -319,13 +412,13 @@ class MarkdownGenerator(object):
   def _PrintNotesSection(self):
     """Prints the NOTES section if needed."""
     if (self._command.IsHidden() or
-        self._command.ReleaseTrack(for_help=True).help_note):
+        self._command.ReleaseTrack().help_note):
       self._Section('NOTES')
       if self._command.IsHidden():
         self._out('This command is an internal implementation detail and may'
                   ' change or disappear without notice.\n\n')
-      if self._command.ReleaseTrack(for_help=True).help_note:
-        self._out(self._command.ReleaseTrack(for_help=True).help_note + '\n\n')
+      if self._command.ReleaseTrack().help_note:
+        self._out(self._command.ReleaseTrack().help_note + '\n\n')
 
   def _Details(self, arg):
     """Returns the detailed help message for the given arg."""
@@ -333,54 +426,20 @@ class MarkdownGenerator(object):
     help_message = help_stuff() if callable(help_stuff) else help_stuff
     # calliope.backend.ArgumentInterceptor.add_argument() sets arg.inverted_help
     # for Boolean flags with auto-generated --no-FLAG inverted counterparts.
+    help_message = textwrap.dedent(help_message)
     inverted_help = getattr(arg, 'inverted_help', None)
     if inverted_help:
-      # Drop trailing space, newlines and periods. A period will be added below.
-      help_message = re.sub('[ .\n]+$', '', help_message)
+      help_message = help_message.rstrip()
       if help_message:
-        help_message += '.' + inverted_help + '\n'
-    return textwrap.dedent(help_message).replace('\n\n', '\n+\n').strip()
-
-  def _Split(self, line):
-    """Splits long example command lines.
-
-    Args:
-      line: str, The line to split.
-
-    Returns:
-      str, The split line.
-    """
-    ret = ''
-    m = self.SPLIT - self.FIRST_INDENT - self.SECTION_INDENT
-    n = len(line)
-    while n > m:
-      indent = self.SUBSEQUENT_INDENT
-      j = m
-      noflag = 0
-      while True:
-        if line[j] == ' ':
-          # Break at a flag if possible.
-          j += 1
-          if line[j] == '-':
-            break
-          # Look back one more operand to see if it's a flag.
-          if noflag:
-            j = noflag
-            break
-          noflag = j
-          j -= 2
+        i = help_message.rfind('\n')
+        if i >= 0 and help_message[i + 1] == ' ':
+          # Preserve example markdown at end of help_message.
+          help_message += '\n\n' + inverted_help.strip() + '\n'
         else:
-          # Line is too long -- force an operand split with no indentation.
-          j -= 1
-          if not j:
-            j = m
-            indent = 0
-            break
-      ret += line[:j] + '\\\n' + ' ' * indent
-      line = line[j:]
-      n = len(line)
-      m = self.SPLIT - self.SUBSEQUENT_INDENT - self.SECTION_INDENT
-    return ret + line
+          if not help_message.endswith('.'):
+            help_message += '.'
+          help_message += inverted_help + '\n'
+    return help_message.replace('\n\n', '\n+\n').strip()
 
   def _ExpandFormatReferences(self):
     """Expand {...} references."""
@@ -388,7 +447,7 @@ class MarkdownGenerator(object):
 
     # Split long $ ... example lines.
     pat = re.compile(r'^ *(\$ .{%d,})$' % (
-        self.SPLIT - self.FIRST_INDENT - self.SECTION_INDENT), re.M)
+        _SPLIT - _FIRST_INDENT - _SECTION_INDENT), re.M)
     pos = 0
     rep = ''
     while True:
@@ -396,7 +455,8 @@ class MarkdownGenerator(object):
       if not match:
         break
       rep += (self._doc[pos:match.start(1)] +
-              self._Split(self._doc[match.start(1):match.end(1)]))
+              ExampleCommandLineSplitter().Split(
+                  self._doc[match.start(1):match.end(1)]))
       pos = match.end(1)
     if rep:
       self._doc = rep + self._doc[pos:]
@@ -439,7 +499,7 @@ class MarkdownGenerator(object):
   def _AddManPageLinkMarkdown(self):
     """Add gcloud ...(1) man page link markdown."""
     top = self._command_path[0]
-    pat = re.compile(r'(\*?(' + top + r'((?:[-_ a-z])*))\*?)\(1\)')
+    pat = re.compile(r'(\*?(' + top + r'(?:[-_ a-z])*)\*?)\(1\)')
     pos = 0
     rep = ''
     while True:
@@ -447,10 +507,7 @@ class MarkdownGenerator(object):
       if not match:
         break
       cmd = match.group(2).replace('_', ' ')
-      ref = match.group(3).replace('_', ' ')
-      if ref:
-        ref = ref[1:]
-      ref = '/'.join(ref.split(' '))
+      ref = cmd.replace(' ', '/')
       lnk = '*link:' + ref + '[' + cmd + ']*'
       rep += self._doc[pos:match.start(2)] + lnk
       pos = match.end(1)
@@ -466,7 +523,7 @@ class MarkdownGenerator(object):
 
     This is a subjective choice for aesthetically pleasing renderings.
     """
-    pat = re.compile(r"(``([^`]*\w{2,}[^`']*)'')")
+    pat = re.compile(r"[^`](``([^`]*\w{2,}[^`']*)'')")
     pos = 0
     rep = ''
     while True:
