@@ -16,25 +16,28 @@
 """
 
 import argparse
+import difflib
 import re
 import StringIO
 import sys
 import textwrap
 
 from googlecloudsdk.calliope import arg_parsers
+from googlecloudsdk.calliope import base
 from googlecloudsdk.core.console import console_io
 
 LINE_WIDTH = 80
 HELP_INDENT = 25
 
-MARKDOWN_BOLD = '*'
-MARKDOWN_ITALIC = '_'
-MARKDOWN_CODE = '`'
+
+def IsSuppressed(arg):
+  """Returns True if arg is suppressed."""
+  return arg.help == argparse.SUPPRESS
 
 
 def FilterOutSuppressed(args):
   """Returns a copy of args containing only non-suppressed arguments."""
-  return [a for a in args if a.help != argparse.SUPPRESS]
+  return [a for a in args if not IsSuppressed(a)]
 
 
 class HelpInfo(object):
@@ -58,8 +61,6 @@ class CommandChoiceSuggester(object):
   """Utility to suggest mistyped commands.
 
   """
-  TEST_QUOTA = 5000
-  MAX_DISTANCE = 5
   _SYNONYM_SETS = [
       set(['create', 'add']),
       set(['delete', 'remove']),
@@ -68,9 +69,6 @@ class CommandChoiceSuggester(object):
   ]
 
   def __init__(self, choices=None):
-    self.cache = {}
-    self.inf = float('inf')
-    self._quota = self.TEST_QUOTA
     # A mapping of 'thing typed' to the suggestion that should be offered.
     # Often, these will be the same, but this allows for offering more currated
     # suggestions for more commonly misused things.
@@ -116,52 +114,6 @@ class CommandChoiceSuggester(object):
         # aliases for this synonym set.
         self.AddAliases(s_set, choice)
 
-  def _Deletions(self, s):
-    return [s[:i] + s[i + 1:] for i in range(len(s))]
-
-  def _GetDistance(self, longer, shorter):
-    """Get the edit distance between two words.
-
-    They must be in the correct order, since deletions and mutations only happen
-    from 'longer'.
-
-    Args:
-      longer: str, The longer of the two words.
-      shorter: str, The shorter of the two words.
-
-    Returns:
-      int, The number of substitutions or deletions on longer required to get
-      to shorter.
-    """
-
-    if longer == shorter:
-      return 0
-
-    try:
-      return self.cache[(longer, shorter)]
-    except KeyError:
-      pass
-
-    self.cache[(longer, shorter)] = self.inf
-    best_distance = self.inf
-
-    if len(longer) > len(shorter):
-      if self._quota < 0:
-        return self.inf
-      self._quota -= 1
-      for m in self._Deletions(longer):
-        best_distance = min(best_distance, self._GetDistance(m, shorter) + 1)
-
-    if len(longer) == len(shorter):
-      # just count how many letters differ
-      best_distance = 0
-      for i in range(len(longer)):
-        if longer[i] != shorter[i]:
-          best_distance += 1
-
-    self.cache[(longer, shorter)] = best_distance
-    return best_distance
-
   def GetSuggestion(self, arg):
     """Find the item that is closest to what was attempted.
 
@@ -171,33 +123,11 @@ class CommandChoiceSuggester(object):
     Returns:
       str, The closest match.
     """
-
-    min_distance = self.inf
-    bestchoice = None
-    for choice in self._choices:
-      self._quota = self.TEST_QUOTA
-      first, second = arg, choice
-      if len(first) < len(second):
-        first, second = second, first
-      if len(first) - len(second) > self.MAX_DISTANCE:
-        # Don't bother if they're too different.
-        continue
-      d = self._GetDistance(first.lower(), second.lower())
-      if d < min_distance:
-        min_distance = d
-        bestchoice = choice
-
-    if not bestchoice:
-      return None
-    # MAX_DISTANCE doesn't work very well for shorter strings (two strings of
-    # length n can have edit distance of at most n); any length-4 string will
-    # match any length-3 or length-4 string with MAX_DISTANCE 5, which is
-    # confusing. This trick prevents that.
-    if min_distance > min(self.MAX_DISTANCE, len(bestchoice) - 1, len(arg) - 1):
+    if not self._choices:
       return None
 
-    # Return the suggestion for the best choice.
-    return self._choices[bestchoice]
+    match = difflib.get_close_matches(arg.lower(), self._choices, 1)
+    return self._choices[match[0]] if match else None
 
 
 def WrapMessageInNargs(msg, nargs):
@@ -244,7 +174,7 @@ def PositionalDisplayString(arg, markdown=False):
   msg = arg.metavar or arg.dest.upper()
   if markdown:
     msg = re.sub(r'(\b[a-zA-Z][-a-zA-Z_0-9]*)',
-                 MARKDOWN_ITALIC + r'\1' + MARKDOWN_ITALIC, msg)
+                 base.MARKDOWN_ITALIC + r'\1' + base.MARKDOWN_ITALIC, msg)
   return ' ' + WrapMessageInNargs(msg, arg.nargs)
 
 
@@ -270,19 +200,20 @@ def FlagDisplayString(arg, brief=False, markdown=False):
         metavar=GetFlagMetavar(metavar, arg))
   if arg.nargs == 0:
     if markdown:
-      display_string = ', '.join([MARKDOWN_BOLD + x + MARKDOWN_BOLD
+      display_string = ', '.join([base.MARKDOWN_BOLD + x + base.MARKDOWN_BOLD
                                   for x in arg.option_strings])
     else:
       display_string = ', '.join(arg.option_strings)
   else:
     if markdown:
       metavar = re.sub('(\\b[a-zA-Z][-a-zA-Z_0-9]*)',
-                       MARKDOWN_ITALIC + '\\1' + MARKDOWN_ITALIC, metavar)
+                       base.MARKDOWN_ITALIC + '\\1' + base.MARKDOWN_ITALIC,
+                       metavar)
     display_string = ', '.join(
         ['{bb}{flag}{be}{metavar}'.format(
-            bb=MARKDOWN_BOLD if markdown else '',
+            bb=base.MARKDOWN_BOLD if markdown else '',
             flag=option_string,
-            be=MARKDOWN_BOLD if markdown else '',
+            be=base.MARKDOWN_BOLD if markdown else '',
             metavar=GetFlagMetavar(metavar, arg))
          for option_string in arg.option_strings])
     if not arg.required and arg.default:
@@ -379,7 +310,7 @@ def GenerateUsage(command, argument_interceptor, topic=False):
       usage_parts.append(PositionalDisplayString(arg))
 
     for arg in argument_interceptor.flag_args:
-      if arg.help == argparse.SUPPRESS:
+      if IsSuppressed(arg):
         continue
       if not arg.required:
         optional_messages = True
@@ -453,6 +384,107 @@ def ExpandHelpText(command, text):
       description=long_help)
 
 
+def GetFlagHeading(category):
+  """Returns the flag section heading for a flag category."""
+  return category if 'FLAGS' in category else category + ' FLAGS'
+
+
+def GetFlagSections(command, argument_interceptor):
+  """Returns the flag sections in document order.
+
+  Args:
+    command: calliope._CommandCommon, The command object with all subelements
+      already loaded.
+    argument_interceptor: calliope._ArgumentInterceptor, the object that tracks
+        all of the flags for this command or group.
+
+  Returns:
+    ([section], has_global_flags)
+      section - (heading, is_priority, flags)
+        heading - The section heading.
+        is_priority - True if this is a priority section. Priority sections are
+          grouped first. The first 2 priority sections appear in short help.
+        flags - The list of flags in the section.
+      has_global_flags - True if command has global flags not included in the
+        section list.
+  """
+  # Partition the non-GLOBAL flag groups dict into categorized sections. A
+  # group is REQUIRED if any flag in it is required, categorized if any flag
+  # in it is categorized, otherwise its OTHER.  REQUIRED takes precedence
+  # over categorized.
+  categories = {}
+  has_global_flags = False
+  is_top_element = command.IsRoot()
+  for arg in (argument_interceptor.flag_args +
+              argument_interceptor.ancestor_flag_args):
+    if not IsSuppressed(arg):
+      if arg.is_global and not is_top_element:
+        has_global_flags = True
+      else:
+        if arg.required:
+          category = 'REQUIRED'
+        elif arg.category:
+          category = arg.category
+        else:
+          category = 'OTHER'
+        if category not in categories:
+          categories[category] = []
+        categories[category].append(arg)
+
+  sections = []
+  other_flags_heading = 'FLAGS'
+
+  # Collect the priority sections first.
+  for category, other in (('REQUIRED', 'OPTIONAL'),
+                          (base.COMMONLY_USED_FLAGS, 'OTHER'),
+                          ('OTHER', None)):
+    if category in categories:
+      if other:
+        other_flags_heading = other
+        heading = category
+      elif len(categories) > 1:
+        heading = 'FLAGS'
+      else:
+        heading = other_flags_heading
+      sections.append(
+          (GetFlagHeading(heading), other is not None, categories[category]))
+      # This prevents the category from being re-added in the loop below.
+      del categories[category]
+
+  # Add the remaining categories in sorted order.
+  for category, flags in sorted(categories.iteritems()):
+    sections.append((GetFlagHeading(category), False, flags))
+
+  return sections, has_global_flags
+
+
+def TextIfExists(title, messages):
+  """Generates the text for the given section.
+
+  This printing is done by collecting a list of rows. If the row is just a
+  string, that means print it without decoration. If the row is a tuple, use
+  WrapWithPrefix to print that tuple in aligned columns.
+
+  Args:
+    title: str, The name of this section.
+    messages: str or [(str, str)], The item or items to print in this section.
+
+  Returns:
+    str, The generated text.
+  """
+  if not messages:
+    return None
+  textbuf = StringIO.StringIO()
+  textbuf.write('%s\n' % title)
+  if isinstance(messages, str):
+    textbuf.write('  ' + messages + '\n')
+  else:
+    for (arg, helptxt) in messages:
+      WrapWithPrefix(arg, helptxt, HELP_INDENT, LINE_WIDTH,
+                     spacing='  ', writer=textbuf)
+  return textbuf.getvalue()
+
+
 def ShortHelpText(command, argument_interceptor):
   """Get a command's short help text.
 
@@ -470,26 +502,6 @@ def ShortHelpText(command, argument_interceptor):
 
   buf = StringIO.StringIO()
 
-  required_messages = []
-  common_messages = []
-  optional_messages = []
-  has_global_flags = False
-
-  # Sorting for consistency and readability.
-  for arg in (argument_interceptor.flag_args +
-              argument_interceptor.ancestor_flag_args):
-    if arg.help == argparse.SUPPRESS:
-      continue
-    message = (FlagDisplayString(arg), arg.help or '')
-    if arg.is_global and not command.IsRoot():
-      has_global_flags = True
-    elif arg.required:
-      required_messages.append(message)
-    elif arg.is_common:
-      common_messages.append(message)
-    else:
-      optional_messages.append(message)
-
   positional_messages = []
 
   # Explicitly not sorting here - order matters.
@@ -519,34 +531,10 @@ def ShortHelpText(command, argument_interceptor):
   buf.write('\n\n')
 
   # Third, print out the short help for everything that can come on
-  # the command line, grouped into required flags, optional flags,
-  # sub groups, sub commands, and positional arguments.
+  # the command line, grouped into flag sections, sub groups, sub commands,
+  # and positional arguments.
 
-  def TextIfExists(title, messages):
-    """Generates the text for the given section.
-
-    This printing is done by collecting a list of rows. If the row is just a
-    string, that means print it without decoration. If the row is a tuple, use
-    WrapWithPrefix to print that tuple in aligned columns.
-
-    Args:
-      title: str, The name of this section.
-      messages: str or [(str, str)], The item or items to print in this section.
-
-    Returns:
-      str, The generated text.
-    """
-    if not messages:
-      return None
-    textbuf = StringIO.StringIO()
-    textbuf.write('%s\n' % title)
-    if type(messages) == str:
-      textbuf.write('  ' + messages + '\n')
-    else:
-      for (arg, helptxt) in messages:
-        WrapWithPrefix(arg, helptxt, HELP_INDENT, LINE_WIDTH,
-                       spacing='  ', writer=textbuf)
-    return textbuf.getvalue()
+  command_path = ' '.join(command.GetPath())
 
   if topic:
     all_messages = [
@@ -555,28 +543,23 @@ def ShortHelpText(command, argument_interceptor):
   else:
     all_messages = [
         TextIfExists('positional arguments:', positional_messages),
-        TextIfExists('required flags:', sorted(required_messages)),
     ]
 
-    # If this command has flags tagged as common, only show those flags, and
-    # print a message to use the long help to see all the flags (if there are
-    # others).
-    if common_messages:
-      all_messages.append(
-          TextIfExists('commonly used flags:', sorted(common_messages)))
-      command_path = ' '.join(command.GetPath())
-      if optional_messages:
-        all_messages.append(
-            TextIfExists(
-                'other flags:',
-                'Run: `{0} --help`\n  for the full list of available flags for '
-                'this command.'.format(command_path)))
-    # If nothing tagged as common, just display all the optional flags as we
-    # normally do.
-    else:
-      optional_flags_tag = 'optional flags:' if required_messages else 'flags:'
-      all_messages.append(
-          TextIfExists(optional_flags_tag, sorted(optional_messages)))
+    sections, has_global_flags = GetFlagSections(command, argument_interceptor)
+
+    # Add the top 2 is_priority sections.
+    for index, (heading, is_priority, flags) in enumerate(sections):
+      if not is_priority or index >= 2:
+        # More sections remain. Long help will list those.
+        all_messages.append(TextIfExists(
+            'flags:' if len(all_messages) == 1 else 'other flags:',
+            'Run `{0} --help`\n  for the full list of available flags for '
+            'this command.'.format(command_path)))
+        break
+      messages = []
+      for flag in flags:
+        messages.append((FlagDisplayString(flag), flag.help or ''))
+      all_messages.append(TextIfExists(heading.lower() + ':', sorted(messages)))
 
     if has_global_flags:
       root_command_name = command.GetPath()[0]
@@ -591,6 +574,9 @@ def ShortHelpText(command, argument_interceptor):
         TextIfExists('commands:', sorted(command_messages)),
     ])
   buf.write('\n'.join([msg for msg in all_messages if msg]))
+  buf.write(
+      '\n\nFor more detailed information on this command and its flags, run:\n'
+      '  {0} --help\n'.format(command_path))
 
   return buf.getvalue()
 
