@@ -93,18 +93,7 @@ _cloudsdk_root_dir() {
 }
 CLOUDSDK_ROOT_DIR=$(_cloudsdk_root_dir "$0")
 
-# Cloud SDK requires python 2 (2.6 or 2.7)
-case $CLOUDSDK_PYTHON in
-*python2*)
-  ;;
-*python[0-9]*)
-  CLOUDSDK_PYTHON=
-  ;;
-esac
-# if CLOUDSDK_PYTHON is empty
-if [ -z "$CLOUDSDK_PYTHON" ]; then
-  CLOUDSDK_PYTHON=/usr/local/bin/python2.7
-fi
+CLOUDSDK_PYTHON=/usr/local/bin/python2.7
 
 # if CLOUDSDK_PYTHON_SITEPACKAGES and VIRTUAL_ENV are empty
 case :$CLOUDSDK_PYTHON_SITEPACKAGES:$VIRTUAL_ENV: in
@@ -263,6 +252,7 @@ $env:CLOUDSDK_PYTHON_ARGS = $cloudsdk_python_args
 # </cloud-sdk-ps1-preamble>
 """
 
+
 class InstallationConfig(object):
   """Loads configuration constants from the core config file.
 
@@ -275,6 +265,8 @@ class InstallationConfig(object):
       calls.
     documentation_url: str, The URL where we can redirect people when they need
       more information.
+    release_notes_url: str, The URL where we host a nice looking version of our
+      release notes.
     snapshot_url: str, The url for the component manager to look at for
       updates.
     disable_updater: bool, True to disable the component manager for this
@@ -353,14 +345,16 @@ class InstallationConfig(object):
     return time.mktime(InstallationConfig.ParseRevision(revision))
 
   def __init__(self, version, revision, user_agent, documentation_url,
-               snapshot_url, disable_updater, disable_usage_reporting,
-               snapshot_schema_version, release_channel, config_suffix):
+               release_notes_url, snapshot_url, disable_updater,
+               disable_usage_reporting, snapshot_schema_version,
+               release_channel, config_suffix):
     # JSON returns all unicode.  We know these are regular strings and using
     # unicode in environment variables on Windows doesn't work.
     self.version = str(version)
     self.revision = revision
     self.user_agent = str(user_agent)
     self.documentation_url = str(documentation_url)
+    self.release_notes_url = str(release_notes_url)
     self.snapshot_url = str(snapshot_url)
     self.disable_updater = disable_updater
     self.disable_usage_reporting = disable_usage_reporting
@@ -388,6 +382,7 @@ CLOUDSDK_CLIENT_NOTSOSECRET = 'ZmssLNjJy2998hD4CTg2ejr2'
 
 CLOUDSDK_USER_AGENT = INSTALLATION_CONFIG.user_agent
 
+# Do not add more scopes here, see http://b/19019218.
 CLOUDSDK_SCOPES = (
     'https://www.googleapis.com/auth/userinfo.email',
     'https://www.googleapis.com/auth/cloud-platform',
@@ -434,57 +429,28 @@ class Paths(object):
       current workspace, or None if not in a workspace.
   """
   # Name of the directory that roots a cloud SDK workspace.
-  _CLOUDSDK_WORKSPACE_CONFIG_WORD = ('gcloud' +
-                                     INSTALLATION_CONFIG.config_suffix)
-  CLOUDSDK_WORKSPACE_CONFIG_DIR_NAME = '.%s' % _CLOUDSDK_WORKSPACE_CONFIG_WORD
+  _CLOUDSDK_GLOBAL_CONFIG_DIR_NAME = ('gcloud' +
+                                      INSTALLATION_CONFIG.config_suffix)
 
   CLOUDSDK_STATE_DIR = '.install'
-
   CLOUDSDK_PROPERTIES_NAME = 'properties'
 
-  # The pointer to the user's active config is stored in
-  # $GLOBAL_CONFIG/$CLOUDSDK_NAMED_CONFIG_ACTIVATOR_FILE_NAME, e.g.
-  # $HOME/.config/gcloud/active_config
-  CLOUDSDK_NAMED_CONFIG_ACTIVATOR_FILE_NAME = 'active_config'
-
-  CLOUDSDK_NAMED_CONFIG_DIRECTORY = 'configurations'
-
-  CLOUDSDK_NAMED_CONFIG_FILE_PREFIX = 'config_'
-
   def __init__(self):
     if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
       try:
         default_config_path = os.path.join(
-            os.environ['APPDATA'], Paths._CLOUDSDK_WORKSPACE_CONFIG_WORD)
+            os.environ['APPDATA'], Paths._CLOUDSDK_GLOBAL_CONFIG_DIR_NAME)
       except KeyError:
         # This should never happen unless someone is really messing with things.
         drive = os.environ.get('SystemDrive', 'C:')
         default_config_path = os.path.join(
-            drive, '\\', Paths._CLOUDSDK_WORKSPACE_CONFIG_WORD)
+            drive, '\\', Paths._CLOUDSDK_GLOBAL_CONFIG_DIR_NAME)
     else:
       default_config_path = os.path.join(
           os.path.expanduser('~'), '.config',
-          Paths._CLOUDSDK_WORKSPACE_CONFIG_WORD)
+          Paths._CLOUDSDK_GLOBAL_CONFIG_DIR_NAME)
     self.global_config_dir = os.getenv(CLOUDSDK_CONFIG, default_config_path)
 
-    try:
-      # It is possible for some filesystems (like fakefs) to not report current
-      # working directories.  In the event this happens, just don't try
-      # searching for a workspace.
-      cwd = os.getcwd()
-    except OSError:
-      cwd = None
-
-    self.workspace_dir = None
-    if cwd:
-      self.workspace_dir = file_utils.FindDirectoryContaining(
-          cwd, Paths.CLOUDSDK_WORKSPACE_CONFIG_DIR_NAME)
-
-    self.workspace_config_dir = None
-    if self.workspace_dir:
-      self.workspace_config_dir = os.path.join(
-          self.workspace_dir, Paths.CLOUDSDK_WORKSPACE_CONFIG_DIR_NAME)
-
   @property
   def sdk_root(self):
     """Searches for the Cloud SDK root directory.
@@ -590,20 +556,16 @@ class Paths(object):
     Returns:
       str, The path to the file.
     """
-    return os.path.join(self.global_config_dir,
-                        self.CLOUDSDK_NAMED_CONFIG_ACTIVATOR_FILE_NAME)
+    return os.path.join(self.global_config_dir, 'active_config')
 
   @property
-  def workspace_properties_path(self):
-    """Gets the path to the properties file in your local workspace.
+  def named_config_directory(self):
+    """Gets the path to the directory that stores the named configs.
 
     Returns:
-      str, The path to the file, or None if there is no local workspace.
+      str, The path to the directory.
     """
-    if not self.workspace_config_dir:
-      return None
-    return os.path.join(self.workspace_config_dir,
-                        self.CLOUDSDK_PROPERTIES_NAME)
+    return os.path.join(self.global_config_dir, 'configurations')
 
   @property
   def container_config_path(self):
@@ -695,12 +657,3 @@ class Paths(object):
       str, The path to the GCE cache.
     """
     return os.path.join(self.global_config_dir, 'gce')
-
-
-def GetProxyTypeMap():
-  # pylint:disable=g-import-not-at-top, We only want to import this if needed.
-  from httplib2 import socks
-  return {'socks4': socks.PROXY_TYPE_SOCKS4,
-          'socks5': socks.PROXY_TYPE_SOCKS5,
-          'http': socks.PROXY_TYPE_HTTP,
-          'http_no_tunnel': socks.PROXY_TYPE_HTTP_NO_TUNNEL}
