@@ -61,15 +61,15 @@ Usage:
                                             normal=normal))
 
   # Read one character from stdin with echo disabled.
-  c = con.GetRawChar()
+  c = con.GetRawKey()
   if c is None:
     print 'EOF\n'
 
-  # Return the print width of a string that may contain FontCode() chars.
-  print_width = con.PrintWidth(string)
+  # Return the display width of a string that may contain FontCode() chars.
+  display_width = con.DisplayWidth(string)
 
   # Reset the memoized state.
-  con = console_attr.GetConsoleAttr(reset=True)
+  con = console_attr.ResetConsoleAttr()
 
   # Print the console width and height in characters.
   width, height = con.GetTermSize()
@@ -88,6 +88,7 @@ Usage:
 
 
 import os
+import unicodedata
 
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_attr_os
@@ -101,82 +102,56 @@ class BoxLineCharacters(object):
   """
 
 
-class BoxLineCharactersUtf8(BoxLineCharacters):
-  """UTF-8 Box/line drawing characters."""
-  dl = '┐'
-  dr = '┌'
-  h = '─'
-  hd = '┬'
-  hu = '┴'
-  ul = '┘'
-  ur = '└'
-  v = '│'
-  vh = '┼'
-  vl = '┤'
-  vr = '├'
-  d_dl = '╗'
-  d_dr = '╔'
-  d_h = '═'
-  d_hd = '╦'
-  d_hu = '╩'
-  d_ul = '╝'
-  d_ur = '╚'
-  d_v = '║'
-  d_vh = '╬'
-  d_vl = '╣'
-  d_vr = '╠'
-
-
-class BoxLineCharactersWindows(BoxLineCharacters):
-  """Windows cp437 Box/line drawing characters."""
-  dl = '\xBF'
-  dr = '\xDA'
-  h = '\xC4'
-  hd = '\xC2'
-  hu = '\xC1'
-  ul = '\xD9'
-  ur = '\xC0'
-  v = '\xB3'
-  vh = '\xC5'
-  vl = '\xB4'
-  vr = '\xC3'
-  d_dl = '\xBB'
-  d_dr = '\xC9'
-  d_h = '\xCD'
-  d_hd = '\xCB'
-  d_hu = '\xCA'
-  d_ul = '\xBC'
-  d_ur = '\xC8'
-  d_v = '\xBA'
-  d_vh = '\xCE'
-  d_vl = '\xB9'
-  d_vr = '\xCC'
+class BoxLineCharactersUnicode(BoxLineCharacters):
+  """unicode Box/line drawing characters (cp437 compatible unicode)."""
+  dl = u'┐'
+  dr = u'┌'
+  h = u'─'
+  hd = u'┬'
+  hu = u'┴'
+  ul = u'┘'
+  ur = u'└'
+  v = u'│'
+  vh = u'┼'
+  vl = u'┤'
+  vr = u'├'
+  d_dl = u'╗'
+  d_dr = u'╔'
+  d_h = u'═'
+  d_hd = u'╦'
+  d_hu = u'╩'
+  d_ul = u'╝'
+  d_ur = u'╚'
+  d_v = u'║'
+  d_vh = u'╬'
+  d_vl = u'╣'
+  d_vr = u'╠'
 
 
 class BoxLineCharactersAscii(BoxLineCharacters):
   """ASCII Box/line drawing characters."""
-  dl = '+'
-  dr = '+'
-  h = '-'
-  hd = '+'
-  hu = '+'
-  ul = '+'
-  ur = '+'
-  v = '|'
-  vh = '+'
-  vl = '+'
-  vr = '+'
-  d_dl = '#'
-  d_dr = '#'
-  d_h = '='
-  d_hd = '#'
-  d_hu = '#'
-  d_ul = '#'
-  d_ur = '#'
-  d_v = '#'
-  d_vh = '#'
-  d_vl = '#'
-  d_vr = '#'
+  dl = u'+'
+  dr = u'+'
+  h = u'-'
+  hd = u'+'
+  hu = u'+'
+  ul = u'+'
+  ur = u'+'
+  v = u'|'
+  vh = u'+'
+  vl = u'+'
+  vr = u'+'
+  d_dl = u'#'
+  d_dr = u'#'
+  d_h = u'='
+  d_hd = u'#'
+  d_hu = u'#'
+  d_ul = u'#'
+  d_ur = u'#'
+  d_v = u'#'
+  d_vh = u'#'
+  d_vl = u'#'
+  d_vr = u'#'
 
 
 class ConsoleAttr(object):
@@ -198,7 +173,7 @@ class ConsoleAttr(object):
         win: Windows code page 437.
     _font_bold: The ANSI bold font embellishment code string.
     _font_italic: The ANSI italic font embellishment code string.
-    _get_raw_char: A function that reads one character from stdin with no echo.
+    _get_raw_key: A function that reads one keypress from stdin with no echo.
     _out: The console output file stream.
     _term: TERM environment variable value.
     _term_size: The terminal (x, y) dimensions in characters.
@@ -214,8 +189,8 @@ class ConsoleAttr(object):
       }
   _ANSI_COLOR_RESET = '39;0m'
 
-  _BULLETS_UTF8 = ('●', '■', '◆', '○', '□', '◇')
-  _BULLETS_WINDOWS = ('\x07', '\xFE', '\x04')
+  _BULLETS_UNICODE = (u'▪', u'◆', u'▸', u'▫', u'◇', u'▹')
+  _BULLETS_WINDOWS = (u'■', u'≡', u'∞', u'Φ', u'·')  # cp437 compatible unicode
   _BULLETS_ASCII = ('o', '*', '+', '-')
 
   def __init__(self, encoding=None, out=None):
@@ -230,15 +205,17 @@ class ConsoleAttr(object):
     """
     self._out = out or log.out
     # Normalize the encoding name.
+    console_encoding = None
     if not encoding:
-      encoding = 'ascii'
       if hasattr(self._out, 'encoding') and self._out.encoding:
         console_encoding = self._out.encoding.lower()
         if 'utf-8' in console_encoding:
           encoding = 'utf8'
         elif 'cp437' in console_encoding:
-          encoding = 'win'
-    self._encoding = encoding
+          encoding = 'cp437'
+    elif encoding == 'win':
+      encoding = 'cp437'
+    self._encoding = encoding or 'ascii'
     self._term = os.getenv('TERM', '').lower()
 
     # ANSI "standard" attributes.
@@ -257,17 +234,17 @@ class ConsoleAttr(object):
 
     # Encoded character attributes.
     if self._encoding == 'utf8':
-      self._box_line_characters = BoxLineCharactersUtf8()
-      self._bullets = self._BULLETS_UTF8
-    elif self._encoding == 'win':
-      self._box_line_characters = BoxLineCharactersWindows()
+      self._box_line_characters = BoxLineCharactersUnicode()
+      self._bullets = self._BULLETS_UNICODE
+    elif self._encoding == 'cp437':
+      self._box_line_characters = BoxLineCharactersUnicode()
       self._bullets = self._BULLETS_WINDOWS
     else:
       self._box_line_characters = BoxLineCharactersAscii()
       self._bullets = self._BULLETS_ASCII
 
     # OS specific attributes.
-    self._get_raw_char = [console_attr_os.GetRawCharFunction()]
+    self._get_raw_key = [console_attr_os.GetRawKeyFunction()]
     self._term_size = console_attr_os.GetTermSize()
 
   def Colorize(self, string, color, justify=None):
@@ -287,10 +264,26 @@ class ConsoleAttr(object):
           color_code=self._ANSI_COLOR[color],
           reset_code=self._ANSI_COLOR_RESET,
           string=string))
-      # TODO(user): Add elif self._encoding == 'win': code here.
+      # TODO(user): Add elif self._encoding == 'cp437': code here.
     else:
       self._out.write(string)
 
+  def ConvertOutputToUnicode(self, buf):
+    """Converts a console output string buf to unicode.
+
+    Mainly used for testing. Allows test comparisons in unicode while ensuring
+    that unicode => encoding => unicode works.
+
+    Args:
+      buf: The console output string to convert.
+
+    Returns:
+      The console output string buf converted to unicode.
+    """
+    if isinstance(buf, unicode):
+      buf = buf.encode(self._encoding)
+    return unicode(buf, self._encoding, 'replace')
+
   def GetBoxLineCharacters(self):
     """Returns the box/line drawing characters object.
 
@@ -326,7 +319,7 @@ class ConsoleAttr(object):
   def GetControlSequenceLen(self, buf):
     """Returns the control sequence length at the beginning of buf.
 
-    Used in print width computations. Control sequences have print width 0.
+    Used in display width computations. Control sequences have display width 0.
 
     Args:
       buf: The string to check for a control sequence.
@@ -370,13 +363,14 @@ class ConsoleAttr(object):
       codes.append(self._font_italic)
     return '{csi}{codes}m'.format(csi=self._csi, codes=';'.join(codes))
 
-  def GetRawChar(self):
-    """Reads one character from stdin with no echo.
+  def GetRawKey(self):
+    """Reads one key press from stdin with no echo.
 
     Returns:
-      One raw character.
+      The key name, None for EOF, <KEY-*> for function keys, otherwise a
+      character.
     """
-    return self._get_raw_char[0]()
+    return self._get_raw_key[0]()
 
   def GetTermSize(self):
     """Returns the terminal (x, y) dimensions in characters.
@@ -386,26 +380,26 @@ class ConsoleAttr(object):
     """
     return self._term_size
 
-  def PrintWidth(self, buf):
-    """Returns the print width of buf which may contain ANSI control sequences.
+  def DisplayWidth(self, buf):
+    """Returns the display width of buf, handling unicode and ANSI controls.
 
     Args:
       buf: The string to count from.
 
     Returns:
-      The print width of buf.
+      The display width of buf, handling unicode and ANSI controls.
     """
-    if not self._csi:
+    if not isinstance(buf, basestring):
+      # Handle non-string objects like Colorizer().
       return len(buf)
     width = 0
     i = 0
     while i < len(buf):
-      csi_index = buf.find(self._csi, i)
-      if csi_index < 0:
-        width += len(buf) - i
-        break
-      width += csi_index - i
-      i = csi_index + self.GetControlSequenceLen(buf[csi_index:])
+      if self._csi and buf[i:].startswith(self._csi):
+        i += self.GetControlSequenceLen(buf[i:])
+      else:
+        width += GetCharacterDisplayWidth(buf[i])
+        i += 1
     return width
 
   def SplitIntoNormalAndControl(self, buf):
@@ -441,7 +435,7 @@ class ConsoleAttr(object):
         width.
 
     Returns:
-      A list of chunks, all but the last with print width == width.
+      A list of chunks, all but the last with display width == width.
     """
     lines = []
     chunk = ''
@@ -499,7 +493,7 @@ class Colorizer(object):
     return 0
 
   def __len__(self):
-    return len(self._string)
+    return self._con.DisplayWidth(self._string)
 
   def __str__(self):
     return self._string
@@ -514,7 +508,7 @@ class Colorizer(object):
 
 
 def GetConsoleAttr(encoding=None, out=None, reset=False):
-  """Get the console attribute state.
+  """Gets the console attribute state.
 
   If this is the first call or reset is True or encoding is not None and does
   not match the current encoding or out is not None and does not match the
@@ -526,7 +520,7 @@ def GetConsoleAttr(encoding=None, out=None, reset=False):
 
   Args:
     encoding: Encoding override.
-      ascii -- ASCII art. This is the default.
+      ascii -- ASCII. This is the default.
       utf8 -- UTF-8 unicode.
       win -- Windows code page 437.
     out: The console output file stream, ConsoleAttr default if None.
@@ -547,3 +541,114 @@ def GetConsoleAttr(encoding=None, out=None, reset=False):
     attr = ConsoleAttr(encoding=encoding, out=out)
     ConsoleAttr._CONSOLE_ATTR_STATE = attr  # pylint: disable=protected-access
   return attr
+
+
+def ResetConsoleAttr():
+  """Resets the console attribute state to the console default.
+
+  Returns:
+    The global ConsoleAttr state object.
+  """
+  return GetConsoleAttr(reset=True)
+
+
+def GetCharacterDisplayWidth(char):
+  """Returns the monospaced terminal display width of char.
+
+  Assumptions:
+    - monospaced display
+    - ambiguous or unknown chars default to width 1
+    - ASCII control char width is 1 => don't use this for control chars
+
+  Args:
+    char: The character to determine the display width of.
+
+  Returns:
+    The monospaced terminal display width of char: either 0, 1, or 2.
+  """
+  if not isinstance(char, unicode):
+    # Non-unicode chars have width 1. Don't use this function on control chars.
+    return 1
+
+  # Normalize to avoid special cases.
+  char = unicodedata.normalize('NFC', char)
+
+  if unicodedata.combining(char) != 0:
+    # Modifies the previous character and does not move the cursor.
+    return 0
+  elif unicodedata.category(char) == 'Cf':
+    # Unprintable formatting char.
+    return 0
+  elif unicodedata.east_asian_width(char) in 'FW':
+    # Fullwidth or Wide chars take 2 character positions.
+    return 2
+  else:
+    # Don't use this function on control chars.
+    return 1
+
+
+def EncodeForOutput(string, encoding=None, escape=True):
+  r"""Returns string encoded to prevent output codec exceptions.
+
+  This function can be used to massage output strings that may have been encoded
+  with an encoding incompatible with the standard output encoding. This happens
+  frequently in tests and error messages where the string is obtained from a
+  source of unknown encoding.
+
+  Args:
+    string: A string that may contain an encoding incompatible with the
+      standard output encoding.
+    encoding: The output encoding name. Defaults to
+      GetConsoleAttr().GetEncoding().
+    escape: Replace unencodable characters with a \uXXXX or \xXX equivalent if
+      True. Otherwise replace unencodable characters with an appropriate unknown
+      character, '?' for ASCII, and the unicode unknown replacement character
+      \uFFFE for unicode.
+
+  Returns:
+    The string encoded to avoid output codec exceptions.
+  """
+  if not encoding:
+    encoding = GetConsoleAttr().GetEncoding()
+  try:
+    # No change needed if the string encodes to the output encoding.
+    string.encode(encoding)
+    return string
+  except UnicodeError:
+    pass
+  try:
+    # The string is encoded using the standard output encoding. Decoding here
+    # gets the string in a form that will print to the standard output.
+    string = string.decode(encoding)
+  except UnicodeError:
+    # We don't know the string encoding.
+    try:
+      # Check if the encoding is UTF-8. If the output encoding were UTF-8 it
+      # would have been caught above.
+      string = string.decode('utf8')
+    except UnicodeError:
+      # Its not UTF-8. That's OK. The fallback below will handle it.
+      if isinstance(string, str):
+        # This branch works around a Python str.encode() "feature" that throws
+        # an ASCII *decode* exception on str strings that contain 8th bit set
+        # bytes. For example, this sequence throws an exception:
+        #   string = '\xdc'  # iso-8859-1 'Ü'
+        #   string = string.encode('ascii', 'backslashreplace')
+        # even though 'backslashreplace' is documented to handle encoding
+        # errors. We work around the problem by first decoding the str string
+        # from an 8-bit encoding to unicode, selecting any 8-bit encoding that
+        # uses all 256 bytes (such as ISO-8559-1):
+        #   string = string.decode('iso-8859-1')
+        # Using this produces a sequence that works:
+        #   string = '\xdc'
+        #   string = string.decode('iso-8859-1')
+        #   string = string.encode('ascii', 'backslashreplace')
+        try:
+          string = string.decode('iso-8859-1')
+        except UnicodeError:
+          pass
+    # Fall back to the output encoding with unencodable characters handled
+    # according to escape.
+    string = string.encode(
+        encoding, 'backslashreplace' if escape else 'replace')
+  return string
