@@ -14,7 +14,6 @@
 
 """General console printing utilities used by the Cloud SDK."""
 
-import logging
 import os
 import re
 import sys
@@ -55,209 +54,6 @@ class OperationCancelledError(Error):
     super(OperationCancelledError, self).__init__('Operation cancelled.')
 
 
-class TablePrinter(object):
-  """Provides the ability to print a list of items as a formatted table.
-
-  Using this class helps you adhere to the gcloud style guide.
-
-  The table will auto size the columns to fit the maximum item length for that
-  column.  You can also choose how to justify each column and to add extra
-  padding to each column.
-  """
-
-  JUSTIFY_LEFT = '<'
-  JUSTIFY_RIGHT = '>'
-  JUSTIFY_CENTER = '^'
-
-  def __init__(self, headers, title=None,
-               justification=None, column_padding=None):
-    """Creates a new TablePrinter.
-
-    Args:
-      headers: A tuple of strings that represent the column headers titles.
-        This can be a tuple of empty strings or None's if you do not want
-        headers displayed.  The number of empty elements in the tuple must match
-        the number of columns you want to display.
-      title: str, An optional title for the table.
-      justification: A tuple of JUSTIFY_LEFT, JUSTIFY_RIGHT, JUSTIFY_CENTER that
-        describes the justification for each column.  This must have the same
-        number of items as the headers tuple.
-      column_padding: A tuple of ints that describes the extra padding that
-        should be added to each column.  This must have the same
-        number of items as the headers tuple.
-
-    Raises:
-      ValueError: If the justification or column_padding tuples are not of the
-        correct type or length.
-    """
-    self.__headers = [h if h else '' for h in headers]
-    self.__title = title
-    self.__num_columns = len(self.__headers)
-    self.__header_widths = [len(str(x)) for x in self.__headers]
-
-    self.__column_padding = column_padding
-    if self.__column_padding is None:
-      self.__column_padding = tuple([0] * self.__num_columns)
-    if (not isinstance(self.__column_padding, (tuple)) or
-        len(self.__column_padding) != self.__num_columns):
-      raise ValueError('Column padding tuple does not have {0} columns'
-                       .format(self.__num_columns))
-
-    self.__justification = justification
-    if self.__justification is None:
-      self.__justification = tuple([TablePrinter.JUSTIFY_LEFT] *
-                                   self.__num_columns)
-    if (not isinstance(self.__justification, tuple) or
-        len(self.__justification) != self.__num_columns):
-      raise ValueError('Justification tuple does not have {0} columns'
-                       .format(self.__num_columns))
-    for value in self.__justification:
-      if not (value is TablePrinter.JUSTIFY_LEFT or
-              value is TablePrinter.JUSTIFY_RIGHT or
-              value is TablePrinter.JUSTIFY_CENTER):
-        raise ValueError('Justification values must be one of JUSTIFY_LEFT, '
-                         'JUSTIFY_RIGHT, or JUSTIFY_CENTER')
-
-  def SetTitle(self, title):
-    """Sets the title of the table.
-
-    Args:
-      title: str, The new title.
-    """
-    self.__title = title
-
-  def Log(self, rows, logger=None, level=logging.INFO):
-    """Logs the given rows to the given logger.
-
-    Args:
-      rows: list of tuples, The rows to log the formatted table for.
-      logger: logging.Logger, The logger to do the logging.  If None, the root
-        logger will be used.
-      level: logging level, An optional override for the logging level, INFO by
-        default.
-    """
-    if not logger:
-      logger = log.getLogger()
-    lines = self.GetLines(rows)
-    for line in lines:
-      logger.log(level, line)
-
-  def Print(self, rows, output_stream=None, indent=0):
-    """Prints the given rows to stdout.
-
-    Args:
-      rows: list of tuples, The rows to print the formatted table for.
-      output_stream: file-like object, The stream to wire the rows to.  Defaults
-        to log.out if not given.
-      indent: int, The number of spaces to indent all lines of the table.
-    """
-    if not output_stream:
-      output_stream = log.out
-    lines = self.GetLines(rows, indent=indent)
-    for line in lines:
-      output_stream.write(line + '\n')
-
-  def GetLines(self, rows, indent=0):
-    """Gets a list of strings of formatted lines for the given rows.
-
-    Args:
-      rows: list of tuples, The rows to get the formatted table for.
-      indent: int, The number of spaces to indent all lines of the table.
-
-    Returns:
-      list of str, The lines of the formatted table that can be printed.
-
-    Raises:
-      ValueError: If any row does not have the correct number of columns.
-    """
-    column_widths = list(self.__header_widths)
-    for row in rows:
-      if len(row) != self.__num_columns:
-        raise ValueError('Row [{row}] does not have {rows} columns'
-                         .format(row=row, rows=self.__num_columns))
-      # Find the max width of each column
-      for i in range(self.__num_columns):
-        column_widths[i] = max(column_widths[i], len(str(row[i])))
-
-    # Add padding
-    column_widths = [column_widths[i] + self.__column_padding[i]
-                     for i in range(self.__num_columns)]
-    total_width = (len(column_widths) - 1) * 3
-    for width in column_widths:
-      total_width += width
-
-    edge_line = ('--' +
-                 '---'.join(['-' * width for width in column_widths]) +
-                 '--')
-    title_divider_line = ('|-' +
-                          '---'.join(['-' * width for width in column_widths]) +
-                          '-|')
-    divider_line = ('|-' +
-                    '-+-'.join(['-' * width for width in column_widths]) +
-                    '-|')
-    lines = [edge_line]
-
-    if self.__title:
-      title_line = '| {{title:{justify}{width}s}} |'.format(
-          justify=TablePrinter.JUSTIFY_CENTER, width=total_width).format(
-              title=self.__title)
-      lines.append(title_line)
-      lines.append(title_divider_line)
-
-    # Generate format strings with the correct width for each column
-    column_formats = []
-    for i in range(self.__num_columns):
-      column_formats.append('{{i{i}:{justify}{width}s}}'.format(
-          i=i, justify=self.__justification[i], width=column_widths[i]))
-    pattern = '| ' + ' | '.join(column_formats) + ' |'
-
-    def _ParameterizedArrayDict(array):
-      return dict(('i{i}'.format(i=i), array[i]) for i in range(len(array)))
-
-    if [h for h in self.__headers if h]:
-      # Only print headers if there is at least one non-empty header
-      lines.append(pattern.format(**_ParameterizedArrayDict(self.__headers)))
-      lines.append(divider_line)
-
-    lines.extend([pattern.format(**_ParameterizedArrayDict(row))
-                  for row in rows])
-    lines.append(edge_line)
-
-    if indent:
-      return [(' ' * indent) + l for l in lines]
-    return lines
-
-
-class ListPrinter(object):
-  """Provides the ability to print a list of items as a formatted list.
-
-  Using this class helps you adhere to the gcloud style guide.
-  """
-
-  def __init__(self, title):
-    """Create a titled list printer that can print rows to stdout.
-
-    Args:
-      title: A string for the title of the list.
-    """
-    self.__title = title
-
-  def Print(self, rows, output_stream=None):
-    """Print this list with the provided rows to stdout.
-
-    Args:
-      rows: A list of objects representing the rows of this list. Before being
-          printed, they will be converted to strings.
-      output_stream: file-like object, The stream to wire the rows to.  Defaults
-        to log.out if not given.
-    """
-    if not output_stream:
-      output_stream = log.out
-    output_stream.write(self.__title + '\n')
-    for row in rows:
-      output_stream.write(' - ' + str(row) + '\n')
-
-
 TEXTWRAP = textwrap.TextWrapper(replace_whitespace=False,
                                 drop_whitespace=False,
                                 break_on_hyphens=False)
@@ -593,17 +389,20 @@ def PrintExtendedList(items, col_fetchers):
         row.append(value)
     rows.append(row)
 
+  attr = console_attr.GetConsoleAttr()
   max_col_widths = [0] * len(col_fetchers)
   for row in rows:
     for col in range(len(row)):
-      max_col_widths[col] = max(max_col_widths[col], len(unicode(row[col]))+2)
+      max_col_widths[col] = max(max_col_widths[col],
+                                attr.DisplayWidth(unicode(row[col]))+2)
 
   for row in rows:
     for col in range(len(row)):
       width = max_col_widths[col]
       item = unicode(row[col])
-      if len(item) < width and col != len(row)-1:
-        item += ' ' * (width - len(item))
+      item_width = attr.DisplayWidth(item)
+      if item_width < width and col != len(row) - 1:
+        item += u' ' * (width - item_width)
       log.out.write(item)
     log.out.write('\n')
   if not total_items:
@@ -631,6 +430,7 @@ class ProgressTracker(object):
     self._detail_message_callback = detail_message_callback
     self._last_message_size = 0
     self._tick_delay = tick_delay
+    self._is_tty = IsInteractive(output=True, error=True)
 
   def _GetPrefix(self):
     if self._detail_message_callback:
@@ -653,13 +453,20 @@ class ProgressTracker(object):
     return self
 
   def Tick(self):
-    """Give a visual indication to the user that some progress has been made."""
-    with self._lock:
-      if not self._done:
-        self._ticks += 1
-        self._Print(ProgressTracker.SPIN_MARKS[
-            self._ticks % len(ProgressTracker.SPIN_MARKS)])
-      return self._done
+    """Give a visual indication to the user that some progress has been made.
+
+    Output is sent to sys.stderr. Nothing is shown if output is not a TTY.
+
+    Returns:
+      Whether progress has completed.
+    """
+    if self._is_tty:
+      with self._lock:
+        if not self._done:
+          self._ticks += 1
+          self._Print(ProgressTracker.SPIN_MARKS[
+              self._ticks % len(ProgressTracker.SPIN_MARKS)])
+    return self._done
 
   def _Print(self, message=''):
     """Reprints the prefix followed by an optional message."""
@@ -852,15 +659,15 @@ class ProgressBar(object):
       label += ' ' * diff
     left = self._box.d_vr + self._box.d_h
     right = self._box.d_h + self._box.d_vl
-    self._label = '{left} {label} {right}'.format(left=left, label=label,
-                                                  right=right)
+    self._label = u'{left} {label} {right}'.format(left=left, label=label,
+                                                   right=right)
 
   def Start(self):
     """Starts the progress bar by writing the top rule and label."""
     if self._first or self._redraw:
       left = self._box.d_dr if self._first else self._box.d_vr
       right = self._box.d_dl if self._first else self._box.d_vl
-      rule = '{left}{middle}{right}\n'.format(
+      rule = u'{left}{middle}{right}\n'.format(
           left=left, middle=self._box.d_h * self._total_ticks, right=right)
       self._stream.write(rule)
     self._stream.write(self._label + '\n')
@@ -936,7 +743,8 @@ def More(contents, out=None, prompt=None, check_pager=True):
       if less is None:
         os.environ['LESS'] = '-R'
       p = subprocess.Popen(pager, stdin=subprocess.PIPE, shell=True)
-      p.communicate(input=contents)
+      encoding = console_attr.GetConsoleAttr().GetEncoding()
+      p.communicate(input=contents.encode(encoding))
       p.wait()
       if less is None:
         os.environ.pop('LESS')
