@@ -205,7 +205,9 @@ class _MetricsCollector(object):
         ('cd3', 'environment', properties.GetMetricsEnvironment()),
         ('cd4', 'interactive', console_io.IsInteractive(error=True,
                                                         heuristic=True)),
-        ('cd5', 'python_version', platform.python_version())]
+        ('cd5', 'python_version', platform.python_version()),
+        ('cd7', 'environment_version',
+         properties.VALUES.metrics.environment_version.Get())]
 
     self._ga_params = [
         ('v', '1'),
@@ -329,8 +331,8 @@ class _MetricsCollector(object):
       self._metrics = []
 
     # TODO(user): make this not depend on the file.
-    reporting_script_path = os.path.join(os.path.dirname(__file__),
-                                         'metrics_reporter.py')
+    reporting_script_path = os.path.realpath(
+        os.path.join(os.path.dirname(__file__), 'metrics_reporter.py'))
     execution_args = execution_utils.ArgsForPythonTool(
         reporting_script_path, temp_metrics_file.name)
 
