@@ -17,6 +17,7 @@
 import re
 
 from googlecloudsdk.core.resource import resource_exceptions
+from googlecloudsdk.core.resource import resource_filter
 from googlecloudsdk.core.resource import resource_lex
 from googlecloudsdk.core.resource import resource_projection_spec
 from googlecloudsdk.core.resource import resource_transform
@@ -43,30 +44,28 @@ class Parser(object):
 
   Attributes:
     __key_attributes_only: Parse projection key list for attributes only.
-    _ordinal: The projection key ordinal counting from 1.
     _projection: The resource_projection_spec.ProjectionSpec to parse into.
     _root: The projection _Tree tree root node.
     _snake_headings: Dict used to disambiguate key attribute labels.
     _snake_re: Compiled re for converting key names to angry snake case.
   """
 
-  _BOOLEAN_ATTRIBUTES = ['reverse']
+  _BOOLEAN_ATTRIBUTES = ['optional', 'reverse']
 
-  def __init__(self, defaults=None, symbols=None, compiler=None):
+  def __init__(self, defaults=None, symbols=None, aliases=None, compiler=None):
     """Constructor.
 
     Args:
       defaults: resource_projection_spec.ProjectionSpec defaults.
       symbols: Transform function symbol table dict indexed by function name.
+      aliases: Resource key alias dictionary.
       compiler: The projection compiler method for nested projections.
     """
     self.__key_attributes_only = False
-    self._ordinal = 0
     self._projection = resource_projection_spec.ProjectionSpec(
-        defaults=defaults, symbols=symbols, compiler=compiler)
+        defaults=defaults, symbols=symbols, aliases=aliases, compiler=compiler)
     self._snake_headings = {}
     self._snake_re = None
-    self._builtin_transforms = resource_transform
 
   class _Tree(object):
     """Defines a Projection tree node.
@@ -87,64 +86,53 @@ class Parser(object):
     formatter uses the label attribute for the column heading for the key.
 
     Attributes:
+      align: The column alignment name: left, center, or right.
       flag: The projection algorithm flag, one of DEFAULT, INNER, PROJECT.
-      ordinal: The left-to-right column order counting from 1.
+      hidden: Attribute is projected but not displayed.
+      label: A string associated with each projection key.
+      optional: Column data is optional if True.
       order: The column sort order, None if not ordered. Lower values have
         higher sort precedence.
-      label: A string associated with each projection key.
       reverse: Reverse column sort if True.
-      align: The column alignment name: left, center, or right.
-      transform: obj = func(obj,...) function applied during projection.
       subformat: Sub-format string.
+      transform: obj = func(obj,...) function applied during projection.
     """
 
     def __init__(self, flag):
-      self.flag = flag
-      self.ordinal = None
-      self.order = None
-      self.label = None
-      self.reverse = None
       self.align = resource_projection_spec.ALIGN_DEFAULT
-      self.transform = []
+      self.flag = flag
+      self.hidden = False
+      self.label = None
+      self.optional = None
+      self.order = None
+      self.reverse = None
       self.subformat = None
+      self.transform = None
 
     def __str__(self):
+      option = []
+      if self.hidden:
+        option.append('hidden')
+      if self.optional:
+        option.append('optional')
+      if self.reverse:
+        option.append('reverse')
+      if self.subformat:
+        option.append('subformat')
+      if option:
+        options = ', [{0}]'.format('|'.join(option))
+      else:
+        options = ''
       return (
-          '({flag}, {ordinal}, {order}, {label}, {align}, {active},'
-          ' {transform})'.format(
+          '({flag}, {order}, {label}, {align}, {active},'
+          ' {transform}{options})'.format(
               flag=self.flag,
-              ordinal=('DEFAULT' if self.ordinal is None
-                       else str(self.ordinal)),
               order=('UNORDERED' if self.order is None else str(self.order)),
               label=repr(self.label),
               align=self.align,
-              active=self.transform[0].active if self.transform else None,
-              transform='[{0}]'.format('.'.join(map(str, self.transform)))))
-
-  class _Transform(object):
-    """A key transform function with actual args.
-
-    Attributes:
-      name: The transform function name.
-      func: The transform function.
-      active: The parent projection active level. A transform is active if
-        transform.active is None or equal to the current projection.active.
-      map_transform: If r is a list then apply the transform to each list item.
-      args: List of function call actual arg strings.
-      kwargs: List of function call actual keyword arg strings.
-    """
-
-    def __init__(self, name, func, active, map_transform, args, kwargs):
-      self.name = name
-      self.func = func
-      self.active = active
-      self.map_transform = map_transform
-      self.args = args
-      self.kwargs = kwargs
-
-    def __str__(self):
-      return '{0}{1}({2})'.format('map().' if self.map_transform else '',
-                                  self.name, ','.join(self.args))
+              active=self.transform.active if self.transform else None,
+              transform=self.transform,
+              options=options))
 
   def _AngrySnakeCase(self, key):
     """Returns an ANGRY_SNAKE_CASE string representation of a parsed key.
@@ -194,12 +182,23 @@ class Parser(object):
 
     # Add or update the terminal node.
     tree = projection.tree
-    # self.key == [] => a function on the entire object.
+    # self.key == [] => . or a function on the entire object.
     name = key[-1] if key else ''
     name_in_tree = name in tree
     if name_in_tree:
       # Already added.
-      attribute = tree[name].attribute
+      if (not self.__key_attributes_only and
+          any([col for col in self._projection.Columns() if col.key == key])):
+        # A duplicate column. A projection can only have one attribute object
+        # per key. The first <key, attribute> pair added to the current set of
+        # columns remains in the projection. Projection columns may have
+        # duplicate keys (e.g., table columns with the same key but different
+        # transforms). The attribute copy, with attribute_add merged in, is
+        # added to the projection columns but not the projection tree.
+        attribute = copy.copy(tree[name].attribute)
+      else:
+        attribute = tree[name].attribute
+      attribute.hidden = False
     elif isinstance(name, (int, long)) and None in tree:
       # New projection for explicit name using slice defaults.
       tree[name] = copy.deepcopy(tree[None])
@@ -207,7 +206,10 @@ class Parser(object):
     else:
       # New projection.
       attribute = attribute_add
-      tree[name] = self._Tree(attribute)
+      if self.__key_attributes_only and attribute.order:
+        attribute.hidden = True
+      if key or attribute.transform:
+        tree[name] = self._Tree(attribute)
 
     # Propagate non-default values from attribute_add to attribute.
     if attribute_add.order is not None:
@@ -218,6 +220,10 @@ class Parser(object):
       attribute.label = self._AngrySnakeCase(key)
     if attribute_add.align != resource_projection_spec.ALIGN_DEFAULT:
       attribute.align = attribute_add.align
+    if attribute_add.optional is not None:
+      attribute.optional = attribute_add.optional
+    elif attribute.optional is None:
+      attribute.optional = False
     if attribute_add.reverse is not None:
       attribute.reverse = attribute_add.reverse
     elif attribute.reverse is None:
@@ -228,51 +234,14 @@ class Parser(object):
       attribute.subformat = attribute_add.subformat
     self._projection.AddAlias(attribute.label, key)
 
-    if not self.__key_attributes_only:
+    if not self.__key_attributes_only or attribute.hidden:
       # This key is in the projection.
       attribute.flag = self._projection.PROJECT
-      self._ordinal += 1
-      attribute.ordinal = self._ordinal
       self._projection.AddKey(key, attribute)
     elif not name_in_tree:
       # This is a new attributes only key.
       attribute.flag = self._projection.DEFAULT
 
-  def _ParseTransform(self, func_name, active, map_transform):
-    """Parses a transform function call.
-
-    Args:
-      func_name: The transform function name.
-      active: The transform active level or None if always active.
-      map_transform: Apply the transform to each resource list item.
-
-    Returns:
-      A _Transform call item. The caller appends these to a list that is used
-      to apply the transform functions.
-
-    Raises:
-      ExpressionSyntaxError: The expression has a syntax error.
-    """
-    here = self._lex.GetPosition()
-    if (not self._projection.symbols or
-        func_name not in self._projection.symbols):
-      raise resource_exceptions.ExpressionSyntaxError(
-          'Unknown transform function {0} [{1}].'.format(
-              func_name, self._lex.Annotate(here)))
-    kwargs = {}
-    func = self._projection.symbols[func_name]
-    if func == self._builtin_transforms.TransformFormat:
-      args = [self._projection] + self._lex.Args()
-    else:
-      args = []
-      for arg in self._lex.Args():
-        name, sep, val = arg.partition('=')
-        if sep:
-          kwargs[name] = val
-        else:
-          args.append(arg)
-    return self._Transform(func_name, func, active, map_transform, args, kwargs)
-
   def _ParseKeyAttributes(self, key, attribute):
     """Parses one or more key attributes and adds them to attribute.
 
@@ -321,6 +290,8 @@ class Parser(object):
         attribute.subformat = value or ''
       elif name == 'label':
         attribute.label = value or ''
+      elif name == 'optional':
+        attribute.optional = value
       elif name == 'reverse':
         attribute.reverse = value
       elif name == 'sort':
@@ -343,43 +314,30 @@ class Parser(object):
       The parsed key.
     """
     key = self._lex.Key()
-    here = self._lex.GetPosition()
     attribute = self._Attribute(self._projection.PROJECT)
     if self._lex.IsCharacter('(', eoi_ok=True):
       func_name = key.pop()
-      active = self._projection.active
-      map_transform = False
-      while True:
-        transform = self._ParseTransform(func_name, active, map_transform)
-        if transform.func == self._builtin_transforms.TransformAlways:
-          active = None  # Always active.
-          func_name = None
-        elif transform.func == self._builtin_transforms.TransformMap:
-          map_transform = True
-          func_name = None
-        else:
-          # always() applies to all transforms for key.
-          # map() applies to the next transform.
-          map_transform = False
-          attribute.transform.append(transform)
-        if not self._lex.IsCharacter('.', eoi_ok=True):
-          break
-        call = self._lex.Key()
-        here = self._lex.GetPosition()
-        if not self._lex.IsCharacter('('):
-          raise resource_exceptions.ExpressionSyntaxError(
-              'Transform function expected [{0}].'.format(
-                  self._lex.Annotate(here)))
-        if len(call) != 1:
-          raise resource_exceptions.ExpressionSyntaxError(
-              'Unknown transform function {0} [{1}].'.format(
-                  '.'.join(call), self._lex.Annotate(here)))
-        func_name = call.pop()
+      attribute.transform = self._lex.Transform(func_name,
+                                                self._projection.active)
+      func_name = attribute.transform.Name()
     else:
       func_name = None
     self._lex.SkipSpace()
     if self._lex.IsCharacter(':'):
       self._ParseKeyAttributes(key, attribute)
+    if attribute.transform and attribute.transform.conditional:
+      # Parse and evaluate if() transform conditional expression,
+      conditionals = self._projection.symbols.get(
+          resource_transform.GetTypeDataName('conditionals'))
+
+      def GlobalRestriction(key):
+        return getattr(conditionals, key, None)
+
+      defaults = resource_projection_spec.ProjectionSpec(
+          symbols={'global': GlobalRestriction})
+      if not resource_filter.Compile(attribute.transform.conditional,
+                                     defaults=defaults).Evaluate(conditionals):
+        return
     if func_name and attribute.label is None and not key:
       attribute.label = self._AngrySnakeCase([func_name])
     self._AddKey(key, attribute)
@@ -452,8 +410,7 @@ class Parser(object):
     self._projection.SetEmpty(
         self._Tree(self._Attribute(self._projection.PROJECT)))
     if expression:
-      self._lex = resource_lex.Lexer(expression,
-                                     aliases=self._projection.aliases)
+      self._lex = resource_lex.Lexer(expression, self._projection)
       defaults = False
       self.__key_attributes_only = False
       while self._lex.SkipSpace():
@@ -461,7 +418,6 @@ class Parser(object):
           if not self.__key_attributes_only:
             defaults = False
             self._projection.Defaults()
-            self._ordinal = 0
           self._ParseKeys()
         elif self._lex.IsCharacter('['):
           self._ParseAttributes()
@@ -482,17 +438,18 @@ class Parser(object):
     return self._projection
 
 
-def Parse(expression, defaults=None, symbols=None, compiler=None):
+def Parse(expression, defaults=None, symbols=None, aliases=None, compiler=None):
   """Parses a resource projector expression.
 
   Args:
     expression: The resource projection expression string.
     defaults: resource_projection_spec.ProjectionSpec defaults.
     symbols: Transform function symbol table dict indexed by function name.
+    aliases: Resource key alias dictionary.
     compiler: The projection compiler method for nested projections.
 
   Returns:
     A ProjectionSpec for the expression.
   """
-  return Parser(
-      defaults=defaults, symbols=symbols, compiler=compiler).Parse(expression)
+  return Parser(defaults=defaults, symbols=symbols, aliases=aliases,
+                compiler=compiler).Parse(expression)
