@@ -14,37 +14,73 @@
 
 """Resource property Get."""
 
+import json
 
-def _GetMetaData(resource, name, value, default=None):
-  """Gets the metadata dict in resource that contains {name: value, ...}.
+
+def _GetMetaDict(items, key, value):
+  """Gets the dict in items that contains key==value.
+
+  A metadict object is a list of dicts of the form:
+    [
+      {key: value-1, ...},
+      {key: value-2, ...},
+      ...
+    ]
+
+  Args:
+    items: A list of dicts.
+    key: The dict key name.
+    value: The dict key value.
+
+  Returns:
+    The dict in items that contains key==value or None if no match.
+  """
+  try:
+    for item in items:
+      if item.get(key) == value:
+        return item
+  except (AttributeError, IndexError, TypeError, ValueError):
+    pass
+  return None
+
+
+def _GetMetaDataValue(items, name, deserialize=False):
+  """Gets the metadata value for the item in items with key == name.
 
   A metadata object is a list of dicts of the form:
     [
-      {'name': key-name-1, 'field_1': field-1-value-2, ...},
-      {'name': key-name-2, 'field_1': field-1-value-2, ...},
+      {'key': key-name-1, 'value': field-1-value-string},
+      {'key': key-name-2, 'value': field-2-value-string},
       ...
     ]
-  Get() on key 'name.foo.field_1' calls _GetMetaData(resource, 'name', 'foo')
-  to find the dict in resource with 'name'=='foo' and then looks up 'field_1' in
-  that dict.
+
+  Examples:
+    x.metadata[windows-keys].email
+      Deserializes the 'windows-keys' metadata value and gets the email value.
+    x.metadata[windows-keys]
+      Gets the 'windows-key' metadata string value.
+    x.metadata[windows-keys][]
+      Gets the deserialized 'windows-key' metadata value.
 
   Args:
-    resource: The metadata resource object.
-    name: The metadata key field name.
-    value: The metadata key field value.
-    default: This value returned if not found or resource is not metadata.
+    items: The metadata items list.
+    name: The metadata name (which must match one of the 'key' values).
+    deserialize: If True then attempt to deserialize a compact JSON string.
 
   Returns:
-    The metadata dict containing {name: value, ...} or default if not found or
-    if resource is not a metadata object.
+    The metadata value for name or None if not found or if items is not a
+    metadata dict list.
   """
-  try:
-    for item in resource:
-      if item.get(name) == value:
-        return item
-  except (IndexError, TypeError):
-    pass
-  return default
+  item = _GetMetaDict(items, 'key', name)
+  if item is None:
+    return None
+  value = item.get('value', None)
+  if deserialize:
+    try:
+      return json.loads(value)
+    except (TypeError, ValueError):
+      pass
+  return value
 
 
 def Get(resource, key, default=None):
@@ -75,22 +111,18 @@ def Get(resource, key, default=None):
   """
   if isinstance(resource, set):
     resource = sorted(resource)
-  metadata = None
+  meta = None
   for i, index in enumerate(key):
-
     # This if-ladder ordering checks builtin object attributes last. For
     # example, with resource = {'items': ...}, Get() treats 'items' as a dict
     # key rather than the builtin 'items' attribute of resource.
 
-    if metadata:
-      # MetaData-like
-      resource = _GetMetaData(resource, metadata, index)
-      metadata = None
-
-    elif resource is None:
+    if resource is None:
       # None is different than an empty dict or list.
       return default
-
+    elif meta:
+      resource = _GetMetaDict(resource, meta, index)
+      meta = None
     elif hasattr(resource, 'iteritems'):
       # dict-like
       if index is None:
@@ -102,11 +134,18 @@ def Get(resource, key, default=None):
           return resource
       elif index in resource:
         resource = resource[index]
+      elif 'items' in resource:
+        # It would be nice if there were a better metadata indicator.
+        # _GetMetaDataValue() returns None if resource['items'] isn't really
+        # metadata, so there is a bit more verification than just 'items' in
+        # resource.
+        resource = _GetMetaDataValue(
+            resource['items'], index, deserialize=i + 1 < len(key))
       else:
         return default
 
     elif isinstance(index, basestring) and hasattr(resource, index):
-      # class-like -- done here to catch metadata
+      # class-like
       resource = getattr(resource, index, default)
 
     elif hasattr(resource, '__iter__') or isinstance(resource, basestring):
@@ -119,10 +158,14 @@ def Get(resource, key, default=None):
         else:
           # Trailing slice: *.[]
           return resource
-      elif isinstance(index, basestring):
-        # Try _GetMetaData() index lookup on the next iteration.
-        metadata = index
       elif not isinstance(index, (int, long)):
+        if (isinstance(index, basestring) and
+            isinstance(resource, list) and
+            len(resource) and
+            isinstance(resource[0], dict)):
+          # Let the next iteration check for a meta dict.
+          meta = index
+          continue
         # Index mismatch.
         return default
       elif index in xrange(-len(resource), len(resource)):
@@ -137,9 +180,6 @@ def Get(resource, key, default=None):
     if isinstance(resource, set):
       resource = sorted(resource)
 
-  if metadata:
-    return default
-
   return resource
 
 
