@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,10 +23,10 @@ class ResourceInfo(object):
 
   Attributes:
     async_collection: The operations collection when --async is set.
+    bypass_cache: True if cache_command output should be used instead of cache.
     collection: Memoized collection name set by Get().
     cache_command: The gcloud command string that updates the URI cache.
     list_format: The default list format string for resource_printer.Print().
-    simple_format: The --simple-list format string for resource_printer.Print().
     defaults: The resource projection transform defaults.
     transforms: Memoized combined transform symbols dict set by GetTransforms().
 
@@ -37,14 +37,14 @@ class ResourceInfo(object):
     'none': Do not print anything.
   """
 
-  def __init__(self, async_collection=None, cache_command=None,
-               list_format=None, simple_format=None, defaults=None,
+  def __init__(self, async_collection=None, bypass_cache=False,
+               cache_command=None, list_format=None, defaults=None,
                transforms=None):
     self.collection = None  # memoized by Get().
     self.async_collection = async_collection
+    self.bypass_cache = bypass_cache
     self.cache_command = cache_command
     self.list_format = list_format
-    self.simple_format = simple_format
     self.defaults = defaults
     self.transforms = transforms  # memoized by GetTransforms().
 
@@ -74,48 +74,26 @@ class ResourceInfo(object):
 
 RESOURCE_REGISTRY = {
 
-    # apheleia
-
-    'apheleia.projects.regions.functions': ResourceInfo(
-        list_format="""
-          table(
-            name,
-            status,
-            triggers.len()
-          )
-        """,
-    ),
-
     # appengine
 
-    'app.module_versions': ResourceInfo(
-        list_format="""
-          table(
-            module,
-            version,
-            format("%.2f", traffic_split)
-          )
-        """,
-    ),
-    'app.versions': ResourceInfo(
-        list_format="""
-          table(
-            service,
-            id:label=VERSION,
-            format("{0:.2f}", traffic_split):label=TRAFFIC_SPLIT,
-            last_deployed_time.date("%Y-%m-%dT%H:%M:%S"):label=LAST_DEPLOYED,
-            version.servingStatus
-          )
-        """,
-    ),
-
     'app.instances': ResourceInfo(
         list_format="""
           table(
             service:sort=1,
             version:sort=2,
             id:sort=3,
-            instance.status
+            instance.vmStatus.yesno(no="N/A"),
+            instance.vmUnlocked.yesno(yes="YES", no=""):label=DEBUG_MODE
+          )
+        """,
+    ),
+
+    'app.module_versions': ResourceInfo(
+        list_format="""
+          table(
+            module,
+            version,
+            traffic_split.format("{0:.2f}", .)
           )
         """,
     ),
@@ -129,15 +107,15 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
-    # autoscaler
-
-    'autoscaler.instances': ResourceInfo(
+    'app.versions': ResourceInfo(
         list_format="""
           table(
-            name,
-            description.yesno(no="-"),
-            state.yesno(no="-"),
-            state_details.yesno(no="-")
+            service,
+            id:label=VERSION,
+            traffic_split.format("{0:.2f}", .),
+            last_deployed_time.date("%Y-%m-%dT%H:%M:%S%Oz", undefined="-")
+              :label=LAST_DEPLOYED,
+            version.servingStatus:label=SERVING_STATUS
           )
         """,
     ),
@@ -206,19 +184,109 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    # bigtable
+
+    'bigtable.clusters.list.alpha': ResourceInfo(
+        list_format="""
+          table[box](
+            displayName:label=NAME,
+            clusterId:label=ID,
+            zoneId:label=ZONE,
+            serveNodes:label=NODES
+          )
+        """,
+    ),
+
+    'bigtable.clusters.list': ResourceInfo(
+        list_format="""
+          table(
+            name.segment(3):sort=1:label=INSTANCE,
+            name.basename():sort=2:label=NAME,
+            location.basename():label=ZONE,
+            serveNodes:label=NODES,
+            defaultStorageType:label=STORAGE,
+            state
+          )
+        """,
+    ),
+
+    'bigtable.instances.list': ResourceInfo(
+        list_format="""
+          table(
+            name.basename():sort=1,
+            displayName,
+            state
+          )
+        """,
+    ),
+
+    # cloud billing
+
+    'cloudbilling.billingAccounts': ResourceInfo(
+        cache_command='billing accounts list',
+        # TODO(b/22402915) Delete this when OP resource completion is supported.
+        bypass_cache=True,
+        list_format="""
+          table(
+            name.basename():label=ID,
+            displayName:label=NAME,
+            open
+          )
+        """,
+    ),
+
+    'cloudbilling.projectBillingInfo': ResourceInfo(
+        list_format="""
+          table(
+            projectId,
+            billingAccountName.basename():label=BILLING_ACCOUNT_ID,
+            billingEnabled
+          )
+        """,
+    ),
+
+    # cloud build
+
+    # TODO(user): use durations instead of finishTime with cl/123214184.
+    'cloudbuild.projects.builds': ResourceInfo(
+        cache_command='cloud build list',
+        bypass_cache=True,
+        async_collection='cloudbuild.projects.builds',
+        list_format="""
+          table(
+            id,
+            createTime.date('%Y-%m-%dT%H:%M:%S%Oz', undefined='-'),
+            startTime.date('%Y-%m-%dT%H:%M:%S%Oz', undefined='-'),
+            finishTime.date('%Y-%m-%dT%H:%M:%S%Oz', undefined='-'),
+            status
+          )
+        """,
+    ),
+
     # cloud resource manager
 
     'cloudresourcemanager.projects': ResourceInfo(
         cache_command='projects list',
         list_format="""
           table(
-            projectId,
+            projectId:sort=101,
             name,
             projectNumber
           )
         """,
     ),
 
+    'cloudresourcemanager.organizations': ResourceInfo(
+        cache_command='organizations list',
+        list_format="""
+          table(
+            displayName,
+            organizationId:sort=101,
+            owner.directoryCustomerId
+          )
+        """,
+    ),
+
     # Cloud SDK client side resources
 
     # 'coudsdk.*': ...
@@ -250,7 +318,6 @@ RESOURCE_REGISTRY = {
     ),
 
     'compute.backendBuckets': ResourceInfo(
-        cache_command='compute backend-buckets list',
         list_format="""
           table(
             name,
@@ -259,7 +326,13 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
-    'compute.backendService': ResourceInfo(
+    'compute.backendServiceGroupHealth': ResourceInfo(
+        list_format="""
+          default
+        """,
+    ),
+
+    'compute.backendServices': ResourceInfo(
         cache_command='compute backend-services list',
         list_format="""
           table(
@@ -270,6 +343,32 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'compute.backendServices.alpha': ResourceInfo(
+        cache_command='compute backend-services list',
+        list_format="""
+          table(
+            name,
+            backends[].group.list():label=BACKENDS,
+            protocol,
+            loadBalancingScheme,
+            healthChecks.map().basename().list()
+          )
+        """,
+    ),
+
+    'compute.regionBackendServices': ResourceInfo(
+        cache_command='compute backend-services list',
+        list_format="""
+          table(
+            name,
+            backends[].group.list():label=BACKENDS,
+            protocol,
+            loadBalancingScheme,
+            healthChecks.map().basename().list()
+          )
+        """,
+    ),
+
     'compute.disks': ResourceInfo(
         cache_command='compute disks list',
         list_format="""
@@ -332,6 +431,16 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'compute.healthChecks': ResourceInfo(
+        cache_command='compute health-checks list',
+        list_format="""
+          table(
+            name,
+            type:label=PROTOCOL
+          )
+        """,
+    ),
+
     'compute.httpHealthChecks': ResourceInfo(
         cache_command='compute http-health-checks list',
         list_format="""
@@ -362,7 +471,7 @@ RESOURCE_REGISTRY = {
           table(
             name,
             selfLink.map().scope(projects).segment(0):label=PROJECT,
-            image_alias():label=ALIAS,
+            family,
             deprecated.state:label=DEPRECATED,
             status
           )
@@ -374,7 +483,8 @@ RESOURCE_REGISTRY = {
         list_format="""
           table(
             name,
-            zone.basename(),
+            location():label=LOCATION,
+            location_scope():label=SCOPE,
             network.basename(),
             isManaged:label=MANAGED,
             size:label=INSTANCES
@@ -387,10 +497,11 @@ RESOURCE_REGISTRY = {
         list_format="""
           table(
             name,
-            zone.basename(),
+            location():label=LOCATION,
+            location_scope():label=SCOPE,
             baseInstanceName,
             size,
-            instanceGroup.basename():label=GROUP,
+            targetSize,
             instanceTemplate.basename(),
             autoscaled
           )
@@ -403,7 +514,7 @@ RESOURCE_REGISTRY = {
           table(
             name,
             zone.basename(),
-            machineType.basename(),
+            machineType.machine_type(),
             scheduling.preemptible.yesno(yes=true, no=''),
             networkInterfaces[0].networkIP:label=INTERNAL_IP,
             networkInterfaces[0].accessConfigs[0].natIP:label=EXTERNAL_IP,
@@ -417,13 +528,25 @@ RESOURCE_REGISTRY = {
         list_format="""
           table(
             name,
-            properties.machineType,
+            properties.machineType.machine_type(),
             properties.scheduling.preemptible.yesno(yes=true, no=''),
             creationTimestamp
           )
         """,
     ),
 
+    'compute.invalidations': ResourceInfo(
+        cache_command='beta compute url-maps list-cdn-cache-invalidations',
+        list_format="""
+          table(
+            description,
+            operation_http_status():label=HTTP_STATUS,
+            status,
+            insertTime:label=TIMESTAMP
+          )
+        """,
+    ),
+
     'compute.machineTypes': ResourceInfo(
         cache_command='compute machine-types list',
         list_format="""
@@ -431,7 +554,7 @@ RESOURCE_REGISTRY = {
             name,
             zone.basename(),
             guestCpus:label=CPUS,
-            memoryMb.size(units_in=MiB, units_out=GiB):label=MEMORY_GB,
+            memoryMb.size(units_in=MiB, units_out=GiB, precision=2):label=MEMORY_GB,
             deprecated.state:label=DEPRECATED
           )
         """,
@@ -442,6 +565,7 @@ RESOURCE_REGISTRY = {
         list_format="""
           table(
             name,
+            x_gcloud_mode:label=MODE,
             IPv4Range:label=IPV4_RANGE,
             gatewayIPv4
           )
@@ -484,37 +608,13 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
-    'compute.replicaPools': ResourceInfo(
+    'compute.routers': ResourceInfo(
+        cache_command='compute routers list',
         list_format="""
           table(
             name,
-            currentNumReplicas
-          )
-        """,
-    ),
-
-    'compute.replicaPoolsReplicas': ResourceInfo(
-        list_format="""
-          table(
-            name,
-            status.templateVersion,
-            status.state:label=STATUS
-          )
-        """,
-    ),
-
-    'compute.resourceViews': ResourceInfo(
-        list_format="""
-          value(
-            uri()
-          )
-        """,
-    ),
-
-    'compute.resourceViewsResources': ResourceInfo(
-        list_format="""
-          value(
-            uri()
+            region.basename(),
+            network.basename()
           )
         """,
     ),
@@ -526,7 +626,11 @@ RESOURCE_REGISTRY = {
             name,
             network.basename(),
             destRange,
-            firstof(nextHopInstance, nextHopGateway, nextHopIp).scope()
+            firstof(
+                nextHopInstance,
+                nextHopGateway,
+                nextHopIp,
+                nextHopVpnTunnel).scope()
               :label=NEXT_HOP,
             priority
           )
@@ -534,7 +638,7 @@ RESOURCE_REGISTRY = {
     ),
 
     'compute.snapshots': ResourceInfo(
-        cache_command='compute snapshts list',
+        cache_command='compute snapshots list',
         list_format="""
           table(
             name,
@@ -555,6 +659,18 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'compute.subnetworks': ResourceInfo(
+        cache_command='compute networks subnets list',
+        list_format="""
+          table(
+            name,
+            region.basename(),
+            network.basename(),
+            ipCidrRange:label=RANGE
+          )
+        """,
+    ),
+
     'compute.targetHttpProxies': ResourceInfo(
         cache_command='compute target-http-proxies list',
         list_format="""
@@ -588,8 +704,14 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'compute.targetPoolInstanceHealth': ResourceInfo(
+        list_format="""
+          default
+        """,
+    ),
+
     'compute.targetPools': ResourceInfo(
-        cache_command='compute pools list',
+        cache_command='compute target-pools list',
         list_format="""
           table(
             name,
@@ -601,8 +723,20 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'compute.targetSslProxies': ResourceInfo(
+        cache_command='compute target-ssl-proxies list',
+        list_format="""
+          table(
+            name,
+            proxyHeader,
+            service.basename(),
+            sslCertificates.map().basename().list():label=SSL_CERTIFICATES
+          )
+        """,
+    ),
+
     'compute.targetVpnGateways': ResourceInfo(
-        cache_command='compute vpn-gateways list',
+        cache_command='compute target-vpn-gateways list',
         list_format="""
           table(
             name,
@@ -617,7 +751,7 @@ RESOURCE_REGISTRY = {
         list_format="""
           table(
             name,
-            defaultService.basename()
+            defaultService
           )
         """,
     ),
@@ -659,31 +793,59 @@ RESOURCE_REGISTRY = {
     ),
 
     # container
+    'container.images': ResourceInfo(
+        list_format="""
+          table(
+            name
+          )
+        """,
+    ),
+
+    'container.tags': ResourceInfo(
+        list_format="""
+          table(
+            digest.slice(7:19).join(''),
+            tags.list(),
+            timestamp.date()
+          )
+        """,
+    ),
 
     'container.projects.zones.clusters': ResourceInfo(
         list_format="""
           table(
             name,
             zone,
-            clusterApiVersion,
+            currentMasterVersion:label=MASTER_VERSION,
             endpoint:label=MASTER_IP,
-            machineType,
-            sourceImage,
-            numNodes:label=NODES,
+            nodePools[0].config.machineType,
+            currentNodeVersion:label=NODE_VERSION,
+            currentNodeCount:label=NUM_NODES,
             status
           )
         """,
     ),
 
+    'container.projects.zones.clusters.nodePools': ResourceInfo(
+        list_format="""
+          table(
+            name,
+            config.machineType,
+            config.diskSizeGb,
+            version:label=NODE_VERSION
+          )
+        """,
+    ),
+
     'container.projects.zones.operations': ResourceInfo(
         list_format="""
           table(
             name,
             operationType:label=TYPE,
             zone,
-            target,
-            status,
-            errorMessage
+            targetLink.basename():label=TARGET,
+            statusMessage,
+            status
           )
         """,
     ),
@@ -702,6 +864,17 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'dataflow.logs': ResourceInfo(
+        list_format="""
+          table[no-heading,pad=1](
+            messageImportance.enum(dataflow.JobMessage),
+            time.date(tz=LOCAL):label=TIME,
+            id,
+            messageText:label=TEXT
+          )
+        """,
+    ),
+
     # dataproc
 
     'dataproc.clusters': ResourceInfo(
@@ -710,7 +883,7 @@ RESOURCE_REGISTRY = {
             clusterName:label=NAME,
             configuration.numWorkers:label=WORKER_COUNT,
             status.state:label=STATUS,
-            configuration.gceClusterConfiguration.zoneUri.zone()
+            configuration.gceClusterConfiguration.zoneUri.scope(zone)
           )
         """,
     ),
@@ -735,10 +908,74 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    # debug
+
+    'debug.logpoints': ResourceInfo(
+        list_format="""
+          table(
+            short_status():label=STATUS,
+            userEmail.if(all_users),
+            location,
+            condition,
+            logLevel,
+            logMessageFormat,
+            id)
+            :(isFinalState:sort=101, createTime:sort=102)
+        """,
+    ),
+
+    'debug.logpoints.create': ResourceInfo(
+        list_format="""
+          list(
+            format("id: {0}", id),
+            format("location: {0}", location),
+            format("logLevel: {0}", logLevel),
+            format("logMessageFormat: {0}", logMessageFormat),
+            format("condition: {0}", condition),
+            format("logViewUrl: {0}", logViewUrl),
+            format("status: {0}", full_status())
+          )
+        """,
+    ),
+
+    'debug.snapshots': ResourceInfo(
+        list_format="""
+          table(
+            short_status():label=STATUS,
+            userEmail.if(all_users),
+            location,
+            condition,
+            finalTime.if(include_inactive != 0):label=COMPLETED_TIME,
+            id,
+            consoleViewUrl:label=VIEW
+          )
+        """
+    ),
+
+    'debug.snapshots.create': ResourceInfo(
+        list_format="""
+          list(
+            format("id: {0}", id),
+            format("location: {0}", location),
+            format("status: {0}", full_status()),
+            format("consoleViewUrl: {0}", consoleViewUrl)
+          )
+        """
+    ),
+
+    'debug.targets': ResourceInfo(
+        list_format="""
+          table(
+            name,
+            target_id:label=ID,
+            description
+          )
+        """
+    ),
+
     # deployment manager v2
 
     'deploymentmanager.deployments': ResourceInfo(
-        async_collection='deployments.operations',
         list_format="""
           table(
             name,
@@ -746,13 +983,7 @@ RESOURCE_REGISTRY = {
             operation.status,
             description,
             manifest.basename(),
-            update.errors.group(code, message)
-          )
-        """,
-        simple_format="""
-          [log=status,empty-legend="No Deployments were found in your project!"]
-          value(
-            name
+            operation.error.errors.group(code, message)
           )
         """,
     ),
@@ -767,28 +998,32 @@ RESOURCE_REGISTRY = {
             error.errors.group(code, message)
           )
         """,
-        simple_format="""
-          [log=status,empty-legend="No Operations were found in your project!"]
-          value(
-            name
+    ),
+
+    'deploymentmanager.resources': ResourceInfo(
+        async_collection='deploymentmanager.operations',
+        list_format="""
+          table(
+            name,
+            type,
+            update.state.yesno(no="COMPLETED"),
+            update.error.errors.group(code, message)
           )
         """,
     ),
 
-    'deploymentmanager.resources': ResourceInfo(
+    'deploymentmanager.resources_and_outputs': ResourceInfo(
+        async_collection='deploymentmanager.operations',
         list_format="""
           table(
-            name,
-            operationType,
-            status.yesno(no="COMPLETED"):label=UPDATE_STATE,
-            update.error.errors.group(code, message)
-          )
-        """,
-        simple_format="""
-          [log=status,
-           empty-legend="No Resources were found in your deployment!"]
-          value(
-            name
+            resources:format='table(
+              name,
+              type,
+              update.state.yesno(no="COMPLETED"),
+              update.error.errors.group(code, message))',
+            outputs:format='table(
+              name:label=OUTPUTS,
+              finalValue:label=VALUE)'
           )
         """,
     ),
@@ -827,8 +1062,42 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    # functions
+
+    'functions.projects.regions.functions': ResourceInfo(
+        list_format="""
+          table(
+            name.basename(),
+            status,
+            triggers.len()
+          )
+        """,
+    ),
+
     # genomics
 
+    'genomics.alignments': ResourceInfo(
+        list_format="""
+          table(
+            alignment.position.referenceName,
+            alignment.position.position,
+            alignment.position.reverseStrand,
+            fragmentName,
+            alignedSequence:label=SEQUENCE
+          )
+        """,
+    ),
+
+    'genomics.callSets': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            name,
+            variantSetIds.list()
+          )
+        """,
+    ),
+
     'genomics.datasets': ResourceInfo(
         list_format="""
           table(
@@ -838,21 +1107,88 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    'genomics.readGroupSets': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            name,
+            referenceSetId
+          )
+        """,
+    ),
+
+    'genomics.references': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            name,
+            length,
+            sourceUri,
+            sourceAccessions.list():label=ACCESSIONS
+          )
+        """,
+    ),
+
+    'genomics.referenceSets': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            assemblyId,
+            sourceAccessions.list()
+          )
+        """,
+    ),
+
+    'genomics.variants': ResourceInfo(
+        list_format="""
+          table(
+            variantSetId,
+            referenceName,
+            start,
+            end,
+            referenceBases,
+            alternateBases
+          )
+        """,
+    ),
+
+    'genomics.variantsets': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            name,
+            description
+          )
+        """,
+    ),
+
+    # iam
+
+    'iam.service_accounts': ResourceInfo(
+        list_format="""
+          table(
+            displayName:label=NAME,
+            email
+          )
+        """,
+    ),
+
+    'iam.service_accounts.keys': ResourceInfo(
+        list_format="""
+          table(
+            name.scope(keys):label=KEY_ID,
+            validAfterTime:label=CREATED_AT,
+            validBeforeTime:label=EXPIRES_AT
+          )
+        """,
+    ),
+
     # logging
 
     'logging.logs': ResourceInfo(
         list_format="""
           table(
-            name
-          )
-        """,
-    ),
-
-    'logging.sinks': ResourceInfo(
-        list_format="""
-          table(
-            name,
-            destination
+            name.scope(logs):label=ID
           )
         """,
     ),
@@ -867,12 +1203,24 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
-    'logging.typedSinks': ResourceInfo(
+    'logging.resourceDescriptors': ResourceInfo(
+        list_format="""
+          table(
+            type,
+            description,
+            labels[].key.list()
+          )
+        """,
+    ),
+
+    'logging.sinks': ResourceInfo(
         list_format="""
           table(
             name,
             destination,
-            type
+            type,
+            format,
+            filter
           )
         """,
     ),
@@ -889,6 +1237,102 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
+    # pubsub
+
+    'pubsub.projects.topics': ResourceInfo(
+        list_format="""
+          table[box](
+            topicId:label=TOPIC,
+            success:label=SUCCESS,
+            reason:label=REASON
+          )
+        """,
+    ),
+
+    'pubsub.topics.publish': ResourceInfo(
+        list_format="""
+          table[box](
+            messageIds:label=MESSAGE_ID,
+          )
+        """,
+    ),
+
+    'pubsub.pull': ResourceInfo(
+        list_format="""
+          table[box](
+            message.data.decode(base64),
+            message.messageId,
+            message.attributes.list(separator=' '),
+            ackId.if(NOT auto_ack)
+          )
+        """,
+    ),
+
+    'pubsub.subscriptions': ResourceInfo(
+        list_format="""
+          table[box](
+            projectId:label=PROJECT,
+            subscriptionId:label=SUBSCRIPTION,
+            topicId:label=TOPIC,
+            type,
+            ackDeadlineSeconds:label=ACK_DEADLINE
+          )
+        """,
+    ),
+
+    'replicapoolupdater.rollingUpdates': ResourceInfo(
+        list_format="""
+          table(
+            id,
+            instanceGroupManager.basename():label=GROUP_NAME,
+            instanceTemplate.basename():label=TEMPLATE_NAME,
+            status,
+            statusMessage
+          )
+        """,
+    ),
+
+    'replicapoolupdater.rollingUpdates.instanceUpdates': ResourceInfo(
+        list_format="""
+          table(
+            instance.basename():label=INSTANCE_NAME,
+            status
+          )
+        """,
+    ),
+
+    # runtime config
+
+    'runtimeconfig.configurations': ResourceInfo(
+        list_format="""
+          table(
+            name,
+            description
+          )
+        """,
+    ),
+
+    'runtimeconfig.variables': ResourceInfo(
+        list_format="""
+          table(
+            name,
+            updateTime.date()
+          )
+        """,
+    ),
+
+    'runtimeconfig.waiters': ResourceInfo(
+        async_collection='runtimeconfig.waiters',
+        list_format="""
+          table(
+            name,
+            createTime.date(),
+            waiter_status(),
+            error.message
+          )
+        """,
+    ),
+
     # service management (inception)
 
     'servicemanagement-v1.services': ResourceInfo(
@@ -898,19 +1342,49 @@ RESOURCE_REGISTRY = {
             serviceConfig.title
           )
         """,
-        simple_format="""
-          value(
-            serviceName
+    ),
+
+    # service registry
+
+    'service_registry.endpoints': ResourceInfo(
+        async_collection='service_registry.operations',
+        list_format="""
+          table[box](
+            name,
+            state,
+            addresses[].map().endpoint_address().list(separator=' | '):label=ADDRESSES
+          )
+        """,
+    ),
+
+    'service_registry.operations': ResourceInfo(
+        list_format="""
+          table(
+            name,
+            operationType:label=TYPE,
+            status,
+            targetLink.basename():label=TARGET,
+            insertTime.date(format="%Y-%m-%d"):label=DATE,
+            error.errors.group(code, message)
           )
         """,
     ),
 
     # source
 
-    'source.jobs.list': ResourceInfo(
+    'source.captures': ResourceInfo(
         list_format="""
           table(
-            name.YesNo(no="default"):label=REPO_NAME,
+            project_id,
+            id:label=CAPTURE_ID
+          )
+        """,
+    ),
+
+    'source.jobs': ResourceInfo(
+        list_format="""
+          table(
+            name.yesno(no="default"):label=REPO_NAME,
             projectId,
             vcs,
             state,
@@ -981,6 +1455,7 @@ RESOURCE_REGISTRY = {
     ),
 
     'sql.operations': ResourceInfo(
+        async_collection='default',
         list_format="""
           table(
             operation,
@@ -994,6 +1469,7 @@ RESOURCE_REGISTRY = {
     ),
 
     'sql.operations.v1beta4': ResourceInfo(
+        async_collection='default',
         list_format="""
           table(
             name,
@@ -1001,7 +1477,7 @@ RESOURCE_REGISTRY = {
             startTime.iso():label=START,
             endTime.iso():label=END,
             error[0].code.yesno(no="-"):label=ERROR,
-            state:label=STATUS
+            status:label=STATUS
           )
         """,
     ),
@@ -1036,7 +1512,7 @@ RESOURCE_REGISTRY = {
             id:label=DEVICE_ID,
             manufacturer:label=MAKE,
             name:label=MODEL_NAME,
-            form.color(blue=VIRTUAL,yellow=PHYSICAL),
+            form.color(blue=VIRTUAL,yellow=PHYSICAL):label=FORM,
             format("{0:4} x {1}", screenY, screenX):label=RESOLUTION,
             supportedVersionIds.list("none"):label=OS_VERSION_IDS,
             tags.list().color(green=default,red=deprecated,yellow=preview)
@@ -1101,31 +1577,20 @@ RESOURCE_REGISTRY = {
         """,
     ),
 
-    # updater
+    # special IAM roles completion case
 
-    'replicapoolupdater.rollingUpdates': ResourceInfo(
-        list_format="""
-          table(
-            id,
-            instanceGroupManager.basename():label=GROUP_NAME,
-            instanceTemplate.basename("-"):label=TEMPLATE_NAME,
-            status,
-            statusMessage
-          )
-        """,
-    ),
-
-    'replicapoolupdater.rollingUpdates.instanceUpdates': ResourceInfo(
-        list_format="""
-          table(
-            instance.basename():label=INSTANCE_NAME,
-            status
-          )
-        """,
+    'iam.roles': ResourceInfo(
+        bypass_cache=True,
     ),
 
     # generic
 
+    'default': ResourceInfo(
+        list_format="""
+          default
+        """,
+    ),
+
     'uri': ResourceInfo(
         list_format="""
           table(
