@@ -25,24 +25,30 @@ Pythonicness of the Transform*() methods:
   (1) The docstrings are used to generate external user documentation.
   (2) The method prototypes are included in the documentation. In particular the
       prototype formal parameter names are stylized for the documentation.
-  (3) The types of some args, like r, are not fixed until runtime. Other args
+  (3) The 'r', 'kwargs', and 'projection' args are not included in the external
+      documentation. Docstring descriptions, other than the Args: line for the
+      arg itself, should not mention these args. Assume the reader knows the
+      specific item the transform is being applied to. When in doubt refer to
+      the output of $ gcloud topic projections.
+  (4) The types of some args, like r, are not fixed until runtime. Other args
       may have either a base type value or string representation of that type.
       It is up to the transform implementation to silently do the string=>type
       conversions. That's why you may see e.g. int(arg) in some of the methods.
-  (4) Unless it is documented to do so, a transform function must not raise any
+  (5) Unless it is documented to do so, a transform function must not raise any
       exceptions related to the resource r. The `undefined' arg is used to
       handle all unusual conditions, including ones that would raise exceptions.
       Exceptions for arguments explicitly under the caller's control are OK.
 """
 
-import cStringIO
 import datetime
 import re
+import StringIO
+import urllib2
 
 from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.resource import resource_exceptions
 from googlecloudsdk.core.resource import resource_property
-from googlecloudsdk.core.util import timezone
+from googlecloudsdk.core.util import times
 
 
 def TransformAlways(r):
@@ -66,18 +72,18 @@ def TransformAlways(r):
 
 
 def TransformBaseName(r, undefined=''):
-  """Returns the last path component in r.
+  """Returns the last path component.
 
   Args:
     r: A URI or unix/windows file path.
-    undefined: This value is returned if r or the basename is empty.
+    undefined: Returns this value if the resource or basename is empty.
 
   Returns:
     The last path component.
   """
   if not r:
     return undefined
-  s = str(r)
+  s = unicode(r)
   for separator in ('/', '\\'):
     i = s.rfind(separator)
     if i >= 0:
@@ -85,10 +91,24 @@ def TransformBaseName(r, undefined=''):
   return s or undefined
 
 
-def TransformColor(r, red=None, yellow=None, green=None, blue=None, **kwargs):
-  """Colorizes the string value of r.
+def TransformCollection(r, undefined=''):  # pylint: disable=unused-argument
+  """Returns the current resource collection.
 
-  The resource string is searched for an re pattern match in Roy.G.Biv order.
+  Args:
+    r: A JSON-serializable object.
+    undefined: This value is returned if r or the collection is empty.
+
+  Returns:
+    The current resource collection, undefined if unknown.
+  """
+  # This method will most likely be overridden by a resource printer.
+  return undefined
+
+
+def TransformColor(r, red=None, yellow=None, green=None, blue=None, **kwargs):
+  """Colorizes the resource string value.
+
+  The resource string is searched for an RE pattern match in Roy.G.Biv order.
   The first pattern that matches colorizes the resource string with that color.
 
   Args:
@@ -103,7 +123,7 @@ def TransformColor(r, red=None, yellow=None, green=None, blue=None, **kwargs):
     A console_attr.Colorizer() object if any color substring matches, r
     otherwise.
   """
-  string = str(r)
+  string = unicode(r)
   for color, pattern in (('red', red), ('yellow', yellow), ('green', green),
                          ('blue', blue)):
     if pattern and re.search(pattern, string):
@@ -112,31 +132,54 @@ def TransformColor(r, red=None, yellow=None, green=None, blue=None, **kwargs):
 
 
 # pylint: disable=redefined-builtin, external expression expects format kwarg.
-def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='', tz=None):
-  """Formats r to a strftime() format.
+def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='',
+                  tz=None, tz_default=None):
+  """Formats the resource as a strftime() format.
 
   Args:
     r: A timestamp number or an object with 3 or more of these fields: year,
       month, day, hour, minute, second, millisecond, microsecond, nanosecond.
     format: The strftime(3) format.
-    unit: Timestamp r divided by unit yields seconds.
-    undefined: Returns this if r does not contain a valid time.
-    tz: Fixed timezone string, local timezone if None. For example, EST5EDT,
-      US/Pacific, UTC, WEST.
+    unit: If the resource is a Timestamp then divide by _unit_ to yield seconds.
+    undefined: Returns this value if the resource is not a valid time.
+    tz: Return the time relative to the tz timezone if specified, the explicit
+      timezone in the resource if it has one, otherwise the local timezone.
+      For example, ...date(tz=EST5EDT, tz_default=UTC).
+    tz_default: The default timezone if the resource does not have a timezone
+      suffix.
 
   Returns:
     The strftime() date format for r or undefined if r does not contain a valid
     time.
   """
-  tz = timezone.GetTimeZone(tz)
+  # Check if r has an isoformat() method.
+  try:
+    r = r.isoformat()
+  except (AttributeError, TypeError, ValueError):
+    pass
+
+  tz_in = times.GetTimeZone(tz_default) if tz_default else None
   # Check if r is a timestamp.
   try:
     timestamp = float(r) / float(unit)
-    dt = datetime.datetime.fromtimestamp(timestamp, tz)
-    return dt.strftime(format)
+    dt = times.GetDateTimeFromTimeStamp(timestamp, tz_in)
+    return times.FormatDateTime(dt, format)
   except (TypeError, ValueError):
     pass
 
+  # Check if r is a serialized datetime object.
+  original_repr = resource_property.Get(r, ['datetime'], None)
+  if original_repr and isinstance(original_repr, basestring):
+    r = original_repr
+
+  tz_out = times.GetTimeZone(tz) if tz else None
+  # Check if r is a date/time string.
+  try:
+    dt = times.ParseDateTime(r, tz_in)
+    return times.FormatDateTime(dt, format, tz_out)
+  except (AttributeError, ImportError, TypeError, ValueError):
+    pass
+
   def _FormatFromParts():
     """Returns the formatted time from broken down time parts in r.
 
@@ -149,12 +192,12 @@ def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='', tz=None):
     """
     valid = 0
     parts = []
-    now = datetime.datetime.now(tz)
+    now = datetime.datetime.now(tz_in)
     for part in ('year', 'month', 'day', 'hour', 'minute', 'second'):
       value = resource_property.Get(r, [part], None)
       if value is None:
         # Missing parts default to now.
-        value = getattr(now, part)
+        value = getattr(now, part, 0)
       else:
         valid += 1
       parts.append(int(value))
@@ -169,9 +212,9 @@ def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='', tz=None):
     # combination of 3 non-subsecond date/time parts.
     if valid < 3:
       raise ValueError
-    parts.append(tz)
+    parts.append(tz_in)
     dt = datetime.datetime(*parts)
-    return dt.strftime(format)
+    return times.FormatDateTime(dt, format, tz_out)
 
   try:
     return _FormatFromParts()
@@ -182,13 +225,33 @@ def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='', tz=None):
   return undefined
 
 
-def TransformDuration(r, unit=1, undefined=''):
-  """Formats r to a duration string.
+def TransformDecode(r, encoding, undefined=''):
+  """Returns the decoded value of the resource that was encoded by encoding.
 
   Args:
     r: A JSON-serializable object.
-    unit: r divided by unit yields seconds.
-    undefined: Returns this if r/unit is not a valid timestamp.
+    encoding: The encoding name. *base64* and *utf-8* are supported.
+    undefined: Returns this value if the decoding fails.
+
+  Returns:
+    The decoded resource.
+  """
+  # Some codecs support 'replace', all support 'strict' (the default).
+  for errors in ('replace', 'strict'):
+    try:
+      return r.decode(encoding, errors)
+    except:  # pylint: disable=bare-except, undefined for any exception
+      pass
+  return undefined
+
+
+def TransformDuration(r, unit=1, undefined=''):
+  """Formats the resource as a duration string.
+
+  Args:
+    r: A JSON-serializable object.
+    unit: Divide the resource numeric value by _unit_ to yield seconds.
+    undefined: Returns this value if the resource is not a valid timestamp.
 
   Returns:
     The duration string for r or undefined if r is not a duration.
@@ -196,22 +259,88 @@ def TransformDuration(r, unit=1, undefined=''):
   try:
     timestamp = float(r) / unit
     d = datetime.timedelta(seconds=timestamp)
-    return str(d).replace(' ', '')
+    return unicode(d).replace(' ', '')
   except (TypeError, ValueError):
     return undefined
 
 
+def TransformEncode(r, encoding, undefined=''):
+  """Returns the encoded value of the resource using encoding.
+
+  Args:
+    r: A JSON-serializable object.
+    encoding: The encoding name. *base64* and *utf-8* are supported.
+    undefined: Returns this value if the encoding fails.
+
+  Returns:
+    The encoded resource.
+  """
+  # Some codecs support 'replace', all support 'strict' (the default).
+  for errors in ('replace', 'strict'):
+    try:
+      return r.encode(encoding, errors).rstrip('\n')
+    except:  # pylint: disable=bare-except, undefined for any exception
+      pass
+  return undefined
+
+
+def TransformEnum(r, projection, enums, inverse=False, undefined=''):
+  """Returns the enums dictionary description for the resource.
+
+  Args:
+    r: A JSON-serializable object.
+    projection: The parent ProjectionSpec.
+    enums: The name of a message enum dictionary.
+    inverse: Do inverse lookup.
+    undefined: Returns this value if there is no matching enum description.
+
+  Returns:
+    The enums dictionary description for the resource.
+  """
+  type_name = GetTypeDataName(enums, 'inverse-enum' if inverse else 'enum')
+  descriptions = projection.symbols.get(type_name)
+  if not descriptions and inverse:
+    normal = projection.symbols.get(GetTypeDataName(enums, 'enum'))
+    if normal:
+      # Create the inverse dict and memoize it in projection.symbols.
+      descriptions = {}
+      for k, v in normal.iteritems():
+        descriptions[v] = k
+      projection.symbols[type_name] = descriptions
+  return descriptions.get(r, undefined) if descriptions else undefined
+
+
 def TransformError(r, message=None):
   """Raises an Error exception that does not generate a stack trace.
 
   Args:
     r: A JSON-serializable object.
-    message: An error message. If None then r is formatted as the error message.
+    message: An error message. If not specified then the resource is formatted
+      as the error message.
 
   Raises:
     Error: This will not generate a stack trace.
   """
-  raise resource_exceptions.Error(message if message is not None else str(r))
+  if message is None:
+    message = unicode(r)
+  raise resource_exceptions.Error(message)
+
+
+def TransformExtract(r, *keys):
+  """Extract an ordered list of values from the resource for the specified keys.
+
+  Args:
+    r: A JSON-serializable object.
+    *keys: The list of keys in the resource whose associated values will be
+        included in the result.
+
+  Returns:
+    The list of extracted values.
+  """
+  try:
+    return [r[k] for k in keys if k in r]
+  except TypeError:
+    return []
 
 
 def TransformFatal(r, message=None):
@@ -219,7 +348,8 @@ def TransformFatal(r, message=None):
 
   Args:
     r: A JSON-serializable object.
-    message: An error message. If None then r is formatted as the error message.
+    message: An error message. If not specified then the resource is formatted
+      as the error message.
 
   Raises:
     InternalError: This generates a stack trace.
@@ -229,11 +359,11 @@ def TransformFatal(r, message=None):
 
 
 def TransformFirstOf(r, *args):
-  """Returns the first non-empty r.name value for name in args.
+  """Returns the first non-empty .name attribute value for name in args.
 
   Args:
     r: A JSON-serializable object.
-    *args: Names to check for values in r.
+    *args: Names to check for resource attribute values,
 
   Returns:
     The first non-empty r.name value for name in args, '' otherwise.
@@ -249,18 +379,63 @@ def TransformFirstOf(r, *args):
   return ''
 
 
+def TransformFloat(r, precision=6, spec=None, undefined=''):
+  """Returns the string representation of a floating point number.
+
+  One of these formats is used (1) ". _precision_ _spec_" if _spec_ is specified
+  (2) ". _precision_" unless 1e-04 <= abs(number) < 1e+09 (3) ".1f" otherwise.
+
+  Args:
+    r: A JSON-serializable object.
+    precision: The maximum number of digits before and after the decimal point.
+    spec: The printf(3) floating point format "e", "f" or "g" spec character.
+    undefined: Returns this value if the resource is not a float.
+
+  Returns:
+    The string representation of the floating point number r.
+  """
+  # TransformFloat vs. float.str() comparison:
+  #
+  #   METHOD          PRECISION   NO-EXPONENT-RANGE
+  #   TransformFloat()        6   1e-04 <= x < 1e+9
+  #   float.str()            12   1e-04 <= x < 1e+11
+  #
+  # The TransformFloat default avoids implementation dependent floating point
+  # roundoff differences in the fraction digits.
+  #
+  # round(float(r), precision) won't work here because it only works for
+  # significant digits immediately after the decimal point. For example,
+  # round(0.0000000000123456789, 6) is 0, not 1.23457e-11.
+
+  try:
+    number = float(r)
+  except (TypeError, ValueError):
+    return undefined
+  if spec is not None:
+    fmt = '{{number:.{precision}{spec}}}'.format(precision=precision, spec=spec)
+    return fmt.format(number=number)
+  fmt = '{{number:.{precision}}}'.format(precision=precision)
+  representation = fmt.format(number=number)
+  exponent_index = representation.find('e+')
+  if exponent_index >= 0:
+    exponent = int(representation[exponent_index + 2:])
+    if exponent < 9:
+      return '{number:.1f}'.format(number=number)
+  return representation
+
+
 # The 'format' transform is special: it has no kwargs and the second argument
 # is the ProjectionSpec of the calling projection.
 def TransformFormat(r, projection, fmt, *args):
-  """Formats a sub-projection of r.
+  """Formats resource key values.
 
   Args:
     r: A JSON-serializable object.
     projection: The parent ProjectionSpec.
     fmt: The format string with {0} ... {nargs-1} references to the resource
-      key arg values.
-    *args: The resource key args to format. The args values form a projection on
-      r. The projection symbols and aliases are available in the sub-projection.
+      attribute name arg values.
+    *args: The resource attribute key expression to format. The printer
+      projection symbols and aliases may be used in key expressions.
 
   Returns:
     The formatted string.
@@ -293,7 +468,7 @@ def TransformGroup(r, *args):
   """
   if not r:
     return '[]'
-  buf = cStringIO.StringIO()
+  buf = StringIO.StringIO()
   sep = None
   for item in r:
     if sep:
@@ -301,7 +476,7 @@ def TransformGroup(r, *args):
     else:
       sep = ' '
     if not args:
-      buf.write('[{0}]'.format(str(item)))
+      buf.write('[{0}]'.format(unicode(item)))
     else:
       buf.write('[')
       sub = None
@@ -311,26 +486,76 @@ def TransformGroup(r, *args):
           sub = ', '
         else:
           sub = ': '
-        buf.write(str(getattr(item, attr)))
+        if isinstance(item, dict):
+          value = item.get(attr)
+        else:
+          value = getattr(item, attr, None)
+        if value is not None:
+          buf.write(unicode(value))
       buf.write(']')
   return buf.getvalue()
 
 
-def TransformIso(r, undefined='T'):
-  """Formats r to the numeric ISO time format.
+def TransformIf(r, expr):
+  """Disables the projection key if the flag name filter expr is false.
 
   Args:
     r: A JSON-serializable object.
-    undefined: Returns this if r does not have an isoformat() attribute.
+    expr: A command flag filter name expression. See `gcloud topic filters` for
+      details on filter expressions. The expression variables are flag names
+      without the leading *--* prefix and dashes replaced by underscores.
+
+  Example:
+    The "table(name, value.if(NOT short_format))" format will list a value
+    column if the *--short-format* command line flag is not specified.
+
+  Returns:
+    r
+  """
+  _ = expr
+  return r
+
+
+def TransformIso(r, undefined='T'):
+  """Formats the resource to numeric ISO time format.
+
+  Args:
+    r: A JSON-serializable object.
+    undefined: Returns this value if the resource does not have an isoformat()
+      attribute.
 
   Returns:
     The numeric ISO time format for r or undefined if r is not a time.
   """
-  return r.isoformat() if hasattr(r, 'isoformat') else undefined
+  return TransformDate(r, format='%Y-%m-%dT%H:%M:%S.%3f%Oz',
+                       undefined=undefined)
+
+
+def TransformJoin(r, sep='/', undefined=''):
+  """Joins the elements of the resource list by the value of sep.
+
+  A string resource is treated as a list of characters.
+
+  Args:
+    r: A string or list.
+    sep: The separator value to use when joining.
+    undefined: Returns this value if the result after joining is empty.
+
+  Returns:
+    A new string containing the resource values joined by sep.
+
+  Example:
+    "a/b/c/d".split("/").join("!") returns "a!b!c!d"
+  """
+  try:
+    parts = [unicode(i) for i in r]
+    return sep.join(parts) or undefined
+  except (AttributeError, TypeError):
+    return undefined
 
 
 def TransformLen(r):
-  """Returns the length of r if r is non-empty, 0 otherwise.
+  """Returns the length of the resource if it is non-empty, 0 otherwise.
 
   Args:
     r: A JSON-serializable object.
@@ -344,12 +569,14 @@ def TransformLen(r):
     return 0
 
 
-def TransformList(r, undefined='', separator=','):
+def TransformList(r, show='', undefined='', separator=','):
   """Formats a dict or list into a compact comma separated list.
 
   Args:
     r: A JSON-serializable object.
-    undefined: Return this if r is empty.
+    show: If show=*keys* then list dict keys; if show=*values* then list dict
+      values; otherwise list dict key=value pairs.
+    undefined: Return this if the resource is empty.
     separator: The list item separator string.
 
   Returns:
@@ -358,15 +585,20 @@ def TransformList(r, undefined='', separator=','):
     list.
   """
   if isinstance(r, dict):
-    return separator.join(['{key}={value}'.format(key=key, value=value)
-                           for key, value in sorted(r.iteritems())])
+    if show == 'keys':
+      return separator.join([unicode(k) for k in sorted(r)])
+    elif show == 'values':
+      return separator.join([unicode(v) for _, v in sorted(r.iteritems())])
+    else:
+      return separator.join([u'{k}={v}'.format(k=k, v=v)
+                             for k, v in sorted(r.iteritems())])
   if isinstance(r, list):
-    return separator.join(map(str, r))
+    return separator.join(map(unicode, r))
   return r or undefined
 
 
 def TransformMap(r):
-  """Applies the next transform in the sequence to each item in list resource r.
+  """Applies the next transform in the sequence to each resource list item.
 
   Example:
     list_field.map().foo().bar() applies foo() to each item in list_field and
@@ -390,7 +622,7 @@ def TransformResolution(r, undefined='', transpose=False):
 
   Args:
     r: object, A JSON-serializable object containing an x/y resolution.
-    undefined: Returns this if a recognizable resolution was not found.
+    undefined: Returns this value if a recognizable resolution was not found.
     transpose: Returns the y/x resolution if True.
 
   Returns:
@@ -439,7 +671,7 @@ def TransformResolution(r, undefined='', transpose=False):
 
 
 def TransformScope(r, *args):
-  """Gets the /args/ suffix from URI r.
+  """Gets the /args/ suffix from a URI.
 
   Args:
     r: A URI.
@@ -451,11 +683,12 @@ def TransformScope(r, *args):
       component in r if none found.
 
   Example:
-    scope('https://abc/foo/projects/bar/zyx', 'projects') returns 'bar/xyz'.
-    scope("https://abc/foo/rergions/abc") returns 'abc'.
+    "https://abc/foo/projects/bar/xyz".scope("projects") returns "bar/xyz".
+    "https://xyz/foo/regions/abc".scope() returns "abc".
   """
   if not r:
     return ''
+  r = urllib2.unquote(unicode(r))
   if '/' not in r:
     return r
   # Checking for regions and/or zones is the most common use case.
@@ -469,20 +702,20 @@ def TransformScope(r, *args):
 
 
 def TransformSegment(r, index=-1, undefined=''):
-  """Returns the index-th URI path segment in r.
+  """Returns the index-th URI path segment.
 
   Args:
     r: A URI path.
     index: The path segment index to return counting from 0.
-    undefined: This value is returned if r or segment index is empty.
+    undefined: Returns this value if the resource or segment index is empty.
 
   Returns:
     The index-th URI path segment in r
   """
   if not r:
     return undefined
-  s = str(r)
-  segments = s.split('/')
+  r = urllib2.unquote(unicode(r))
+  segments = r.split('/')
   try:
     return segments[int(index)] or undefined
   except IndexError:
@@ -490,17 +723,19 @@ def TransformSegment(r, index=-1, undefined=''):
 
 
 # pylint: disable=redefined-builtin, params match the transform spec
-def TransformSize(r, zero='0', units_in=None, units_out=None, min=0):
+def TransformSize(r, zero='0', precision=1, units_in=None, units_out=None,
+                  min=0):
   """Formats a human readable size in bytes.
 
   Args:
     r: A size in bytes.
     zero: Returns this if size==0. Ignored if None.
+    precision: The number of digits displayed after the decimal point.
     units_in: A unit suffix (only the first character is checked) or unit size.
-      The size of r is multiplied by this size. The defaults is 1.0.
+      The size is multiplied by this. The default is 1.0.
     units_out: A unit suffix (only the first character is checked) or unit size.
-      The size of r is divided by this size. The defaults is 1.0.
-    min: Sizes < min will be listed as "< MIN".
+      The size is divided by this. The default is 1.0.
+    min: Sizes < _min_ will be listed as "< _min_".
 
   Returns:
     A human readable scaled size in bytes.
@@ -552,7 +787,8 @@ def TransformSize(r, zero='0', units_in=None, units_out=None, min=0):
   (units_out_suffix, units_out_size) = _UnitSuffixAndSize(units_out)
   if units_out_suffix:
     size /= units_out_size
-    return '{0:.1f}'.format(size)
+    fmt = '{{0:.{precision}f}}'.format(precision=precision)
+    return fmt.format(size)
   the_unit = 'PiB'
   for unit in ['bytes', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB']:
     if size < 1024.0:
@@ -564,11 +800,94 @@ def TransformSize(r, zero='0', units_in=None, units_out=None, min=0):
   if size == int(size):
     return '{0}{1}{2}'.format(prefix, int(size), the_unit)
   else:
-    return '{0}{1:.1f}{2}'.format(prefix, size, the_unit)
+    fmt = '{{0}}{{1:.{precision}f}}{{2}}'.format(precision=precision)
+    return fmt.format(prefix, size, the_unit)
+
+
+def TransformSlice(r, op=':', undefined=''):
+  """Returns a list slice specified by op.
+
+  The op parameter consists of up to three colon-delimeted integers: start, end,
+  and step. The parameter supports half-open ranges: start and end values can
+  be omitted, representing the first and last positions of the resource
+  respectively.
+
+  The step value represents the increment between items in the resource included
+  in the slice. A step of 2 results in a slice that contains every other item in
+  the resource.
+
+  Negative values for start and end indicate that the positons should start from
+  the last position of the resource. A negative value for step indicates that
+  the slice should contain items in reverse order.
+
+  If op contains no colons, the slice consists of the single item at the
+  specified position in the resource.
+
+  Args:
+    r: A JSON-serializable string or array.
+    op: The slice operation.
+    undefined: Returns this value if the slice cannot be created, or the
+        resulting slice is empty.
+
+  Returns:
+    A new array containing the specified slice of the resource.
+
+  Example:
+    [1,2,3].slice(1:) returns [2,3].
+    [1,2,3].slice(:2) returns [1,2].
+    [1,2,3].slice(-1:) returns [3].
+    [1,2,3].slice(::-1) returns [3,2,1].
+    [1,2,3].slice(1) returns [2].
+  """
+  op = op.strip()
+  if not op:
+    return undefined
+
+  # Construct a list of integer and None values from op to be passed to slice().
+  try:
+    ops = [int(sp) if sp else None for sp in (p.strip() for p in op.split(':'))]
+  except (AttributeError, TypeError, ValueError):
+    return undefined
+
+  # Handle the case where the user specifies only an index by
+  # constructing a slice of i:i+1. E.g., the equivalent slice of index
+  # 1 is [1:2]. If ops[0] + 1 == 0, use None as the slice end instead.
+  # The slice [-1:0] returns an empty set; [-1:] returns a set
+  # containing the last element.
+  if len(ops) == 1:
+    ops.append(ops[0] + 1 or None)
+
+  try:
+    return list(r[slice(*ops)]) or undefined
+  except (TypeError, ValueError):
+    return undefined
+
+
+def TransformSplit(r, sep='/', undefined=''):
+  """Splits a string by the value of sep.
+
+  Args:
+    r: A string.
+    sep: The separator value to use when splitting.
+    undefined: Returns this value if the result after splitting is empty.
+
+  Returns:
+    A new array containing the split components of the resource.
+
+  Example:
+    "a/b/c/d".split() returns ["a", "b", "c", "d"]
+  """
+  if not r:
+    return undefined
+
+  try:
+    return r.split(sep)
+  except (AttributeError, TypeError, ValueError):
+    return undefined
 
 
 def TransformUri(r, undefined='.'):
-  """Gets the URI for r.
+  """Gets the resource URI.
 
   Args:
     r: A JSON-serializable object.
@@ -578,8 +897,6 @@ def TransformUri(r, undefined='.'):
     The URI for r or undefined if not defined.
   """
 
-  names = ('selfLink', 'SelfLink', 'instance')
-
   def _GetAttr(attr):
     """Returns the string value for attr or None if the value is not a string.
 
@@ -599,20 +916,21 @@ def TransformUri(r, undefined='.'):
     if r.startswith('https://'):
       return r
   elif r:
-    for name in names:
-      attr = _GetAttr(resource_property.Get(r, [name], None))
-      if attr:
-        return attr
+    for name in ('selfLink', 'SelfLink'):
+      uri = _GetAttr(resource_property.Get(r, [name], None))
+      if uri:
+        return uri
   return undefined
 
 
 def TransformYesNo(r, yes=None, no='No'):
-  """Returns no if r is empty, yes or r otherwise.
+  """Returns no if the resource is empty, yes or the resource itself otherwise.
 
   Args:
     r: A JSON-serializable object.
-    yes: If r is not empty then returns yes or r.
-    no: Returns this string if r is empty.
+    yes: If the resource is not empty then returns _yes_ or the resource itself
+      if _yes_ is not defined.
+    no: Returns this value if the resource is empty.
 
   Returns:
     yes or r if r is not empty, no otherwise.
@@ -624,15 +942,23 @@ def TransformYesNo(r, yes=None, no='No'):
 _BUILTIN_TRANSFORMS = {
     'always': TransformAlways,
     'basename': TransformBaseName,
+    'collection': TransformCollection,
     'color': TransformColor,
     'date': TransformDate,
+    'decode': TransformDecode,
     'duration': TransformDuration,
+    'encode': TransformEncode,
+    'enum': TransformEnum,
     'error': TransformError,
+    'extract': TransformExtract,
     'fatal': TransformFatal,
     'firstof': TransformFirstOf,
+    'float': TransformFloat,
     'format': TransformFormat,
     'group': TransformGroup,
+    'if': TransformIf,
     'iso': TransformIso,
+    'join': TransformJoin,
     'len': TransformLen,
     'list': TransformList,
     'map': TransformMap,
@@ -640,6 +966,8 @@ _BUILTIN_TRANSFORMS = {
     'scope': TransformScope,
     'segment': TransformSegment,
     'size': TransformSize,
+    'slice': TransformSlice,
+    'split': TransformSplit,
     'uri': TransformUri,
     'yesno': TransformYesNo,
 }
@@ -650,6 +978,13 @@ _BUILTIN_TRANSFORMS = {
 #   method_name: A method name in the module that returns the transform dict.
 _API_TO_TRANSFORMS = {
     'compute': ('googlecloudsdk.api_lib.compute.transforms', 'GetTransforms'),
+    'debug': ('googlecloudsdk.api_lib.debug.transforms', 'GetTransforms'),
+    'runtimeconfig': (
+        'googlecloudsdk.api_lib.deployment_manager.runtime_configs.transforms',
+        'GetTransforms'
+    ),
+    'service_registry': ('googlecloudsdk.api_lib.service_registry.transforms',
+                         'GetTransforms'),
 }
 
 
@@ -676,3 +1011,16 @@ def GetTransforms(collection=None):
   module = __import__(module_path, fromlist=[method_name])
   method = getattr(module, method_name)
   return method()
+
+
+def GetTypeDataName(name, type_name='object'):
+  """Returns the data name for name of type type_name.
+
+  Args:
+    name: The data name.
+    type_name: The data type name.
+
+  Returns:
+    The data name for name of type type_name.
+  """
+  return '{name}::{type_name}'.format(name=name, type_name=type_name)
