@@ -15,6 +15,7 @@
 """YAML format printer."""
 
 from googlecloudsdk.core.resource import resource_printer_base
+from googlecloudsdk.core.resource import resource_transform
 
 
 class YamlPrinter(resource_printer_base.ResourcePrinter):
@@ -42,19 +43,38 @@ class YamlPrinter(resource_printer_base.ResourcePrinter):
   """
 
   def __init__(self, *args, **kwargs):
-    super(YamlPrinter, self).__init__(*args, **kwargs)
+    super(YamlPrinter, self).__init__(*args, retain_none_values=True, **kwargs)
     # pylint:disable=g-import-not-at-top, Delay import for performance.
     import yaml
     self._yaml = yaml
 
-    def LiteralPresenter(dumper, data):
+    def _FloatPresenter(unused_dumper, data):
+      return yaml.nodes.ScalarNode('tag:yaml.org,2002:float',
+                                   resource_transform.TransformFloat(data))
+
+    def _LiteralLinesPresenter(dumper, data):
       return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
 
-    self._yaml.add_representer(YamlPrinter._LiteralString, LiteralPresenter,
+    def _UndefinedPresenter(dumper, data):
+      r = repr(data)
+      if r == '[]':
+        return dumper.represent_list([])
+      if r == '{}':
+        return dumper.represent_dict({})
+      dumper.represent_undefined(data)
+
+    self._yaml.add_representer(float,
+                               _FloatPresenter,
+                               Dumper=yaml.dumper.SafeDumper)
+    self._yaml.add_representer(YamlPrinter._LiteralLines,
+                               _LiteralLinesPresenter,
+                               Dumper=yaml.dumper.SafeDumper)
+    self._yaml.add_representer(None,
+                               _UndefinedPresenter,
                                Dumper=yaml.dumper.SafeDumper)
 
-  class _LiteralString(str):
-    """A type used to inform the yaml printer about how it should look."""
+  class _LiteralLines(unicode):
+    """A yaml representer hook for literal strings containing newlines."""
 
   def _UpdateTypesForOutput(self, val):
     """Dig through a dict of list of primitives to help yaml output.
@@ -66,7 +86,7 @@ class YamlPrinter(resource_printer_base.ResourcePrinter):
       An updated version of val.
     """
     if isinstance(val, basestring) and '\n' in val:
-      return YamlPrinter._LiteralString(val)
+      return YamlPrinter._LiteralLines(val)
     if isinstance(val, list):
       for i in range(len(val)):
         val[i] = self._UpdateTypesForOutput(val[i])
