@@ -20,17 +20,19 @@ will be accepted, and a consistent python object will be returned for use in
 code.
 """
 
+import functools
 import re
 import types
 import urllib
 
+from googlecloudsdk.core import apis as core_apis
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import properties
 from googlecloudsdk.third_party.apitools.base.py import base_api
-from googlecloudsdk.third_party.apitools.base.py import util
 from googlecloudsdk.third_party.py27 import py27_collections as collections
 from googlecloudsdk.third_party.py27 import py27_copy as copy
 
+import uritemplate
 
 _COLLECTION_SUB_RE = r'[a-zA-Z_]+(?:\.[a-zA-Z0-9_]+)+'
 
@@ -43,123 +45,15 @@ _URL_RE = re.compile(r'(https?://[^/]+/[^/]+/[^/]+/)(.+)')
 _METHOD_ID_RE = re.compile(r'(?P<collection>{collection})\.get'.format(
     collection=_COLLECTION_SUB_RE))
 _HTTP_RE = re.compile(r'^https?://')
-
-
-class ClientDef(object):
-
-  def __init__(self, api_name, api_version,
-               base_import_path='googlecloudsdk.third_party.apis',
-               urls_only=False):
-    self.api_name = api_name
-    self.api_version = api_version
-    self.base_import_path = base_import_path
-    self.urls_only = urls_only
-
-  def Import(self):
-    """Imports the client module and returns the client class.
-
-    Returns:
-      type: Class that can be used to contruct an instance of the client.
-    """
-    client_cls_name = _CamelCase(self.api_name) + _CamelCase(self.api_version)
-    module_obj = __import__(
-        '{base}.{api_name}.{api_version}.{api_name}_{api_version}_client'
-        .format(base=self.base_import_path,
-                api_name=self.api_name, api_version=self.api_version),
-        fromlist=[client_cls_name])
-    return getattr(module_obj, client_cls_name)
-
-
-# Map of all known APIs and their clients.
-_API_CLIENT_MAP = {
-    'autoscaler': {
-        'v1beta2': ClientDef('autoscaler', 'v1beta2')
-    },
-    'bigquery': {
-        'v2': ClientDef('bigquery', 'v2')
-    },
-    'cloudresourcemanager': {
-        'v1beta1': ClientDef('cloudresourcemanager', 'v1beta1')
-    },
-    'compute': {
-        'v1': ClientDef('compute', 'v1'),
-        'alpha': ClientDef('compute', 'alpha', urls_only=True),
-        'beta': ClientDef('compute', 'beta', urls_only=True),
-    },
-    'clouduseraccounts': {
-        'beta': ClientDef('clouduseraccounts', 'beta'),
-        'alpha': ClientDef('clouduseraccounts', 'alpha', urls_only=True),
-    },
-    'container': {
-        'v1': ClientDef('container', 'v1'),
-    },
-    'dataflow': {
-        'v1b3': ClientDef('dataflow', 'v1b3')
-    },
-    'dataproc': {
-        'v1beta1': ClientDef('dataproc', 'v1beta1')
-    },
-    'datastore': {
-        'v1beta3': ClientDef('datastore', 'v1beta3')
-    },
-    'dns': {
-        'v1': ClientDef('dns', 'v1')
-    },
-    'genomics': {
-        'v1': ClientDef('genomics', 'v1')
-    },
-    'logging': {
-        'v1beta3': ClientDef('logging', 'v1beta3')
-    },
-    'manager': {
-        'v1beta2': ClientDef('manager', 'v1beta2')
-    },
-    'replicapool': {
-        'v1beta2': ClientDef('replicapool', 'v1beta2')
-    },
-    'replicapoolupdater': {
-        'v1beta1': ClientDef('replicapoolupdater', 'v1beta1')
-    },
-    'resourceviews': {
-        'v1beta1': ClientDef('resourceviews', 'v1beta1')
-    },
-    'sql': {
-        'v1beta3': ClientDef('sqladmin', 'v1beta3'),
-        'v1beta4': ClientDef('sqladmin', 'v1beta4', urls_only=True)
-    },
-    'storage': {
-        'v1': ClientDef('storage', 'v1')
-    },
-    'testing': {
-        'v1': ClientDef('testing', 'v1')
-    },
-    'toolresults': {
-        'v1beta3': ClientDef('toolresults', 'v1beta3')
-    },
-}
-
-
-def _DefaultVersionForApiName(api_name):
-  for version, client_def in _API_CLIENT_MAP.get(api_name, {}).items():
-    if not client_def.urls_only:
-      return version
-  return None
+_GCS_URL_RE = re.compile('^gs://([^/]*)(?:/(.*))?$')
+_GCS_URL = 'https://www.googleapis.com/storage/v1/'
+_GCS_ALT_URL = 'https://storage.googleapis.com/'
 
 
 class Error(Exception):
   """Exceptions for this module."""
 
 
-class UnknownAPIException(Error):
-  """Exception we are trying to register an unknown API."""
-
-  def __init__(self, api, version=None):
-    msg = 'trying to register unknown API [{0}]'.format(api)
-    if version:
-      msg += ' (version [{0}])'.format(version)
-    super(UnknownAPIException, self).__init__(msg)
-
-
 class InvalidEndpointException(Error):
   """Exception for when an API endpoint is malformed."""
 
@@ -324,9 +218,13 @@ class _ResourceParser(object):
     for param, field in zip(self.method_config.ordered_params, fields):
       setattr(request, param, field)
 
+    api, resource_collection = self.collection.split('.', 1)
+    param_defaults = functools.partial(
+        self.registry.GetParamDefault, api, resource_collection)
     resource = Resource(
-        self.collection, request, self.method_config.ordered_params, kwargs,
-        collection_path, self)
+        self.collection, self.method_config.relative_path, request,
+        self.method_config.ordered_params, kwargs,
+        collection_path, self.client.url, param_defaults)
 
     if resolve:
       resource.Resolve()
@@ -417,8 +315,9 @@ class _ResourceParser(object):
 class Resource(object):
   """Information about a Cloud resource."""
 
-  def __init__(self, collection, request, ordered_params, resolvers,
-               collection_path, parser):
+  def __init__(self, collection, relative_path, request,
+               ordered_params, resolvers,
+               collection_path, endpoint_url, param_defaults):
     """Create a Resource object that may be partially resolved.
 
     To allow resolving of unknown params to happen after parse-time, the
@@ -427,6 +326,7 @@ class Resource(object):
 
     Args:
       collection: str, The collection name for this resource.
+      relative_path: str, relative path uri template.
       request: protorpc.messages.Message (not imported) subclass, An instance
           of a request that can be used to fetch the actual entity in the
           collection.
@@ -435,16 +335,20 @@ class Resource(object):
           be used to fill in values that were not specified in the command line.
       collection_path: str, The original command-line argument used to create
           this Resource.
-      parser: _ResourceParser, The parser used to create this Resource.
+      endpoint_url: str, service endpoint url for this resource.
+      param_defaults: func(param) -> default value for given parameter
+          in ordered_params.
     """
     self.__collection = collection
+    self._relative_path = relative_path
     self.__request = request
     self.__name = None
     self.__self_link = None
     self.__ordered_params = ordered_params
     self.__resolvers = resolvers
     self.__collection_path = collection_path
-    self.__parser = parser
+    self._endpoint_url = endpoint_url
+    self._param_defaults = param_defaults
     for param in ordered_params:
       setattr(self, param, getattr(request, param))
 
@@ -465,6 +369,8 @@ class Resource(object):
     return self.__self_link
 
   def Request(self):
+    for param in self.__ordered_params:
+      setattr(self.__request, param, getattr(self, param))
     return self.__request
 
   def Resolve(self):
@@ -488,27 +394,18 @@ class Resource(object):
       if getattr(self, param, None):
         continue
 
-      def ResolveParam(value):
-        # This is intended to close over param
-        # pylint:disable=cell-var-from-loop
-        setattr(self, param, value)
-        setattr(self.__request, param, value)
-        # pylint:enable=cell-var-from-loop
-
-        # First try the resolvers given to this resource explicitly.
+      # First try the resolvers given to this resource explicitly.
       resolver = self.__resolvers.get(param)
       if resolver:
         if callable(resolver):
-          ResolveParam(resolver())
+          setattr(self, param, resolver())
         else:
-          ResolveParam(resolver)
+          setattr(self, param, resolver)
         continue
 
       # Then try the registered defaults.
-      api, collection = self.__collection.split('.', 1)
       try:
-        value = self.__parser.registry.GetParamDefault(api, collection, param)
-        ResolveParam(value)
+        setattr(self, param, self._param_defaults(param))
       except properties.RequiredPropertyError:
         # If property lookup fails, that's ok.  Just don't resolve anything.
         pass
@@ -517,14 +414,14 @@ class Resource(object):
         [(k, getattr(self, k) or '*') for k in self.__ordered_params])
 
     self.__self_link = '%s%s' % (
-        self.__parser.client.url,
-        util.ExpandRelativePath(self.__parser.method_config, effective_params))
+        self._endpoint_url,
+        uritemplate.expand(self._relative_path, effective_params))
 
     if (self.Collection().startswith('compute.') or
         self.Collection().startswith('clouduseraccounts.') or
-        self.Collection().startswith('resourceviews.')):
+        self.Collection().startswith('storage.')):
       # TODO(user): Unquote URLs for compute, clouduseraccounts, and
-      # resourceviews pending b/15425944.
+      # storage pending b/15425944.
       self.__self_link = urllib.unquote(self.__self_link)
 
     if self.__ordered_params:
@@ -540,6 +437,11 @@ class Resource(object):
     # return '{collection}::{path}'.format(
     #     collection=self.__collection, path=path)
 
+  def __eq__(self, other):
+    if isinstance(other, Resource):
+      return self.SelfLink() == other.SelfLink()
+    return False
+
 
 def _CopyNestedDictSpine(maybe_dictionary):
   if type(maybe_dictionary) is types.DictType:
@@ -572,7 +474,7 @@ def _APINameAndVersionFromURL(url):
   endpoint_overrides = properties.VALUES.api_endpoint_overrides.AllValues()
   for name, overridden_url in endpoint_overrides.iteritems():
     if overridden_url == url:
-      return name, _DefaultVersionForApiName(name)
+      return name, core_apis.GetDefaultVersion(name)
 
   tokens = _StripUrl(url).split('/')
   domain = tokens[0]
@@ -588,7 +490,7 @@ def _APINameAndVersionFromURL(url):
     if len(tokens) > 1:
       version = tokens[1]
     else:
-      version = _DefaultVersionForApiName(api_name)
+      version = core_apis.GetDefaultVersion(api_name)
   return api_name, version
 
 
@@ -643,11 +545,9 @@ class Registry(object):
     Args:
       api_name: str, The API name.
       api_version: if available, the version of the API being registered.
-
-    Raises:
-      UnknownAPIException: If the API to register is not in _API_CLIENT_MAP.
     """
-    if api_version and api_version in self.registered_apis.get(api_name, []):
+    registered_versions = self.registered_apis.get(api_name, [])
+    if api_version and api_version in registered_versions:
       # This API version has been registered.
       return
     if not api_version and api_name in self.registered_apis:
@@ -655,47 +555,30 @@ class Registry(object):
       # registered under this name.
       return
 
-    api_clients = _API_CLIENT_MAP.get(api_name, None)
+    api_client = core_apis.GetClientInstance(api_name, api_version,
+                                             no_http=True)
+    self._RegisterAPI(api_client,
+                      urls_only=bool(registered_versions),
+                      api_version=api_version)
 
-    if not api_clients:
-      raise UnknownAPIException(api_name)
-
-    if api_version:
-      try:
-        api_client = api_clients[api_version]
-        api_client_class = api_client.Import()
-        self._RegisterAPI(api_client_class(get_credentials=False),
-                          urls_only=api_client.urls_only,
-                          api_version=api_version)
-      except KeyError:
-        # This might get re-raised with the full collection path later.
-        # If it doesn't, this error is good enough.
-        raise InvalidResourceException('{0}/{1}'.format(api_name, api_version))
-    else:
-      for api_version, api_client in api_clients.items():
-        if not api_client.urls_only:
-          api_client_class = api_client.Import()
-          self._RegisterAPI(api_client_class(get_credentials=False),
-                            api_version=api_version)
-
-  def _RegisterAPI(self, api, urls_only=False, api_version=None):
+  def _RegisterAPI(self, api_client, urls_only=False, api_version=None):
     """Register a generated API with this registry.
 
     Args:
-      api: base_api.BaseApiClient, The client for a Google Cloud API.
+      api_client: base_api.BaseApiClient, The client for a Google Cloud API.
       urls_only: bool, True if this API should only be used to interpret URLs,
           and not to interpret collection-paths.
       api_version: str, the version of the API if it's not in the API client
           URL.
     """
-    for potential_service in api.__dict__.itervalues():
+    for potential_service in api_client.__dict__.itervalues():
       if not isinstance(potential_service, base_api.BaseApiService):
         continue
       try:
-        self._RegisterService(api, potential_service, urls_only)
+        self._RegisterService(api_client, potential_service, urls_only)
       except _ResourceWithoutGetException:
         pass
-    api_name, parsed_api_version = _APINameAndVersionFromURL(api.url)
+    api_name, parsed_api_version = _APINameAndVersionFromURL(api_client.url)
     self.registered_apis[api_name].add(parsed_api_version or api_version)
 
   def _RegisterService(self, api, service, urls_only):
@@ -865,7 +748,7 @@ class Registry(object):
     return self.parsers_by_collection[collection].ParseCollectionPath(
         collection_path, kwargs, resolve)
 
-  def ParseURL(self, url, collection):
+  def ParseURL(self, url):
     """Parse a URL into a Resource.
 
     This method does not yet handle "api.google.com" in place of
@@ -887,8 +770,6 @@ class Registry(object):
 
     Args:
       url: str, The URL of the resource.
-      collection: str, The intended collection for the parsed resource, or None
-          if unconstrained.
 
     Returns:
       Resource, The resource indicated by the provided URL.
@@ -896,8 +777,6 @@ class Registry(object):
     Raises:
       InvalidResourceException: If the provided URL could not be turned into
           a cloud resource.
-      WrongResourceCollectionException: If the provided URL points into a
-          collection other than the one specified.
     """
     match = _URL_RE.match(url)
     if not match:
@@ -910,12 +789,12 @@ class Registry(object):
     api_name, api_version = _APINameAndVersionFromURL(url)
     try:
       self._RegisterAPIByName(api_name, api_version=api_version)
-    except InvalidResourceException:
+    except (core_apis.UnknownAPIError, core_apis.UnknownVersionError):
       # The caught InvalidResourceException has a less detailed message.
       raise InvalidResourceException(url)
 
     cur_level = self.parsers_by_url
-    for token in tokens:
+    for i, token in enumerate(tokens):
       if token in cur_level:
         # If the literal token is already here, follow it down.
         cur_level = cur_level[token]
@@ -925,10 +804,20 @@ class Registry(object):
         param = cur_level.keys()[0]
         if not param.startswith('{') or not param.endswith('}'):
           raise InvalidResourceException(url)
+
+        next_level = cur_level[param]
+        if len(next_level) == 1 and None in next_level:
+          # This is the last parameter so we can combine the remaining tokens.
+          token = '/'.join(tokens[i:])
+          params[param[1:-1]] = urllib.unquote(token)
+          cur_level = next_level
+          break
+
         # Clean up the provided value
         params[param[1:-1]] = urllib.unquote(token)
+
         # Keep digging down.
-        cur_level = cur_level[param]
+        cur_level = next_level
       else:
         # If the token we want isn't here, and there isn't a single parameter,
         # the URL we've been given doesn't match anything we know about.
@@ -942,14 +831,26 @@ class Registry(object):
     if None not in cur_level:
       raise InvalidResourceException(url)
     parser = cur_level[None]
-    resource = parser.ParseCollectionPath(None, params, resolve=True)
+    return parser.ParseCollectionPath(None, params, resolve=True)
 
-    if collection and resource.Collection() != collection:
-      raise WrongResourceCollectionException(
-          expected=collection, got=resource.Collection(), path=url)
-    return resource
+  def ParseStorageURL(self, url):
+    """Parse gs://bucket/object_path into storage.v1 api resource."""
+    match = _GCS_URL_RE.match(url)
+    if not match:
+      raise InvalidResourceException('Invalid storage url: [{0}]'.format(url))
+    if match.group(2):
+      return self.ParseCollectionPath(
+          collection='storage.objects',
+          collection_path=None,
+          kwargs={'bucket': match.group(1), 'object': match.group(2)})
 
-  def Parse(self, line, params=None, collection=None, resolve=True):
+    return self.ParseCollectionPath(
+        collection='storage.buckets',
+        collection_path=None,
+        kwargs={'bucket': match.group(1)})
+
+  def Parse(self, line, params=None, collection=None,
+            enforce_collection=True, resolve=True):
     """Parse a Cloud resource from a command line.
 
     Args:
@@ -960,6 +861,8 @@ class Registry(object):
         will be searched for what remains.
       collection: str, The resource's collection, or None if it should be
         inferred from the line.
+      enforce_collection: bool, fail unless parsed resource is of this
+        specified collection, this is applicable only if line is URL.
       resolve: bool, If True, call the resource's .Resolve() method before
           returning, ensuring that all of the resource parameters are defined.
           If False, don't call them, under the assumption that it will be called
@@ -972,19 +875,70 @@ class Registry(object):
       InvalidResourceException: If the line is invalid.
       UnknownCollectionException: If no collection is provided or can be
           inferred.
+      WrongResourceCollectionException: If the provided URL points into a
+          collection other than the one specified.
     """
-    if line and (line.startswith('https://') or line.startswith('http://')):
-      return self.ParseURL(line, collection)
-    else:
-      if not collection:
-        match = _COLLECTIONPATH_RE.match(line)
-        if not match:
-          raise InvalidResourceException(line)
-        collection, unused_path = match.groups()
+    if line:
+      if line.startswith('https://') or line.startswith('http://'):
+        try:
+          resource = self.ParseURL(line)
+        except InvalidResourceException:
+          # TODO(b/29573201): Make sure ParseURL handles this logic by default.
+          bucket = None
+          if line.startswith(_GCS_URL):
+            bucket_prefix, bucket, object_prefix, objectpath = (
+                line[len(_GCS_URL):].split('/', 3))
+            if (bucket_prefix, object_prefix) != ('b', 'o'):
+              raise
+          elif line.startswith(_GCS_ALT_URL):
+            line = line[len(_GCS_ALT_URL):]
+            if '/' in line:
+              bucket, objectpath = line.split('/', 1)
+            else:
+              return self.ParseCollectionPath(
+                  collection='storage.buckets',
+                  collection_path=None,
+                  kwargs={'bucket': line})
+          if bucket is not None:
+            return self.ParseCollectionPath(
+                collection='storage.objects',
+                collection_path=None,
+                kwargs={'bucket': bucket, 'object': objectpath})
+          raise
+        # TODO(user): consider not doing this here.
+        # Validation of the argument is a distict concern.
+        if (enforce_collection and collection and
+            resource.Collection() != collection):
+          raise WrongResourceCollectionException(
+              expected=collection,
+              got=resource.Collection(),
+              path=resource.SelfLink())
+        return resource
+      elif line.startswith('gs://'):
+        return self.ParseStorageURL(line)
+
+    if not collection:
+      match = _COLLECTIONPATH_RE.match(line)
+      if not match:
+        raise InvalidResourceException(line)
+      collection, unused_path = match.groups()
       if not collection:
         raise UnknownCollectionException(line)
+    # Special handle storage collection paths.
+    if collection == 'storage.objects':
+      p = dict(params or {})
+      if 'bucket' not in p or 'object' not in p:
+        if '/' not in line:
+          raise InvalidResourceException(
+              'Expected bucket/object in "{0}"'.format(line))
+        p['bucket'], p['object'] = line.split('/', 1)
 
-      return self.ParseCollectionPath(collection, line, params or {}, resolve)
+      return self.ParseCollectionPath(
+          collection='storage.objects',
+          collection_path=None,
+          kwargs=p)
+
+    return self.ParseCollectionPath(collection, line, params or {}, resolve)
 
   def Create(self, collection, **params):
     """Create a Resource from known collection and params.
@@ -1088,9 +1042,3 @@ def _StripUrl(url):
   if not _HTTP_RE.match(url):
     raise InvalidEndpointException(url)
   return url[url.index(':') + 1:].strip('/')
-
-
-def _CamelCase(snake_case):
-  parts = snake_case.split('_')
-  return ''.join(s.capitalize() for s in parts)
-
