@@ -44,9 +44,9 @@ class InvalidSDKRootError(Error):
 
   def __init__(self):
     super(InvalidSDKRootError, self).__init__(
-        'The update action could not be performed because this Cloud SDK '
-        'installation is managed by an external package manager.  If you would '
-        'like to get the latest version, please see our main download page at:\n  '
+        'You cannot perform this action because this Cloud SDK installation '
+        'is managed by an external package manager.  If you would like to get'
+        'the latest version, please see our main download page at:\n  '
         + config.INSTALLATION_CONFIG.documentation_url)
 
 
