@@ -28,6 +28,7 @@ from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.resource import resource_printer
+from googlecloudsdk.core.updater import installers
 from googlecloudsdk.core.updater import local_state
 from googlecloudsdk.core.updater import release_notes
 from googlecloudsdk.core.updater import snapshots
@@ -230,6 +231,13 @@ class UpdateManager(object):
       local_state.InvalidSDKRootError: If the Cloud SDK root cannot be found.
       MissingUpdateURLError: If we don't know what manifest to download.
     """
+    # Check sdk root before URL, as error about non-installed gcloud is more
+    # informative.
+    self.__sdk_root = sdk_root
+    if not self.__sdk_root:
+      self.__sdk_root = config.Paths().sdk_root
+    if not self.__sdk_root:
+      raise local_state.InvalidSDKRootError()
 
     if not url:
       url = properties.VALUES.component_manager.snapshot_url.Get()
@@ -241,11 +249,6 @@ class UpdateManager(object):
     if not url:
       raise MissingUpdateURLError()
 
-    self.__sdk_root = sdk_root
-    if not self.__sdk_root:
-      self.__sdk_root = config.Paths().sdk_root
-    if not self.__sdk_root:
-      raise local_state.InvalidSDKRootError()
     self.__sdk_root = os.path.realpath(self.__sdk_root)
     self.__base_url = url
     self.__platform_filter = platform_filter
@@ -480,7 +483,8 @@ version [{1}].  To clear your fixed version setting, run:
         log.debug('Checking for updates...')
         # It's time to do an update check and refresh the notification cache.
         try:
-          (_, diff) = self._GetStateAndDiff(command_path='UPDATE_MANAGER')
+          (_, diff) = self._GetStateAndDiff(
+              command_path=installers.UPDATE_MANAGER_COMMAND_PATH)
           last_update_check.SetFromSnapshot(
               diff.latest, bool(diff.AvailableUpdates()), force=force)
         except snapshots.IncompatibleSchemaVersionError:
@@ -490,94 +494,33 @@ version [{1}].  To clear your fixed version setting, run:
       last_update_check.Notify(command_path)
       return last_update_check.UpdatesAvailable()
 
-  def List(self, show_versions=False):
+  def List(self):
     """Lists all of the components and their current state.
 
     This pretty prints the list of components along with whether they are up
     to date, require an update, etc.
 
-    Args:
-      show_versions: bool, True to print versions in the table.  Defaults to
-        False.
-
     Returns:
-      The list of snapshots.ComponentDiffs for all components.
+      The list of snapshots.ComponentDiffs for all components that are not
+      hidden.
     """
     try:
       _, diff = self._GetStateAndDiff(command_path='components.list')
     except snapshots.IncompatibleSchemaVersionError as e:
-      return self._ReinstallOnError(e)
+      self._ReinstallOnError(e)
+      return ([], None, None)
 
     # Only show the latest version if we are not pinned.
     latest_msg = ('The latest available version is: '
                   if not self.__fixed_version else None)
     current_version, latest_version = self._PrintVersions(
         diff, latest_msg=latest_msg)
-    to_print = [diff.AvailableUpdates(), diff.Removed(),
-                diff.AvailableToInstall(), diff.UpToDate()]
 
-    self._PrintTable('Packages', show_versions=False, to_print=to_print,
-                     func=lambda x: x.is_configuration, ignore_if_empty=True)
-    self._PrintTable('Components', show_versions=show_versions,
-                     to_print=to_print, func=lambda x: not x.is_configuration,
-                     ignore_if_empty=False)
+    to_print = (diff.AvailableUpdates() + diff.Removed() +
+                diff.AvailableToInstall() + diff.UpToDate())
+    to_print = [c for c in to_print if not c.is_hidden]
 
-    self.__Write(
-        log.status,
-        """\
-To install or remove components at your current SDK version [{current}], run:
-  $ gcloud components install COMPONENT_ID
-  $ gcloud components remove COMPONENT_ID
-
-To update your SDK installation to the latest version [{latest}], run:
-  $ gcloud components update
-""".format(current=current_version, latest=latest_version), word_wrap=False)
-    return diff.AllDiffs()
-
-  def _PrintTable(self, title, show_versions, to_print, func, ignore_if_empty):
-    """Prints a table of updatable components.
-
-    Args:
-      title: str, The title for the table.
-      show_versions: bool, True to print versions in the table.
-      to_print: list(list(snapshots.ComponentDiff)), The available components
-        divided by state.
-      func: func(snapshots.ComponentDiff) -> bool, Decides whether the component
-        should be printed.
-      ignore_if_empty: bool, True to not show the table at all if there are no
-        matching components.
-    """
-    attributes = [
-        'box',
-        'legend=" "',
-        ]
-    if ignore_if_empty:
-      attributes.append('no-empty-legend')
-    else:
-      attributes.append('empty-legend="No updates."')
-    if title:
-      attributes.append('title="{title}"'.format(title=title))
-    columns = [
-        'state.name:label=Status',
-        'name:label=Name',
-        ]
-    if show_versions:
-      columns.extend([
-          'current.version.version_string:label=Installed:align=right',
-          'latest.version.version_string:label=Latest:align=right',
-          ])
-    columns.extend([
-        'id:label=ID',
-        'size.size(zero="",min=1048576):label=Size:align=right',
-        ])
-    fmt = 'table[{attributes}]({columns})'.format(
-        attributes=','.join(attributes), columns=','.join(columns))
-    printer = resource_printer.Printer(fmt)
-    for components in to_print:
-      for c in components:
-        if not c.is_hidden and func(c):
-          printer.AddRecord(c)
-    printer.Finish()
+    return (to_print, current_version, latest_version)
 
   def _PrintVersions(self, diff, latest_msg=None):
     """Prints the current and latest version.
@@ -644,7 +587,7 @@ To update your SDK installation to the latest version [{latest}], run:
         ]
     fmt = 'table[{attributes}]({columns})'.format(
         attributes=','.join(attributes), columns=','.join(columns))
-    resource_printer.Print(components, fmt)
+    resource_printer.Print(components, fmt, out=log.status)
 
   def _UpdateWithProgressBar(self, components, action, action_func, first=False,
                              last=False):
@@ -792,7 +735,7 @@ To update your SDK installation to the latest version [{latest}], run:
     config.EnsureSDKWriteAccess(self.__sdk_root)
     self._RestartIfUsingBundledPython(args=restart_args)
 
-    if self._IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
+    if self.IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
       log.warn(BUNDLED_PYTHON_REMOVAL_WARNING)
 
     # If explicitly listing components, you are probably installing and not
@@ -879,7 +822,7 @@ To update your SDK installation to the latest version [{latest}], run:
           log.status,
           """\
 To revert your SDK to the previously installed version, you may run:
-  $ gcloud components update --version [{current}]
+  $ gcloud components update --version {current}
 """.format(current=current_version), word_wrap=False)
 
     if self.__warn:
@@ -892,6 +835,14 @@ Please remove the following to avoid accidentally invoking these old tools:
 {0}
 
 """.format('\n'.join(bad_commands)))
+
+    if platforms.PythonVersion().IsPython26():
+      log.warn("""\
+You are running Python 2.6, which is no longer receiving security
+patches as of October 2013.  The Cloud SDK will stop supporting Python 2.6 on
+September 1, 2016.  Please update your Python installation to 2.7 to ensure
+compatibility with future Cloud SDK versions.""")
+
     return True
 
   def FindAllOldToolsOnPath(self, path=None):
@@ -975,7 +926,7 @@ Please remove the following to avoid accidentally invoking these old tools:
     config.EnsureSDKWriteAccess(self.__sdk_root)
     self._RestartIfUsingBundledPython()
 
-    if self._IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
+    if self.IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
       log.warn(BUNDLED_PYTHON_REMOVAL_WARNING)
 
     message = self._GetDontCancelMessage(disable_backup)
@@ -1025,7 +976,7 @@ Please remove the following to avoid accidentally invoking these old tools:
     backup_has_bundled_python = (
         BUNDLED_PYTHON_COMPONENT in
         install_state.BackupInstallationState().InstalledComponents())
-    if self._IsPythonBundled() and not backup_has_bundled_python:
+    if self.IsPythonBundled() and not backup_has_bundled_python:
       log.warn(BUNDLED_PYTHON_REMOVAL_WARNING)
 
     if not console_io.PromptContinue(
@@ -1206,7 +1157,7 @@ prompt, or run:
     # Restart the original command.
     RestartCommand(command)
 
-  def _IsPythonBundled(self):
+  def IsPythonBundled(self):
     return _IsPythonBundled(self.__sdk_root)
 
   def _RestartIfUsingBundledPython(self, args=None, command=None):
@@ -1261,7 +1212,7 @@ prompt, or run:
                   'to view the failures.')
 
 
-def _CopyPython():
+def CopyPython():
   """Copy the current Python to temporary directory and return its path."""
   # We don't want to clean this up when we're done, because we use it later.
   temp_dir = file_utils.TemporaryDirectory()
@@ -1279,6 +1230,27 @@ def RestartIfUsingBundledPython(sdk_root, args=None, command=None):
   current_os = platforms.OperatingSystem.Current()
   if (current_os is platforms.OperatingSystem.WINDOWS and
       _IsPythonBundled(sdk_root)):
+    if not console_io.CanPrompt():
+      # If we're in non-interactive mode, updates using bundled Python will
+      # usually not work as intended. This process will terminate and not wait
+      # for the child process to finish. This can yield inconsistent state and
+      # mask errors (especially in the context of the installer).
+      gcloud_cmd_path = os.path.realpath(
+          os.path.join(config.Paths().sdk_bin_path or '', 'gcloud.cmd'))
+      log.error('''\
+Cannot use bundled Python installation to update Cloud SDK in
+non-interactive mode. Please run again in interactive mode.\n\n
+
+If you really want to run in non-interactive mode, please run the
+following command before re-running this one:\n\n
+
+  FOR /F "delims=" %i in ( '""{0}"" components copy-bundled-python'
+  ) DO (
+    SET CLOUDSDK_PYTHON=%i
+  )
+
+(Substitute `%%i` for `%i` if in a .bat script.)'''.format(gcloud_cmd_path))
+      sys.exit(1)
     # On Windows, you can't use a Python installed within a directory to move
     # that directory, which means that with a bundled Python, updates will
     # fail. To get around this, we copy the Python interpreter to a temporary
@@ -1286,7 +1258,7 @@ def RestartIfUsingBundledPython(sdk_root, args=None, command=None):
     # There's no issue if the `.py` files themselves are inside the install
     # directory, because the Python interpreter loads them into memory and
     # closes them immediately.
-    RestartCommand(args=args, command=command, python=_CopyPython(),
+    RestartCommand(args=args, command=command, python=CopyPython(),
                    block=False)
     sys.exit(0)
 
