@@ -19,12 +19,14 @@ import hashlib
 import logging
 import os
 import shutil
+import stat
 import sys
 import tempfile
 import time
 import traceback
 
 from googlecloudsdk.core.util import platforms
+from googlecloudsdk.core.util import retry
 
 NUM_RETRIES = 10
 
@@ -84,13 +86,14 @@ def _WaitForRetry(retries_left):
   time.sleep(time_to_wait)
 
 
-RETRY_ERROR_CODES = [32, 145]
+RETRY_ERROR_CODES = [5, 32, 145]
 
 
 def _ShouldRetryOperation(func, exc_info):
   """Matches specific error types that should be retried.
 
   This will retry the following errors:
+    WindowsError(5, 'Access is denied'), When trying to delete a readonly file
     WindowsError(32, 'The process cannot access the file because it is being '
       'used by another process'), When a file is in use.
     WindowsError(145, 'The directory is not empty'), When a directory cannot be
@@ -154,6 +157,11 @@ def _HandleRemoveError(func, failed_path, exc_info):
   logging.debug('Handling file system error: %s, %s, %s',
                 func, failed_path, exc_info)
 
+  # Access denied on Windows. This happens when trying to delete a readonly
+  # file. Change the permissions and retry the delete.
+  if exc_info[0] == WindowsError and exc_info[1].winerror == 5:
+    os.chmod(failed_path, stat.S_IWUSR)
+
   # Don't remove the trailing comma in the passed arg tuple.  It indicates that
   # it is a tuple of 1, rather than a tuple of characters that will get expanded
   # by *args.
@@ -164,6 +172,7 @@ def _HandleRemoveError(func, failed_path, exc_info):
     raise exc_info[0], exc_info[1], exc_info[2]
 
 
+# TODO(user): Add unit tests for Windows specific code paths, b/28869930
 def RmTree(path):
   """Calls shutil.rmtree() with error handling to fix Windows problems.
 
@@ -595,11 +604,12 @@ class Checksum(object):
     return self.__files
 
 
-def OpenForWritingPrivate(path):
+def OpenForWritingPrivate(path, access_mode='w'):
   """Open a file for writing, with the right permissions for user-private files.
 
   Args:
     path: str, The full path to the file.
+    access_mode: Can be 'w' or 'wb'. Default to 'w'.
 
   Returns:
     A file context manager.
@@ -615,7 +625,7 @@ def OpenForWritingPrivate(path):
     flags |= os.O_NOINHERIT
 
   fd = os.open(path, flags, 0600)
-  return os.fdopen(fd, 'w')
+  return os.fdopen(fd, access_mode)
 
 
 class Context(object):
@@ -649,3 +659,165 @@ class ChDir(object):
 
   def __exit__(self, typ, value, tb):
     os.chdir(self.__original_dir)
+
+
+class FileLockLockingError(Error):
+  pass
+
+
+class FileLockTimeoutError(FileLockLockingError):
+  """A case of FileLockLockingError."""
+  pass
+
+
+class FileLockUnlockingError(Error):
+  pass
+
+
+class FileLock(object):
+  """A file lock for interprocess (not interthread) mutual exclusion.
+
+  At most one FileLock instance may be locked at a time for a given local file
+  path. FileLock instances may be used as context objects.
+  """
+
+  def __init__(self, path, timeout_secs=None):
+    """Constructs the FileLock.
+
+    Args:
+      path: str, the path to the file to lock. The directory containing the
+        file must already exist when Lock() is called.
+      timeout_secs: int, seconds Lock() may wait for the lock to become
+        available. If None, Lock() may block forever.
+    """
+    self._path = path
+    self._timeout_secs = timeout_secs
+    self._file = None
+    self._locked = False
+    if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
+      self._impl = _WindowsLocking()
+    else:
+      self._impl = _PosixLocking()
+
+  def Lock(self):
+    """Opens and locks the file. A no-op if this FileLock is already locked.
+
+    The lock file is created if it does not already exist.
+
+    Raises:
+      FileLockLockingError: if the file could not be opened (or created when
+        necessary).
+      FileLockTimeoutError: if the file could not be locked before the timeout
+        elapsed.
+    """
+    if self._locked:
+      return
+    try:
+      self._file = open(self._path, 'w')
+    except IOError as e:
+      raise FileLockLockingError(e)
+
+    max_wait_ms = None
+    if self._timeout_secs is not None:
+      max_wait_ms = 1000 * self._timeout_secs
+
+    r = retry.Retryer(max_wait_ms=max_wait_ms)
+    try:
+      r.RetryOnException(self._impl.TryLock, args=[self._file.fileno()],
+                         sleep_ms=100)
+    except retry.RetryException as e:
+      self._file.close()
+      self._file = None
+      raise FileLockTimeoutError(
+          'Timed-out waiting to lock file: {0}'.format(self._path))
+    else:
+      self._locked = True
+
+  def Unlock(self):
+    """Unlocks and closes the file.
+
+    A no-op if this object is not locked.
+
+    Raises:
+      FileLockUnlockingError: if a problem was encountered when unlocking the
+        file. There is no need to retry.
+    """
+    if not self._locked:
+      return
+    try:
+      self._impl.Unlock(self._file.fileno())
+    except IOError as e:
+      # We don't expect Unlock() to ever raise an error, but can't be sure.
+      raise FileLockUnlockingError(e)
+    finally:
+      self._file.close()
+      self._file = None
+      self._locked = False
+
+  def __enter__(self):
+    """Locks and returns this FileLock object."""
+    self.Lock()
+    return self
+
+  def __exit__(self, exc_type, exc_val, exc_tb):
+    """Unlocks, logging any errors encountered."""
+    try:
+      self.Unlock()
+    except Error as e:
+      logging.debug('Encountered error unlocking file %s: %s', self._path, e)
+    # Have Python re-raise the exception which caused the context to exit, if
+    # any.
+    return False
+
+
+# Imports fcntl, which is only available on POSIX.
+class _PosixLocking(object):
+  """Exclusive, non-blocking file locking on POSIX systems."""
+
+  def TryLock(self, fd):
+    """Raises IOError on failure."""
+    # pylint: disable=g-import-not-at-top
+    import fcntl
+    # Exclusive lock, non-blocking
+    fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+
+  def Unlock(self, fd):
+    import fcntl  # pylint: disable=g-import-not-at-top
+    fcntl.flock(fd, fcntl.LOCK_UN)
+
+
+# Imports msvcrt, which is only available on Windows.
+class _WindowsLocking(object):
+  """Exclusive, non-blocking file locking on Windows."""
+
+  def TryLock(self, fd):
+    """Raises IOError on failure."""
+    # pylint: disable=g-import-not-at-top
+    import msvcrt
+    # Exclusive lock, non-blocking
+    msvcrt.locking(fd, msvcrt.LK_NBLCK, 1)
+
+  def Unlock(self, fd):
+    import msvcrt  # pylint: disable=g-import-not-at-top
+    msvcrt.locking(fd, msvcrt.LK_UNLCK, 1)
+
+
+def GetFileContents(path):
+  """Returns the contents of the specified file.
+
+  Args:
+    path: str, The path of the file to read.
+
+  Raises:
+    Error: If the file cannot be read.
+
+  Returns:
+    The contents of the file.
+  """
+  try:
+    with open(path) as in_file:
+      return in_file.read()
+  except EnvironmentError:
+    # EnvironmentError is parent of IOError, OSError and WindowsError.
+    # Raised when file does not exist or can't be opened/read.
+    raise Error('Unable to read file [{0}]'.format(path))
