@@ -42,7 +42,11 @@ class InvalidEnumValue(Error):
 
 
 def GetHomePath():
-  return os.path.expanduser('~')
+  return ExpandHomePath('~')
+
+
+def ExpandHomePath(path):
+  return os.path.expanduser(path)
 
 
 class OperatingSystem(object):
@@ -61,7 +65,8 @@ class OperatingSystem(object):
   LINUX = _OS('LINUX', 'Linux', 'linux')
   CYGWIN = _OS('CYGWIN', 'Cygwin', 'cygwin')
   MSYS = _OS('MSYS', 'Msys', 'msys')
-  _ALL = [WINDOWS, MACOSX, LINUX, CYGWIN, MSYS]
+  OPENBSD = _OS('OPENBSD', 'OpenBSD', 'openbsd')
+  _ALL = [WINDOWS, MACOSX, LINUX, CYGWIN, MSYS, OPENBSD]
 
   @staticmethod
   def AllValues():
@@ -114,6 +119,8 @@ class OperatingSystem(object):
       return OperatingSystem.MACOSX
     elif 'cygwin' in sys.platform:
       return OperatingSystem.CYGWIN
+    elif 'openbsd' in sys.platform:
+      return OperatingSystem.OPENBSD
     # TODO(user): More reliable handling of OS types
     # TODO(user): What happens when we use jython, does it actually use the
     # 'java' os name?
@@ -135,7 +142,7 @@ class Architecture(object):
   x86_64 = _ARCH('x86_64', 'x86_64', 'x86_64')
   ppc = _ARCH('PPC', 'PPC', 'ppc')
   _ALL = [x86, x86_64, ppc]
-  _MACHINE_TO_ARCHITECTURE = {'AMD64': x86_64, 'x86_64': x86_64,
+  _MACHINE_TO_ARCHITECTURE = {'AMD64': x86_64, 'x86_64': x86_64, 'amd64': x86_64,
                               'i386': x86, 'i686': x86, 'x86': x86,
                               'Power Macintosh': ppc}
 
@@ -247,6 +254,9 @@ class Platform(object):
                      if self.architecture == Architecture.ppc else 'Intel')
       return format_string.format(
           name=arch_string, version=platform.release())
+    elif self.operating_system == OperatingSystem.OPENBSD:
+      return '({name} {version})'.format(
+          name=self.operating_system.name, version=platform.release())
     else:
       return '()'
 
@@ -274,21 +284,6 @@ class Platform(object):
       args['close_fds'] = True
     return args
 
-  def IsSupported(self):
-    """Ensure that we support the given platform.
-
-    This will print an error message if not supported.
-
-    Returns:
-      bool, True if the platform is valid, False otherwise.
-    """
-    if (self.operating_system == OperatingSystem.CYGWIN and
-        self.architecture == Architecture.x86_64):
-      sys.stderr.write('ERROR: Cygwin 64 bit is not supported by the Google '
-                       'Cloud SDK.  Please use a 32 bit version of Cygwin.')
-      return False
-    return True
-
 
 class PythonVersion(object):
   """Class to validate the Python version we are using.
@@ -311,13 +306,9 @@ class PythonVersion(object):
     else:
       self.version = None
 
-  def __MinVersionString(self):
-    return '%s.%s' % (str(PythonVersion.MIN_REQUIRED_VERSION[0]),
-                      str(PythonVersion.MIN_REQUIRED_VERSION[1]))
-
   def MinSupportedVersionString(self):
-    return '%s.%s' % (str(PythonVersion.MIN_SUPPORTED_VERSION[0]),
-                      str(PythonVersion.MIN_SUPPORTED_VERSION[1]))
+    return '{0}.{1}.x'.format(PythonVersion.MIN_SUPPORTED_VERSION[0],
+                              PythonVersion.MIN_SUPPORTED_VERSION[1])
 
   def __PrintEnvVarMessage(self):
     """Prints how to set CLOUDSDK_PYTHON."""
@@ -342,17 +333,17 @@ class PythonVersion(object):
     error = None
     if not self.version:
       error = ('ERROR: Your current version of Python is not compatible with '
-               'the Google Cloud SDK. Please upgrade to Python %s or greater.\n'
-               % (self.__MinVersionString()))
-    if self.version[0] >= 3:
+               'the Google Cloud SDK. Please upgrade to Python {0}\n'
+               .format(self.MinSupportedVersionString()))
+    elif self.version[0] >= 3:
       error = ('ERROR: Python 3 and later is not compatible with by the Google '
-               'Cloud SDK. Please use a Python 2.x version that is %s or '
-               'greater.\n' % (self.__MinVersionString()))
-    if self.version < PythonVersion.MIN_REQUIRED_VERSION:
-      error = ('ERROR: Python %s.%s is not compatible with the Google Cloud '
-               'SDK. Please upgrade to version %s or greater.\n'
-               % (str(self.version[0]), str(self.version[1]),
-                  self.__MinVersionString()))
+               'Cloud SDK. Please use a Python {0} version.\n'
+               .format(self.MinSupportedVersionString()))
+    elif self.version < PythonVersion.MIN_REQUIRED_VERSION:
+      error = ('ERROR: Python {0}.{1} is not compatible with the Google Cloud '
+               'SDK. Please upgrade to Python {2}\n'
+               .format(self.version[0], self.version[1],
+                       self.MinSupportedVersionString()))
 
     if error:
       if print_errors:
@@ -364,10 +355,18 @@ class PythonVersion(object):
   def IsSupported(self):
     """Return whether this Python version is recommended.
 
-    Only version 2.7 is compatible.
+    Only version 2.7 is supported.
 
     Returns:
       bool, True if the Python version is recommended
     """
     return (self.IsCompatible(print_errors=False) and
             self.version >= self.MIN_SUPPORTED_VERSION)
+
+  def IsPython26(self):
+    """Check specifically if we are running on Python 2.6 so we can warn.
+
+    Returns:
+      True, if running on Python 2.6, False otherwise.
+    """
+    return self.version == (2, 6)
