@@ -1,3 +1,5 @@
+#!/usr/bin/env python
+#
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -67,8 +69,6 @@ if hasattr(signal, 'SIGPIPE'):
 def _DoStartupChecks():
   if not platforms.PythonVersion().IsCompatible():
     sys.exit(1)
-  if not platforms.Platform.Current().IsSupported():
-    sys.exit(1)
 
 _DoStartupChecks()
 
@@ -111,6 +111,8 @@ def CreateCLI(surfaces):
   loader.AddReleaseTrack(base.ReleaseTrack.BETA,
                          os.path.join(pkg_root, 'surface', 'beta'),
                          component='beta')
+  loader.AddReleaseTrack(base.ReleaseTrack.PREVIEW,
+                         os.path.join(pkg_root, 'surface', 'preview'))
 
   for dot_path, dir_path in surfaces:
     loader.AddModule(dot_path, dir_path, component=None)
