@@ -18,6 +18,6 @@
 from googlecloudsdk.calliope import base
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
 class Auth(base.Group):
   """Manage oauth2 credentials for the Google Cloud SDK."""
