@@ -15,6 +15,8 @@
 """The command to perform any necessary post installation steps."""
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.static_completion import table
+from googlecloudsdk.core import remote_completion
 from googlecloudsdk.core.updater import local_state
 
 
@@ -27,5 +29,12 @@ class PostProcess(base.SilentCommand):
     parser.add_argument('data', nargs='*', default='')
 
   def Run(self, args):
+    # Re-complile python files.
     state = local_state.InstallationState.ForCurrent()
     state.CompilePythonFiles()
+
+    # Re-set remote completion cache.
+    remote_completion.RemoteCompletion.ResetCache()
+
+    # Re-generate static completion table.
+    table.Update(self.cli)
