@@ -14,11 +14,14 @@
 """Command for creating images."""
 
 from googlecloudsdk.api_lib.compute import base_classes
-from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.api_lib.compute import utils
+from googlecloudsdk.calliope import arg_parsers
+from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.compute import flags
 
 
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class Create(base_classes.BaseAsyncCreator):
   """Create Google Compute Engine images."""
 
@@ -58,7 +61,19 @@ class Create(base_classes.BaseAsyncCreator):
         help='The zone of the disk specified by --source-disk.')
     source_disk_zone.detailed_help = ("""\
         The zone of the disk specified by --source-disk.
-        """ + constants.ZONE_PROPERTY_EXPLANATION)
+        """ + flags.ZONE_PROPERTY_EXPLANATION)
+
+    parser.add_argument(
+        '--family',
+        help=('The family of the image. When creating an instance or disk, '
+              'specifying a family will cause the latest non-deprecated image '
+              'in the family to be used.')
+    )
+
+    parser.add_argument(
+        '--licenses',
+        type=arg_parsers.ArgList(),
+        help='Comma-separated list of URIs to license resources.')
 
     parser.add_argument(
         'name',
@@ -83,7 +98,8 @@ class Create(base_classes.BaseAsyncCreator):
     image = self.messages.Image(
         name=args.name,
         description=args.description,
-        sourceType=self.messages.Image.SourceTypeValueValuesEnum.RAW)
+        sourceType=self.messages.Image.SourceTypeValueValuesEnum.RAW,
+        family=args.family)
 
     # Validate parameters.
     if args.source_disk_zone and not args.source_disk:
@@ -102,6 +118,9 @@ class Create(base_classes.BaseAsyncCreator):
           resource_type='disks')
       image.sourceDisk = source_disk_ref.SelfLink()
 
+    if args.licenses:
+      image.licenses = args.licenses
+
     request = self.messages.ComputeImagesInsertRequest(
         image=image,
         project=self.project)
@@ -109,6 +128,14 @@ class Create(base_classes.BaseAsyncCreator):
     return [request]
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
+class CreateBeta(Create):
+
+  @staticmethod
+  def Args(parser):
+    Create.Args(parser)
+
+
 Create.detailed_help = {
     'brief': 'Create Google Compute Engine images',
     'DESCRIPTION': """\
@@ -127,3 +154,5 @@ Create.detailed_help = {
         link:https://cloud.google.com/compute/docs/creating-custom-image[].
         """,
 }
+
+CreateBeta.detailed_help = Create.detailed_help
