@@ -11,10 +11,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for deleting access configs from virtual machine instances."""
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import constants
-from googlecloudsdk.api_lib.compute import utils
+from googlecloudsdk.command_lib.compute import flags
 
 
 class DeleteAccessConfig(base_classes.NoOutputAsyncMutator):
@@ -26,7 +28,7 @@ class DeleteAccessConfig(base_classes.NoOutputAsyncMutator):
 
   @staticmethod
   def Args(parser):
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser,
         resource_type='instance',
         operation_type='delete an access config from')
