@@ -11,15 +11,15 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for setting machine type for virtual machine instances."""
 
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import instance_utils
-from googlecloudsdk.api_lib.compute import utils
-from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute import flags
+from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
 class SetMachineType(base_classes.NoOutputAsyncMutator):
   """Set machine type for Google Compute Engine virtual machine instances."""
 
@@ -32,10 +32,10 @@ class SetMachineType(base_classes.NoOutputAsyncMutator):
         completion_resource='compute.instances',
         help='The name of the instance to change the machine type for.')
 
-    instance_utils.AddMachineTypeArgs(parser)
-    instance_utils.AddCustomMachineTypeArgs(parser)
+    instances_flags.AddMachineTypeArgs(parser)
+    instances_flags.AddCustomMachineTypeArgs(parser)
 
-    utils.AddZoneFlag(
+    flags.AddZoneFlag(
         parser,
         resource_type='instance',
         operation_type='change the machine type for')
@@ -56,9 +56,13 @@ class SetMachineType(base_classes.NoOutputAsyncMutator):
     """Returns a list of request necessary for setting scheduling options."""
     instance_ref = self.CreateZonalReference(args.name, args.zone)
 
-    machine_type = instance_utils.InterpretMachineType(args)
+    machine_type = instance_utils.InterpretMachineType(
+        machine_type=args.machine_type,
+        custom_cpu=args.custom_cpu,
+        custom_memory=args.custom_memory)
 
-    instance_utils.CheckCustomCpuRamRatio(self, instance_ref.zone, machine_type)
+    instance_utils.CheckCustomCpuRamRatio(
+        self.compute_client, self.project, instance_ref.zone, machine_type)
 
     machine_type_uri = self.CreateZonalReference(
         machine_type, instance_ref.zone,
