@@ -24,7 +24,6 @@ from googlecloudsdk.core import resources as resource_exceptions
 def AddFlags(parser, is_ga):
   """Helper function for adding flags dependant on the release track."""
   base_classes.BaseDescriber.Args(parser)
-  base_classes.AddFieldsFlag(parser, 'operations')
 
   scope = parser.add_mutually_exclusive_group()
 
@@ -110,8 +109,8 @@ class DescribeGA(base_classes.BaseDescriber):
         ref = self.CreateZonalReference(
             args.name, args.zone, resource_type='zoneOperations')
       elif not self._ga and args.user_accounts:
-        ref = self.CreateAccountsReference(
-            args.name, resource_type='globalAccountsOperations')
+        ref = self.clouduseraccounts_resources.Parse(
+            args.name, collection='clouduseraccounts.globalAccountsOperations')
       else:
         # TODO(user): Instead of raising here, we should really just
         # prompt for {global, <list of regions>, <list of zones>}, but
