@@ -13,8 +13,9 @@
 # limitations under the License.
 
 """Command for removing a BGP peer from a router."""
+
 from googlecloudsdk.api_lib.compute import base_classes
-from googlecloudsdk.api_lib.compute import utils
+from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.third_party.py27 import py27_copy as copy
 
@@ -39,7 +40,7 @@ class RemoveBgpPeer(base_classes.ReadWriteCommand):
         required=True,
         help='The name of the peer being removed.')
 
-    utils.AddRegionFlag(
+    flags.AddRegionFlag(
         parser,
         resource_type='router',
         operation_type='update')
@@ -79,7 +80,7 @@ class RemoveBgpPeer(base_classes.ReadWriteCommand):
   def Modify(self, args, existing):
     replacement = copy.deepcopy(existing)
 
-    # remove peer
+    # remove peer if exists
     peer = None
     for p in replacement.bgpPeers:
       if p.name == args.peer_name:
@@ -90,17 +91,10 @@ class RemoveBgpPeer(base_classes.ReadWriteCommand):
     if peer is None:
       raise PeerNotFoundError(args.peer_name)
 
-    # remove interface if exists
-    for i in replacement.interfaces:
-      if i.name == peer.interfaceName:
-        replacement.interfaces.remove(i)
-        break
-
     return replacement
 
 
 RemoveBgpPeer.detailed_help = {
-    'brief': 'Remove a BGP peer to a router.',
     'DESCRIPTION': """
         *{command}* removes a BGP peer from a router.
         """,
