@@ -15,9 +15,6 @@
 """Configurations command group."""
 
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions as c_exc
-from googlecloudsdk.core import config
-from googlecloudsdk.core import properties
 
 
 class Configurations(base.Group):
@@ -31,14 +28,3 @@ class Configurations(base.Group):
           configurations.
           """,
   }
-
-  def Filter(self, context, args):
-    """Modify the context that will be given to this group's commands when run.
-
-    Args:
-      context: {str:object}, A set of key-value pairs that can be used for
-          common initialization among commands.
-      args: argparse.Namespace: The same namespace given to the corresponding
-          .Run() invocation.
-    """
-    pass
