@@ -15,8 +15,9 @@
 """Command to activate named configuration."""
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.config import completers
 from googlecloudsdk.core import log
-from googlecloudsdk.core import named_configs
+from googlecloudsdk.core.configurations import named_configs
 
 
 class Activate(base.SilentCommand):
@@ -43,12 +44,13 @@ class Activate(base.SilentCommand):
   @staticmethod
   def Args(parser):
     """Adds args for this command."""
-    parser.add_argument(
+    configuration_arg = parser.add_argument(
         'configuration_name',
-        help='Configuration name to activate')
+        help='Name of the configuration to activate')
+    configuration_arg.completer = completers.NamedConfigCompleter
 
   def Run(self, args):
-    named_configs.ActivateNamedConfig(args.configuration_name)
+    named_configs.ConfigurationStore.ActivateConfig(args.configuration_name)
     log.status.write('Activated [{0}].\n'.format(args.configuration_name))
     return args.configuration_name
 
