@@ -15,10 +15,11 @@
 """Command to list properties."""
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.config import completers
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
-from googlecloudsdk.core import named_configs
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.configurations import named_configs
 
 
 class BadConfigListInvocation(exceptions.Error):
@@ -26,10 +27,12 @@ class BadConfigListInvocation(exceptions.Error):
 
 
 class List(base.Command):
-  """View Google Cloud SDK properties.
+  """View Cloud SDK properties.
 
-  List all currently available Cloud SDK properties associated with your current
-  workspace or global configuration.
+  Lists all properties in your active configuration. These include the
+  account used to authorize access to the Cloud Platform, the current Cloud
+  Platform project, and the default Compute Engine region and zone, if set. See
+  `gcloud topic configurations` for more information.
   """
 
   detailed_help = {
@@ -64,7 +67,7 @@ class List(base.Command):
         nargs='?',
         help='The property to be listed. Note that SECTION/ is optional while '
         'referring to properties in the core section.')
-    property_arg.completer = List.group_class.PropertiesCompleter
+    property_arg.completer = completers.PropertiesCompleter
 
   def _GetPropertiesToDisplay(self, args):
     """List available regular properties."""
@@ -83,9 +86,7 @@ class List(base.Command):
       raise BadConfigListInvocation('`gcloud config list` cannot take both '
                                     'a property name and the `--all` flag.')
 
-    config_name = named_configs.GetNameOfActiveNamedConfig()
-    if config_name is None:
-      config_name = named_configs.RESERVED_NAMED_CONFIG_NAME_NONE
+    config_name = named_configs.ConfigurationStore.ActiveConfig().name
     log.status.write('Your active configuration is: [{0}]\n\n'.format(
         config_name))
     return self._GetPropertiesToDisplay(args)
