@@ -16,8 +16,8 @@
 
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as c_exc
-from googlecloudsdk.core import log
-from googlecloudsdk.core import named_configs
+from googlecloudsdk.command_lib.config import completers
+from googlecloudsdk.command_lib.config import flags
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import remote_completion
 
@@ -38,10 +38,11 @@ DETAILED_HELP = {
 
 
 class Set(base.Command):
-  """Edit Google Cloud SDK properties.
+  """Set a Cloud SDK property.
 
-  Set the value for an option, so that Cloud SDK tools can use them as
-  configuration.
+  By default, sets the property in your active configuration only. Use the
+  `--installation` flag to set the property across all configurations. See
+  `gcloud topic configurations` for more information.
   """
 
   detailed_help = DETAILED_HELP
@@ -54,7 +55,7 @@ class Set(base.Command):
         metavar='SECTION/PROPERTY',
         help='The property to be set. Note that SECTION/ is optional while '
         'referring to properties in the core section.')
-    property_arg.completer = Set.group_class.PropertiesCompleter
+    property_arg.completer = completers.PropertiesCompleter
     value_arg = parser.add_argument(
         'value',
         completion_resource='cloudresourcemanager.projects',
@@ -62,9 +63,7 @@ class Set(base.Command):
         help='The value to be set.')
     value_arg.completer = Set.ValueCompleter
 
-    scope_args = parser.add_mutually_exclusive_group()
-    Set.group_class.DEPRECATED_SCOPE_FLAG.AddToParser(scope_args)
-    Set.group_class.INSTALLATION_FLAG.AddToParser(scope_args)
+    flags.INSTALLATION_FLAG.AddToParser(parser)
 
   @staticmethod
   def ValueCompleter(prefix, parsed_args, **unused_kwargs):
@@ -88,21 +87,14 @@ class Set(base.Command):
     return completer(prefix=prefix, parsed_args=parsed_args, **unused_kwargs)
 
   def Run(self, args):
-    if args.scope:
-      log.err.Print('The `--scope` flag is deprecated.  Please run `gcloud '
-                    'help topic configurations` and `gcloud help config set` '
-                    'for more information.')
-
-    requested_scope = Set.group_class.RequestedScope(args)
-
-    if not requested_scope:
-      named_configs.TryEnsureWriteableNamedConfig()
+    scope = (properties.Scope.INSTALLATION if args.installation
+             else properties.Scope.USER)
 
     prop = properties.FromString(args.property)
     if not prop:
       raise c_exc.InvalidArgumentException(
           'property', 'Must be in the form: [SECTION/]PROPERTY')
-    properties.PersistProperty(prop, args.value, scope=requested_scope)
+    properties.PersistProperty(prop, args.value, scope=scope)
 
 
 
