@@ -21,7 +21,7 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
 
 
-class Delete(base.Command):
+class Delete(base.DeleteCommand):
   """Delete a cluster."""
 
   detailed_help = {
@@ -36,6 +36,7 @@ class Delete(base.Command):
   @staticmethod
   def Args(parser):
     parser.add_argument('name', help='The name of the cluster to delete.')
+    base.ASYNC_FLAG.AddToParser(parser)
 
   @util.HandleHttpError
   def Run(self, args):
@@ -44,8 +45,9 @@ class Delete(base.Command):
 
     cluster_ref = util.ParseCluster(args.name, self.context)
 
-    request = messages.DataprocProjectsClustersDeleteRequest(
+    request = messages.DataprocProjectsRegionsClustersDeleteRequest(
         clusterName=cluster_ref.clusterName,
+        region=cluster_ref.region,
         projectId=cluster_ref.projectId)
 
     if not console_io.PromptContinue(
@@ -53,12 +55,16 @@ class Delete(base.Command):
         'deleted.'.format(args.name)):
       raise exceptions.ToolException('Deletion aborted by user.')
 
-    operation = client.projects_clusters.Delete(request)
+    operation = client.projects_regions_clusters.Delete(request)
+
+    if args.async:
+      log.status.write(
+          'Deleting [{0}] with operation [{1}].'.format(
+              cluster_ref, operation.name))
+      return operation
+
     operation = util.WaitForOperation(
         operation, self.context, 'Waiting for cluster deletion operation')
     log.DeletedResource(cluster_ref)
 
     return operation
-
-  def Display(self, args, result):
-    self.format(result)
