@@ -18,7 +18,7 @@ from googlecloudsdk.api_lib.dataproc import util
 from googlecloudsdk.calliope import base
 
 
-class Describe(base.Command):
+class Describe(base.DescribeCommand):
   """View the details of a cluster."""
 
   detailed_help = {
@@ -41,8 +41,5 @@ class Describe(base.Command):
     cluster_ref = util.ParseCluster(args.name, self.context)
     request = cluster_ref.Request()
 
-    cluster = client.projects_clusters.Get(request)
+    cluster = client.projects_regions_clusters.Get(request)
     return cluster
-
-  def Display(self, args, result):
-    self.format(result)
