@@ -18,7 +18,7 @@ from googlecloudsdk.api_lib.dataproc import util
 from googlecloudsdk.calliope import base
 
 
-class Describe(base.Command):
+class Describe(base.DescribeCommand):
   """View the details of a job."""
 
   detailed_help = {
@@ -44,8 +44,5 @@ class Describe(base.Command):
     job_ref = util.ParseJob(args.id, self.context)
     request = job_ref.Request()
 
-    job = client.projects_jobs.Get(request)
+    job = client.projects_regions_jobs.Get(request)
     return job
-
-  def Display(self, args, result):
-    self.format(result)
