@@ -31,10 +31,7 @@ class Hive(base_classes.JobSubmitter):
 
           To submit a Hive job with inline queries, run:
 
-            $ {command} --cluster my_cluster \\
-                -e "CREATE EXTERNAL TABLE foo(bar int) LOCATION \
-'gs://my_bucket/'" \\
-                -e "SELECT * FROM foo WHERE bar > 2"
+            $ {command} --cluster my_cluster -e "CREATE EXTERNAL TABLE foo(bar int) LOCATION 'gs://my_bucket/'" -e "SELECT * FROM foo WHERE bar > 2"
           """,
   }
 
