@@ -18,7 +18,7 @@ from googlecloudsdk.api_lib.dataproc import util
 from googlecloudsdk.calliope import base
 
 
-class Describe(base.Command):
+class Describe(base.DescribeCommand):
   """View the details of an operation."""
 
   detailed_help = {
@@ -40,11 +40,8 @@ class Describe(base.Command):
     client = self.context['dataproc_client']
     messages = self.context['dataproc_messages']
 
-    request = messages.DataprocOperationsGetRequest(
+    request = messages.DataprocProjectsRegionsOperationsGetRequest(
         name=args.operation)
 
-    operation = client.operations.Get(request)
+    operation = client.projects_regions_operations.Get(request)
     return operation
-
-  def Display(self, args, result):
-    self.format(result)
