@@ -11,26 +11,26 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""The super-group for the compute CLI."""
+"""The super-group for the IAM CLI."""
 import argparse
 import sys
 
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import base
+from googlecloudsdk.core import apis
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
-from googlecloudsdk.third_party.apis.iam.v1 import iam_v1_client
-from googlecloudsdk.third_party.apis.iam.v1 import iam_v1_messages
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Iam(base.Group):
+  """Manage IAM service accounts and keys."""
+  detailed_help = {
+      'brief': 'Manage IAM service accounts and keys.',
+  }
 
   def Filter(self, context, args):
-    context['iam-client'] = iam_v1_client.IamV1(
-        get_credentials=False,
-        http=self.Http(),
-        url=properties.VALUES.api_endpoint_overrides.iam.Get())
-    context['iam-messages'] = iam_v1_messages
+    context['iam-client'] = apis.GetClientInstance('iam', 'v1')
+    context['iam-messages'] = apis.GetMessagesModule('iam', 'v1')
     context['iam-resources'] = resources
