@@ -14,36 +14,39 @@
 """Command for to list all of a project's service accounts."""
 
 
-from googlecloudsdk.api_lib.iam import base_classes
-from googlecloudsdk.api_lib.iam import data_formats
 from googlecloudsdk.api_lib.iam import utils
+from googlecloudsdk.api_lib.util import http_error_handler
+from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.iam import base_classes
+from googlecloudsdk.core import properties
 from googlecloudsdk.third_party.apitools.base.py import list_pager
 
 
-class List(base_classes.BaseIamCommand):
+class List(base_classes.BaseIamCommand, base.ListCommand):
   """List all of a project's service accounts."""
 
+  def Collection(self):
+    return 'iam.service_accounts'
+
   @staticmethod
   def Args(parser):
-    parser.add_argument('--limit',
-                        type=int,
-                        help='The maximum number of service accounts to '
-                        'return.')
+    base.ASYNC_FLAG.RemoveFromParser(parser)
+    base.PAGE_SIZE_FLAG.RemoveFromParser(parser)
+    base.URI_FLAG.RemoveFromParser(parser)
 
-  @utils.CatchHttpErrors
+  @http_error_handler.HandleHttpErrors
   def Run(self, args):
     if args.limit is not None:
       if args.limit < 1:
-        raise ValueError('Limit size must be >=1')
+        raise exceptions.ToolException('Limit size must be >=1')
 
-    # TODO(user): We can't use the default list printing functions until
-    # there is support for atomic names. This property is the equivalent of
-    # a COLUMN_MAP for the list printer. To be removed in the future.
-    self.data_format = data_formats.SERVICE_ACCOUNT_COLUMNS
-    return list_pager.YieldFromList(
+    project = properties.VALUES.core.project.Get(required=True)
+    for item in list_pager.YieldFromList(
         self.iam_client.projects_serviceAccounts,
         self.messages.IamProjectsServiceAccountsListRequest(
-            name=utils.ProjectToProjectResourceName(self.project.Get())),
+            name=utils.ProjectToProjectResourceName(project)),
         field='accounts',
         limit=args.limit,
-        batch_size_attribute='pageSize')
+        batch_size_attribute='pageSize'):
+      yield item
