@@ -17,8 +17,8 @@
 from googlecloudsdk.api_lib.logging import util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
-from googlecloudsdk.core import list_printer
 from googlecloudsdk.core import log
+from googlecloudsdk.third_party.apitools.base.py import exceptions as apitools_exceptions
 
 
 class Update(base.Command):
@@ -43,6 +43,9 @@ class Update(base.Command):
               'https://cloud.google.com/logging/docs/api/introduction_v2'),
         choices=('V1', 'V2'))
 
+  def Collection(self):
+    return 'logging.sinks'
+
   def GetLogSink(self):
     """Returns a log sink specified by the arguments."""
     client = self.context['logging_client_v1beta3']
@@ -123,12 +126,20 @@ class Update(base.Command):
 
     # Calling Update on a non-existing sink creates it.
     # We need to make sure it exists, otherwise we would create it.
-    if args.log:
-      sink = self.GetLogSink()
-    elif args.service:
-      sink = self.GetLogServiceSink()
-    else:
-      sink = self.GetProjectSink()
+    try:
+      if args.log:
+        sink = self.GetLogSink()
+      elif args.service:
+        sink = self.GetLogServiceSink()
+      else:
+        sink = self.GetProjectSink()
+    except apitools_exceptions.HttpError as error:
+      project_sink = not args.log and not args.service
+      # Suggest the user to add --log or --log-service flag.
+      if project_sink and error.response.status == 404:
+        log.status.Print(('Project sink was not found. '
+                          'Did you forget to add --log or --log-service flag?'))
+      raise error
 
     # Only update fields that were passed to the command.
     if args.destination:
@@ -158,17 +169,11 @@ class Update(base.Command):
         sink_data['outputVersionFormat'] = sink.outputVersionFormat.name
       result = util.TypedLogSink(self.UpdateProjectSink(sink_data))
     log.UpdatedResource(sink_ref)
+    self._epilog_result_destination = result.destination
     return result
 
-  def Display(self, unused_args, result):
-    """This method is called to print the result of the Run() method.
-
-    Args:
-      unused_args: The arguments that command was run with.
-      result: The value returned from the Run() method.
-    """
-    list_printer.PrintResourceList('logging.typedSinks', [result])
-    util.PrintPermissionInstructions(result.destination)
+  def Epilog(self, unused_resources_were_displayed):
+    util.PrintPermissionInstructions(self._epilog_result_destination)
 
 
 Update.detailed_help = {
