@@ -20,7 +20,7 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 
 
-class Write(base.Command):
+class Write(base.SilentCommand):
   """Writes a log entry."""
 
   SEVERITY_ENUM = ('DEFAULT', 'DEBUG', 'INFO', 'NOTICE', 'WARNING',
@@ -63,7 +63,8 @@ class Write(base.Command):
                              args.severity.upper())
 
     entry = messages.LogEntry(
-        logName=util.CreateLogResourceName(project, args.log_name),
+        logName=util.CreateLogResourceName('projects/{0}'.format(project),
+                                           args.log_name),
         resource=messages.MonitoredResource(type='global'),
         severity=severity_value)
 
