@@ -12,23 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""The command group for the Projects CLI."""
+"""The command group for the projects CLI."""
 
-from googlecloudsdk.api_lib.projects import util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.core import resources
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA,
-                    base.ReleaseTrack.BETA,
-                    base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Projects(base.Group):
-  """Manage your Projects.
+  """Manage your projects.
 
-  Commands to get information about your Google Developer Projects.
+  Commands to get information about your Google Developer projects.
   """
+  pass
 
-  def Filter(self, context, _):
-    context['projects_resources'] = resources
-    context['projects_client'] = util.GetClient(self.Http())
-    context['projects_messages'] = util.GetMessages()
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class ProjectsAlpha(base.Group):
+  """Manage your projects.
+
+  Commands to get information about your Google Developer projects.
+  """
+  pass
