@@ -15,11 +15,11 @@
 import json
 from googlecloudsdk.api_lib.pubsub import util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.core.console import console_io as io
+from googlecloudsdk.core import resources
 from googlecloudsdk.third_party.apitools.base.py import exceptions as api_ex
 
 
-class Create(base.Command):
+class Create(base.CreateCommand):
   """Creates one or more Cloud Pub/Sub topics."""
 
   @staticmethod
@@ -28,6 +28,9 @@ class Create(base.Command):
     parser.add_argument('topic', nargs='+',
                         help='One or more topic names to create.')
 
+  def Collection(self):
+    return util.TOPICS_COLLECTION
+
   def Run(self, args):
     """This is what gets called when the user runs this command.
 
@@ -35,43 +38,20 @@ class Create(base.Command):
       args: an argparse namespace. All the arguments that were provided to this
         command invocation.
 
-    Returns:
-      A 2-tuple of lists, one populated with the topic paths that were
-      successfully created, the other one with the list of topic names
-      that could not be created.
+    Yields:
+      A serialized object (dict) describing the results of the operation.
+      This description fits the Resource described in the ResourceRegistry under
+      'pubsub.projects.topics'.
     """
     msgs = self.context['pubsub_msgs']
     pubsub = self.context['pubsub']
 
-    succeeded = []
-    failed = []
-
     for topic_name in args.topic:
+      topic_name = resources.Parse(topic_name,
+                                   collection=util.TOPICS_COLLECTION).Name()
       topic = msgs.Topic(name=util.TopicFormat(topic_name))
       try:
-        succeeded.append(pubsub.projects_topics.Create(topic))
+        yield util.TopicDisplayDict(pubsub.projects_topics.Create(topic))
       except api_ex.HttpError as exc:
-        failed.append((topic_name, json.loads(exc.content)['error']['message']))
-
-    return succeeded, failed
-
-  def Display(self, args, result):
-    """This method is called to print the result of the Run() method.
-
-    Args:
-      args: The arguments that command was run with.
-      result: The value returned from the Run() method.
-    """
-    succeeded, failed = result
-    successes = len(succeeded)
-    failures = len(failed)
-
-    if successes:
-      success_printer = io.ListPrinter(
-          '{0} topic(s) created successfully'.format(successes))
-      success_printer.Print([topic.name for topic in succeeded])
-
-    if failures:
-      fail_printer = io.ListPrinter('{0} topic(s) failed'.format(failures))
-      fail_printer.Print(
-          ['{0} (reason: {1})'.format(topic, desc) for topic, desc in failed])
+        yield util.TopicDisplayDict(topic,
+                                    json.loads(exc.content)['error']['message'])
