@@ -28,14 +28,3 @@ class BackupRuns(base.Group):
   instance.
   """
 
-  @staticmethod
-  def Args(parser):
-    parser.add_argument(
-        '--instance',
-        '-i',
-        completion_resource='sql.instances',
-        help='Cloud SQL instance ID.')
-
-  def Filter(self, tool_context, args):
-    if not args.instance:
-      raise exceptions.ToolException('argument --instance/-i is required')
