@@ -32,24 +32,11 @@ class _BaseResetSslConfig(object):
           on the command line after this command. Positional arguments are
           allowed.
     """
+    base.ASYNC_FLAG.AddToParser(parser)
     parser.add_argument(
         'instance',
         completion_resource='sql.instances',
         help='Cloud SQL instance ID.')
-    parser.add_argument(
-        '--async',
-        action='store_true',
-        help='Do not wait for the operation to complete.')
-
-  def Display(self, unused_args, result):
-    """Display prints information about what just happened to stdout.
-
-    Args:
-      unused_args: The same as the args in Run.
-      result: A dict object representing the operations resource describing the
-          resetSslConfig operation if the reset-ssl-config was successful.
-    """
-    self.format(result)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
