@@ -35,6 +35,7 @@ import signal
 import sys
 import textwrap
 import threading
+import time
 import traceback
 
 import boto
@@ -52,6 +53,10 @@ from gslib.name_expansion import NameExpansionIterator
 from gslib.name_expansion import NameExpansionResult
 from gslib.parallelism_framework_util import AtomicDict
 from gslib.plurality_checkable_iterator import PluralityCheckableIterator
+from gslib.sig_handling import ChildProcessSignalHandler
+from gslib.sig_handling import GetCaughtSignals
+from gslib.sig_handling import KillProcess
+from gslib.sig_handling import MultithreadedMainSignalHandler
 from gslib.sig_handling import RegisterSignalHandler
 from gslib.storage_url import StorageUrlFromString
 from gslib.third_party.storage_apitools import storage_v1_messages as apitools_messages
@@ -70,9 +75,6 @@ from gslib.wildcard_iterator import CreateWildcardIterator
 
 OFFER_GSUTIL_M_SUGGESTION_THRESHOLD = 5
 
-if IS_WINDOWS:
-  import ctypes  # pylint: disable=g-import-not-at-top
-
 
 def _DefaultExceptionHandler(cls, e):
   cls.logger.exception(e)
@@ -258,6 +260,18 @@ def InitializeMultiprocessingVariables():
   failure_count = multiprocessing.Value('i', 0)
 
 
+def TeardownMultiprocessingProcesses():
+  """Should be called by signal handlers prior to shut down."""
+  # Shut down all processes in consumer pools in preparation for exiting.
+  ShutDownGsutil()
+  # Shut down command and util's multiprocessing.Manager().
+  # pylint: disable=global-variable-not-assigned,global-variable-undefined
+  global manager
+  # pylint: enable=global-variable-not-assigned,global-variable-undefined
+  manager.shutdown()
+  gslib.util.manager.shutdown()
+
+
 def InitializeThreadingVariables():
   """Initializes module-level variables used when running multi-threaded.
 
@@ -897,24 +911,6 @@ class Command(HelpProvider):
           'command' % (url_str, self.command_name))
     return list(plurality_iter)[0]
 
-  def _HandleMultiProcessingSigs(self, signal_num, unused_cur_stack_frame):
-    """Handles signals INT AND TERM during a multi-process/multi-thread request.
-
-    Kills subprocesses.
-
-    Args:
-      unused_signal_num: signal generated by ^C.
-      unused_cur_stack_frame: Current stack frame.
-    """
-    # Note: This only works under Linux/MacOS. See
-    # https://github.com/GoogleCloudPlatform/gsutil/issues/99 for details
-    # about why making it work correctly across OS's is harder and still open.
-    ShutDownGsutil()
-    if signal_num == signal.SIGINT:
-      sys.stderr.write('Caught ^C - exiting\n')
-    # Simply calling sys.exit(1) doesn't work - see above bug for details.
-    KillProcess(os.getpid())
-
   def GetSingleBucketUrlFromArg(self, arg, bucket_fields=None):
     """Gets a single bucket URL based on the command arguments.
 
@@ -1300,15 +1296,18 @@ class Command(HelpProvider):
     """
     is_main_thread = self.recursive_apply_level == 0
 
-    # Catch SIGINT and SIGTERM under Linux/MacOs so we can do cleanup before
-    # exiting.
     if not IS_WINDOWS and is_main_thread:
-      # Register as a final signal handler because this handler kills the
-      # main gsutil process (so it must run last).
-      RegisterSignalHandler(signal.SIGINT, self._HandleMultiProcessingSigs,
-                            is_final_handler=True)
-      RegisterSignalHandler(signal.SIGTERM, self._HandleMultiProcessingSigs,
-                            is_final_handler=True)
+      # For multi-thread or multi-process scenarios, the main process must
+      # kill itself on a terminating signal, because sys.exit(1) only exits
+      # the currently executing thread, leaving orphaned processes. The main
+      # thread is responsible for cleaning up multiprocessing variables such
+      # as manager processes. Therefore, the main thread's signal handling
+      # chain is:
+      # 1: __main__._CleanupSignalHandler (clean up processes)
+      # 2: MultithreadedSignalHandler (kill self)
+      for signal_num in (signal.SIGINT, signal.SIGTERM):
+        RegisterSignalHandler(signal_num, MultithreadedMainSignalHandler,
+                              is_final_handler=True)
 
     if not task_queues:
       # The process we create will need to access the next recursive level
@@ -1426,6 +1425,16 @@ class Command(HelpProvider):
                         (which is true iff process_count == 1), implying that
                         _ApplyThreads must behave as a blocking call.
     """
+    # _ApplyThreads is called in two situations: as the main process when
+    # process_count == 1, and as a separate multiprocessing.Process when
+    # process_count > 1. Separate processes should exit on a terminating signal,
+    # but to avoid race conditions only the main process should handle
+    # multiprocessing cleanup. Override child processes to use a single signal
+    # handler.
+    if process_count > 1:
+      for catch_signal in GetCaughtSignals():
+        signal.signal(catch_signal, ChildProcessSignalHandler)
+
     self._ResetConnectionPool()
     self.recursive_apply_level = recursive_apply_level
 
@@ -1449,7 +1458,14 @@ class Command(HelpProvider):
 
     num_enqueued = 0
     while True:
-      worker_semaphore.acquire()
+      while not worker_semaphore.acquire(blocking=False):
+        # Because Python signal handlers are only called in between atomic
+        # instructions, if we block the main thread on an available worker
+        # thread, we won't be able to respond to signals such as a
+        # user-initiated CTRL-C until a worker thread completes a task.
+        # We poll the semaphore periodically as a compromise between
+        # efficiency and user responsiveness.
+        time.sleep(0.01)
       task = task_queue.get()
       if task.args != ZERO_TASKS_TO_DO_ARGUMENT:
         # If we have no tasks to do and we're performing a blocking call, we
@@ -1494,28 +1510,6 @@ class _ConsumerPool(object):
       KillProcess(process.pid)
 
 
-def KillProcess(pid):
-  """Make best effort to kill the given process.
-
-  We ignore all exceptions so a caller looping through a list of processes will
-  continue attempting to kill each, even if one encounters a problem.
-
-  Args:
-    pid: The process ID.
-  """
-  try:
-    # os.kill doesn't work in 2.X or 3.Y on Windows for any X < 7 or Y < 2.
-    if IS_WINDOWS and ((2, 6) <= sys.version_info[:3] < (2, 7) or
-                       (3, 0) <= sys.version_info[:3] < (3, 2)):
-      kernel32 = ctypes.windll.kernel32
-      handle = kernel32.OpenProcess(1, 0, pid)
-      kernel32.TerminateProcess(handle, 0)
-    else:
-      os.kill(pid, signal.SIGKILL)
-  except:  # pylint: disable=bare-except
-    pass
-
-
 class Task(namedtuple('Task', (
     'func args caller_id exception_handler should_return_results arg_checker '
     'fail_on_error'))):
@@ -1839,6 +1833,14 @@ def _IncrementFailureCount():
     failure_count.value += 1
 
 
+def DecrementFailureCount():
+  global failure_count
+  if isinstance(failure_count, int):
+    failure_count -= 1
+  else:  # Otherwise it's a multiprocessing.Value() of type 'i'.
+    failure_count.value -= 1
+
+
 # pylint: disable=global-variable-undefined
 def GetFailureCount():
   """Returns the number of failures processed during calls to Apply()."""
