@@ -16,6 +16,7 @@
 
 from __future__ import absolute_import
 
+from contextlib import contextmanager
 import httplib
 import json
 import logging
@@ -44,6 +45,8 @@ from gslib.cloud_api import AccessDeniedException
 from gslib.cloud_api import ArgumentException
 from gslib.cloud_api import BadRequestException
 from gslib.cloud_api import CloudApi
+from gslib.cloud_api import CryptoTuple
+from gslib.cloud_api import EncryptionException
 from gslib.cloud_api import NotEmptyException
 from gslib.cloud_api import NotFoundException
 from gslib.cloud_api import PreconditionException
@@ -53,8 +56,11 @@ from gslib.cloud_api import ResumableUploadAbortException
 from gslib.cloud_api import ResumableUploadException
 from gslib.cloud_api import ResumableUploadStartOverException
 from gslib.cloud_api import ServiceException
+from gslib.cloud_api_helper import ListToGetFields
 from gslib.cloud_api_helper import ValidateDstObjectMetadata
 from gslib.cred_types import CredTypes
+from gslib.encryption_helper import Base64Sha256FromBase64EncryptionKey
+from gslib.encryption_helper import FindMatchingCryptoKey
 from gslib.exception import CommandException
 from gslib.gcs_json_media import BytesTransferredContainer
 from gslib.gcs_json_media import DownloadCallbackConnectionClassFactory
@@ -79,6 +85,7 @@ from gslib.translation_helper import CreateObjectNotFoundException
 from gslib.translation_helper import DEFAULT_CONTENT_TYPE
 from gslib.translation_helper import PRIVATE_DEFAULT_OBJ_ACL
 from gslib.translation_helper import REMOVE_CORS_CONFIG
+from gslib.util import AddAcceptEncodingGzipIfNeeded
 from gslib.util import GetBotoConfigFileList
 from gslib.util import GetCertsFile
 from gslib.util import GetCredentialStoreFilename
@@ -88,7 +95,7 @@ from gslib.util import GetMaxRetryDelay
 from gslib.util import GetNewHttp
 from gslib.util import GetNumRetries
 from gslib.util import GetPrintableExceptionString
-
+from gslib.util import JsonResumableChunkSizeDefined
 
 # Implementation supports only 'gs' URLs, so provider is unused.
 # pylint: disable=unused-argument
@@ -125,17 +132,18 @@ HTTP_TRANSFER_EXCEPTIONS = (apitools_exceptions.TransferRetryError,
                             ssl.SSLError,
                             ValueError)
 
-_VALIDATE_CERTIFICATES_503_MESSAGE = (
-    """Service Unavailable. If you have recently changed
-    https_validate_certificates from True to False in your boto configuration
-    file, please delete any cached access tokens in your filesystem (at %s)
-    and try again.""" % GetCredentialStoreFilename())
-
-
 # Fields requiring projection=full across all API calls.
 _ACL_FIELDS_SET = set(['acl', 'defaultObjectAcl', 'items/acl',
                        'items/defaultObjectAcl', 'items/owner', 'owner'])
 
+# Fields that may be encrypted.
+_ENCRYPTED_HASHES_SET = set(['crc32c', 'md5Hash'])
+
+
+# During listing, if we attempt to decrypt an object but it has been removed,
+# we skip including the object in the listing.
+_SKIP_LISTING_OBJECT = 'skip'
+
 
 class GcsJsonApi(CloudApi):
   """Google Cloud Storage JSON implementation of gsutil Cloud API."""
@@ -163,7 +171,7 @@ class GcsJsonApi(CloudApi):
                                      perf_trace_token=perf_trace_token)
     no_op_credentials = False
     if not credentials:
-      loaded_credentials = self._CheckAndGetCredentials(logger)
+      loaded_credentials = self._CheckAndGetCredentials()
 
       if not loaded_credentials:
         loaded_credentials = NoOpCredentials()
@@ -261,7 +269,7 @@ class GcsJsonApi(CloudApi):
     if self.perf_trace_token:
       headers['cookie'] = self.perf_trace_token
 
-  def _CheckAndGetCredentials(self, logger):
+  def _CheckAndGetCredentials(self):
     configured_cred_types = []
     try:
       if self._HasOauth2UserAccountCreds():
@@ -296,16 +304,17 @@ class GcsJsonApi(CloudApi):
       devshell_creds = self._GetDevshellCreds()
       return user_creds or service_account_creds or gce_creds or devshell_creds
     except:  # pylint: disable=bare-except
-
       # If we didn't actually try to authenticate because there were multiple
       # types of configured credentials, don't emit this warning.
       if failed_cred_type:
+        if self.logger.isEnabledFor(logging.DEBUG):
+          self.logger.debug(traceback.format_exc())
         if os.environ.get('CLOUDSDK_WRAPPER') == '1':
-          logger.warn(
+          self.logger.warn(
               'Your "%s" credentials are invalid. Please run\n'
               '  $ gcloud auth login', failed_cred_type)
         else:
-          logger.warn(
+          self.logger.warn(
               'Your "%s" credentials are invalid. For more help, see '
               '"gsutil help creds", or re-run the gsutil config command (see '
               '"gsutil help config").', failed_cred_type)
@@ -405,6 +414,15 @@ class GcsJsonApi(CloudApi):
     """
     return fields is None or _ACL_FIELDS_SET.intersection(set(fields))
 
+  def _ValidateEncryptionFields(self, fields=None):
+    """Ensures customerEncryption field is included if hashes are requested."""
+    if (fields
+        and _ENCRYPTED_HASHES_SET.intersection(set(fields))
+        and 'customerEncryption' not in fields):
+      raise ArgumentException(
+          'gsutil client error: customerEncryption must be included when '
+          'requesting potentially encrypted fields %s' % _ENCRYPTED_HASHES_SET)
+
   def GetBucket(self, bucket_name, provider=None, fields=None):
     """See CloudApi class for function doc strings."""
     projection = (apitools_messages.StorageBucketsGetRequest
@@ -507,9 +525,9 @@ class GcsJsonApi(CloudApi):
     metadata.name = bucket_name
 
     if metadata.location:
-      metadata.location = metadata.location.upper()
+      metadata.location = metadata.location.lower()
     if metadata.storageClass:
-      metadata.storageClass = metadata.storageClass.upper()
+      metadata.storageClass = metadata.storageClass.lower()
 
     project_id = PopulateProjectId(project_id)
 
@@ -610,32 +628,84 @@ class GcsJsonApi(CloudApi):
       fields = set(fields)
       if 'nextPageToken' not in fields:
         fields.add('nextPageToken')
+      if ListToGetFields(list_fields=fields).intersection(
+          _ENCRYPTED_HASHES_SET):
+        # We may need to make a follow-up request to decrypt the hashes,
+        # so ensure the required fields are present.
+        fields.add('items/customerEncryption')
+        fields.add('items/generation')
+        fields.add('items/name')
       global_params.fields = ','.join(fields)
 
-    try:
-      object_list = self.api_client.objects.List(apitools_request,
-                                                 global_params=global_params)
-    except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
-      self._TranslateExceptionAndRaise(e, bucket_name=bucket_name)
+    list_page = True
+    next_page_token = None
 
-    for object_or_prefix in self._YieldObjectsAndPrefixes(object_list):
-      yield object_or_prefix
-
-    while object_list.nextPageToken:
+    while list_page:
+      list_page = False
       apitools_request = apitools_messages.StorageObjectsListRequest(
           bucket=bucket_name, prefix=prefix, delimiter=delimiter,
           versions=all_versions, projection=projection,
-          pageToken=object_list.nextPageToken,
-          maxResults=NUM_OBJECTS_PER_LIST_PAGE)
+          pageToken=next_page_token, maxResults=NUM_OBJECTS_PER_LIST_PAGE)
       try:
         object_list = self.api_client.objects.List(apitools_request,
                                                    global_params=global_params)
       except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
         self._TranslateExceptionAndRaise(e, bucket_name=bucket_name)
 
+      if object_list.nextPageToken:
+        list_page = True
+        next_page_token = object_list.nextPageToken
+
       for object_or_prefix in self._YieldObjectsAndPrefixes(object_list):
+        if object_or_prefix.datatype == CloudApi.CsObjectOrPrefixType.OBJECT:
+          decrypted_metadata = self._DecryptHashesIfPossible(
+              bucket_name, object_or_prefix.data,
+              fields=ListToGetFields(fields))
+          if decrypted_metadata == _SKIP_LISTING_OBJECT:
+            continue
+          elif decrypted_metadata:
+            object_or_prefix.data = decrypted_metadata
+          else:
+            # Object metadata was unencrypted or we did not have a matching
+            # key; yield what we received in the initial listing.
+            pass
+
         yield object_or_prefix
 
+  def _DecryptHashesIfPossible(self, bucket_name, object_metadata,
+                               fields=None):
+    """Attempts to decrypt object metadata.
+
+    This function issues a GetObjectMetadata request with a decryption key if
+    a matching one is found.
+
+    Args:
+      bucket_name: String of bucket containing the object.
+      object_metadata: apitools Object describing the object. Must include
+          name, generation, and customerEncryption fields.
+      fields: ListObjects-format fields to return.
+
+    Returns:
+      apitools Object with decrypted hashes if decryption was performed,
+      None if the object was not encrypted or a matching decryption key was not
+      found, or _SKIP_LISTING_OBJECT if the object no longer exists.
+    """
+    get_fields = ListToGetFields(list_fields=fields)
+    if self._ShouldTryWithEncryptionKey(object_metadata, fields=get_fields):
+      decryption_key = FindMatchingCryptoKey(
+          object_metadata.customerEncryption.keySha256)
+      if decryption_key:
+        # Issue a get request per-object with the encryption key.
+        # This requires an API call per-object and may be slow.
+        try:
+          return self._GetObjectMetadataWithDecryptionKey(
+              bucket_name, object_metadata.name, CryptoTuple(decryption_key),
+              object_metadata.generation, fields=get_fields)
+        except NotFoundException:
+          # If the object was deleted between the listing request and
+          # the get request, do not include it in the list.
+          return _SKIP_LISTING_OBJECT
+
   def _YieldObjectsAndPrefixes(self, object_list):
     # ls depends on iterating objects before prefixes for proper display.
     if object_list.items:
@@ -647,9 +717,83 @@ class GcsJsonApi(CloudApi):
         yield CloudApi.CsObjectOrPrefix(prefix,
                                         CloudApi.CsObjectOrPrefixType.PREFIX)
 
-  def GetObjectMetadata(self, bucket_name, object_name, generation=None,
-                        provider=None, fields=None):
-    """See CloudApi class for function doc strings."""
+  @contextmanager
+  def _ApitoolsRequestHeaders(self, headers):
+    """Wrapped code makes apitools requests with the specified headers."""
+    if headers:
+      old_headers = self.api_client.additional_http_headers.copy()
+      self.api_client.additional_http_headers.update(headers)
+    yield
+    if headers:
+      self.api_client.additional_http_headers = old_headers
+
+  def _EncryptionHeadersFromTuple(self, crypto_tuple=None):
+    """Returns headers dict matching crypto_tuple, or empty dict."""
+    headers = {}
+    if crypto_tuple:
+      headers['x-goog-encryption-algorithm'] = crypto_tuple.crypto_alg
+      headers['x-goog-encryption-key'] = crypto_tuple.crypto_key
+      headers['x-goog-encryption-key-sha256'] = (
+          Base64Sha256FromBase64EncryptionKey(crypto_tuple.crypto_key))
+    return headers
+
+  def _RewriteCryptoHeadersFromTuples(self, decryption_tuple=None,
+                                      encryption_tuple=None):
+    """Returns headers dict matching the provided tuples, or empty dict."""
+    headers = {}
+    if decryption_tuple:
+      headers['x-goog-copy-source-encryption-algorithm'] = (
+          decryption_tuple.crypto_alg)
+      headers['x-goog-copy-source-encryption-key'] = (
+          decryption_tuple.crypto_key)
+      headers['x-goog-copy-source-encryption-key-sha256'] = (
+          Base64Sha256FromBase64EncryptionKey(decryption_tuple.crypto_key))
+
+    if encryption_tuple:
+      headers['x-goog-encryption-algorithm'] = encryption_tuple.crypto_alg
+      headers['x-goog-encryption-key'] = encryption_tuple.crypto_key
+      headers['x-goog-encryption-key-sha256'] = (
+          Base64Sha256FromBase64EncryptionKey(encryption_tuple.crypto_key))
+
+    return headers
+
+  def _GetObjectMetadataWithDecryptionKey(
+      self, bucket_name, object_name, decryption_tuple, generation=None,
+      fields=None):
+    """Helper function to get encrypted object metadata.
+
+    Args:
+      bucket_name: Bucket containing the object.
+      object_name: Object name.
+      decryption_tuple: CryptoTuple with decryption key.
+      generation: Generation of the object to retrieve.
+      fields: If present, return only these Object metadata fields, for
+              example, ['acl', 'updated'].
+
+    Raises:
+      ArgumentException for errors during input validation.
+      ServiceException for errors interacting with cloud storage providers.
+
+    Returns:
+      Object object.
+    """
+    apitools_request = self._GetApitoolsObjectMetadataRequest(
+        bucket_name, object_name, generation=generation, fields=fields)
+    global_params = self._GetApitoolsObjectMetadataGlobalParams(fields=fields)
+
+    try:
+      with self._ApitoolsRequestHeaders(
+          self._EncryptionHeadersFromTuple(crypto_tuple=decryption_tuple)):
+        return self.api_client.objects.Get(
+            apitools_request, global_params=global_params)
+    except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
+      self._TranslateExceptionAndRaise(e, bucket_name=bucket_name,
+                                       object_name=object_name,
+                                       generation=generation)
+
+  def _GetApitoolsObjectMetadataRequest(self, bucket_name, object_name,
+                                        generation=None, fields=None):
+    self._ValidateEncryptionFields(fields=fields)
     projection = (apitools_messages.StorageObjectsGetRequest
                   .ProjectionValueValuesEnum.noAcl)
     if self._FieldsContainsAclField(fields):
@@ -659,27 +803,84 @@ class GcsJsonApi(CloudApi):
     if generation:
       generation = long(generation)
 
-    apitools_request = apitools_messages.StorageObjectsGetRequest(
+    return apitools_messages.StorageObjectsGetRequest(
         bucket=bucket_name, object=object_name, projection=projection,
         generation=generation)
+
+  def _GetApitoolsObjectMetadataGlobalParams(self, fields=None):
     global_params = apitools_messages.StandardQueryParameters()
     if fields:
       global_params.fields = ','.join(set(fields))
+    return global_params
+
+  def GetObjectMetadata(self, bucket_name, object_name, generation=None,
+                        provider=None, fields=None):
+    """See CloudApi class for function doc strings."""
+    apitools_request = self._GetApitoolsObjectMetadataRequest(
+        bucket_name, object_name, generation=generation, fields=fields)
+    global_params = self._GetApitoolsObjectMetadataGlobalParams(fields=fields)
 
     try:
-      return self.api_client.objects.Get(apitools_request,
-                                         global_params=global_params)
+      object_metadata = self.api_client.objects.Get(apitools_request,
+                                                    global_params=global_params)
+      if self._ShouldTryWithEncryptionKey(object_metadata, fields=fields):
+        key_sha256 = object_metadata.customerEncryption.keySha256
+        decryption_key = FindMatchingCryptoKey(key_sha256)
+        if not decryption_key:
+          raise EncryptionException(
+              'Missing decryption key with SHA256 hash %s. No decryption key '
+              'matches object %s://%s/%s' % (key_sha256, self.provider,
+                                             bucket_name, object_name))
+        return self._GetObjectMetadataWithDecryptionKey(
+            bucket_name, object_name, CryptoTuple(decryption_key),
+            generation=generation, fields=fields)
+      else:
+        return object_metadata
     except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
       self._TranslateExceptionAndRaise(e, bucket_name=bucket_name,
                                        object_name=object_name,
                                        generation=generation)
 
+  def _ShouldTryWithEncryptionKey(self, object_metadata, fields=None):
+    """Checks if a metadata request should be re-issued with a decryption key.
+
+    Args:
+      object_metadata: apitools Object metadata from a request without a
+          decryption key; must include customerEncryption field if hashes
+          are requested.
+      fields: Requested fields.
+
+    Returns:
+      True if request requires a decryption key and should be re-issued, False
+      otherwise.
+
+    Raises:
+      ServiceException if service did not provide at least one of the requested
+      hashes or the customerEncryption field.
+    """
+    if fields is not None and not _ENCRYPTED_HASHES_SET.intersection(fields):
+      # No potentially encrypted metadata requested.
+      return False
+
+    if object_metadata.customerEncryption:
+      # Object is encrypted.
+      return True
+
+    need_hash = fields is None or 'md5Hash' in fields or 'crc32c' in fields
+    has_hash = object_metadata.md5Hash or object_metadata.crc32c
+    if need_hash and not has_hash:
+      raise ServiceException('Service did not provide requested hashes, '
+                             'but customerEncryption field is missing.')
+
+    return False
+
   def GetObjectMedia(
       self, bucket_name, object_name, download_stream,
       provider=None, generation=None, object_size=None,
+      compressed_encoding=False,
       download_strategy=CloudApi.DownloadStrategy.ONE_SHOT, start_byte=0,
       end_byte=None, progress_callback=None, serialization_data=None,
-      digesters=None):
+      digesters=None, decryption_tuple=None):
     """See CloudApi class for function doc strings."""
     # This implementation will get the object metadata first if we don't pass it
     # in via serialization_data.
@@ -716,6 +917,15 @@ class GcsJsonApi(CloudApi):
     self.download_http.connections = {'https': download_http_class}
 
     if serialization_data:
+      # If we have an apiary trace token, add it to the URL.
+      # TODO: Add query parameter support to apitools downloads so there is
+      # a well-defined way to express query parameters. Currently, we assume
+      # the URL ends in ?alt=media, and this will break if that changes.
+      if self.trace_token:
+        serialization_dict = json.loads(serialization_data)
+        serialization_dict['url'] += '&trace=token%%3A%s' % self.trace_token
+        serialization_data = json.dumps(serialization_dict)
+
       apitools_download = apitools_transfer.Download.FromData(
           download_stream, serialization_data, self.api_client.http,
           num_retries=self.num_retries)
@@ -739,13 +949,18 @@ class GcsJsonApi(CloudApi):
         return self._PerformResumableDownload(
             bucket_name, object_name, download_stream, apitools_request,
             apitools_download, bytes_downloaded_container,
+            compressed_encoding=compressed_encoding,
             generation=generation, start_byte=start_byte, end_byte=end_byte,
-            serialization_data=serialization_data)
+            serialization_data=serialization_data,
+            decryption_tuple=decryption_tuple)
       else:
         return self._PerformDownload(
             bucket_name, object_name, download_stream, apitools_request,
-            apitools_download, generation=generation, start_byte=start_byte,
-            end_byte=end_byte, serialization_data=serialization_data)
+            apitools_download, generation=generation,
+            compressed_encoding=compressed_encoding,
+            start_byte=start_byte, end_byte=end_byte,
+            serialization_data=serialization_data,
+            decryption_tuple=decryption_tuple)
     except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
       self._TranslateExceptionAndRaise(e, bucket_name=bucket_name,
                                        object_name=object_name,
@@ -754,15 +969,18 @@ class GcsJsonApi(CloudApi):
   def _PerformResumableDownload(
       self, bucket_name, object_name, download_stream, apitools_request,
       apitools_download, bytes_downloaded_container, generation=None,
-      start_byte=0, end_byte=None, serialization_data=None):
+      compressed_encoding=False, start_byte=0, end_byte=None,
+      serialization_data=None, decryption_tuple=None):
     retries = 0
     last_progress_byte = start_byte
     while retries <= self.num_retries:
       try:
         return self._PerformDownload(
             bucket_name, object_name, download_stream, apitools_request,
-            apitools_download, generation=generation, start_byte=start_byte,
-            end_byte=end_byte, serialization_data=serialization_data)
+            apitools_download, generation=generation,
+            compressed_encoding=compressed_encoding, start_byte=start_byte,
+            end_byte=end_byte, serialization_data=serialization_data,
+            decryption_tuple=decryption_tuple)
       except HTTP_TRANSFER_EXCEPTIONS, e:
         self._ValidateHttpAccessTokenRefreshError(e)
         start_byte = download_stream.tell()
@@ -787,8 +1005,9 @@ class GcsJsonApi(CloudApi):
 
   def _PerformDownload(
       self, bucket_name, object_name, download_stream, apitools_request,
-      apitools_download, generation=None, start_byte=0, end_byte=None,
-      serialization_data=None):
+      apitools_download, generation=None, compressed_encoding=False,
+      start_byte=0, end_byte=None, serialization_data=None,
+      decryption_tuple=None):
     if not serialization_data:
       try:
         self.api_client.objects.Get(apitools_request,
@@ -814,10 +1033,15 @@ class GcsJsonApi(CloudApi):
     # Since bytes_http is created in this function, we don't get the
     # user-agent header from api_client's http automatically.
     additional_headers = {
-        'accept-encoding': 'gzip',
-        'user-agent': self.api_client.user_agent
+        'user-agent': self.api_client.user_agent,
+        'accept-encoding': 'gzip'
     }
+    AddAcceptEncodingGzipIfNeeded(additional_headers,
+                                  compressed_encoding=compressed_encoding)
+
     self._AddPerfTraceTokenToHeaders(additional_headers)
+    additional_headers.update(
+        self._EncryptionHeadersFromTuple(decryption_tuple))
 
     if start_byte or end_byte is not None:
       apitools_download.GetRange(additional_headers=additional_headers,
@@ -875,9 +1099,9 @@ class GcsJsonApi(CloudApi):
                                        generation=generation)
 
   def _UploadObject(self, upload_stream, object_metadata, canned_acl=None,
-                    size=None, preconditions=None, provider=None, fields=None,
-                    serialization_data=None, tracker_callback=None,
-                    progress_callback=None,
+                    size=None, preconditions=None, encryption_tuple=None,
+                    provider=None, fields=None, serialization_data=None,
+                    tracker_callback=None, progress_callback=None,
                     apitools_strategy=apitools_transfer.SIMPLE_UPLOAD,
                     total_size=0):
     # pylint: disable=g-doc-args
@@ -907,7 +1131,8 @@ class GcsJsonApi(CloudApi):
 
     callback_class_factory = UploadCallbackConnectionClassFactory(
         bytes_uploaded_container, total_size=total_size,
-        progress_callback=progress_callback)
+        progress_callback=progress_callback, logger=self.logger,
+        debug=self.debug)
 
     upload_http_class = callback_class_factory.GetConnectionClass()
     self.upload_http.connections = {'http': upload_http_class,
@@ -924,6 +1149,7 @@ class GcsJsonApi(CloudApi):
       content_type = None
       apitools_request = None
       global_params = None
+
       if not serialization_data:
         # This is a new upload, set up initial upload state.
         content_type = object_metadata.contentType
@@ -942,6 +1168,9 @@ class GcsJsonApi(CloudApi):
         if fields:
           global_params.fields = ','.join(set(fields))
 
+      encryption_headers = self._EncryptionHeadersFromTuple(
+          crypto_tuple=encryption_tuple)
+
       if apitools_strategy == apitools_transfer.SIMPLE_UPLOAD:
         # One-shot upload.
         apitools_upload = apitools_transfer.Upload(
@@ -950,11 +1179,19 @@ class GcsJsonApi(CloudApi):
         apitools_upload.strategy = apitools_strategy
         apitools_upload.bytes_http = self.authorized_upload_http
 
-        return self.api_client.objects.Insert(
-            apitools_request,
-            upload=apitools_upload,
-            global_params=global_params)
+        with self._ApitoolsRequestHeaders(encryption_headers):
+          return self.api_client.objects.Insert(
+              apitools_request,
+              upload=apitools_upload,
+              global_params=global_params)
       else:  # Resumable upload.
+        # Since bytes_http is created in this function, we don't get the
+        # user-agent header from api_client's http automatically.
+        additional_headers = {
+            'user-agent': self.api_client.user_agent
+        }
+        additional_headers.update(encryption_headers)
+
         return self._PerformResumableUpload(
             upload_stream, self.authorized_upload_http, content_type, size,
             serialization_data, apitools_strategy, apitools_request,
@@ -988,10 +1225,11 @@ class GcsJsonApi(CloudApi):
             num_retries=self.num_retries)
         apitools_upload.strategy = apitools_strategy
         apitools_upload.bytes_http = authorized_upload_http
-        self.api_client.objects.Insert(
-            apitools_request,
-            upload=apitools_upload,
-            global_params=global_params)
+        with self._ApitoolsRequestHeaders(addl_headers):
+          self.api_client.objects.Insert(
+              apitools_request,
+              upload=apitools_upload,
+              global_params=global_params)
       # Disable retries in apitools. We will handle them explicitly here.
       apitools_upload.retry_func = (
           apitools_http_wrapper.RethrowExceptionHandler)
@@ -1004,6 +1242,7 @@ class GcsJsonApi(CloudApi):
       # from the server how many bytes it already has. Update our
       # callback class with this information.
       bytes_uploaded_container.bytes_transferred = apitools_upload.progress
+
       if tracker_callback:
         tracker_callback(json.dumps(apitools_upload.serialization_data))
 
@@ -1014,7 +1253,7 @@ class GcsJsonApi(CloudApi):
           # TODO: On retry, this will seek to the bytes that the server has,
           # causing the hash to be recalculated. Make HashingFileUploadWrapper
           # save a digest according to json_resumable_chunk_size.
-          if size:
+          if size and not JsonResumableChunkSizeDefined():
             # If size is known, we can send it all in one request and avoid
             # making a round-trip per chunk.
             http_response = apitools_upload.StreamMedia(
@@ -1087,18 +1326,18 @@ class GcsJsonApi(CloudApi):
 
   def UploadObject(self, upload_stream, object_metadata, canned_acl=None,
                    size=None, preconditions=None, progress_callback=None,
-                   provider=None, fields=None):
+                   encryption_tuple=None, provider=None, fields=None):
     """See CloudApi class for function doc strings."""
     return self._UploadObject(
         upload_stream, object_metadata, canned_acl=canned_acl,
         size=size, preconditions=preconditions,
-        progress_callback=progress_callback, fields=fields,
-        apitools_strategy=apitools_transfer.SIMPLE_UPLOAD)
+        progress_callback=progress_callback, encryption_tuple=encryption_tuple,
+        fields=fields, apitools_strategy=apitools_transfer.SIMPLE_UPLOAD)
 
-  def UploadObjectStreaming(self, upload_stream, object_metadata,
-                            canned_acl=None, preconditions=None,
-                            progress_callback=None, provider=None,
-                            fields=None):
+  def UploadObjectStreaming(
+      self, upload_stream, object_metadata, canned_acl=None, preconditions=None,
+      progress_callback=None, encryption_tuple=None, provider=None,
+      fields=None):
     """See CloudApi class for function doc strings."""
     # Streaming indicated by not passing a size.
     # Resumable capabilities are present up to the resumable chunk size using
@@ -1106,24 +1345,27 @@ class GcsJsonApi(CloudApi):
     return self._UploadObject(
         upload_stream, object_metadata, canned_acl=canned_acl,
         preconditions=preconditions, progress_callback=progress_callback,
-        fields=fields, apitools_strategy=apitools_transfer.RESUMABLE_UPLOAD,
-        total_size=None)
+        encryption_tuple=encryption_tuple, fields=fields,
+        apitools_strategy=apitools_transfer.RESUMABLE_UPLOAD, total_size=None)
 
   def UploadObjectResumable(
       self, upload_stream, object_metadata, canned_acl=None, preconditions=None,
-      provider=None, fields=None, size=None, serialization_data=None,
-      tracker_callback=None, progress_callback=None):
+      size=None, serialization_data=None, tracker_callback=None,
+      progress_callback=None, encryption_tuple=None, provider=None,
+      fields=None):
     """See CloudApi class for function doc strings."""
     return self._UploadObject(
         upload_stream, object_metadata, canned_acl=canned_acl,
         preconditions=preconditions, fields=fields, size=size,
         serialization_data=serialization_data,
         tracker_callback=tracker_callback, progress_callback=progress_callback,
+        encryption_tuple=encryption_tuple,
         apitools_strategy=apitools_transfer.RESUMABLE_UPLOAD)
 
   def CopyObject(self, src_obj_metadata, dst_obj_metadata, src_generation=None,
                  canned_acl=None, preconditions=None, progress_callback=None,
-                 max_bytes_per_call=None, provider=None, fields=None):
+                 max_bytes_per_call=None, encryption_tuple=None,
+                 decryption_tuple=None, provider=None, fields=None):
     """See CloudApi class for function doc strings."""
     ValidateDstObjectMetadata(dst_obj_metadata)
     predefined_acl = None
@@ -1153,6 +1395,13 @@ class GcsJsonApi(CloudApi):
         new_fields.add('resource/' + field)
       global_params.fields = ','.join(set(new_fields))
 
+    dec_key_sha256 = (
+        Base64Sha256FromBase64EncryptionKey(decryption_tuple.crypto_key)
+        if decryption_tuple else None)
+    enc_key_sha256 = (
+        Base64Sha256FromBase64EncryptionKey(encryption_tuple.crypto_key)
+        if encryption_tuple else None)
+
     # Check to see if we are resuming a rewrite.
     tracker_file_name = GetRewriteTrackerFilePath(
         src_obj_metadata.bucket, src_obj_metadata.name, dst_obj_metadata.bucket,
@@ -1161,29 +1410,33 @@ class GcsJsonApi(CloudApi):
         src_obj_metadata, dst_obj_metadata, projection,
         src_generation=src_generation, gen_match=preconditions.gen_match,
         meta_gen_match=preconditions.meta_gen_match,
-        canned_acl=predefined_acl, fields=global_params.fields,
-        max_bytes_per_call=max_bytes_per_call)
+        canned_acl=predefined_acl, max_bytes_per_call=max_bytes_per_call,
+        src_dec_key_sha256=dec_key_sha256, dst_enc_key_sha256=enc_key_sha256,
+        fields=global_params.fields)
     resume_rewrite_token = ReadRewriteTrackerFile(tracker_file_name,
                                                   rewrite_params_hash)
+    crypto_headers = self._RewriteCryptoHeadersFromTuples(
+        decryption_tuple=decryption_tuple, encryption_tuple=encryption_tuple)
 
     progress_cb_with_backoff = None
     try:
       last_bytes_written = 0L
       while True:
-        apitools_request = apitools_messages.StorageObjectsRewriteRequest(
-            sourceBucket=src_obj_metadata.bucket,
-            sourceObject=src_obj_metadata.name,
-            destinationBucket=dst_obj_metadata.bucket,
-            destinationObject=dst_obj_metadata.name,
-            projection=projection, object=dst_obj_metadata,
-            sourceGeneration=src_generation,
-            ifGenerationMatch=preconditions.gen_match,
-            ifMetagenerationMatch=preconditions.meta_gen_match,
-            destinationPredefinedAcl=predefined_acl,
-            rewriteToken=resume_rewrite_token,
-            maxBytesRewrittenPerCall=max_bytes_per_call)
-        rewrite_response = self.api_client.objects.Rewrite(
-            apitools_request, global_params=global_params)
+        with self._ApitoolsRequestHeaders(crypto_headers):
+          apitools_request = apitools_messages.StorageObjectsRewriteRequest(
+              sourceBucket=src_obj_metadata.bucket,
+              sourceObject=src_obj_metadata.name,
+              destinationBucket=dst_obj_metadata.bucket,
+              destinationObject=dst_obj_metadata.name,
+              projection=projection, object=dst_obj_metadata,
+              sourceGeneration=src_generation,
+              ifGenerationMatch=preconditions.gen_match,
+              ifMetagenerationMatch=preconditions.meta_gen_match,
+              destinationPredefinedAcl=predefined_acl,
+              rewriteToken=resume_rewrite_token,
+              maxBytesRewrittenPerCall=max_bytes_per_call)
+          rewrite_response = self.api_client.objects.Rewrite(
+              apitools_request, global_params=global_params)
         bytes_written = long(rewrite_response.totalBytesRewritten)
         if progress_callback and not progress_cb_with_backoff:
           progress_cb_with_backoff = ProgressCallbackWithBackoff(
@@ -1233,7 +1486,8 @@ class GcsJsonApi(CloudApi):
                                        generation=generation)
 
   def ComposeObject(self, src_objs_metadata, dst_obj_metadata,
-                    preconditions=None, provider=None, fields=None):
+                    preconditions=None, encryption_tuple=None,
+                    provider=None, fields=None):
     """See CloudApi class for function doc strings."""
     ValidateDstObjectMetadata(dst_obj_metadata)
 
@@ -1254,21 +1508,26 @@ class GcsJsonApi(CloudApi):
     src_objs_compose_request = apitools_messages.ComposeRequest(
         sourceObjects=src_objs_metadata, destination=dst_obj_metadata)
 
-    apitools_request = apitools_messages.StorageObjectsComposeRequest(
-        composeRequest=src_objs_compose_request,
-        destinationBucket=dst_bucket_name,
-        destinationObject=dst_obj_name,
-        ifGenerationMatch=preconditions.gen_match,
-        ifMetagenerationMatch=preconditions.meta_gen_match)
-    try:
-      return self.api_client.objects.Compose(apitools_request,
-                                             global_params=global_params)
-    except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
-      # We can't be sure which object was missing in the 404 case.
-      if isinstance(e, apitools_exceptions.HttpError) and e.status_code == 404:
-        raise NotFoundException('One of the source objects does not exist.')
-      else:
-        self._TranslateExceptionAndRaise(e)
+    encryption_headers = self._EncryptionHeadersFromTuple(
+        crypto_tuple=encryption_tuple)
+
+    with self._ApitoolsRequestHeaders(encryption_headers):
+      apitools_request = apitools_messages.StorageObjectsComposeRequest(
+          composeRequest=src_objs_compose_request,
+          destinationBucket=dst_bucket_name,
+          destinationObject=dst_obj_name,
+          ifGenerationMatch=preconditions.gen_match,
+          ifMetagenerationMatch=preconditions.meta_gen_match)
+      try:
+        return self.api_client.objects.Compose(apitools_request,
+                                               global_params=global_params)
+      except TRANSLATABLE_APITOOLS_EXCEPTIONS, e:
+        # We can't be sure which object was missing in the 404 case.
+        if (isinstance(e, apitools_exceptions.HttpError)
+            and e.status_code == 404):
+          raise NotFoundException('One of the source objects does not exist.')
+        else:
+          self._TranslateExceptionAndRaise(e)
 
   def WatchBucket(self, bucket_name, address, channel_id, token=None,
                   provider=None, fields=None):
@@ -1369,6 +1628,9 @@ class GcsJsonApi(CloudApi):
       Translated CloudApi exception, or the original exception if it was not
       translatable.
     """
+    if self.logger.isEnabledFor(logging.DEBUG):
+      self.logger.debug(
+          'TranslateExceptionAndRaise: %s', traceback.format_exc())
     translated_exception = self._TranslateApitoolsException(
         e, bucket_name=bucket_name, object_name=object_name,
         generation=generation, not_found_exception=not_found_exception)
@@ -1391,11 +1653,7 @@ class GcsJsonApi(CloudApi):
   def _TranslateApitoolsResumableUploadException(self, e):
     if isinstance(e, apitools_exceptions.HttpError):
       message = self._GetMessageFromHttpError(e)
-      if (e.status_code == 503 and
-          self.http.disable_ssl_certificate_validation):
-        return ServiceException(_VALIDATE_CERTIFICATES_503_MESSAGE,
-                                status=e.status_code)
-      elif e.status_code >= 500:
+      if e.status_code >= 500:
         return ResumableUploadException(
             message or 'Server Error', status=e.status_code)
       elif e.status_code == 429:
@@ -1463,14 +1721,14 @@ class GcsJsonApi(CloudApi):
           return AccessDeniedException(
               'The bucket you tried to create requires domain ownership '
               'verification. Please see '
-              'https://developers.google.com/storage/docs/bucketnaming'
+              'https://cloud.google.com/storage/docs/naming'
               '?hl=en#verification for more details.', status=e.status_code)
         elif 'User Rate Limit Exceeded' in str(e):
           return AccessDeniedException('Rate limit exceeded. Please retry this '
                                        'request later.', status=e.status_code)
         elif 'Access Not Configured' in str(e):
           return AccessDeniedException(
-              'Access Not Configured. Please go to the Google Developers '
+              'Access Not Configured. Please go to the Google Cloud Platform '
               'Console (https://cloud.google.com/console#/project) for your '
               'project, select APIs and Auth and enable the '
               'Google Cloud Storage JSON API.',
@@ -1501,10 +1759,6 @@ class GcsJsonApi(CloudApi):
             'Bucket %s already exists.' % bucket_name, status=e.status_code)
       elif e.status_code == 412:
         return PreconditionException(message, status=e.status_code)
-      elif (e.status_code == 503 and
-            not self.http.disable_ssl_certificate_validation):
-        return ServiceException(_VALIDATE_CERTIFICATES_503_MESSAGE,
-                                status=e.status_code)
       return ServiceException(message, status=e.status_code)
     elif isinstance(e, apitools_exceptions.TransferInvalidError):
       return ServiceException('Transfer invalid (possible encoding error: %s)'
